Package: mapme.biodiversity
Title: Efficient Monitoring of Global Biodiversity Portfolios
Version: 0.9.6
Authors@R: c(person(given = "Darius A.", 
                    family = "G\u00F6rgen", 
                    role = "aut",
                    comment = c(ORCID = "0009-0008-5503-7704")),
             person(given = "Om Prakash", 
                    family = "Bhandari", 
                    role = "aut"),
             person(given = "Andreas",
                    family = "Petutschnig",
                    role = "ctb",
                    comment = c(ORCID = "0000-0001-5029-2425")),
             person(given = "Sven", 
                    family = "Bergtold",
                    role = "ctb"),
             person(given = "Zivan", 
                    family = "Karaman",
                    email =  "zivan.karaman@gmail.com",
                    role = c("ctb", "cre"),
                    comment = c(ORCID = "0000-0002-8933-4589")),
             person(given = "MAPME-Initiative",
                    email = "contact@mapme-initiative.org",
                    role = c("cph", "fnd"))
                    )
Description: Biodiversity areas, especially primary forest, serve a
    multitude of functions for local economy, regional functionality of
    the ecosystems as well as the global health of our planet. Recently,
    adverse changes in human land use practices and climatic responses to
    increased greenhouse gas emissions, put these biodiversity areas under
    a variety of different threats. The present package helps to analyse a
    number of biodiversity indicators based on freely available
    geographical datasets. It supports computational efficient routines
    that allow the analysis of potentially global biodiversity portfolios.
    The primary use case of the package is to support evidence based
    reporting of an organization's effort to protect biodiversity areas
    under threat and to identify regions were intervention is most duly
    needed.
License: GPL (>= 3)
URL: https://mapme-initiative.github.io/mapme.biodiversity/,
        https://github.com/mapme-initiative/mapme.biodiversity/
BugReports: https://github.com/mapme-initiative/mapme.biodiversity/issues
Depends: R (>= 3.5.0)
SystemRequirements: GDAL (>= 3.7.0), PROJ (>= 4.8.0)
Imports: curl, dplyr, furrr, httr2, jsonlite, magrittr, purrr, sf,
        terra, tibble, tidyr, utils
Suggests: exactextractr, future, knitr, landscapemetrics, progressr,
        rmarkdown, rstac, rvest, testthat (>= 3.0.0)
VignetteBuilder: knitr
Config/Needs/website: DiagrammeR
Config/testthat/edition: 3
Encoding: UTF-8
Language: en-GB
RoxygenNote: 7.3.3
Collate: 'register.R' 'calc_biodiversity_intactness_index.R'
        'calc_biome.R' 'calc_burned_area.R' 'calc_carbon.R'
        'calc_deforestation_drivers.R' 'calc_drought_indicator.R'
        'calc_ecoregion.R' 'calc_elevation.R'
        'calc_exposed_population_acled.R'
        'calc_exposed_population_ucdp.R' 'calc_fatalities_acled.R'
        'calc_fatalities_ucdp.R' 'calc_gsw_change.R'
        'calc_gsw_occurrence.R' 'calc_gsw_recurrence.R'
        'calc_gsw_seasonality.R' 'calc_gsw_time_series.R'
        'calc_gsw_transitions.R' 'calc_hfp.R' 'get_resources.R'
        'calc_indicators.R' 'calc_ipbes_biomes.R' 'calc_iucn.R'
        'calc_key_biodiversity_areas.R' 'calc_landcover.R'
        'calc_mangroves_area.R' 'calc_population_count.R'
        'calc_precipitation_chelsa.R' 'calc_precipitation_chirps.R'
        'calc_precipitation_wc.R' 'calc_slope.R'
        'calc_soilproperties.R' 'calc_temperature_max_wc.R'
        'calc_temperature_min_wc.R' 'calc_traveltime.R'
        'calc_traveltime_2000.R' 'calc_treecover_area.R'
        'calc_treecover_area_and_emissions.R'
        'calc_treecoverloss_emissions.R' 'calc_tri.R' 'chunking.R'
        'engines.R' 'get_accessibility_2000.R' 'get_acled.R'
        'get_biodiversity_intactness_index.R' 'get_carbon.R'
        'get_chelsa.R' 'get_chirps.R' 'get_esalandcover.R'
        'get_fritz_et_al.R' 'get_gfw_emissions.R' 'get_gfw_lossyear.R'
        'get_gfw_treecover.R' 'get_gmw.R' 'get_gsw.R'
        'get_gsw_time_series.R' 'get_hfp.R' 'get_ipbes_biomes.R'
        'get_iucn.R' 'get_key_biodiversity_areas.R' 'get_mcd64A1.R'
        'get_nasa_grace.R' 'get_nasa_srtm.R' 'get_nelson_et_al.R'
        'get_soilgrids.R' 'get_teow.R' 'get_ucdp_ged.R'
        'get_worldclim.R' 'get_worldpop.R' 'mapme.biodiversity-pkg.R'
        'portfolio.R' 'spatial-utils.R' 'utils.R'
NeedsCompilation: no
Packaged: 2026-04-27 07:13:17 UTC; zivankaraman
Author: Darius A. Görgen [aut] (ORCID: <https://orcid.org/0009-0008-5503-7704>),
  Om Prakash Bhandari [aut],
  Andreas Petutschnig [ctb] (ORCID:
    <https://orcid.org/0000-0001-5029-2425>),
  Sven Bergtold [ctb],
  Zivan Karaman [ctb, cre] (ORCID:
    <https://orcid.org/0000-0002-8933-4589>),
  MAPME-Initiative [cph, fnd]
Maintainer: Zivan Karaman <zivan.karaman@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-27 09:20:12 UTC
Built: R 4.5.2; ; 2026-04-27 09:39:15 UTC; unix

Package: mapme.biodiversity
Title: Efficient Monitoring of Global Biodiversity Portfolios
Version: 0.9.6
Authors@R: c(person(given = "Darius A.", 
                    family = "G\u00F6rgen", 
                    role = "aut",
                    comment = c(ORCID = "0009-0008-5503-7704")),
             person(given = "Om Prakash", 
                    family = "Bhandari", 
                    role = "aut"),
             person(given = "Andreas",
                    family = "Petutschnig",
                    role = "ctb",
                    comment = c(ORCID = "0000-0001-5029-2425")),
             person(given = "Sven", 
                    family = "Bergtold",
                    role = "ctb"),
             person(given = "Zivan", 
                    family = "Karaman",
                    email =  "zivan.karaman@gmail.com",
                    role = c("ctb", "cre"),
                    comment = c(ORCID = "0000-0002-8933-4589")),
             person(given = "MAPME-Initiative",
                    email = "contact@mapme-initiative.org",
                    role = c("cph", "fnd"))
                    )
Description: Biodiversity areas, especially primary forest, serve a
    multitude of functions for local economy, regional functionality of
    the ecosystems as well as the global health of our planet. Recently,
    adverse changes in human land use practices and climatic responses to
    increased greenhouse gas emissions, put these biodiversity areas under
    a variety of different threats. The present package helps to analyse a
    number of biodiversity indicators based on freely available
    geographical datasets. It supports computational efficient routines
    that allow the analysis of potentially global biodiversity portfolios.
    The primary use case of the package is to support evidence based
    reporting of an organization's effort to protect biodiversity areas
    under threat and to identify regions were intervention is most duly
    needed.
License: GPL (>= 3)
URL: https://mapme-initiative.github.io/mapme.biodiversity/,
        https://github.com/mapme-initiative/mapme.biodiversity/
BugReports: https://github.com/mapme-initiative/mapme.biodiversity/issues
Depends: R (>= 3.5.0)
SystemRequirements: GDAL (>= 3.7.0), PROJ (>= 4.8.0)
Imports: curl, dplyr, furrr, httr2, jsonlite, magrittr, purrr, sf,
        terra, tibble, tidyr, utils
Suggests: exactextractr, future, knitr, landscapemetrics, progressr,
        rmarkdown, rstac, rvest, testthat (>= 3.0.0)
VignetteBuilder: knitr
Config/Needs/website: DiagrammeR
Config/testthat/edition: 3
Encoding: UTF-8
Language: en-GB
RoxygenNote: 7.3.3
Collate: 'register.R' 'calc_biodiversity_intactness_index.R'
        'calc_biome.R' 'calc_burned_area.R' 'calc_carbon.R'
        'calc_deforestation_drivers.R' 'calc_drought_indicator.R'
        'calc_ecoregion.R' 'calc_elevation.R'
        'calc_exposed_population_acled.R'
        'calc_exposed_population_ucdp.R' 'calc_fatalities_acled.R'
        'calc_fatalities_ucdp.R' 'calc_gsw_change.R'
        'calc_gsw_occurrence.R' 'calc_gsw_recurrence.R'
        'calc_gsw_seasonality.R' 'calc_gsw_time_series.R'
        'calc_gsw_transitions.R' 'calc_hfp.R' 'get_resources.R'
        'calc_indicators.R' 'calc_ipbes_biomes.R' 'calc_iucn.R'
        'calc_key_biodiversity_areas.R' 'calc_landcover.R'
        'calc_mangroves_area.R' 'calc_population_count.R'
        'calc_precipitation_chelsa.R' 'calc_precipitation_chirps.R'
        'calc_precipitation_wc.R' 'calc_slope.R'
        'calc_soilproperties.R' 'calc_temperature_max_wc.R'
        'calc_temperature_min_wc.R' 'calc_traveltime.R'
        'calc_traveltime_2000.R' 'calc_treecover_area.R'
        'calc_treecover_area_and_emissions.R'
        'calc_treecoverloss_emissions.R' 'calc_tri.R' 'chunking.R'
        'engines.R' 'get_accessibility_2000.R' 'get_acled.R'
        'get_biodiversity_intactness_index.R' 'get_carbon.R'
        'get_chelsa.R' 'get_chirps.R' 'get_esalandcover.R'
        'get_fritz_et_al.R' 'get_gfw_emissions.R' 'get_gfw_lossyear.R'
        'get_gfw_treecover.R' 'get_gmw.R' 'get_gsw.R'
        'get_gsw_time_series.R' 'get_hfp.R' 'get_ipbes_biomes.R'
        'get_iucn.R' 'get_key_biodiversity_areas.R' 'get_mcd64A1.R'
        'get_nasa_grace.R' 'get_nasa_srtm.R' 'get_nelson_et_al.R'
        'get_soilgrids.R' 'get_teow.R' 'get_ucdp_ged.R'
        'get_worldclim.R' 'get_worldpop.R' 'mapme.biodiversity-pkg.R'
        'portfolio.R' 'spatial-utils.R' 'utils.R'
NeedsCompilation: no
Packaged: 2026-04-27 07:13:17 UTC; zivankaraman
Author: Darius A. Görgen [aut] (ORCID: <https://orcid.org/0009-0008-5503-7704>),
  Om Prakash Bhandari [aut],
  Andreas Petutschnig [ctb] (ORCID:
    <https://orcid.org/0000-0001-5029-2425>),
  Sven Bergtold [ctb],
  Zivan Karaman [ctb, cre] (ORCID:
    <https://orcid.org/0000-0002-8933-4589>),
  MAPME-Initiative [cph, fnd]
Maintainer: Zivan Karaman <zivan.karaman@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-27 09:20:12 UTC
Built: R 4.6.0; ; 2026-04-27 09:55:00 UTC; unix

# Generated by roxygen2: do not edit by hand

export(available_indicators)
export(available_resources)
export(calc_biodiversity_intactness_index)
export(calc_biome)
export(calc_burned_area)
export(calc_deforestation_drivers)
export(calc_drought_indicator)
export(calc_ecoregion)
export(calc_elevation)
export(calc_exposed_population_acled)
export(calc_exposed_population_ucdp)
export(calc_fatalities_acled)
export(calc_fatalities_ucdp)
export(calc_gsw_change)
export(calc_gsw_occurrence)
export(calc_gsw_recurrence)
export(calc_gsw_seasonality)
export(calc_gsw_time_series)
export(calc_gsw_transitions)
export(calc_humanfootprint)
export(calc_indicators)
export(calc_ipbes_biomes)
export(calc_irr_carbon)
export(calc_key_biodiversity_area)
export(calc_landcover)
export(calc_man_carbon)
export(calc_mangroves_area)
export(calc_population_count)
export(calc_precipitation_chelsa)
export(calc_precipitation_chirps)
export(calc_precipitation_wc)
export(calc_slope)
export(calc_soilproperties)
export(calc_species_richness)
export(calc_temperature_max_wc)
export(calc_temperature_min_wc)
export(calc_traveltime)
export(calc_traveltime_2000)
export(calc_treecover_area)
export(calc_treecover_area_and_emissions)
export(calc_treecoverloss_emissions)
export(calc_tri)
export(calc_vul_carbon)
export(check_available_years)
export(check_engine)
export(check_namespace)
export(check_stats)
export(get_accessibility_2000)
export(get_acled)
export(get_biodiversity_intactness_index)
export(get_chelsa)
export(get_chirps)
export(get_esalandcover)
export(get_fritz_et_al)
export(get_gfw_emissions)
export(get_gfw_lossyear)
export(get_gfw_treecover)
export(get_global_surface_water_change)
export(get_global_surface_water_occurrence)
export(get_global_surface_water_recurrence)
export(get_global_surface_water_seasonality)
export(get_global_surface_water_transitions)
export(get_gmw)
export(get_gsw_time_series)
export(get_humanfootprint)
export(get_ipbes_biomes)
export(get_irr_carbon)
export(get_iucn)
export(get_key_biodiversity_areas)
export(get_man_carbon)
export(get_mcd64a1)
export(get_nasa_grace)
export(get_nasa_srtm)
export(get_nelson_et_al)
export(get_resources)
export(get_soilgrids)
export(get_teow)
export(get_ucdp_ged)
export(get_vul_carbon)
export(get_worldclim_max_temperature)
export(get_worldclim_min_temperature)
export(get_worldclim_precipitation)
export(get_worldpop)
export(make_footprints)
export(make_global_grid)
export(mapme_options)
export(portfolio_long)
export(portfolio_wide)
export(prep_resources)
export(read_portfolio)
export(register_indicator)
export(register_resource)
export(select_engine)
export(spds_exists)
export(write_portfolio)
import(sf)
import(terra)
importFrom(curl,has_internet)
importFrom(httr2,req_headers)
importFrom(httr2,req_perform)
importFrom(httr2,req_retry)
importFrom(httr2,request)
importFrom(httr2,resp_body_json)
importFrom(httr2,resp_status)
importFrom(magrittr,"%>%")
importFrom(stats,sd)
importFrom(stats,var)
importFrom(tibble,as_tibble)
importFrom(tibble,tibble)
importFrom(utils,download.file)
importFrom(utils,unzip)

<!-- badges: start -->

[![status](https://joss.theoj.org/papers/0d1960441cd29c1a250366db291c96c9/status.svg)](https://joss.theoj.org/papers/0d1960441cd29c1a250366db291c96c9) [![R-CMD-check](https://github.com/mapme-initiative/mapme.biodiversity/actions/workflows/R-CMD-check.yaml/badge.svg?branch=main)](https://github.com/mapme-initiative/mapme.biodiversity/actions) [![Coverage Status](https://img.shields.io/codecov/c/github/mapme-initiative/mapme.biodiversity/master.svg)](https://app.codecov.io/github/mapme-initiative/mapme.biodiversity?branch=main) [![CRAN status](https://badges.cranchecks.info/worst/mapme.biodiversity.svg)](https://cran.r-project.org/web/checks/check_results_mapme.biodiversity.html) [![CRAN version](https://www.r-pkg.org/badges/version/mapme.biodiversity)](https://CRAN.R-project.org/package=mapme.biodiversity) [![License](https://img.shields.io/badge/License-GPL%20(%3E=3)-brightgreen.svg?style=flat)](https://choosealicense.com/licenses/gpl-3.0/)

<!-- badges: end -->

# mapme.biodiversity <img src="man/figures/logo.png" align="right" height="110"/>

## About

Biodiversity areas, especially primary forests, provide multiple ecosystem services for the local population and the planet as a whole. The rapid expansion of human land use into natural ecosystems and the impacts of the global climate crisis put natural ecosystems and the global biodiversity under threat.

The `{mapme.biodiversity}` package helps to analyse a number of biodiversity related indicators and biodiversity threats based on freely available geodata-sources such as the Global Forest Watch. It supports computational efficient routines and heavy parallel computing in cloud-infrastructures such as AWS or Microsoft Azure using in the statistical programming language R. The package allows for the analysis of global biodiversity portfolios with a thousand or millions of AOIs which is normally only possible on dedicated platforms such as the Google Earth Engine. It provides the possibility to e.g. analyse the World Database of Protected Areas (WDPA) for a number of relevant indicators. The primary use case of this package is to support scientific analysis and data science for individuals and organizations who seek to preserve the planet biodiversity. Its development is funded by the German Development Bank KfW.

## Installation

### Stable version

The package and its dependencies can be installed from CRAN via:

``` r
install.packages("mapme.biodiversity", dependencies = TRUE)
```

Windows and macOS binary packages are available from here.

### Development version

To install the development version, use the following command:

``` r
if (isFALSE(require("remotes", quietly = TRUE))) install.packages("remotes", dependencies = TRUE)
remotes::install_github("https://github.com/mapme-initiative/mapme.biodiversity", dependencies = TRUE)
```

## Available resources and indicators

Below is a list of the resources currently supported by `{mapme.biodiversity}`.

    ## mapme.biodiversity 0.9.6

| name | description | licence |
|:---|:---|:---|
| accessibility_2000 | Accessibility data for the year 2000 from the Global Accessibility Map project | See JRC data policy: <https://joint-research-centre.ec.europa.eu/about-us/service-standards-and-principles/data-policy_en> |
| acled | Armed Conflict Location & Event Data (ACLED) | Visit acleddata.com |
| biodiversity_intactness_index | Biodiversity Intactness Index | CC-BY-4.0 |
| chelsa | Climatologies at High resolution for the Earth Land Surface Areas (CHELSA) | Unknown - Must cite! |
| chirps | Climate Hazards Group InfraRed Precipitation with Station data (CHIRPS) | CC - unknown |
| esalandcover | Copernicus Land Monitoring Service (CLMS) 100 meter land cover product | CC-BY 4.0 |
| fritz_et_al | Drivers of deforestation in the tropics | CC-BY 4.0 |
| gfw_emissions | Global Forest Watch - CO2 Emssions caused by forest cover loss | CC-BY 4.0 |
| gfw_lossyear | Global Forest Watch - Year of forest cover loss occurrence | CC-BY 4.0 |
| gfw_treecover | Global Forest Watch - Percentage of canopy closure in 2000 | CC-BY 4.0 |
| global_surface_water_change | Global Surface Water - Change of water occurrence intensity | <https://www.copernicus.eu/en/access-data> |
| global_surface_water_occurrence | Global Surface Water - Percentage of water occurrence | <https://www.copernicus.eu/en/access-data> |
| global_surface_water_recurrence | Global Surface Water - Percentage of water recurrence | <https://www.copernicus.eu/en/access-data> |
| global_surface_water_seasonality | Global Surface Water - Seasonality of water occurrence | <https://www.copernicus.eu/en/access-data> |
| global_surface_water_transitions | Global Surface Water - Transition classes | <https://www.copernicus.eu/en/access-data> |
| gmw | Global Mangrove Watch - Vector data of mangrove extent | CC BY 4.0 |
| gsw_time_series | Global Surface Water - Yearly Time Series | <https://global-surface-water.appspot.com/download> |
| humanfootprint | Time series on human pressures on natural ecosystems. | CC BY 4.0 |
| ipbes_biomes | Global Assessment Report on Biodiversity and Ecosystem Services division of the earth’s surface into biomes and anthromes. | CC 4.0 |
| irr_carbon | Amount of carbon irrecoverably lost by a typical land use conversion event until mid-century. | CC NC 4.0 |
| iucn | IUCN Species Richness Raser Dataset | <https://www.iucnredlist.org/terms/terms-of-use> |
| key_biodiversity_areas | Key Biodiversity Areas | <https://www.keybiodiversityareas.org/termsofservice> |
| man_carbon | Amount of carbon that is manageable by humans. | CC NC 4.0 |
| mcd64a1 | MODIS Burned Area Monthly Product (Aqua and Terra) | <https://www.earthdata.nasa.gov/engage/open-data-services-software-policies/data-use-guidance> |
| nasa_grace | NASA Gravity Recovery And Climate Experiment (GRACE) - Measurments of Earth’s mass and water changes | <https://nasagrace.unl.edu/About.aspx> |
| nasa_srtm | NASA Shuttle Radar Topography Mission (SRTM) Digital Elevation Model (DEM) | <https://www.earthdata.nasa.gov/engage/open-data-services-software-policies/data-use-guidance> |
| nelson_et_al | Global maps of traveltime to cities | CC-BY 4.0 |
| soilgrids | ISRIC - Modelled global soil property layers | CC-BY 4.0 |
| teow | Terrestrial Ecosystems of the World (TEOW) from WWF-US | unknown |
| ucdp_ged | UCDP Georeferenced Event Dataset (UCDP GED) | CC-BY 4.0 |
| vul_carbon | Amount of carbon that is vulnerable to a typical land use conversion event. | CC NC 4.0 |
| worldclim_max_temperature | WorldClim - Monthly maximum temperature 1960 - 2024 | <https://www.worldclim.org/about.html> |
| worldclim_min_temperature | WorldClim - Monthly minimum temperature 1960 - 2024 | <https://www.worldclim.org/about.html> |
| worldclim_precipitation | WorldClim - Monthly precipitation 1960 - 2024 | <https://www.worldclim.org/about.html> |
| worldpop | WorldPop - Unconstrained Global Mosaics 2000 - 2020 | CC-BY 4.0 |

A more detailed list of these resources can be found in [this](https://github.com/mapme-initiative/mapme.biodiversity/raw/refs/heads/main/extras/mapme.biodiversity_resources.xlsx) Excel file.

Next, is a list of supported indicators.

| name | description |
|:---|:---|
| biodiversity_intactness_index | Averaged biodiversity intactness index. |
| biome | Areal statistics of biomes from TEOW |
| burned_area | Monthly burned area detected by MODIS satellites |
| deforestation_drivers | Areal statistics of deforestation drivers |
| drought_indicator | Relative wetness statistics based on NASA GRACE |
| ecoregion | Areal statistics of ecoregions based on TEOW |
| elevation | Statistics of elevation based on NASA SRTM |
| exposed_population_acled | Number of people exposed to conflicts based on ACLED |
| exposed_population_ucdp | Number of people exposed to conflicts based on UCDP GED |
| fatalities_acled | Number of fatalities by event type based on ACLED. |
| fatalities_ucdp | Number of fatalities by group of conflict based on UCDP GED |
| gsw_change | Statistics of the surface water change layer by JRC |
| gsw_occurrence | Areal statistic of surface water based on occurrence threshold |
| gsw_recurrence | Areal statistic of surface water based on recurrence threshold |
| gsw_seasonality | Areal statistic of surface water by seasonality |
| gsw_time_series | Global Surface Water - Yearly Time Series area estimation of water classes. |
| gsw_transitions | Areal statistics of surface water grouped by transition class |
| humanfootprint | Statistics of the human footprint data set per polygon. |
| ipbes_biomes | Area distribution of IBPES biomes within a polygon. |
| irr_carbon | Statistics of irrecoverable carbon per polygon. |
| key_biodiversity_areas | Area estimation of intersection with key biodiversity areas. |
| landcover | Areal statistics grouped by landcover class |
| man_carbon | Statistics of manageable carbon per polygon. |
| mangroves_area | Area covered by mangroves |
| population_count | Statistic of population counts |
| precipitation_chelsa | Statistics of CHELSA precipitation layer |
| precipitation_chirps | Statistics of CHIRPS precipitation layer |
| precipitation_wc | Statistics of WorldClim precipitation layer |
| slope | Statistics of slope based on NASA SRTM |
| soilproperties | Statistics of SoilGrids layers |
| species_richness | Species richness statistics based on user-specified raster files. |
| temperature_max_wc | Statistics of WorldClim maximum temperature layer |
| temperature_min_wc | Statistics of WorldClim minimum temperature layer |
| traveltime | Statistics of traveltime to the closests city grouped by city category |
| traveltime_2000 | Statistics of traveltime to the closests city in 2000 |
| treecover_area | Area of forest cover by year |
| treecover_area_and_emissions | Area of forest cover and greenhouse gas emissions caused by forest loss by year |
| treecoverloss_emissions | Greenouse gas emissions cause by forest loss by year |
| tri | Statistics of terrain ruggedness index based on NASA SRTM DEM |
| vul_carbon | Statistics of vulnerable carbon per polygon. |

## Usage example

`{mapme.biodiversity}` works by constructing a portfolio from an sf object. Specific raster and vector resource matching the spatio-temporal extent of the portfolio are made available locally. Once all required resources are available, indicators can be calculated individually for each asset in the portfolio.

Once you have decided on an indicator you are interested in, you can start by making the required resource available for your portfolio. Using `mapme_options()` you can set an output directory, control the maximum size of polygons before they are chunked into smaller parts, and control the verbosity of the package.

A portfolio is represented by an sf-object. It is required for the object to only contain geometries of type `POLYGON` and `MULTIPOLYGON` as assets. We can request the download of a resource for the spatial extent of our portfolio by using the `get_resources()` function. We simply supply our portfolio and one or more resource functions. Once the resources were made available, we can query the calculation of an indicator by using the `calc_indicators()` function. This function also expects the portfolio as input and one or more indicator functions. Once the indicator has been calculated for all assets in a portfolio, the data is returned as a nested list column to the original portfolio object. The output of each indicator is standardized to common format, consisting of a tibble with columns `datetime`, `variable`, `unit`, and `value`. We can transform the the data to long format by using `portfolio_long()`.

``` r
library(mapme.biodiversity)
library(sf)

mapme_options(
  outdir = system.file("res", package = "mapme.biodiversity"),
  chunk_size = 1e6, # in ha
  verbose = FALSE
)

aoi <- system.file("extdata", "sierra_de_neiba_478140_2.gpkg", package = "mapme.biodiversity") %>%
  sf::read_sf() %>%
  get_resources(
    get_gfw_treecover(version = "GFC-2023-v1.11"),
    get_gfw_lossyear(version = "GFC-2023-v1.11"),
    get_gfw_emissions()
  ) %>%
  calc_indicators(calc_treecover_area_and_emissions(years = 2016:2017, min_size = 1, min_cover = 30)) %>%
  portfolio_long()

aoi
```

```         
## Simple feature collection with 4 features and 8 fields
## Geometry type: POLYGON
## Dimension:     XY
## Bounding box:  xmin: -71.80933 ymin: 18.57668 xmax: -71.33201 ymax: 18.69931
## Geodetic CRS:  WGS 84
## # A tibble: 4 × 9
##   WDPAID ISO3  assetid indicator        datetime            variable unit  value
##    <dbl> <chr>   <int> <chr>            <dttm>              <chr>    <chr> <dbl>
## 1 478140 DOM         1 treecover_area_… 2016-01-01 00:00:00 emissio… Mg    4296.
## 2 478140 DOM         1 treecover_area_… 2016-01-01 00:00:00 treecov… ha    2372.
## 3 478140 DOM         1 treecover_area_… 2017-01-01 00:00:00 emissio… Mg    4970.
## 4 478140 DOM         1 treecover_area_… 2017-01-01 00:00:00 treecov… ha    2360.
## # ℹ 1 more variable: geom <POLYGON [°]>
```

## Using cloud storages

`{mapme.biodiversity}` leverages GDAL’s capabilities for data I/O. For users of this package, that means that integrating a cloud storage is as easy as setting up a configuration file and changing the `outdir` argument in `mapme_options()`. While you could also decide to use environment variables, we recommend to set up a GDAL config file. You can find GDAL’s documentation on this topic [here](https://gdal.org/en/latest/user/configoptions.html#gdal-configuration-file).

Suppose that we want to use an AWS S3 bucket that we control to write resource data to. Let’s assume this bucket is already set up and we wish to refer to it in our R code as `mapme-data`. The GDAL configuration file should look something like this:

``` ini
[credentials]

[.mapme-data]
path=/vsis3/mapme-data
AWS_SECRET_ACCESS_KEY=<your-access-key>
AWS_ACCESS_KEY_ID=<your-access-id>
```

The connection will be handled based on GDAL’s virtual file system. You can find documentation on specific options for your cloud provider [here](https://gdal.org/en/latest/user/virtual_file_systems.html#network-based-file-systems).

Ideally, you would also set the following in the `.Renviron` file in your user’s home directory to ensure that GDAL is aware of this configuration when an R session is started:

``` ini
GDAL_CONFIG_FILE = "<path-to-your-config-file>"
```

Then, in your scripts set the `outdir` option to the value specified with the `path` variable in the configuration file:

``` r
mapme_options(outdir = "/vsis3/mapme-data")
```

## A note on parallel computing

`{mapme.biodiversity}` follows the parallel computing paradigm of the [`{future}`](https://cran.r-project.org/package=future) package. That means that you as a user are in the control if and how you would like to set up parallel processing. Since `{mapme.biodiversity} v0.9`, we apply pre-chunking to all assets in the portfolio. That means that assets are split up into components of roughly the size of `chunk_size`. These components can than be iterated over in parallel to speed up processing. Indicator values will be aggregated automatically.

``` r
library(future)
plan(cluster, workers = 6)
```

As another example, with the code below one would apply parallel processing of 2 assets, with each having 4 workers available to process chunks, thus requiring a total of 8 available cores on the host machine. Be sure to not request more workers than available on your machine.

``` r
library(progressr)

plan(cluster, workers = 2)

with_progress({
  aoi <- calc_indicators(
    aoi,
    calc_treecover_area_and_emissions(
      min_size = 1,
      min_cover = 30
    )
  )
})

plan(sequential) # close child processes
```

## How to contribute

Contributions to the package are very welcome. If you want to contribute a new resource or indicator, please check out the [developer guidelines](https://mapme-initiative.github.io/mapme.biodiversity/articles/contributing.html) for more information on how to implement a new resource or indicator. If you want to contribute to the package in any other way or seek support from the community, please open an [issue](https://github.com/mapme-initiative/mapme.biodiversity/issues) or consider opening a [discussion](https://github.com/mapme-initiative/mapme.biodiversity/discussions).

## More info

In addition to extensive [online
documentation](https://mapme-initiative.github.io/mapme.biodiversity/)
that provides detailed information about the package,
`mapme.biodiversity` provides an applied
[workshop](https://github.com/mapme-initiative/demo-madeira) example
using real-world data showcasing the usage of the package to estimate
changes in forest cover caused by flooding as a consequence of dam
building at the Madeira river in Rondônia, Brazil.

#' Forest greenhouse gas emissions
#'
#' This resource is part of the publication by Harris et al. (2021)
#' "Global maps of twenty-first century forest carbon fluxes.". It
#' represents "the greenhouse gas
#' emissions arising from stand-replacing forest disturbances that occurred in
#' each modelled year (megagrams CO2 emissions/ha, between 2001 and 2023).
#' Emissions include all relevant ecosystem carbon pools (aboveground biomass,
#' belowground biomass, dead wood, litter, soil) and greenhouse gases (CO2, CH4,
#' N2O)." The area unit that is downloaded here corresponds to the
#' "megagrams of CO2 emissions/pixel" layer, in order to support the calculation
#' of area-wise emissions.
#'
#' There are no arguments users need to specify. However, users should note
#' that the spatial extent for this dataset does not totally cover the same
#' extent as the \code{treecover2000} and \code{lossyear} resources by Hansen
#' et al. (2013). A missing value (NA) will be inserted for greenhouse gas
#' emissions for areas where no data is available.
#'
#' @name gfw_emissions
#' @keywords resource
#' @returns A function that returns an `sf` footprint object.
#' @references Harris, N.L., Gibbs, D.A., Baccini, A. et al. Global maps of
#' twenty-first century forest carbon fluxes. Nat. Clim. Chang. 11, 234–240
#' (2021). \doi{doi:10.1038/s41558-020-00976-6}
#' @source \url{https://data.globalforestwatch.org/datasets/gfw::forest-greenhouse-gas-emissions/about}
#' @include register.R
#' @export
get_gfw_emissions <- function() {
  function(x,
           name = "gfw_emissions",
           type = "raster",
           outdir = mapme_options()[["outdir"]],
           verbose = mapme_options()[["verbose"]]) {
    index_file <- system.file("extdata", "greenhouse_index.geosjon", package = "mapme.biodiversity")
    spatialindex <- st_read(index_file, quiet = TRUE, check_ring_dir = TRUE)

    row_ids <- unique(unlist(st_intersects(x, spatialindex)))
    tile_str <- spatialindex$tile_id[row_ids]

    urls <- as.character(spatialindex$Mg_CO2e_px_download[row_ids])
    urls <- paste0("/vsicurl/", urls)
    filenames <- sprintf("gfw_forest_carbon_gross_emissions_Mg_CO2e_px_%s.tif", tile_str)

    fps <- spatialindex[row_ids, "geometry"]
    fps[["source"]] <- urls
    make_footprints(fps, filenames,
      what = "raster",
      co = c("-co", "INTERLEAVE=BAND", "-co", "COMPRESS=DEFLATE")
    )
  }
}

register_resource(
  name = "gfw_emissions",
  description = "Global Forest Watch - CO2 Emssions caused by forest cover loss",
  licence = "CC-BY 4.0",
  source = "https://data.globalforestwatch.org/datasets/gfw::forest-greenhouse-gas-emissions/about",
  type = "raster"
)

#' Downloads WorldClim Minimum Temperature layer
#'
#' This resource is published by Fick et al. (2017) "WorldClim 2: new 1-km
#' spatial resolution climate surfaces for global land areas" and
#' represents multiple climatic variables from which we will be requiring minimum
#' temperature, maximum temperature, and mean precipitation layers. The layers are
#' available to download for the period 1960 - 2024 on monthly basis from WorldClim.
#'
#' This resource represents the minimum temperature, layers available to
#' download for the period 1960 - 2024 on monthly basis from WorldClim. Encoded
#' as (°C), representing the minimum temperature per output grid cell.
#'
#' @name worldclim_min_temperature
#' @param years A numeric vector indicating for which years to make the
#'   resource available. Defaults to 2000:2024.
#' @param resolution A character vector indicating the desired resolution:
#' "2.5m", "5m", or "10m".
#' @keywords resource
#' @returns A function that returns an `sf` footprint object.
#' @source \url{https://www.worldclim.org/data/index.html}
#' @importFrom utils unzip
#' @include register.R
#' @export
get_worldclim_min_temperature <- function(years = 2000:2024,
                                          resolution = c("2.5m", "5m", "10m")) {
  years <- check_available_years(years, 1960:2024, "tmin")
  resolution <- match.arg(resolution)

  function(x,
           name = "worldclim_min_temperature",
           type = "raster",
           outdir = mapme_options()[["outdir"]],
           verbose = mapme_options()[["verbose"]]) {
    .get_climatic_variables(x = x, years = years, var = "tmin", res = resolution, verbose)
  }
}

#' Downloads WorldClim Maximum Temperature layer
#'
#' This resource is published by Fick et al. (2017) "WorldClim 2: new 1-km
#' spatial resolution climate surfaces for global land areas" and
#' represents multiple climatic variables from which we will be requiring minimum
#' temperature, maximum temperature, and mean precipitation layers. The layers are
#' available to download for the period 1960 - 2024 on monthly basis from WorldClim.
#'
#' This resource represents the maximum temperature, layers available to
#' download for the period 1960 - 2024 on monthly basis from WorldClim. Encoded
#' as (°C), representing the maximum temperature per output grid cell.
#'
#' @name worldclim_max_temperature
#' @param years A numeric vector indicating for which years to make the
#'   resource available. Defaults to 2000:2024.
#' @param resolution A character vector indicating the desired resolution:
#' "2.5m", "5m", or "10m".
#' @keywords resource
#' @returns  A function that returns an `sf` footprint object.
#' @source \url{https://www.worldclim.org/data/index.html}
#' @importFrom utils unzip
#' @include register.R
#' @export
get_worldclim_max_temperature <- function(years = 2000:2024,
                                          resolution = c("2.5m", "5m", "10m")) {
  years <- check_available_years(years, 1960:2024, "tmax")
  resolution <- match.arg(resolution)

  function(x,
           name = "worldclim_max_temperature",
           type = "raster",
           outdir = mapme_options()[["outdir"]],
           verbose = mapme_options()[["verbose"]]) {
    .get_climatic_variables(x = x, years = years, var = "tmax", res = resolution, verbose)
  }
}

#' Downloads WorldClim Mean Precipitation layer
#'
#' This resource is published by Fick et al. (2017) "WorldClim 2: new 1-km
#' spatial resolution climate surfaces for global land areas" and
#' represents multiple climatic variables from which we will be requiring minimum
#' temperature, maximum temperature, and mean precipitation layers. The layers are
#' available to download for the period 1960 - 2024 on monthly basis from WorldClim.
#'
#' This resource represents the average precipitation, layers available to
#' download for the period 1960 - 2024 on monthly basis from WorldClim. Encoded
#' as (mm), representing the mean precipitation per output grid cell.
#'
#' @name worldclim_precipitation
#' @param years A numeric vector indicating for which years to make the
#'   resource available.
#' @param resolution A character vector indicating the desired resolution.
#' @keywords resource
#' @returns  A function that returns an `sf` footprint object.
#' @source \url{https://www.worldclim.org/data/index.html}
#' @importFrom utils unzip
#' @include register.R
#' @export
get_worldclim_precipitation <- function(years = 1960:2024,
                                        resolution = c("2.5m", "5m", "10m")) {
  years <- check_available_years(years, 1960:2024, "prec")
  resolution <- match.arg(resolution)

  function(x,
           name = "worldclim_precipitation",
           type = "raster",
           outdir = mapme_options()[["outdir"]],
           verbose = mapme_options()[["verbose"]]) {
    .get_climatic_variables(x = x, years = years, var = "prec", res = resolution, verbose)
  }
}


.get_climatic_variables <- function(x,
                                    years = 2000:2024,
                                    var,
                                    res,
                                    verbose = TRUE) {
  urls <- purrr::map(years, function(year) .get_worldclim_url(res, year, var))
  urls <- unlist(urls)
  bbox <- c(xmin = -180., ymin = -90., xmax = 180., ymax = 90.)
  tiles <- st_as_sfc(st_bbox(bbox, crs = "EPSG:4326"))
  tiles <- st_as_sf(rep(tiles, length(urls)))
  tiles[["source"]] <- urls
  make_footprints(tiles, what = "raster", co = c("-co", "COMPRESS=LZW"))
}


#' Helper function to construct climate variable layers urls
#'
#' @param layer A character indicating the target variable name
#' @param year A numeric indicating the target year
#'
#' @return A character vector.
#' @keywords internal
#' @noRd
.get_worldclim_url <- function(res, year, var) {
  stopifnot(length(res) == 1 && res %in% c("2.5m", "5m", "10m"))
  stopifnot(length(year) == 1 && year %in% 1960:2024)
  stopifnot(length(var) == 1 && var %in% c("prec", "tmin", "tmax"))

  base_url <- "https://geodata.ucdavis.edu/climate/worldclim/2_1/hist/cts4.09/"
  zipfile <- "wc2.1_cruts4.09_%s_%s_%s-%s.zip"

  start <- c(1960, 1970, 1980, 1990, 2000, 2010, 2020)
  end <- c(1969, 1979, 1989, 1999, 2009, 2019, 2024)
  index <- which(start <= year & end >= year)

  zipfile <- sprintf(zipfile, res, var, start[index], end[index])

  url <- file.path(base_url, zipfile)

  months <- sprintf("%02d", 1:12)
  dates <- sprintf("%s-%s", year, months)
  tifs <- sprintf("wc2.1_cruts4.09_%s_%s_%s.tif", res, var, dates)
  file.path("/vsizip//vsicurl", url, tifs)
}


register_resource(
  name = "worldclim_min_temperature",
  description = "WorldClim - Monthly minimum temperature 1960 - 2024",
  licence = "https://www.worldclim.org/about.html",
  source = "https://www.worldclim.org/data/index.html",
  type = "raster"
)

register_resource(
  name = "worldclim_max_temperature",
  description = "WorldClim - Monthly maximum temperature 1960 - 2024",
  licence = "https://www.worldclim.org/about.html",
  source = "https://www.worldclim.org/data/index.html",
  type = "raster"
)

register_resource(
  name = "worldclim_precipitation",
  description = "WorldClim - Monthly precipitation 1960 - 2024",
  licence = "https://www.worldclim.org/about.html",
  source = "https://www.worldclim.org/data/index.html",
  type = "raster"
)

test_that(".get_worldclim works", {
  x <- read_sf(
    system.file("extdata", "sierra_de_neiba_478140.gpkg",
      package = "mapme.biodiversity"
    )
  )
  .clear_resources()
  outdir <- file.path(tempdir(), "mapme.data")
  .copy_resource_dir(outdir)
  mapme_options(outdir = outdir, verbose = FALSE)

  gcv <- get_worldclim_min_temperature(years = 2018, resolution = "2.5m")
  files <- sprintf("wc2.1_cruts4.09_2.5m_tmin_2018-%02d.tif", 1:12)
  expect_equal(gcv(x)[["filename"]], files)

  gcv <- get_worldclim_max_temperature(years = 2018, resolution = "2.5m")
  files <- sprintf("wc2.1_cruts4.09_2.5m_tmax_2018-%02d.tif", 1:12)
  expect_equal(gcv(x)[["filename"]], files)

  gcv <- get_worldclim_precipitation(years = 2018, resolution = "2.5m")
  files <- sprintf("wc2.1_cruts4.09_2.5m_prec_2018-%02d.tif", 1:12)
  expect_equal(gcv(x)[["filename"]], files)

  gcv <- get_worldclim_min_temperature(years = 2018, resolution = "5m")
  files <- sprintf("wc2.1_cruts4.09_5m_tmin_2018-%02d.tif", 1:12)
  expect_equal(gcv(x)[["filename"]], files)

  gcv <- get_worldclim_max_temperature(years = 2018, resolution = "5m")
  files <- sprintf("wc2.1_cruts4.09_5m_tmax_2018-%02d.tif", 1:12)
  expect_equal(gcv(x)[["filename"]], files)

  gcv <- get_worldclim_precipitation(years = 2018, resolution = "5m")
  files <- sprintf("wc2.1_cruts4.09_5m_prec_2018-%02d.tif", 1:12)
  expect_equal(gcv(x)[["filename"]], files)

  expect_error(get_worldclim_precipitation(resolution = "na"))
})

#' Terrestrial Human Foootprint
#'
#' This resource is part of the publication by Mu et al. (2022) "A global
#' record of annual terrestrial Human Footprint dataset from 2000 to 2018".
#' It is calculated based on 8 variables representing human pressures on
#' natural ecosystems collected at a yearly cadence between 2000 and 2024
#' sampled at a 1km spatial resolution. The variables are used are
#' the expansion of built environments (expressed as percentage of built-up
#' areas within a grid cell), population density (aggregated at the grid cell),
#' nighttime lights, crop and pasture lands, roads and railways (excluding trails
#' and minor roads), and navigable waterways (compares waterways with nighttime
#' lights dataset). The human footprint was then calculated based on a weighting
#' scheme proposed by Venter et al. (2016), assigning each pixel a value between
#' 0 and 50, with 50 representing the theoretical value of the highest human
#' pressure.
#
#' @note
#' It may be required to increase the timeout option to successfully download
#' theses layers from their source location via e.g. `options(timeout = 600)`.
#' In case an 403 error occurs, you can create an account with Figshare and
#' create an personal access token. If set as `FIGSHARE_PAT` environment
#' variable, it will be used to authenticate.
#'
#' @name humanfootprint_resource
#' @param years A numeric vector indicating the years for which to download
#'   the human footprint data, defaults to \code{2000:2024}.
#' @keywords resource
#' @returns A function that returns an `sf` footprint object.
#' @references Mu, H., Li, X., Wen, Y. et al. A global record of annual
#' terrestrial Human Footprint dataset from 2000 to 2018. Sci Data 9, 176 (2022).
#' \doi{doi:10.1038/s41597-022-01284-8}
#' @source \url{https://figshare.com/articles/figure/An_annual_global_terrestrial_Human_Footprint_dataset_from_2000_to_2018/16571064}
#' @importFrom utils unzip download.file
#' @include register.R
#' @export
get_humanfootprint <- function(years = 2000:2024) {
  available_years <- 2000:2024
  years <- check_available_years(
    years, available_years, "humanfootprint"
  )

  if (is.null(mapme_options()[["outdir"]])) {
    warning(paste(
      "humanfootprint layers must be downloaded from the source location",
      "irrespective if `outdir` was specified or not."
    ))
  }

  function(
      x,
      name = "humanfootprint",
      type = "raster",
      outdir = mapme_options()[["outdir"]],
      verbose = mapme_options()[["verbose"]]) {
    srcs <- .get_hfp_url(years)
    has_outdir <- !is.null(outdir)
    if (has_outdir) {
      dsts <- file.path(outdir, srcs[["filename"]])
    } else {
      dsts <- file.path(tempdir(), srcs[["filename"]])
    }
    dsts <- gsub("zip", "tif", dsts)
    is_available <- purrr::map_lgl(dsts, spds_exists, what = "raster")
    if (all(is_available)) {
      return(make_footprints(dsts, what = "raster"))
    }

    dsts <- purrr::map2_chr(srcs[["url"]], dsts, function(src, dst) {
      if (!spds_exists(dst, what = "raster")) {
        tmp <- file.path(tempdir(), gsub(".tif", ".zip", basename(dst)))
        download.file(src, tmp, mode = ifelse(Sys.info()["sysname"] == "Windows", "wb", "w"))
        unzip(tmp, exdir = tempdir())
        return(file.path(tempdir(), basename(dst)))
      } else {
        return(dst)
      }
    })

    make_footprints(
      dsts,
      what = "raster",
      co = c("-of", "COG", "-co", "COMPRESS=LZW", "-ot", "Float32")
    )
  }
}

#' @noRd
#' @importFrom httr2 request req_perform req_retry resp_status resp_body_json req_headers
.get_hfp_url <- function(years) {
  req <- request("https://api.figshare.com/v2/articles/16571064/files?page_size=1000")
  # req <- req |>
  #   req_headers(page = 1, page_size = 2)
  req <- req_headers(req, page = 1, page_size = 2)

  if (Sys.getenv("FIGSHARE_PAT") != "") {
    token <- sprintf("token %s", Sys.getenv("FIGSHARE_PAT"))
    req <- req_headers(req, "Authorization" = token, .redact = "Authorization")
  }

  is_transient <- function(resp) resp_status(resp) %in% c(403, 429, 503, 504)
  rsp <- req_perform(req_retry(req, max_seconds = 15, is_transient = is_transient))

  cnt <- resp_body_json(rsp)
  data <- lapply(cnt, function(x) data.frame(filename = x[["name"]], url = x[["download_url"]]))
  data <- do.call(rbind, data)
  data <- data[grep("zip", data[["filename"]]), ]
  data["year"] <- as.numeric(substring(data[["filename"]], 4, 7))
  data[data[["year"]] %in% years, ]
}

register_resource(
  name = "humanfootprint",
  description = "Time series on human pressures on natural ecosystems.",
  licence = "CC BY 4.0",
  source = "https://figshare.com/articles/figure/An_annual_global_terrestrial_Human_Footprint_dataset_from_2000_to_2018/16571064",
  type = "raster"
)

#' mapme.biodiversity
#'
#' @name mapme.biodiversity - package
#' @docType package
#' @title mapme.biodiversity - Global biodiversity portfolio analysis
#' @import sf terra
#' @importFrom tibble tibble as_tibble
#' @importFrom magrittr "%>%"
#' @importFrom utils download.file
#' @noRd
#' @keywords internal
globalVariables(c(":=", "!!", ".id", ".", "datetime", "variable", "unit", "value"))
NULL


.copy_resource_dir <- function(target) {
  if (!dir.exists(target)) {
    dir.create(target, showWarnings = FALSE)
    resource_dir <- system.file("res", package = "mapme.biodiversity")
    for (dir in list.dirs(resource_dir)) {
      file.copy(dir, target, recursive = TRUE)
    }
  }
}

#### -------------------------Exported utils------------------------------####

#' Check if a spatial data sets exists
#'
#' This function uses a file path readable by GDAL to check if it can query
#' it for information. Note, this should also work for remote files, e.g.
#' in an S3 bucket. You can use this function in your custom resource
#' function to query if a file is already present at the destination.
#' Note, that performance will be dependent on your connection to the
#' server. It can also be used for files on the local file system.
#'
#' @param path A length 1 character vector with a GDAL readable file path.
#' @param oo Either a list or a character vector with opening options (-oo)
#'   of the respective GDAL driver. A list must have equal length of the
#'   input sources, a vector will be recycled.
#' @param what A character vector indicating if the resource is a vector or raster file.
#'
#' @return A logical, TRUE if the file exists, FALSE if it does not.
#' @keywords utils
#' @export
#'
#' @examples
#'
#' # a vector resource
#' vec <- system.file("shape/nc.shp", package = "sf")
#' spds_exists(vec, what = "vector")
#'
#' # a raster resource
#' ras <- system.file("ex/elev.tif", package = "terra")
#' spds_exists(ras, what = "raster")
#'
#' # a non existing file
#' spds_exists("not-here.gpkg", what = "vector")
#'
spds_exists <- function(path, oo = character(0), what = c("vector", "raster")) {
  what <- match.arg(what)
  # path <- normalizePath(path, mustWork = FALSE)
  path <- .normalize_source_path(path)
  util <- switch(what,
    vector = "ogrinfo",
    raster = "gdalinfo"
  )
  opts <- switch(what,
    vector = c(
      "-json", "-ro", "-so", "-nomd",
      "-nocount", "-noextent", "-nogeomtype", oo
    ),
    raster = c("-json", "-nomd", "-norat", "-noct", oo)
  )
  if (what == "vector" &&
      (utils::compareVersion(sf::sf_extSoftVersion()[["GDAL"]], "3.7.0") < 0)) {
    util <- "gdalinfo"
    opts <- oo
  }
  info <- sf::gdal_utils(
    util = util,
    source = path,
    options = opts,
    quiet = TRUE
  )
  length(info) > 0
}

#' Create footprints for vector or raster data sets
#'
#' With this function you can create footprints for vector or raster datasets.
#' Specify a character vector of GDAL readable sources of either vector
#' or raster type. Internally, GDAL will be used to create an sf object
#' with a single column indicating the source and the geometry indicating
#' the bounding box of the respective source.
#' Note, the performance for remote sources is dependent on your connection to
#' the server. If you have other means to create footprints in your resource
#' function (e.g. by using the output `{rstac::items_bbox()}`) you should prefer
#' those means over this function for remote files.
#'
#' @param srcs A character vector with GDAL readable paths to either vector
#'  or raster sources, then internal footprint functions are called, or an
#'  sf object which will be appended for filenames and potential options.
#' @param filenames A character vector indicating the filenames of the source
#'   data sets if they were written to a destination. Defaults to `basename(srcs)`
#'   in case of character type or `basename(srcs[["source"]])` in case of
#'   an sf object.
#' @param what A character vector indicating if the files are vector or raster
#'   files.
#' @param oo Either a list or a character vector with opening options (-oo)
#'   of the respective GDAL driver. A list must have equal length of the
#'   input sources, a vector will be recycled.
#' @param co Either a list or a character vector with creation options (-co)
#'   of the respective GDAL driver. A list must have equal length of the
#'   input sources, a vector will be recycled.
#' @param precision A numeric indicating the precision of coordinates when
#'   a binary round-trip is done (see `?sf::st_as_binary()`).
#'
#' @return An sf object with a the files sources and the geometry indicating
#'   their spatial footprint.
#' @keywords utils
#' @export
#'
#' @examples
#'
#' # a vector resource
#' # requires GDAL >= 3.7.0
#' if (FALSE) {
#'   vec <- system.file("shape/nc.shp", package = "sf")
#'   make_footprints(vec, what = "vector")
#' }
#'
#' # a raster resource
#' ras <- system.file("ex/elev.tif", package = "terra")
#' make_footprints(ras, what = "raster")
make_footprints <- function(srcs = NULL,
                            filenames = if (inherits(srcs, "sf")) basename(srcs[["source"]]) else basename(srcs),
                            what = c("vector", "raster"),
                            oo = NULL,
                            co = NULL,
                            precision = 1e5) {
  stopifnot(is.null(oo) || (inherits(oo, "list") | inherits(oo, "character")))
  stopifnot(is.null(co) || (inherits(co, "list") | inherits(co, "character")))
  stopifnot(inherits(srcs, "sf") | inherits(srcs, "character"))
  stopifnot(inherits(filenames, "character") | is.null(filenames))
  stopifnot(is.numeric(precision) && length(precision) == 1)

  n <- ifelse(inherits(srcs, "sf"), nrow(srcs), length(srcs))
  if (length(filenames) != n) stop("filenames required to be of equal length of sources.")

  if (inherits(oo, "list") && length(oo) != n) {
    stop("Opening options list is required to be equal length of sources.")
  }

  if (inherits(co, "list") && length(co) != n) {
    stop("Creation options list is required to be equal length of sources.")
  }

  if (inherits(oo, "character") | is.null(oo)) {
    oo <- lapply(seq_len(n), function(i) oo)
  }

  if (inherits(co, "character") | is.null(co)) {
    co <- lapply(seq_len(n), function(i) co)
  }

  if (inherits(srcs, "character")) {
    what <- match.arg(what)
    # srcs <- normalizePath(srcs, mustWork = FALSE)
    srcs <- .normalize_source_path(srcs)
    srcs <- switch(what,
      vector = purrr::map2(srcs, oo, function(src, opt) .vector_footprint(src, opt)),
      raster = purrr::map2(srcs, oo, function(src, opt) .raster_footprint(src, opt)),
      stop("Can make footprints for vector and raster data only.")
    )
    srcs <- purrr::list_rbind(srcs)
  }

  srcs <- st_as_sf(tibble::as_tibble(srcs))
  srcs <- .set_precision(srcs, precision)
  srcs[["location"]] <- srcs[["source"]]
  srcs[["type"]] <- what
  srcs[["filename"]] <- filenames
  srcs[["oo"]] <- oo
  srcs[["co"]] <- co
  st_geometry(srcs) <- "geometry"
  srcs[, c("filename", "location", "type", "oo", "co", "source")]
}

#' Prepare resources for an asset
#'
#' This function reads and crops available resources to the extent of a single
#' asset. Specific resources can be queried. If not supplied (the default), all
#' available resources will be prepared.
#'
#' @param avail_resources A list object of available resources. If NULL (the default),
#'   the available resources will automatically be determined.
#' @param resources A character vector with the resources to be prepared. If it
#'   it is NULL (the default) all available resources will be prepared.
#' @param mode A character indicating the reading mode, e.g. either "portfolio"
#'   (the default) or "asset".
#'
#' @return `prep_resources()` returns a list with prepared vector and raster
#'   resources as `sf` and `SpatRaster`-objects.
#' @name mapme
#' @export
prep_resources <- function(x, avail_resources = NULL, resources = NULL, mode = c("portfolio", "asset")) {
  stopifnot(nrow(x) == 1)
  mode <- match.arg(mode)

  if (is.null(avail_resources)) avail_resources <- .avail_resources()
  if (length(avail_resources) == 0) {
    return(NULL)
  }
  if (is.null(resources)) resources <- names(avail_resources)
  if (!any(resources %in% names(avail_resources))) {
    stop("Some requested resources are not available.")
  }

  out <- purrr::map(resources, function(resource_name) {
    resource <- avail_resources[[resource_name]]
    if (nrow(resource) == 0L) {
      stop(sprintf("Resource %s is empty", resource_name), call. = FALSE)
    }
    resource_type <- unique(resource[["type"]])
    reader <- switch(resource_type,
      raster = .read_raster,
      vector = .read_vector,
      stop(sprintf("Resource type '%s' currently not supported", resource_type))
    )
    reader(x, resource, mode)
  })
  names(out) <- resources
  out
}


#### -------------------------Vector Utils------------------------------####
.vector_footprint <- function(src, oo = NULL) {
  layers_info <- .vector_info(src, oo)
  if (is.null(layers_info)) {
    return(NULL)
  }

  bboxs <- purrr::map_vec(layers_info, .vector_bbox)
  bbox <- st_sf(geometry = st_as_sfc(st_bbox(st_union(bboxs)))) # ordered for S2
  bbox["source"] <- src
  bbox
}

.vector_info <- function(src, oo) {
  stopifnot(is.character(src) && length(src) == 1)
  stopifnot(is.null(oo) || is.character(oo))

  info <- sf::gdal_utils(
    "ogrinfo",
    source = src,
    options = c("-json", "-so", "-ro", "-nomd", "-nocount", oo),
    quiet = TRUE
  )

  if (length(info) == 0) {
    return(NULL)
  }

  info <- jsonlite::parse_json(info)
  info[["layers"]]
}

.vector_bbox <- function(layer) {
  crs <- layer[["geometryFields"]][[1]][["coordinateSystem"]][["wkt"]]
  crs <- st_crs(crs)
  bbox <- as.numeric(layer[["geometryFields"]][[1]][["extent"]])
  names(bbox) <- c("xmin", "ymin", "xmax", "ymax")
  st_as_sfc(st_bbox(bbox, crs = crs))
}

.read_vector <- function(x, tindex, mode = "portfolio") {
  x <- st_as_sfc(st_bbox(x))
  if (st_crs(x) != st_crs(tindex)) {
    x <- st_transform(x, st_crs(tindex))
  }
  matches <- .get_intersection(x, tindex)

  if (nrow(matches) == 0) {
    warning("No intersection with asset.")
    return(NULL)
  }

  paths <- matches[["location"]]

  vectors <- purrr::map(paths, function(path) {
    tmp <- try(read_sf(path, wkt_filter = st_as_text(x)), silent = TRUE)
    if (inherits(tmp, "try-error")) {
      warning(tmp)
      return(NULL)
    }
    if (nrow(tmp) == 0) {
      return(NULL)
    }
    tmp
  })

  is_null <- unlist(lapply(vectors, is.null))
  vectors <- vectors[!is_null]
  if (length(vectors) == 0) {
    return(NULL)
  }
  names(vectors) <- matches[["filename"]][!is_null]
  vectors
}


#### -------------------------Raster Utils------------------------------####
.raster_footprint <- function(src, oo = NULL) {
  info <- .raster_info(src, oo)
  if (is.null(info)) {
    return(NULL)
  }
  bbox <- .raster_bbox(info)
  bbox[["source"]] <- src
  bbox
}

.raster_info <- function(src, oo = NULL) {
  stopifnot(is.character(src) && length(src) == 1)
  stopifnot(is.null(oo) || is.character(oo))

  info <- sf::gdal_utils(
    "gdalinfo",
    source = src,
    options = c("-json", "-norat", "-noct", "-nomd", oo),
    quiet = TRUE
  )

  if (length(info) == 0) {
    return(NULL)
  }

  jsonlite::parse_json(info)
}

.raster_bbox <- function(info) {
  crs <- st_crs(info[["coordinateSystem"]][["wkt"]])
  coords <- info[["cornerCoordinates"]]
  bbox <- st_bbox(c(
    xmin = coords$lowerLeft[[1]],
    xmax = coords$upperRight[[1]],
    ymin = coords$lowerLeft[[2]],
    ymax = coords$upperLeft[[2]]
  ), crs = crs)
  bbox <- st_sf(geometry = st_as_sfc(st_bbox(bbox))) # ordered for S2
  bbox
}

.read_raster <- function(x, tindex, mode = "portfolio") {
  x <- st_as_sfc(st_bbox(x))
  if (st_crs(x) != st_crs(tindex)) {
    x <- st_transform(x, st_crs(tindex))
  }

  matches <- .get_intersection(x, tindex)

  if (nrow(matches) == 0) {
    warning("No intersection with asset.")
    return(NULL)
  }

  geoms <- matches[["geometry"]]
  unique_geoms <- unique(geoms)
  grouped_geoms <- match(geoms, unique_geoms)
  names(grouped_geoms) <- matches[["location"]]
  grouped_geoms <- sort(grouped_geoms)

  n_tiles <- length(unique(grouped_geoms))
  n_timesteps <- unique(table(grouped_geoms))

  if (length(n_timesteps) > 1) {
    stop("Did not find equal number of tiles per timestep.")
  }

  vrts <- sapply(1:n_timesteps, function(i) tempfile(fileext = ".vrt"))
  if (mode == "asset") on.exit(file.remove(vrts))

  out <- purrr::map2(1:n_timesteps, vrts, function(i, vrt) {
    index <- rep(FALSE, n_timesteps)
    index[i] <- TRUE
    filenames <- names(grouped_geoms[index])
    layer_name <- tools::file_path_sans_ext(basename(filenames[1]))
    tmp <- terra::vrt(filenames, filename = vrt)
    names(tmp) <- layer_name
    tmp
  })
  out <- do.call(c, out)

  # crop the source to the extent of the current polygon
  cropped <- try(terra::crop(out, terra::vect(x), snap = "out"))
  if (inherits(cropped, "try-error")) {
    warning(as.character(cropped))
    return(NULL)
  }
  if (mode == "asset") cropped[] <- terra::values(cropped)
  cropped
}

#### -------------------------Unexported utils------------------------------####
.set_precision <- function(data, precision = 1e2) {
  crs <- st_crs(data)
  geoms <- st_geometry(data)
  geoms <- st_sfc(geoms, precision = precision)
  geoms_binary <- st_as_binary(geoms)
  geoms <- st_as_sfc(geoms_binary)
  st_geometry(data) <- geoms
  st_geometry(data) <- "geometry"
  st_crs(data) <- crs
  .geom_last(data)
}

.get_spds <- function(source = NULL,
                      destination = NULL,
                      opts = NULL,
                      what = c("vector", "raster")) {
  what <- match.arg(what)
  stopifnot(is.character(source) && length(source) == 1)
  # source <- normalizePath(source, mustWork = FALSE)
  source <- .normalize_source_path(source)
  stopifnot(is.character(destination) && length(destination) == 1)
  # the destination will in general *NOT* exist, so we must use mustWork = FALSE
  destination <- normalizePath(destination, mustWork = FALSE)
  stopifnot(is.null(opts) || is.character(opts))
  if (is.null(opts)) opts <- character(0)

  does_exist <- spds_exists(destination, what = what)
  if (does_exist) {
    return(TRUE)
  }

  util <- switch(what,
    vector = "vectortranslate",
    raster = "translate"
  )
  try(sf::gdal_utils(
    util = util,
    source = source,
    destination = destination,
    options = opts
  ))

  return(spds_exists(destination, what = what))
}

.get_intersection <- function(x, tindex) {
  # https://github.com/r-spatial/sf/issues/2441
  org <- getOption("s2_oriented")
  options(s2_oriented = TRUE)
  on.exit(options(s2_oriented = org))
  suppressMessages(targets <- st_intersects(x, tindex, sparse = FALSE))
  tindex[which(colSums(targets) > 0), ]
}

.try_make_valid <- function(geom) {
  stopifnot(inherits(geom, "sf"))
  is_invalid <- !st_is_valid(geom)

  if (!all(!is_invalid)) {
    geom[is_invalid, ] <- st_make_valid(geom[is_invalid, ])
    still_invalid <- !st_is_valid(geom[is_invalid, ])
    still_invalid <- which(is_invalid)[still_invalid]

    if (length(still_invalid) > 0) {
      geom <- geom[-still_invalid, ]
    }
  }

  types <- st_geometry_type(geom)
  if (any(types == "GEOMETRYCOLLECTION")) {
    cols <- geom[types == "GEOMETRYCOLLECTION", ]
    cols <- suppressWarnings(st_cast(cols))
    types2 <- st_geometry_type(cols)
    cols <- cols[types2 %in% c("POLYGON", "MULTIPOLYGON"), ]
    geom <- rbind(geom[types != "GEOMETRYCOLLECTION", ], cols)
  }

  geom
}

.normalize_source_path <- function(path) {
  # On Windows, 'normalizePath()' would add drive letter ('C:\\', etc.) at the beginning
  # of the path, which is wrong for remote paths like '/vsicurl/https://...'.
  # Since the source file *MUST* exist, we can use mustWork = TRUE.
  # In general, it is *NOT* recommended to use mustWork = FALSE.
  norm_path <- try(normalizePath(path, mustWork = TRUE), silent = TRUE)
  if (!inherits(norm_path, "try-error")) {
    path <- norm_path
  }
  return(path)
}

.init_gdal_http <- function() {
  # Get GDAL HTTP configuration options if already set
  gdal_env <- Sys.getenv(c("GDAL_HTTP_MAX_RETRY",
                           "GDAL_HTTP_RETRY_DELAY",
                           "GDAL_DISABLE_READDIR_ON_OPEN"),
                         unset = NA)
  read_dir <- gdal_env["GDAL_DISABLE_READDIR_ON_OPEN"]
  if (is.na(read_dir)) read_dir <- "EMPTY_DIR"
  retries <- as.integer(gdal_env["GDAL_HTTP_MAX_RETRY"])
  delay <- as.integer(gdal_env["GDAL_HTTP_RETRY_DELAY"])
  # Set to max between already set and mapme_options()
  retries <- max(retries, mapme_options()[["retries"]], na.rm = TRUE)
  delay <- max(delay, mapme_options()[["delay"]], na.rm = TRUE)
  # update mapme_options
  mapme_options(
    retries = retries,
    delay = delay
  )
  # Set GDAL HTTP configuration options
  Sys.setenv(GDAL_HTTP_MAX_RETRY = retries,
             GDAL_HTTP_RETRY_DELAY = delay,
             GDAL_DISABLE_READDIR_ON_OPEN = read_dir)

  # Sys.setenv(
  #   CPL_VSIL_CURL_ALLOWED_EXTENSIONS = ".tif",
  #   GDAL_HTTP_MAX_RETRY = "10",
  #   GDAL_HTTP_RETRY_DELAY = "2",
  #   GDAL_DISABLE_READDIR_ON_OPEN = "YES" (shuold be "TRUE"?)
  #   GDAL_DISABLE_READDIR_ON_OPEN = "EMPTY_DIR"
  # )

  invisible()
}

test_that("get_biodiversity_intactness_index works", {
  sample_path <- system.file("extdata",
                             "shell_beach_protected_area_41057_B.gpkg",
                             package = "mapme.biodiversity")

  expect_error(
    get_biodiversity_intactness_index(NULL),
    "Expecting path to point towards an existing '.asc' file."
  )
  expect_error(
    get_biodiversity_intactness_index(),
    "Expecting path to point towards an existing '.asc' file."
  )
  expect_error(
    get_biodiversity_intactness_index(sample_path),
    "Expecting path to point towards an existing '.asc' file."
  )

  x <- read_sf(sample_path)
  fname_bii <- system.file("res", "biodiversity_intactness_index/lbii.asc",
                           package = "mapme.biodiversity")
  bii <- get_resources(x,
    get_biodiversity_intactness_index(fname_bii)
  )
  expect_equal(
    names(prep_resources(bii)),
    "biodiversity_intactness_index"
  )
})

test_that("soilpoperties works", {
  skip_on_cran()
  skip_if_not(Sys.getenv("USER") == "darius")

  x <- read_sf(
    system.file("extdata", "sierra_de_neiba_478140.gpkg",
      package = "mapme.biodiversity"
    )
  )

  mapme_options(outdir = NULL, verbose = FALSE)
  suppressWarnings(get_resources(x, get_soilgrids(
    layer = "clay",
    depth = "0-5cm",
    stat = "mean"
  )))
  soilgrids <- prep_resources(x)[["soilgrids"]]
  x <- st_transform(x, st_crs(soilgrids))

  csp <- calc_soilproperties()
  expect_true(is.null(csp(x, NULL)))

  csp <- calc_soilproperties()
  result <- csp(x, soilgrids)
  csp <- calc_soilproperties(engine = "extract", stats = c("mean", "median", "sd"))
  result_multi_stat <- csp(x, soilgrids)
  csp <- calc_soilproperties(engine = "zonal")
  result_zonal <- csp(x, soilgrids)
  csp <- calc_soilproperties(engine = "extract")
  result_extract <- csp(x, soilgrids)
  csp <- calc_soilproperties(engine = "exactextract")
  result_exact <- csp(x, soilgrids)

  expect_silent(.check_single_asset(result))
  expect_silent(.check_single_asset(result_multi_stat))
  expect_silent(.check_single_asset(result_zonal))
  expect_silent(.check_single_asset(result_extract))
  expect_silent(.check_single_asset(result_exact))
  expect_equal(result_zonal$value, result_extract$value, tolerance = 1e-4)
  expect_snapshot(result_exact$value)
})

test_that("get_key_biodiversity_areas works", {
  sample_path <- system.file("extdata", "sierra_de_neiba_478140.gpkg",
      package = "mapme.biodiversity"
    )
  aoi <- read_sf(sample_path)
  aoi <- suppressWarnings(st_cast(aoi, to = "POLYGON")[1, ])

  expect_error(
    get_resources(aoi, get_key_biodiversity_areas()),
    "Expecting path to point towards an existing file."
  )
  expect_error(
    get_resources(aoi, get_key_biodiversity_areas("")),
    "Expecting path to point towards an existing file."
  )
  expect_error(
    get_resources(aoi, get_key_biodiversity_areas(NULL)),
    "Expecting path to point towards an existing file."
  )
  expect_error(
    get_key_biodiversity_areas(path = "inst/resources/"),
    "Expecting path to point towards an existing file."
  )

  res <- get_resources(aoi, get_key_biodiversity_areas(sample_path))
  expect_equal(
    names(prep_resources(res)),
    "key_biodiversity_areas"
  )
})

test_that("prep_resources works correctly", {
  .clear_resources()
  x <- read_sf(
    system.file("extdata", "gfw_sample.gpkg",
      package = "mapme.biodiversity"
    )
  )
  x <- st_as_sf(st_make_grid(x, n = 3))

  outdir <- file.path(tempdir(), "mapme.data")
  .copy_resource_dir(outdir)
  mapme_options(outdir = outdir, verbose = FALSE)

  x <- get_resources(
    x,
    get_gfw_treecover(version = "GFC-2023-v1.11"),
    get_gfw_lossyear(version = "GFC-2023-v1.11")
  )

  available_resources <- .avail_resources()
  required_resources <- available_indicators("treecover_area")[["resources"]][[1]][["name"]]
  output <- prep_resources(x[1, ], available_resources, required_resources)

  expect_equal(length(output), 2)
  expect_equal(names(output), c("gfw_lossyear", "gfw_treecover"))
  expect_true(inherits(output[[1]], "SpatRaster"))

  x2 <- read_sf(list.files(
    system.file("extdata", package = "mapme.biodiversity"),
    pattern = "shell_beach", full.names = TRUE
  ))

  x2 <- get_resources(x2, get_gmw(years = 2016))

  available_resources <- .avail_resources()
  required_resources <- available_indicators("mangroves_area")[["resources"]][[1]][["name"]]
  output <- prep_resources(x2, available_resources, required_resources)

  expect_equal(length(output), 1)
  expect_equal(names(output), "gmw")
  output <- output$gmw
  expect_equal(length(output), 1)
  expect_equal(names(output), "gmw_v3_2016_vec.gpkg")
  expect_true(inherits(output[[1]], "sf"))
  expect_error(prep_resources(x[1, ], available_resources, "not-available"))
})


test_that("VRT deletion works as expected", {
  .clear_resources()
  x <- read_sf(
    system.file("extdata", "gfw_sample.gpkg",
      package = "mapme.biodiversity"
    )
  )

  xb <- st_buffer(x, 10000)
  x2 <- st_as_sf(st_as_sfc(st_bbox(xb)))

  r <- rast(ext(xb), nrows = 100, ncols = 100)
  r[] <- runif(ncell(r))
  f <- tempfile(fileext = ".tif")
  writeRaster(r, f)
  fps <- list(test = make_footprints(f, what = "raster"))

  out <- prep_resources(x, avail_resources = fps, resources = "test", mode = "asset")
  expect_true(inMemory(out$test))
  out <- prep_resources(x2, avail_resources = fps, resources = "test", mode = "portfolio")
  expect_true(file.exists(sources(out$test)))
})


test_that(".read_raster works correctly", {
  dummy <- terra::rast()
  dummy_splitted <- aggregate(dummy, fact = c(ceiling(nrow(dummy) / 4), ceiling(ncol(dummy) / 4)))
  dummy_splitted[] <- 1:16
  polys <- terra::as.polygons(dummy_splitted) %>% st_as_sf()
  dummies <- lapply(1:nrow(polys), function(i) crop(dummy_splitted, polys[i, ]))
  temp_loc <- tempfile()
  dir.create(temp_loc, showWarnings = FALSE)
  purrr::walk(1:length(dummies), function(i) {
    writeRaster(dummies[[i]], filename = file.path(temp_loc, paste0("2000_tile_", i, ".tif")))
    writeRaster(dummies[[i]], filename = file.path(temp_loc, paste0("2001_tile_", i, ".tif")))
  })

  files <- list.files(temp_loc, full.names = TRUE)
  footprints <- make_footprints(files, what = "raster")
  footprints[["location"]] <- files
  extent1 <- c(xmin = -179.9, xmax = 179.9, ymin = -89.9, ymax = 89.9)
  extent2 <- c(xmin = -180, xmax = 180, ymin = -90, ymax = 90)
  x <- st_as_sfc(st_bbox(extent1, crs = "EPSG:4326"))

  tiled_temporal <- .read_raster(x, footprints)
  expect_equal(names(tiled_temporal), c("2000_tile_1", "2001_tile_1"))
  expect_equal(as.vector(ext(tiled_temporal)), extent2)

  tiled <- .read_raster(x, footprints[grep("2001", footprints$location), ])
  expect_equal(names(tiled), "2001_tile_1")
  expect_equal(as.vector(ext(tiled)), extent2)

  temporal <- .read_raster(x, footprints[grep("tile_12.tif", footprints$location), ])
  extent1[c(1:4)] <- c(90, 180, -45, 0)
  expect_equal(names(temporal), c("2000_tile_12", "2001_tile_12"))
  expect_equal(as.vector(ext(temporal)), extent1)

  single <- .read_raster(x, footprints[grep("2000_tile_10.tif", footprints$location), ])
  extent1[c(1:4)] <- c(-90, 0, -45, 0)
  expect_equal(names(single), "2000_tile_10")
  expect_equal(as.vector(ext(single)), extent1)

  expect_error(.read_raster(x, footprints[1:24, ]))
})

test_that(".read_vector works", {
  if (utils::compareVersion(sf::sf_extSoftVersion()[["GDAL"]], "3.7.0") < 0) {
    skip("GDAL < 3.7.0")
  }

  # copy to directory with w permissions, as ogrinfo otherwise fails on CRAN
  org <- system.file("extdata", "burundi.gpkg", package = "mapme.biodiversity")
  outdir <- tempfile()
  dir.create(outdir)
  v <- file.path(outdir, basename(org))
  file.copy(org, v)

  x <- st_as_sf(st_as_sfc(st_bbox(read_sf(v))))
  fps <- make_footprints(v, what = "vector")
  fps[["location"]] <- v
  fps <- rbind(fps, fps)
  expect_silent(out <- .read_vector(x, fps))
  expect_true(class(out) == "list")
  expect_equal(length(out), 2)
  expect_equal(names(out), rep("burundi.gpkg", 2))
  expect_true(inherits(out[[1]], "sf"))
})

test_that("can get info from zipped vector resource", {
  if (utils::compareVersion(sf::sf_extSoftVersion()[["GDAL"]], "3.7.0") < 0) {
    skip("GDAL < 3.7.0")
  }
  if (packageVersion("sf") == "1.0.18") skip("sf == '1.0.18'")
  f <- system.file("extdata", "gfw_sample.gpkg",
    package = "mapme.biodiversity"
  )
  outdir <- tempfile()
  dir.create(outdir)
  fzip <- tempfile(tmpdir = outdir, fileext = ".zip")
  zip(fzip, f, flags = "-j")
  Sys.chmod(outdir, mode = "555")
  src <- paste0("/vsizip/", fzip, "/", basename(f))
  expect_true(spds_exists(src, what = "vector"))
  expect_length(.vector_info(src, oo = character(0)), 1)
  expect_silent(fps <- make_footprints(src))
  expect_true(inherits(fps, "sf"))
  Sys.chmod(outdir, mode = "777")
  unlink(outdir, recursive = TRUE)
})

#' Calculate Monthly Burned Area based on MODIS (MCD64A1)
#'
#' Calculates Monthly Burned Area based on the Terra and Aqua combined
#' MCD64A1 Version 6.1. which s a monthly, global gridded 500 meter (m)
#' product containing per-pixel burned-area information.
#'
#' The required resources for this indicator are:
#'  - [mcd64a1]
#'
#' @name burned_area
#' @param engine The preferred processing functions from either one of "zonal",
#'   "extract" or "exactextract" as character.
#' @keywords indicator
#' @returns A function that returns an indicator tibble with variable burned
#'   area and corresponding area (in ha) as values.
#' @references Giglio, L., C. Justice, L. Boschetti, D. Roy. MODIS/Terra+Aqua
#'   Burned Area Monthly L3 Global 500m SIN Grid V061. 2021, distributed by
#'   NASA EOSDIS Land Processes Distributed Active Archive Center.
#'   \doi{doi:10.5067/MODIS/MCD64A1.061}
#' @include register.R
#' @export
#' @examples
#' \dontshow{
#' mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
#' }
#' \dontrun{
#' library(sf)
#' library(mapme.biodiversity)
#'
#' outdir <- file.path(tempdir(), "mapme-data")
#' dir.create(outdir, showWarnings = FALSE)
#'
#' mapme_options(
#'   outdir = outdir,
#'   verbose = FALSE
#' )
#'
#' aoi <- system.file("extdata", "sierra_de_neiba_478140_2.gpkg",
#'   package = "mapme.biodiversity"
#' ) %>%
#'   read_sf() %>%
#'   get_resources(get_mcd64a1(years = 2010)) %>%
#'   calc_indicators(calc_burned_area(engine = "extract")) %>%
#'   portfolio_long()
#'
#' aoi
#' }
calc_burned_area <- function(engine = "extract") {
  engine <- check_engine(engine)

  function(x,
           mcd64a1 = NULL,
           name = "burned_area",
           mode = "asset",
           aggregation = "sum",
           verbose = mapme_options()[["verbose"]]) {
    if (is.null(mcd64a1)) {
      return(NULL)
    }

    x_proj <- st_transform(x, st_crs(mcd64a1))
    mcd64a1 <- terra::mask(mcd64a1, x_proj)
    mcd64a1 <- terra::subst(mcd64a1, from = c(-2, -1, 0, NA), to = 0, others = 1)
    arearaster <- cellSize(mcd64a1, mask = FALSE, unit = "ha")
    mcd64a1 <- mcd64a1 * arearaster

    stats <- select_engine(
      x = x_proj,
      raster = mcd64a1,
      stats = "sum",
      engine = engine,
      name = "burned_area",
      mode = "asset"
    )

    names(stats) <- "value"
    dates <- gsub("^.*?\\.A([0-9]+)\\..*$", "\\1", names(mcd64a1))
    dates <- as.POSIXct(paste0(as.Date(dates, "%Y%j"), "T00:00:00Z"))
    stats[["datetime"]] <- dates
    stats[["unit"]] <- "ha"
    stats[["variable"]] <- "burned_area"
    stats[, c("datetime", "variable", "unit", "value")]
  }
}

register_indicator(
  name = "burned_area",
  description = "Monthly burned area detected by MODIS satellites",
  resources = "mcd64a1"
)

#' Calculate accessibility statistics for the year 2000
#'
#' Accessibility refers to the ease with which cities can be reached from a
#' certain location. This function allows efficient calculation of accessibility
#' statistics (i.e., travel time to the nearest city) for polygons.  For each
#' polygon, the desired statistic/s (mean, median or sd) is/are returned.
#'
#' The required resource for this indicator is:
#'  - [accessibility_2000]
#'
#' @name traveltime_2000
#' @param engine The preferred processing functions from either one of "zonal",
#'   "extract" or "exactextract" as character.
#' @param stats Function to be applied to compute statistics for polygons either
#'   single or multiple inputs as character. Supported statistics are: "mean",
#'   "median", "sd", "min", "max", "sum", "var".
#' @keywords indicator
#' @return A function that returns an indicator tibble with accessibility statistics
#'   for the year 2000 as variables and corresponding values (in minutes) as values.
#' @include register.R
#' @export
#' @examples
#' \dontshow{
#' mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
#' }
#' \dontrun{
#' library(sf)
#' library(mapme.biodiversity)
#'
#' outdir <- file.path(tempdir(), "mapme-data")
#' dir.create(outdir, showWarnings = FALSE)
#'
#' mapme_options(
#'   outdir = outdir,
#'   verbose = FALSE
#' )
#'
#' aoi <- system.file("extdata", "sierra_de_neiba_478140_2.gpkg",
#'   package = "mapme.biodiversity"
#' ) %>%
#'   read_sf() %>%
#'   get_resources(get_accessibility_2000()) %>%
#'   calc_indicators(
#'     calc_traveltime_2000(stats = c("mean", "median", "sd"), engine = "extract")
#'   ) %>%
#'   portfolio_long()
#'
#' aoi
#' }
calc_traveltime_2000 <- function(engine = "extract", stats = "mean") {
  engine <- check_engine(engine)
  stats <- check_stats(stats)

  function(x,
           accessibility_2000 = NULL,
           name = "traveltime_2000",
           mode = "asset",
           aggregation = "stat",
           verbose = mapme_options()[["verbose"]]) {

    # Ensure the resource is provided
    if (is.null(accessibility_2000)) {
      return(NULL)
    }

    # Set NoData value given by gdalinfo as NULL
    accessibility_2000 <- clamp(accessibility_2000, lower = -2147483646, upper = Inf, values = FALSE)

    # Process the resource using the selected engine and compute the statistics
    result <- select_engine(
      x = x,
      raster = accessibility_2000,
      stats = stats,
      engine = engine,
      name = "traveltime_2000",
      mode = "asset"
    )

    # Return the results in a long format with metadata
    result %>%
      tidyr::pivot_longer(cols = dplyr::everything(), names_to = "variable") %>%
      dplyr::mutate(
        datetime = as.POSIXct("2000-01-01T00:00:00Z"),
        unit = "minutes"
      ) %>%
      dplyr::select(datetime, variable, unit, value)
  }
}

register_indicator(
  name = "traveltime_2000",
  description = "Statistics of traveltime to the closests city in 2000",
  resources = "accessibility_2000"
)

#' Terrestrial and Aquatic Biomes
#'
#' This resource is part of the Global Assessment Report on Biodiversity and
#' Ecosystem Services and represents a division of the Earth's surface
#' into several subcategories. The classification differentiates between
#' biomes and anthromes. Biomes are differentiated between terrestrial and aquatic
#' biomes.
#'
#' Terrestrial biomes include:
#' - Tropical and subtropical dry and humid forests
#' - Temperate and boreal forests and woodlands
#' - Mediterranean forests, woodlands and scrub
#' - Tundra and High Mountain habitats
#' - Tropical and subtropical savannas and grasslands
#' - Temperate Grasslands
#' - Deserts and xeric shrublands
#' - Wetlands – peatlands, mires, bogs
#'
#' Aquatic biomes include:
#' - Cryosphere
#' - Aquaculture areas
#' - Inland surface waters and water bodies/freshwater
#' - Shelf ecosystems (neritic and intertidal/littoral zone)
#' - Open ocean pelagic systems (euphotic zone)
#'
#' @name ipbes_biomes
#' @keywords resource
#' @returns A function that returns an `sf` footprint object.
#' @references IPBES (2019): Summary for policymakers of the global assessment
#' report on biodiversity and ecosystem services of the Intergovernmental
#' Science-Policy Platform on Biodiversity and Ecosystem Services. S. Díaz, J.
#' Settele, E. S. Brondízio, H. T. Ngo, M. Guèze, J. Agard, A. Arneth, P.
#' Balvanera, K. A. Brauman, S. H. M. Butchart, K. M. A. Chan, L. A. Garibaldi,
#' K. Ichii, J. Liu, S. M. Subramanian, G. F. Midgley, P. Miloslavich, Z.
#' Molnár, D. Obura, A. Pfaff, S. Polasky, A. Purvis, J. Razzaque, B. Reyers, R.
#'  Roy Chowdhury, Y. J. Shin, I. J. Visseren-Hamakers, K. J. Willis, and C. N.
#'  Zayas (eds.). IPBES secretariat, Bonn, Germany. 56 pages.
#'  \doi{doi:10.5281/zenodo.3553579}
#' @source \url{https://zenodo.org/records/3975694}
#' @include register.R
#' @export
get_ipbes_biomes <- function() {
  function(
      x,
      name = "ipbes_biomes",
      type = "raster",
      outdir = mapme_options()[["outdir"]],
      verbose = mapme_options()[["verbose"]]) {
    url <- "/vsicurl/https://zenodo.org/records/3975694/files/IPBES_UoA_biomes_JK.tif"
    make_footprints(
      url,
      what = "raster",
      co = c("-co", "INTERLEAVE=BAND", "-co", "COMPRESS=LZW", "-ot", "Float32")
    )
  }
}

register_resource(
  name = "ipbes_biomes",
  description = "Global Assessment Report on Biodiversity and Ecosystem Services division of the earth's surface into biomes and anthromes.",
  licence = "CC 4.0",
  source = "https://zenodo.org/records/3975694",
  type = "raster"
)

#' IUCN Red List of Threatened Species
#'
#' This resource is part of the spatial data set Red List of Threatened Species
#' released by IUCN. It is free to use under a non-commercial licence. For
#' commercial uses, a request has to be sent to Integrated Biodiversity
#' Assessment Tool (IBAT).
#'
#' To use this data in mapme workflows, you will have to manually download the
#' global data set and point towards the file path on your local machine.
#' Please find the available data under the source link given below.
#'
#' @name iucn
#' @param paths A character vector to the respective species range files in GTiff
#'   format. Note, that theses files have to be downloaded manually.
#' @keywords resource
#' @returns A function that returns an `sf` footprint object.
#' @references IUCN (2024). The IUCN Red List of Threatened Species.
#'   \url{https://www.iucnredlist.org}
#' @source \url{https://www.iucnredlist.org/resources/other-spatial-downloads}
#' @include register.R
#' @export
get_iucn <- function(paths = NULL) {
  if (is.null(paths) || !all(endsWith(paths, ".tif")) || !all(sapply(paths, spds_exists, what = "raster"))) {
    stop("Expecting paths to point towards existing GTiff files.")
  }

  if (!all(grepl("_SR_", basename(paths)))) {
    stop("Filenames do match expected schema for IUCN species richness rasters.")
  }

  function(
      x,
      name = "iucn",
      type = "raster",
      outdir = mapme_options()[["outdir"]],
      verbose = mapme_options()[["verbose"]]) {
    make_footprints(
      paths,
      what = "raster",
      co = c("-co", "INTERLEAVE=BAND", "-co", "COMPRESS=LZW", "-ot", "Int16")
    )
  }
}

register_resource(
  name = "iucn",
  description = "IUCN Species Richness Raser Dataset",
  licence = "https://www.iucnredlist.org/terms/terms-of-use",
  source = "https://www.iucnredlist.org/resources/other-spatial-downloads",
  type = "raster"
)

test_that("get_iucn works", {
  iucn_dir <- system.file("res", "iucn", package = "mapme.biodiversity")
  iucn_files <- list.files(iucn_dir, pattern = "_SR_", full.names = TRUE)

  msg <- "Expecting paths to point towards existing GTiff files."
  expect_error(get_iucn(NULL), msg)
  expect_error(get_iucn("does-not-exist.tif"), msg)
  gpkg <- system.file("extdata", "burundi.gpkg", package = "mapme.biodiversity")
  expect_error(get_iucn(gpkg), msg)
  other_tif <- system.file("extdata", "burundi_worldpop.tif", package = "mapme.biodiversity")
  msg <- "Filenames do match expected schema for IUCN species richness rasters."
  expect_error(get_iucn(c(iucn_files, other_tif)), msg)
  fps <- get_iucn(iucn_files)()
  expect_silent(.check_footprints(fps))
})

#' Key Biodiversity Areas
#'
#' This resource contains outlines of key biodiversity areas, which are areas
#' representing sites with specific importance for nature conservation.
#'
#' To use this data in mapme workflows, you will have to manually download the
#' global data set and point towards its file path on your local machine.
#' Please find the available data under the source link given below.
#'
#' @name key_biodiversity_areas_resource
#' @keywords resource
#' @param path A character vector to the key biodiversity areas GPKG file.
#'   Note, that the file has to be downloaded manually.
#' @returns A function that returns an `sf` footprints object.
#' @references BirdLife International (2024). The World Database of
#'   Key Biodiversity Areas. Developed by the KBA Partnership: BirdLife
#'   International, International Union for the Conservation of Nature,
#'   Amphibian Survival Alliance, Conservation International, Critical Ecosystem
#'   Partnership Fund, Global Environment Facility, Re:wild, NatureServe,
#'   Rainforest Trust, Royal Society for the Protection of Birds, Wildlife
#'   Conservation Society and World Wildlife Fund. Available at
#'   www.keybiodiversityareas.org.
#' @source \url{https://www.keybiodiversityareas.org/request-gis-data}
#' @include register.R
#' @export
get_key_biodiversity_areas <- function(path = NULL) {
  if(is.null(path) || !spds_exists(path, what = "raster")) {
    stop("Expecting path to point towards an existing file.")
  }

  function(
      x,
      name = "key_biodiversity_areas",
      type = "vector",
      outdir = mapme_options()[["outdir"]],
      verbose = mapme_options()[["verbose"]]) {
    bbox <- st_bbox(c(xmin = -180.0, xmax = 180.0, ymin = -80.0, ymax = 82.0), crs = "EPSG:4326")
    tile <- st_as_sf(st_as_sfc(st_bbox(bbox)))

    if (!is.null(outdir)) {
      dsn <- file.path(outdir, basename(path))
      tile[["source"]] <- dsn
      if (spds_exists(dsn)) {
        return(make_footprints(tile, what = "vector"))
      }
    } else {
      dsn <- file.path(tempdir(), basename(path))
      tile[["source"]] <- dsn
    }

    x <- .try_make_valid(read_sf(path))
    write_sf(x, dsn = dsn)
    make_footprints(tile, what = "vector")
  }
}

register_resource(
  name = "key_biodiversity_areas",
  description = "Key Biodiversity Areas",
  licence = "https://www.keybiodiversityareas.org/termsofservice",
  source = "https://www.keybiodiversityareas.org/kba-data",
  type = "vector"
)

#' Calculate Terrain Ruggedness Index (TRI) statistics
#'
#' Terrain Ruggedness Index is a measurement developed by Riley, et al. (1999).
#' The elevation difference between the centre pixel and its eight immediate
#' pixels are squared and then averaged and its square root is taken to get
#' the TRI value. This function allows to calculate terrain ruggedness
#' index (tri) statistics for polygons. For each polygon, the desired statistic(s)
#' are returned.
#'
#'  The range of index values and corresponding meaning:
#'   - 0-80 m - level surface
#'   - 81-116 m - nearly level surface
#'   - 117-161 m - slightly rugged surface
#'   - 162-239 m - intermediately rugged surface
#'   - 240-497 m - moderately rugged surface
#'   - 498-958 m - highly rugged surface
#'   - 959-4367 m  extremely rugged surface
#'
#' The required resources for this indicator are:
#'  - [nasa_srtm]
#'
#' @name tri
#' @param engine The preferred processing functions from either one of "zonal",
#'   "extract" or "exactextract" as character.
#' @param stats Function to be applied to compute statistics for polygons either
#'   single or multiple inputs as character. Supported statistics are: "mean",
#'   "median", "sd", "min", "max", "sum" "var".
#' @keywords indicator
#' @returns A function that returns an indicator tibble with tri as variable and
#'   the respective statistic as value.
#'
#' @references Riley, S. J., DeGloria, S. D., & Elliot, R. (1999). Index that quantifies
#'   topographic heterogeneity. Intermountain Journal of Sciences, 5(1-4), 23-27.
#' @include register.R
#' @export
#' @examples
#' \dontshow{
#' mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
#' }
#' \dontrun{
#' library(sf)
#' library(mapme.biodiversity)
#'
#' outdir <- file.path(tempdir(), "mapme-data")
#' dir.create(outdir, showWarnings = FALSE)
#'
#' mapme_options(
#'   outdir = outdir,
#'   verbose = FALSE
#' )
#'
#' aoi <- system.file("extdata", "sierra_de_neiba_478140_2.gpkg",
#'   package = "mapme.biodiversity"
#' ) %>%
#'   read_sf() %>%
#'   get_resources(get_nasa_srtm()) %>%
#'   calc_indicators(
#'     calc_tri(stats = c("mean", "median", "sd", "var"), engine = "extract")
#'   ) %>%
#'   portfolio_long()
#'
#' aoi
#' }
calc_tri <- function(engine = "extract", stats = "mean") {
  engine <- check_engine(engine)
  stats <- check_stats(stats)

  function(x,
           nasa_srtm = NULL,
           name = "tri",
           mode = "asset",
           aggregation = "stat",
           verbose = mapme_options()[["verbose"]]) {
    if (is.null(nasa_srtm)) {
      return(NULL)
    }

    tri <- terra::terrain(
      nasa_srtm,
      v = "TRI",
      unit = "degrees",
      neighbors = 8
    )

    result <- select_engine(
      x = x,
      raster = tri,
      stats = stats,
      engine = engine,
      name = "tri",
      mode = "asset"
    )

    result %>%
      tidyr::pivot_longer(cols = dplyr::everything(), names_to = "variable") %>%
      dplyr::mutate(
        datetime = as.POSIXct("2000-02-01T00:00:00Z"),
        unit = "m"
      ) %>%
      dplyr::select(datetime, variable, unit, value)
  }
}

register_indicator(
  name = "tri",
  description = "Statistics of terrain ruggedness index based on NASA SRTM DEM",
  resources = "nasa_srtm"
)

test_that("terrain ruggedness index works", {
  x <- read_sf(
    system.file("extdata", "sierra_de_neiba_478140.gpkg",
      package = "mapme.biodiversity"
    )
  )
  nasa_srtm <- list.files(system.file("res", "nasa_srtm",
    package = "mapme.biodiversity"
  ), pattern = ".tif$", full.names = TRUE)
  nasa_srtm <- rast(nasa_srtm)

  ctri <- calc_tri()
  expect_true(is.null(ctri(x, NULL)))
  result <- ctri(x, nasa_srtm)
  ctri <- calc_tri(stats = c("mean", "median", "sd"))
  result_multi_stat <- ctri(x, nasa_srtm)
  ctri <- calc_tri(engine = "zonal")
  result_zonal <- ctri(x, nasa_srtm)
  ctri <- calc_tri(engine = "extract")
  result_extract <- ctri(x, nasa_srtm)
  ctri <- calc_tri(engine = "exactextract")
  result_exact <- ctri(x, nasa_srtm)

  expect_silent(.check_single_asset(result))
  expect_silent(.check_single_asset(result_multi_stat))
  expect_silent(.check_single_asset(result_zonal))
  expect_silent(.check_single_asset(result_extract))
  expect_silent(.check_single_asset(result_exact))

  vars <- c("tri_mean", "tri_median", "tri_sd")
  expect_equal(unique(result_multi_stat$variable), vars)
  expect_equal(result_zonal$value, result_extract$value, tolerance = 1e-4)
  expect_snapshot(result_exact$value)
})

#' Carbon Layers
#'
#' These resources are from the publication by Noon et al. (2022) "Mapping the
#' irrecoverable carbon in Earth’s ecosystems". This publication differentiates
#' between 3 different kinds of carbon with varying degrees of manageability
#' by humans. All three layers are available for above and below ground
#' carbon, as well as a layer combining the two.
#'
#' It may be required to increase the timeout option to successfully download
#' theses layers from their source location via e.g. `options(timeout = 600)`.
#'
#' @details
#' Irrecoverable carbon is defined as the amount of carbon, that, if lost today,
#' cannot be recovered until mid 21st century (so within 30 years, considering
#' the publication date).
#'
#' @name carbon_resources
#' @keywords resource
#' @returns A function that returns an `sf` footprint object.
#' @references Noon, M.L., Goldstein, A., Ledezma, J.C. et al. Mapping the
#'   irrecoverable carbon in Earth’s ecosystems. Nat Sustain 5, 37–46 (2022).
#'   \doi{doi:10.1038/s41893-021-00803-6}
#' @source \url{https://zenodo.org/records/4091029}
#' @include register.R
#' @export
get_irr_carbon <- function() {
  if (is.null(mapme_options()[["outdir"]])) {
    warning(paste(
      "Carbon layers must be downloaded from the source location",
      "irrespective if `outdir` was specified or not."
    ))
  }

  function(x,
           name = "irr_carbon",
           type = "raster",
           outdir = mapme_options()[["outdir"]],
           verbose = mapme_options()[["verbose"]]) {
    .fetch_carbon("Irrecoverable", outdir)
  }
}

register_resource(
  name = "irr_carbon",
  description = "Amount of carbon irrecoverably lost by a typical land use conversion event until mid-century.",
  licence = "CC NC 4.0",
  source = "https://zenodo.org/records/4091029",
  type = "raster"
)

#' @details
#' Vulnerable carbon is defined as the amount of carbon that would be lost in a
#' hypothetical but typical conversion event (without including information
#' of the probability of such an event to be actually occurring).
#'
#' @name carbon_resources
#' @include register.R
#' @export
get_vul_carbon <- function() {
  if (is.null(mapme_options()[["outdir"]])) {
    warning(paste(
      "Carbon layers must be downloaded from the source location",
      "irrespective if `outdir` was specified or not."
    ))
  }

  function(x,
           name = "vul_carbon",
           type = "raster",
           outdir = mapme_options()[["outdir"]],
           verbose = mapme_options()[["verbose"]]) {
    .fetch_carbon("Vulnerable", outdir)
  }
}

register_resource(
  name = "vul_carbon",
  description = "Amount of carbon that is vulnerable to a typical land use conversion event.",
  licence = "CC NC 4.0",
  source = "https://zenodo.org/records/4091029",
  type = "raster"
)


#' @details
#' Manageable carbon is defined as all land areas, expect cyrosols, because
#' carbon loss is driven by direct land-use conversion which could be halted or
#' because climate change impacts affecting the area can potentially be directly
#' mitigated through adaptive management.
#'
#' @name carbon_resources
#' @include register.R
#' @export
get_man_carbon <- function() {
  if (is.null(mapme_options()[["outdir"]])) {
    warning(paste(
      "Carbon layers must be downloaded from the source location",
      "irrespective if `outdir` was specified or not."
    ))
  }

  function(x,
           name = "man_carbon",
           type = "raster",
           outdir = mapme_options()[["outdir"]],
           verbose = mapme_options()[["verbose"]]) {
    .fetch_carbon("Manageable", outdir)
  }
}

register_resource(
  name = "man_carbon",
  description = "Amount of carbon that is manageable by humans.",
  licence = "CC NC 4.0",
  source = "https://zenodo.org/records/4091029",
  type = "raster"
)


#' @importFrom utils unzip download.file
.fetch_carbon <- function(layer = NULL, outdir = NULL) {
  tifs <- c(
    "%s_C_Biomass_%s.tif",
    "%s_C_Soil_%s.tif",
    "%s_C_Total_%s.tif"
  )
  tifs <- c(sprintf(tifs, layer, 2010), sprintf(tifs, layer, 2018))
  has_outdir <- !is.null(outdir)
  tmpdir <- tempfile()
  dir.create(tmpdir)

  if (has_outdir) {
    tifs <- file.path(outdir, tifs)
  } else {
    tifs <- file.path(tmpdir, tifs)
  }

  is_available <- purrr::map_lgl(tifs, spds_exists, what = "raster")
  if (all(is_available)) {
    return(make_footprints(tifs, what = "raster"))
  }

  srcs <- .get_goldstein_url(sprintf("^%s_Carbon_\\d{4}\\.zip$", layer))
  srcs$filename <- file.path(tmpdir, srcs[["filename"]])

  purrr::walk2(srcs$url, srcs$filename, function(src, dst) {
    download.file(src, dst, mode = ifelse(Sys.info()["sysname"] == "Windows", "wb", "w"))
    unzip(dst, junkpaths = TRUE, exdir = tmpdir)
  })

  tifs <- list.files(tmpdir, pattern = "*.tif$", recursive = TRUE, full.names = TRUE)
  tifs <- grep(layer, tifs, value = TRUE)
  make_footprints(
    tifs,
    what = "raster",
    co = c("-of", "COG", "-co", "COMPRESS=LZW", "-ot", "UInt16")
  )
}


#' @noRd
#' @importFrom httr2 request req_perform req_retry resp_status resp_body_json
.get_goldstein_url <- function(layer) {
  baseurl <- "https://zenodo.org/api/records/4091029"
  is_transient <- function(resp) resp_status(resp) %in% c(429, 503, 504)
  rsp <- req_perform(req_retry(request(baseurl),
    max_seconds = 10,
    is_transient = is_transient
  ))
  cnt <- resp_body_json(rsp)
  files_df <- lapply(cnt[["files"]], function(x) data.frame(filename = x[["key"]], url = x[["links"]][["self"]]))
  files_df <- do.call(rbind, files_df)
  files_df <- files_df[grep(layer, files_df[["filename"]]), ]
  files_df[grep(".zip$", files_df[["filename"]]), ]
}

test_that("get_carbon works", {
  skip_on_cran()
  outdir <- file.path(tempdir(), "mapme.data")
  .copy_resource_dir(outdir)

  mapme_options(outdir = NULL)
  expect_warning(get_irr_carbon())
  expect_warning(get_man_carbon())
  expect_warning(get_vul_carbon())

  mapme_options(outdir = outdir)
  gca <- get_irr_carbon()
  fps <- gca(outdir = file.path(outdir, "irr_carbon"))
  expect_silent(.check_footprints(fps))
  expect_equal(nrow(fps), 6)

  gca <- get_man_carbon()
  fps <- gca(outdir = file.path(outdir, "man_carbon"))
  expect_silent(.check_footprints(fps))
  expect_equal(nrow(fps), 6)

  gca <- get_vul_carbon()
  fps <- gca(outdir = file.path(outdir, "vul_carbon"))
  expect_silent(.check_footprints(fps))
  expect_equal(nrow(fps), 6)
})

test_that(".get_goldstein_url works", {
  skip_on_cran()
  skip_if_not(Sys.getenv("USER") == "darius")
  expect_silent(result <- .get_goldstein_url("Irrecoverable"))
  expect_equal(nrow(result), 2)
  filenames <- c("Irrecoverable_Carbon_2010.zip", "Irrecoverable_Carbon_2018.zip")
  expect_equal(result$filename, filenames)
})

test_that("calc_irr_carbon works", {
  # CRAN checks fail on some platforms, not reproducible
  skip_on_cran()
  x <- read_sf(system.file("extdata", "shell_beach_protected_area_41057_B.gpkg",
    package = "mapme.biodiversity"
  ))
  .clear_resources()
  outdir <- file.path(tempdir(), "mapme.data")
  .copy_resource_dir(outdir)
  mapme_options(outdir = outdir, verbose = FALSE)

  get_resources(x, get_irr_carbon())
  irr_carbon <- prep_resources(x)[["irr_carbon"]]

  expect_error(calc_irr_carbon(type = "other"))
  cb <- calc_irr_carbon(stats = c("sum", "mean"))

  expect_silent(result <- cb(x, irr_carbon))
  expect_silent(.check_single_asset(result))
  expect_equal(nrow(result), 4)
  vars <- c("irr_carbon_total_sum", "irr_carbon_total_mean")
  expect_equal(unique(result$variable), vars)
  vals <- c(406579.29, 1373.57, 407012.885, 1375.04)
  expect_equal(result$value, vals, tolerance = 0.01)

  cb <- calc_irr_carbon(stats = "sum", type = "biomass")
  expect_silent(result <- cb(x, irr_carbon))
  expect_silent(.check_single_asset(result))
  expect_equal(nrow(result), 2)
  expect_equal(result$variable, rep("irr_carbon_biomass_sum", 2))
  vals <- c(43832.18, 42947.41)
  expect_equal(result$value, vals, tolerance = 0.01)
})


test_that("calc_vul_carbon works", {
  # CRAN checks fail on some platforms, not reproducible
  skip_on_cran()
  x <- read_sf(system.file("extdata", "shell_beach_protected_area_41057_B.gpkg",
    package = "mapme.biodiversity"
  ))
  .clear_resources()
  outdir <- file.path(tempdir(), "mapme.data")
  .copy_resource_dir(outdir)
  mapme_options(outdir = outdir, verbose = FALSE)

  get_resources(x, get_vul_carbon())
  vul_carbon <- prep_resources(x)[["vul_carbon"]]

  expect_error(calc_vul_carbon(type = "other"))
  cb <- calc_vul_carbon(stats = c("sum", "mean"))

  expect_silent(result <- cb(x, vul_carbon))
  expect_silent(.check_single_asset(result))
  expect_equal(nrow(result), 4)
  vars <- c("vul_carbon_total_sum", "vul_carbon_total_mean")
  expect_equal(unique(result$variable), vars)
  vals <- c(696439, 2353, 696192, 2352)
  expect_equal(result$value, vals, tolerance = 0.01)

  cb <- calc_vul_carbon(stats = "sum", type = "biomass")
  expect_silent(result <- cb(x, vul_carbon))
  expect_silent(.check_single_asset(result))
  expect_equal(nrow(result), 2)
  expect_equal(result$variable, rep("vul_carbon_biomass_sum", 2))
  vals <- c(249356, 248878)
  expect_equal(result$value, vals, tolerance = 0.01)
})

test_that("calc_man_carbon works", {
  # CRAN checks fail on some platforms, not reproducible
  skip_on_cran()
  x <- read_sf(system.file("extdata", "shell_beach_protected_area_41057_B.gpkg",
    package = "mapme.biodiversity"
  ))
  .clear_resources()
  outdir <- file.path(tempdir(), "mapme.data")
  .copy_resource_dir(outdir)
  mapme_options(outdir = outdir, verbose = FALSE)

  get_resources(x, get_man_carbon())
  man_carbon <- prep_resources(x)[["man_carbon"]]

  expect_error(calc_man_carbon(type = "other"))
  cb <- calc_man_carbon(stats = c("sum", "mean"))

  expect_silent(result <- cb(x, man_carbon))
  expect_silent(.check_single_asset(result))
  expect_equal(nrow(result), 4)
  vars <- c("man_carbon_total_sum", "man_carbon_total_mean")
  expect_equal(unique(result$variable), vars)
  vals <- c(819413, 2768, 819413, 2768)
  expect_equal(result$value, vals, tolerance = 0.01)

  cb <- calc_man_carbon(stats = "sum", type = "biomass")
  expect_silent(result <- cb(x, man_carbon))
  expect_silent(.check_single_asset(result))
  expect_equal(nrow(result), 2)
  expect_equal(result$variable, rep("man_carbon_biomass_sum", 2))
  vals <- c(248949, 248949)
  expect_equal(result$value, vals, tolerance = 0.01)
})

#' Treecover for the year 2000
#'
#' This resource is part of the publication by Hansen et al. (2013)
#' represents "tree cover in the year 2000, defined as canopy closure for all
#' vegetation taller than 5m in height. Encoded as a percentage per output grid
#' cell, in the range 0–100." Due to changes in the satellites products used
#' in the compilation of the treecover product, results before the year 2011
#' and afterwards are not directly comparable until reprocessing has finished.
#' Users should be aware of this limitation, especially when the timeframe
#' of the analysis spans over the two periods delimited by the year 2011.
#
#' @name gfw_treecover
#' @param version The version of the dataset to download. Defaults to
#'   "GFC-2024-v1.12". Check mapme.biodiversity:::.available_gfw_versions()
#'   to get a list of available versions
#' @keywords resource
#' @returns A function that returns an `sf` footprint object.
#' @references Hansen, M. C., P. V. Potapov, R. Moore, M. Hancher, S. A.
#' Turubanova, A. Tyukavina, D. Thau, S. V. Stehman, S. J. Goetz, T. R.
#' Loveland, A. Kommareddy, A. Egorov, L. Chini, C. O. Justice, and J. R. G.
#' Townshend. 2013. “High-Resolution Global Maps of 21st-Century Forest Cover
#' Change.” Science 342 (15 November): 850–53.
#' @source \url{https://data.globalforestwatch.org/documents/tree-cover-2000/explore}
#' @include register.R
#' @export
get_gfw_treecover <- function(version = "GFC-2024-v1.12") {
  version <- .check_gfw_version(version)

  function(x,
           name = "gfw_treecover",
           type = "raster",
           outdir = mapme_options()[["outdir"]],
           verbose = mapme_options()[["verbose"]]) {
    # make the GFW grid and construct urls for intersecting tiles
    baseurl <- sprintf(
      "/vsicurl/https://storage.googleapis.com/earthenginepartners-hansen/%s/",
      version
    )
    grid_gfc <- make_global_grid(
      xmin = -180, xmax = 170, dx = 10,
      ymin = -50, ymax = 80, dy = 10
    )
    tile_ids <- unique(unlist(st_intersects(x, grid_gfc)))
    if (length(tile_ids) == 0) {
      stop("The extent of the portfolio does not intersect with the GFW grid.",
        call. = FALSE
      )
    }
    ids <- sapply(tile_ids, function(n) .get_gfw_tile_id(grid_gfc[n, ]))
    urls <- sprintf(
      "%sHansen_%s_treecover2000_%s.tif",
      baseurl, version, ids
    )
    fps <- grid_gfc[tile_ids, ]
    fps[["source"]] <- urls
    make_footprints(fps,
      what = "raster",
      co = c("-co", "INTERLEAVE=BAND", "-co", "COMPRESS=LZW", "-ot", "Byte")
    )
  }
}

.available_gfw_versions <- function() {
  c(
    "GFC-2015-v1.3", "GFC-2016-v1.4", "GFC-2017-v1.5",
    "GFC-2018-v1.6", "GFC-2019-v1.7", "GFC-2020-v1.8",
    "GFC-2021-v1.9", "GFC-2022-v1.10", "GFC-2023-v1.11",
    "GFC-2024-v1.12"
  )
}

.check_gfw_version <- function(version) {
  if (!version %in% .available_gfw_versions()) {
    stop(
      sprintf(
        "Wrong version specified for treecover resource. Select one of %s.",
        paste(.available_gfw_versions(), collapse = ", ")
      ),
      call. = FALSE
    )
  }
  invisible(version)
}

.get_gfw_tile_id <- function(tile) {
  min_x <- st_bbox(tile)[1]
  max_y <- st_bbox(tile)[4]

  # prepare tile names
  if (min_x < 0) {
    min_x <- paste0(sprintf("%03i", abs(min_x)), "W")
  } else {
    min_x <- paste0(sprintf("%03i", min_x), "E")
  }
  if (max_y < 0) {
    max_y <- paste0(sprintf("%02i", abs(max_y)), "S")
  } else {
    max_y <- paste0(sprintf("%02i", max_y), "N")
  }

  paste0(max_y, "_", min_x)
}


register_resource(
  name = "gfw_treecover",
  description = "Global Forest Watch - Percentage of canopy closure in 2000",
  licence = "CC-BY 4.0",
  source = "https://data.globalforestwatch.org/documents/tree-cover-2000/explore",
  type = "raster"
)

test_that(".get_gfw_tile_id works", {
  gfw_grid <- make_global_grid()
  expect_equal(.get_gfw_tile_id(gfw_grid[100, ]), "20S_110E")
})

test_that(".get_gfw_treecover works", {
  skip_on_cran()

  x <- read_sf(
    system.file("extdata", "sierra_de_neiba_478140.gpkg",
      package = "mapme.biodiversity"
    )
  )
  expect_error(get_gfw_treecover("na"))
  gt <- get_gfw_treecover()
  expect_silent(.check_resource_fun(gt))
  expect_silent(fps <- gt(x))
  expect_silent(.check_footprints(fps))
  expect_equal(fps$filename, "Hansen_GFC-2024-v1.12_treecover2000_20N_080W.tif")
})

#' Calculate emission statistics
#'
#' This functions allows to efficiently calculate emission statistics for
#' areas of interest. For each year in the analysis timeframe, the forest losses
#' from Hansen et al. (2013) are overlayed with the respective emission layer
#' from Harris et al. (2021) and area-wise emission statistics are calculated
#' for each year.
#'
#' The required resources for this indicator are:
#'  - [gfw_treecover]
#'  - [gfw_lossyear]
#'  - [gfw_emissions]
#'
#' @name treecoverloss_emissions
#' @param years A numeric vector with the years for which to calculate emissions
#'   caused by treecover loss.
#' @param min_size The minimum size of a forest patch in ha.
#' @param min_cover The minimum threshold of stand density for a pixel to be
#'   considered forest in the year 2000.
#' @keywords indicator
#' @returns A function that returns an indicator tibble with emissions as variable
#'   and emitted CO2 equivalent (in Mg) as value.
#' @include register.R
#' @export
#' @examples
#' \dontshow{
#' mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
#' }
#' \dontrun{
#' library(sf)
#' library(mapme.biodiversity)
#'
#' outdir <- file.path(tempdir(), "mapme-data")
#' dir.create(outdir, showWarnings = FALSE)
#'
#' mapme_options(
#'   outdir = outdir,
#'   verbose = FALSE
#' )
#'
#' aoi <- system.file("extdata", "sierra_de_neiba_478140_2.gpkg",
#'   package = "mapme.biodiversity"
#' ) %>%
#'   read_sf() %>%
#'   get_resources(
#'     get_gfw_treecover(version = "GFC-2024-v1.12"),
#'     get_gfw_lossyear(version = "GFC-2024-v1.12"),
#'     get_gfw_emissions()
#'   ) %>%
#'   calc_indicators(
#'     calc_treecoverloss_emissions(years = 2016:2017, min_size = 1, min_cover = 30)
#'   ) %>%
#'   portfolio_long()
#'
#' aoi
#' }
calc_treecoverloss_emissions <- function(years = 2000:2024,
                                         min_size = 10,
                                         min_cover = 35) {
  check_namespace("exactextractr")
  check_namespace("landscapemetrics", error = FALSE)
  min_cover <- .gfw_check_min_cover(min_cover, "treecoverloss_emissions")
  min_size <- .gfw_check_min_size(min_size, "treecoverloss_emissions")
  years <- .gfw_check_years(years, "treecoverloss_emissions")

  function(x,
           gfw_treecover,
           gfw_lossyear,
           gfw_emissions,
           name = "treecoverloss_emissions",
           mode = "asset",
           aggregation = "sum",
           verbose = mapme_options()[["verbose"]]) {
    emissions <- NULL

    if (any(is.null(gfw_treecover), is.null(gfw_lossyear), is.null(gfw_emissions))) {
      return(NULL)
    }
    # mask gfw
    gfw_treecover <- terra::mask(gfw_treecover, x)
    # check if gfw_treecover only contains 0s, e.g. on the ocean
    if (.gfw_empty_raster(gfw_treecover, min_cover)) {
      return(NULL)
    }
    # prepare gfw rasters
    gfw <- .gfw_prep_rasters(x, gfw_treecover, gfw_lossyear, gfw_emissions, min_cover)

    # retrieves maximum lossyear value from layer name
    max_year <- as.numeric(
      gsub(
        ".*GFC-([0-9]+)-.*", "\\1",
        names(gfw_lossyear)
      )
    )

    if (max_year < min(years)) {
      return(NULL)
    }

    gfw_stats <- exactextractr::exact_extract(
      gfw, x, function(data, min_size) {
        # retain only forest pixels and set area to ha
        data <- .prep_gfw_data(data, min_size)
        emissions <- .sum_gfw(data, "emissions", max_year)

        if (all(emissions[["emissions"]] == 0)) {
          result <- tibble::tibble(
            years = years,
            emissions = 0
          )
          return(result)
        }

        emissions <- emissions[emissions[["years"]] %in% years, ]
        emissions[, c("years", "emissions")]
      },
      min_size = min_size, coverage_area = TRUE, summarize_df = TRUE
    )

    gfw_stats %>%
      dplyr::mutate(
        datetime = as.POSIXct(paste0(years, "-01-01T00:00:00Z")),
        variable = "emissions",
        unit = "Mg",
        value = emissions
      ) %>%
      dplyr::select(datetime, variable, unit, value) %>%
      tibble::as_tibble()
  }
}


register_indicator(
  name = "treecoverloss_emissions",
  description = "Greenouse gas emissions cause by forest loss by year",
  resources = c(
    "gfw_treecover",
    "gfw_lossyear",
    "gfw_emissions"
  )
)

#' Climate Hazards Group InfraRed Precipitation with Station data (CHIRPS)
#'
#' This resource is published by Funk et al. (2015) and represents a quasi-global
#' (50°S-50°S) rainfall estimation at a monthly resolution starting with the year
#' 1981 to the near-present. It has a spatial resolution of 0.05°. The data can
#' be used to retrieve information on the amount of rainfall. Due to the availability
#' of +30 years, anomaly detection and long-term average analysis is also possible.
#' The routine will download the complete archive in order to support long-term
#' average and anomaly calculations with respect to the 1981 - 2010 climate normal
#' period. Thus no additional arguments need to be specified.
#'
#'
#' @name chirps
#' @param years A numeric vector of the years to download CHIRPS precipitation
#'   layers. Must be greater 1981, defaults to `c(1981:2020)`.
#' @keywords resource
#' @returns A function that returns an `sf` footprint object.
#' @source \url{https://data.chc.ucsb.edu/products/CHIRPS-2.0/global_monthly/cogs/}
#' @references Funk, C., Peterson, P., Landsfeld, M. et al. The climate hazards
#' infrared precipitation with stations—a new environmental record for
#' monitoring extremes. Sci Data 2, 150066 (2015).
#' \doi{10.1038/sdata.2015.66}
#' @include register.R
#' @export
get_chirps <- function(years = 1981:2020) {
  avail_years <- seq(1981, format(Sys.Date(), "%Y"))
  years <- check_available_years(years, avail_years, "chirps")

  function(x,
           name = "chirps",
           type = "raster",
           outdir = mapme_options()[["outdir"]],
           verbose = mapme_options()[["verbose"]]) {
    urls <- .get_chirps_urls(years)
    filenames <- gsub("cog", "tif", basename(urls))
    co <- c("-of", "COG", "-co", "COMPRESSION=LZW", "-a_nodata", "-9999")

    bbox <- c(xmin = -180., ymin = -50., xmax = 180., ymax = 50.)
    fps <- st_as_sfc(st_bbox(bbox, crs = "EPSG:4326"))
    fps <- st_as_sf(rep(fps, length(urls)))
    fps[["source"]] <- urls

    make_footprints(fps, what = "raster", filenames = filenames)
  }
}

.get_chirps_urls <- function(years = 1981:2020) {
  chirps_url <- "https://data.chc.ucsb.edu/products/CHIRPS-2.0/global_monthly/cogs/"
  rsp <- httr2::req_perform(httr2::request(chirps_url))
  httr2::resp_check_status(rsp)
  chirps_list <- as.character(httr2::resp_body_html(rsp))
  chirps_list <- regmatches(chirps_list, gregexpr(chirps_list, pattern = "<a href=\"(.*?)\""))
  chirps_list <- gsub(".*\"([^`]+)\".*", "\\1", chirps_list[[1]])
  chirps_list <- grep("*.cog$", chirps_list, value = TRUE)

  chirps_list <- grep(
    pattern = paste(years, collapse = "|"),
    chirps_list, value = TRUE
  )

  paste("/vsicurl/", chirps_url, chirps_list, sep = "")
}

register_resource(
  name = "chirps",
  description = "Climate Hazards Group InfraRed Precipitation with Station data (CHIRPS)",
  licence = "CC - unknown",
  source = "https://www.chc.ucsb.edu/data/chirps",
  type = "raster"
)

#' Calculate treecover statistics
#'
#' This functions allows to efficiently calculate treecover statistics for
#' polygons. For each year in the analysis timeframe, the forest losses in
#' preceding and the current years are subtracted from the treecover in the
#' year 2000 and actual treecover figures within the polygon are returned.
#'
#' The required resources for this indicator are:
#'  - [gfw_treecover]
#'  - [gfw_lossyear]
#'
#' @name treecover_area
#' @param years A numeric vector with the years for which to calculate treecover
#'   area.
#' @param min_size The minimum size of a forest patch to be considered as forest in ha.
#' @param min_cover The minimum cover percentage per pixel to be considered as forest.
#' @keywords indicator
#' @returns A function that returns an indicator tibble with variable treecover
#'   and corresponding area (in ha) as value.
#'
#' If the [gfw_treecover] resource for an asset contains only zeros (no trees at all,
#'   e.g. big water body like an ocean or a sea),
#'   the \code{NULL} value is returned instead of the indicator tibble.
#' If it doesn't contain any pixels with value >=  \code{min_cover}, indicator tibble
#'   reports area of zero hectares.
#'
#' @include register.R
#' @export
#' @examples
#' \dontshow{
#' mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
#' }
#' \dontrun{
#' library(sf)
#' library(mapme.biodiversity)
#'
#' outdir <- file.path(tempdir(), "mapme-data")
#' dir.create(outdir, showWarnings = FALSE)
#'
#' mapme_options(
#'   outdir = outdir,
#'   verbose = FALSE
#' )
#'
#' aoi <- system.file("extdata", "sierra_de_neiba_478140_2.gpkg",
#'   package = "mapme.biodiversity"
#' ) %>%
#'   read_sf() %>%
#'   get_resources(
#'     get_gfw_treecover(version = "GFC-2024-v1.12"),
#'     get_gfw_lossyear(version = "GFC-2024-v1.12")
#'   ) %>%
#'   calc_indicators(calc_treecover_area(years = 2016:2017, min_size = 1, min_cover = 30)) %>%
#'   portfolio_long()
#'
#' aoi
#' }
calc_treecover_area <- function(years = 2000:2024,
                                min_size = 10,
                                min_cover = 35) {
  check_namespace("exactextractr")
  check_namespace("landscapemetrics", error = FALSE)
  min_cover <- .gfw_check_min_cover(min_cover, "treecover_area")
  min_size <- .gfw_check_min_size(min_size, "treecover_area")
  years <- .gfw_check_years(years, "treecover_area")

  function(x = NULL,
           gfw_treecover = NULL,
           gfw_lossyear = NULL,
           name = "treecover_area",
           mode = "asset",
           aggregation = "sum",
           verbose = mapme_options()[["verbose"]]) {
    treecover <- NULL
    # handling of return value if resources are missing, e.g. no overlap
    if (any(is.null(gfw_treecover), is.null(gfw_lossyear))) {
      return(NULL)
    }
    # mask gfw
    gfw_treecover <- terra::mask(gfw_treecover, x)
    # check if gfw_treecover only contains 0s, e.g. on the ocean
    if (.gfw_empty_raster(gfw_treecover, min_cover)) {
      return(NULL)
    }
    # prepare gfw rasters
    gfw <- .gfw_prep_rasters(x, gfw_treecover, gfw_lossyear, cover = min_cover)

    # retrieves maximum lossyear value from layer name
    max_year <- as.numeric(
      gsub(
        ".*GFC-([0-9]+)-.*", "\\1",
        names(gfw_lossyear)
      )
    )

    if (max_year < min(years)) {
      return(NULL)
    }

    # apply extraction routine
    gfw_stats <- exactextractr::exact_extract(
      gfw, x, function(data, min_size) {
        # retain only forest pixels and set area to ha
        data <- .prep_gfw_data(data, min_size)
        losses <- .sum_gfw(data, "coverage_area", max_year)
        names(losses)[2] <- "loss"
        org_coverage <- sum(data[["coverage_area"]])

        if (all(losses[["loss"]] == 0)) {
          result <- data.frame(
            years = years,
            treecover = org_coverage
          )
          return(result)
        }

        previous_losses <- sum(losses[["loss"]][losses[["years"]] < years[1]])

        if (previous_losses != 0) {
          org_coverage <- org_coverage - previous_losses
        }

        losses <- losses[losses[["years"]] %in% years, ]
        losses[["treecover"]] <- org_coverage
        losses[["treecover"]] <- losses[["treecover"]] - cumsum(losses[["loss"]])
        losses[, c("years", "treecover")]
      },
      min_size = min_size, coverage_area = TRUE, summarize_df = TRUE
    )

    gfw_stats %>%
      dplyr::mutate(
        datetime = as.POSIXct(paste0(years, "-01-01T00:00:00Z")),
        variable = "treecover",
        unit = "ha",
        value = treecover
      ) %>%
      dplyr::select(datetime, variable, unit, value) %>%
      tibble::as_tibble()
  }
}

# helper functions

.prep_gfw_data <- function(data, min_size) {
  # retain only forest pixels and set area to ha
  data <- data[data[["treecover"]] == 1, ]
  data[["coverage_area"]] <- data[["coverage_area"]] / 10000 # to ha

  # exclude patches smaller than threshold
  patch_sizes <- by(data[["coverage_area"]], data[["patches"]], sum)
  keep_patches <- names(patch_sizes)[which(patch_sizes > min_size)]
  keep_patches <- as.numeric(keep_patches)

  data[data[["patches"]] %in% keep_patches, ]
}

.sum_gfw <- function(data, what, max_year) {
  # calculate loss area by year
  df <- data.frame(years = 2000:max_year, var = 0)
  names(df)[2] <- what
  my_sum <- by(data[[what]], data[["lossyear"]], sum, na.rm = TRUE)
  sum_years <- as.numeric(names(my_sum))
  sum_years <- sum_years + 2000

  index <- match(sum_years, df[["years"]])
  df[[what]][index] <- as.numeric(my_sum)

  df
}

.gfw_check_years <- function(years, indicator) {
  if (any(years < 2000)) {
    msg <- paste("Cannot calculate %s statistics ",
                 "for years smaller than 2000.",
                 sep = ""
    )
    msg <- sprintf(msg, indicator)
    warning(msg)
  }
  years <- years[years >= 2000]
  if (length(years) == 0) {
    stop("GFW not available for the selected time range.")
  }
  return(years)
}

.gfw_empty_raster <- function(gfw, min_cover) {
  minmax <- as.vector(terra::minmax(gfw))
  minmax[is.na(minmax)] <- 0.0
  minmax <- unique(minmax)
  # if (length(minmax) > 1 && max(minmax) >= min_cover) {
  #   return(FALSE)
  # }
  # TRUE
  if (length(minmax) == 1L && max(minmax) == 0.0) {
    # only 0 values, no trees
    return(TRUE)
  } else {
    return(FALSE)
  }
}

.gfw_check_min_cover <- function(min_cover, indicator) {
  msg <- paste("Argument 'min_cover' for indicator '%s' ",
               "must be a numeric value between 0 and 100.",
               sep = ""
  )
  msg <- sprintf(msg, indicator)
  if (length(min_cover) != 1) stop(msg, call. = FALSE)
  if (!is.numeric(min_cover)) stop(msg, call. = FALSE)
  if (min_cover < 0 || min_cover > 100) {
    stop(msg, call. = FALSE)
  }
  min_cover
}

.gfw_check_min_size <- function(min_size, indicator) {
  msg <- paste(
    "Argument 'min_size' for indicator '%s' must be ",
    "a numeric value greater 0.",
    sep = ""
  )
  msg <- sprintf(msg, indicator)
  if (length(min_size) != 1) stop(msg, call. = FALSE)
  if (!is.numeric(min_size) || min_size <= 0) stop(msg, call. = FALSE)
  min_size
}

.gfw_calc_patches <- function(gfw) {
  vmax <- max(terra::minmax(gfw))
  # vmax == 0 if no values in raster > min_cover, can't use
  # landscapemetrics::get_patches(), it raises an error
  if (!requireNamespace("landscapemetrics", quietly = TRUE) || vmax == 0) {
    patched <- terra::patches(gfw, directions = 4, zeroAsNA = TRUE)
  } else {
    patched <- landscapemetrics::get_patches(gfw, class = 1, direction = 4)[[1]][[1]]
  }
  patched
}

.gfw_prep_rasters <- function(x, treecover, lossyear, emissions, cover) {
  # binarize the treecover layer based on min_cover argument
  rcl<- matrix(c(NA, NA, 0,
                 0, cover, 0,
                 cover, 100.001, 1), ncol = 3, byrow = TRUE)
  # need value > 100 to include 100 with 'right = FALSE'
  binary_treecover <- terra::classify(treecover,
                                      rcl = rcl,
                                      right = FALSE, others = NA)
  # mask lossyear
  lossyear <- terra::mask(lossyear, binary_treecover)
  lossyear <- terra::ifel(lossyear == 0, NA, lossyear)
  # create patches
  patched <- .gfw_calc_patches(binary_treecover)

  if (!missing(emissions)) {
    if (terra::ncell(emissions) != terra::ncell(treecover)) {
      emissions <- terra::resample(
        emissions, treecover,
        method = "bilinear"
      )
    }
    emissions <- terra::mask(emissions, lossyear)
    gfw <- c(binary_treecover, lossyear, patched, emissions)
    names(gfw) <- c("treecover", "lossyear", "patches", "emissions")
  } else {
    # prepare return raster
    gfw <- c(binary_treecover, lossyear, patched)
    names(gfw) <- c("treecover", "lossyear", "patches")
  }
  gfw
}

register_indicator(
  name = "treecover_area",
  description = "Area of forest cover by year",
  resources = c(
    "gfw_treecover",
    "gfw_lossyear"
  )
)


# old version
#
# .prep_gfw_data <- function(data, min_size) {
#   # retain only forest pixels and set area to ha
#   data <- data[data[["treecover"]] == 1, ]
#   data[["coverage_area"]] <- data[["coverage_area"]] / 10000 # to ha
#
#   # exclude patches smaller than threshold
#   patch_sizes <- by(data[["coverage_area"]], data[["patches"]], sum)
#   keep_patches <- names(patch_sizes)[which(patch_sizes > min_size)]
#   keep_patches <- as.numeric(keep_patches)
#
#   data[data[["patches"]] %in% keep_patches, ]
# }
#
#
# .sum_gfw <- function(data, what = "coverage_area", max_year = 2023) {
#   # calculate loss area by year
#   df <- data.frame(years = 2000:max_year, var = 0)
#   names(df)[2] <- what
#   my_sum <- by(data[[what]], data[["lossyear"]], sum, na.rm = TRUE)
#   sum_years <- as.numeric(names(my_sum))
#   sum_years <- sum_years + 2000
#
#   index <- match(sum_years, df[["years"]])
#   df[[what]][index] <- as.numeric(my_sum)
#
#   df
# }
#
# .gfw_check_years <- function(years, indicator) {
#   if (any(years < 2000)) {
#     msg <- paste("Cannot calculate %s statistics ",
#       "for years smaller than 2000.",
#       sep = ""
#     )
#     msg <- sprintf(msg, indicator)
#     warning(msg)
#   }
#   years <- years[years >= 2000]
#   if (length(years) == 0) {
#     stop("GFW not available for the selected time range.")
#   }
#   return(years)
# }
#
# .gfw_empty_raster <- function(gfw, min_cover) {
#   minmax <- unique(as.vector(terra::minmax(gfw)))
#   if (length(minmax) > 1 && max(minmax) >= min_cover) {
#     return(FALSE)
#   }
#   TRUE
# }
#
# .gfw_check_min_cover <- function(min_cover, indicator) {
#   msg <- paste("Argument 'min_cover' for indicator '%s' ",
#     "must be a numeric value between 0 and 100.",
#     sep = ""
#   )
#   msg <- sprintf(msg, indicator)
#   if (length(min_cover) != 1) stop(msg, call. = FALSE)
#   if (!is.numeric(min_cover)) stop(msg, call. = FALSE)
#   if (min_cover < 0 || min_cover > 100) {
#     stop(msg, call. = FALSE)
#   }
#   min_cover
# }
#
# .gfw_check_min_size <- function(min_size, indicator) {
#   msg <- paste(
#     "Argument 'min_size' for indicator '%s' must be ",
#     "a numeric value greater 0.",
#     sep = ""
#   )
#   msg <- sprintf(msg, indicator)
#   if (length(min_size) != 1) stop(msg, call. = FALSE)
#   if (!is.numeric(min_size) || min_size <= 0) stop(msg, call. = FALSE)
#   min_size
# }
#
# .gfw_calc_patches <- function(gfw) {
#   if (!requireNamespace("landscapemetrics", quietly = TRUE)) {
#     patched <- terra::patches(gfw, directions = 4, zeroAsNA = TRUE)
#   } else {
#     patched <- landscapemetrics::get_patches(gfw, class = 1, direction = 4)[[1]][[1]]
#   }
#   patched
# }
#
#
# .gfw_prep_rasters <- function(x, treecover, lossyear, emissions, cover) {
#   # binarize the treecover layer based on mcover argument
#   binary_treecover <- terra::classify(treecover,
#     rcl = matrix(c(
#       NA, NA, 0,
#       0, cover, 0,
#       cover, 100, 1
#     ), ncol = 3, byrow = TRUE),
#     include.lowest = TRUE
#   )
#
#   # mask lossyear
#   lossyear <- terra::mask(lossyear, binary_treecover)
#   lossyear <- terra::ifel(lossyear == 0, NA, lossyear)
#   # create patches
#   patched <- .gfw_calc_patches(binary_treecover)
#
#   if (!missing(emissions)) {
#     if (terra::ncell(emissions) != terra::ncell(treecover)) {
#       emissions <- terra::resample(
#         emissions, treecover,
#         method = "bilinear"
#       )
#     }
#     emissions <- terra::mask(emissions, lossyear)
#     gfw <- c(binary_treecover, lossyear, patched, emissions)
#     names(gfw) <- c("treecover", "lossyear", "patches", "emissions")
#   } else {
#     # prepare return raster
#     gfw <- c(binary_treecover, lossyear, patched)
#     names(gfw) <- c("treecover", "lossyear", "patches")
#   }
#   gfw
# }

test_that("test gfw utils", {
  x <- read_sf(system.file("extdata", "gfw_sample.gpkg",
    package = "mapme.biodiversity"
  ))

  treecover <- rast(list.files(
    system.file("res", "gfw_treecover", package = "mapme.biodiversity"),
    pattern = ".tif$", full.names = TRUE
  ))

  lossyear <- rast(list.files(
    system.file("res", "gfw_lossyear", package = "mapme.biodiversity"),
    pattern = ".tif$", full.names = TRUE
  ))

  emissions <- rast(list.files(
    system.file("res", "gfw_emissions", package = "mapme.biodiversity"),
    pattern = ".tif$", full.names = TRUE
  ))

  expect_warning(y <- .gfw_check_years(1999:2000, "treecover"))
  expect_length(y, 1)
  expect_error(expect_warning(.gfw_check_years(1999, "treecover")))

  expect_false(.gfw_empty_raster(treecover, 1))
  dummy <- treecover
  dummy[] <- NA
  expect_true(.gfw_empty_raster(dummy))
  dummy[] <- 0
  expect_true(.gfw_empty_raster(dummy))

  expect_error(.gfw_check_min_cover(1:2, "treecover"), "Argument 'min_cover'")
  expect_error(.gfw_check_min_cover("a", "treecover"), "Argument 'min_cover'")
  expect_error(.gfw_check_min_cover(-1, "treecover"), "Argument 'min_cover'")
  expect_error(.gfw_check_min_cover(101, "treecover"), "Argument 'min_cover'")
  expect_silent(y <- .gfw_check_min_cover(50, "treecover"))

  expect_error(.gfw_check_min_size(1:2, "treecover"), "Argument 'min_size'")
  expect_error(.gfw_check_min_size("a", "treecover"), "Argument 'min_size'")
  expect_error(.gfw_check_min_size(-1, "treecover"), "Argument 'min_size'")
  expect_silent(.gfw_check_min_size(5, "treecover"))

  expect_true(inherits(.gfw_calc_patches(treecover), "SpatRaster"))

  lossyear <- treecover
  lossyear[] <- round(runif(ncell(lossyear), 0, 20))
  emissions <- emissions
  emissions[] <- round(runif(ncell(lossyear), 0, 100))

  y <- .gfw_prep_rasters(x, treecover, lossyear, cover = 30)
  expect_true(inherits(y, "SpatRaster"))
  expect_equal(names(y), c("treecover", "lossyear", "patches"))
  y <- .gfw_prep_rasters(x, treecover, lossyear, emissions, 30)
  expect_true(inherits(y, "SpatRaster"))
  expect_equal(names(y), c("treecover", "lossyear", "patches", "emissions"))
  expect_error(.gfw_prep_rasters(x, treecover, lossyear, 30))


  df <- exactextractr::exact_extract(y, x, coverage_area = TRUE)[[1]]
  expect_silent(df <- .prep_gfw_data(df, 5))
  expect_equal(names(df), c(
    "treecover", "lossyear",
    "patches", "emissions", "coverage_area"
  ))

  expect_silent(area <- .sum_gfw(df, "coverage_area", 2023))
  expect_equal(names(area), c("years", "coverage_area"))
  expect_silent(emis <- .sum_gfw(df, "emissions", 2023))
  expect_equal(names(emis), c("years", "emissions"))
})

test_that("treecover works", {
  x <- read_sf(system.file("extdata", "gfw_sample.gpkg",
    package = "mapme.biodiversity"
  ))

  .clear_resources()
  outdir <- file.path(tempdir(), "mapme.data")
  .copy_resource_dir(outdir)
  mapme_options(outdir = outdir, verbose = FALSE)
  get_resources(
    x,
    get_gfw_treecover(version = "GFC-2023-v1.11"),
    get_gfw_lossyear(version = "GFC-2023-v1.11"),
    get_gfw_emissions()
  )
  gfw_treecover <- prep_resources(x, resources = "gfw_treecover")[[1]]
  gfw_lossyear <- prep_resources(x, resources = "gfw_lossyear")[[1]]
  gfw_emissions <- prep_resources(x, resources = "gfw_emissions")[[1]]

  years <- 2000:2005
  ta <- calc_treecover_area(years = years, min_size = 1, min_cover = 10)
  expect_true(is.null(ta(x, gfw_treecover, NULL)))
  result <- ta(x, gfw_treecover, gfw_lossyear)
  expect_silent(.check_single_asset(result))
  expect_equal(format(result$datetime, "%Y"), as.character(years))
  expect_snapshot(result$value)

  tae <- calc_treecover_area_and_emissions(
    years = years,
    min_size = 1,
    min_cover = 10
  )
  stats_treeloss <- tae(x, gfw_treecover, gfw_lossyear, gfw_emissions)
  stats_treeloss <- stats_treeloss[stats_treeloss$variable == "treecover", ]
  expect_equal(result$value, stats_treeloss$value)

  years <- 2023:2024
  ta <- calc_treecover_area(years = years, min_size = 1, min_cover = 10)
  expect_equal(nrow(ta(x, gfw_treecover, gfw_lossyear)), 1)

  years <- 2024:2025
  ta <- calc_treecover_area(years = years, min_size = 1, min_cover = 10)
  expect_true(is.null(ta(x, gfw_treecover, gfw_lossyear)))
})

.pkgenv <- new.env(parent = emptyenv())

.pkgenv$resources <- list()
.pkgenv$indicators <- list()
.pkgenv$avail_resources <- list()

.onAttach <- function(libname, pkgname) {
  packageStartupMessage(paste(pkgname, utils::packageVersion(pkgname)))
  invisible()
}

.onLoad <- function(libname, pkgname) {
  outdir <- tempfile(pattern = "mapme-data-")
  dir.create(outdir, showWarnings = FALSE)
  mapme_options(
    outdir = outdir,
    chunk_size = 100000,
    retries = 3L,
    delay = 10L,
    log_dir = NULL,
    verbose = TRUE
  )
  .check_system_requirements()
  invisible()
}

.probe_dsn <- function(dsn) {
  if (is.null(dsn)) {
    return()
  }
  dst <- file.path(dsn, "mapme-probe.tif")
  r <- terra::rast(resolution = c(180, 180))
  r[] <- 1L:2L

  status <- try(
    writeRaster(r, dst, datatype = "INT1U", overwrite = TRUE),
    silent = TRUE
  )

  if (inherits(status, "try-error")) {
    msg <- sprintf("Destination '%s' is not writeable via GDAL.", dsn)
    stop(msg)
  }
}

#' Portfolio methods for mapme.biodiversity
#'
#' `mapme_options()` sets default options for mapme.biodiversity to control the
#' behaviour of downstream functions. Mainly, the output path as well as the
#' chunk size (in ha), can be set. Additionally, the verbosity can be set and
#' the path to a log directory can be controlled. It also allow to set the number of
#' retries  and the delay between the successive retries when retrieving the resources.
#' Might be extended by other
#' options in the future.
#'
#' @param ... ignored
#' @param outdir A length one character indicating the output path.
#' @param chunk_size A numeric of length one giving the maximum chunk area in ha.
#'   Defaults to 100,000 ha. It refers to the area of an asset's bounding box.
#'   If it lies above the value `chunk_size`, splitting and chunking is considered.
#'   An asset will be processes as-is with an bounding box area below the specified
#'   value.
#' @param retries An integer of length one indicating the number of retries
#'   the package should attempt to make a resource available.
#'   This commands the GDAL option `GDAL_HTTP_MAX_RETRY` (see
#'   [here](https://gdal.org/en/stable/user/configoptions.html#:~:text=GDAL_HTTP_MAX_RETRY)).
#'   Defaults to 3.
#' @param delay An integer of length one indicating the number of seconds
#'   the package should wait between the successive retries.
#'   This commands the GDAL option `GDAL_HTTP_RETRY_DELAY`  (see
#'   [here](https://gdal.org/en/stable/user/configoptions.html#:~:text=GDAL_HTTP_RETRY_DELAY)).
#'   Defaults to 10.
#' @param verbose A logical, indicating if informative messages should be printed.
#' @param log_dir A character path pointing toward a GDAL-writable destination
#'   used to log erroneous assets. Defaults to NULL, meaning that erroneous
#'   assets will not be serialized to disk. If specified, a GPKG file named
#'   `file.path(log_dir, paste0(Sys.Date(), "_mapme-error-assets.gpkg"))` will
#'   be created and appended to in case of erroneous assets. In the case some of the
#'   resources could not be downloaded, their list will be written to a GPKG file named
#'   `file.path(log_dir, paste0(Sys.Date(), "_", {resource_name}, "_mapme-error-resources.gpkg"))`.
#' @return `mapme_options()` returns a list of options if no arguments are
#'   specified. Otherwise sets matching arguments to new values in the package's
#'   internal environment.
#' @name mapme
#' @export
#'
#' @examples
#' library(mapme.biodiversity)
#' mapme_options()
mapme_options <- function(..., outdir, chunk_size, retries, delay, verbose, log_dir) {
  if (!missing(outdir)) {
    stopifnot(is.null(outdir) | (is.character(outdir) && length(outdir) == 1))
    .probe_dsn(outdir)
    .pkgenv$outdir <- outdir
  }

  if (!missing(chunk_size)) {
    stopifnot(is.null(chunk_size) | (length(chunk_size) == 1 && is.numeric(chunk_size)))
    .pkgenv$chunk_size <- chunk_size
  }

  if (!missing(retries)) {
    stopifnot(length(retries) == 1 && is.numeric(retries))
    .pkgenv$retries <- max(as.integer(retries), 0L) # make sure it is non-negative
  }

  if (!missing(delay)) {
    stopifnot(length(delay) == 1 && is.numeric(delay))
    .pkgenv$delay <- max(as.integer(delay), 0L) # make sure it is non-negative
  }

  if (!missing(verbose)) {
    stopifnot(is.logical(verbose))
    .pkgenv$verbose <- verbose
  }

  if (!missing(log_dir)) {
    stopifnot(is.null(log_dir) | (is.character(log_dir) && length(log_dir) == 1))
    .pkgenv$log_dir <- log_dir
  }

  if (nargs() == 0) {
    return(list(
      outdir = .pkgenv$outdir,
      chunk_size = .pkgenv$chunk_size,
      retries = .pkgenv$retries,
      delay = .pkgenv$delay,
      verbose = .pkgenv$verbose,
      log_dir = .pkgenv$log_dir
    ))
  }
}

.check_char <- function(obj, name) {
  if (!inherits(obj, "character") || length(obj) > 1 || nchar(obj) == 0) {
    stop(paste0(name, " needs to be a single charachter string"))
  }
}
#' Register or list resources in mapme.biodiversity
#'
#' `register_resource()` is used to register a new resource function with base
#' information to the package's internal environment used to inform users about
#' available resources. Note, registering a custom resource will
#' only have effect for the current R session.
#'
#' @param name A character vector indicating the name of the resource.
#' @param description A character vector with a basic description
#' @param licence A character vector indicating the licence of the resource.
#'   In case it is a custom licence, put a link to the licence text.
#' @param source Optional, preferably a URL where the data is found.
#' @param type A character vector indicating the type of the resource. Either
#'   'vector' or 'raster'.
#' @param source Optional, preferably a URL where the data is found.
#'
#' @return `register_resource()` is called for the side-effect of registering a
#'   resource.
#' @name resources
#' @export
#'
#' @examples
#' \dontrun{
#' register_resource(
#'   name = "gfw_treecover",
#'   description = "Global Forest Watch - Percentage of canopy closure in 2000",
#'   licence = "CC-BY 4.0",
#'   source = "https://data.globalforestwatch.org/documents/tree-cover-2000/explore",
#'   type = "raster"
#' )
#' }
register_resource <- function(name = NULL,
                              description = NULL,
                              licence = NULL,
                              source = NULL,
                              type = NULL) {
  if (any(is.null(name), is.null(description), is.null(licence), is.null(source), is.null(type))) {
    stop("neither name, description, licence, source, nor type can be NULL")
  }

  .check_char(name, "name")
  .check_char(description, "description")
  .check_char(licence, "licence")
  .check_char(source, "source")
  .check_char(type)

  if (name %in% names(.pkgenv$resources)) {
    warning(paste("resource with name", name, "already registered"))
  }

  if (!type %in% c("vector", "raster")) {
    stop("type needs to be one of 'vector' or 'raster'")
  }

  resource <- tibble(
    name = name, description = description, licence = licence,
    source = source, type = type
  )
  .pkgenv$resources <- rbind(.pkgenv$resources, resource)
}



#' Register or list indicators in mapme.biodiversity
#'
#' `register_indicator()` is used to register a new indicator function with base
#' information to the package's internal environment used to inform users about
#' available indicators. Note, registering a custom indicator will
#' only have effect for the current R session.
#' @param name A character vector indicating the name of the indicator.
#' @param description A character vector with a basic description
#' @param resources A character vector of the required resources
#'   that need to be available to calculate the indicator. The names must
#'   correspond with already registered resources.
#'
#' @return `register_indicator()` is called for the side-effect of registering
#'   an indicator
#' @name indicators
#' @export
#'
#' @examples
#' \dontrun{
#' register_indicator(
#'   name = "treecover_area",
#'   description = "Area of forest cover by year",
#'   resources = c(
#'     "gfw_treecover",
#'     "gfw_lossyear"
#'   )
#' )
#' }
register_indicator <- function(name = NULL, description = NULL, resources = NULL) {
  if (any(is.null(name), is.null(description), is.null(resources))) {
    stop("neither name, description nor resources can be NULL")
  }

  .check_char(name, "name")
  .check_char(description, "description")

  if (name %in% names(.pkgenv$indicators)) {
    warning(paste("indicator with name", name, "already registered"))
  }

  if (!inherits(resources, "character")) {
    stop("resources needs to be a charachter vector")
  }

  indicator <- tibble(name = name, description = description, resources = list(resources))
  .pkgenv$indicators <- rbind(.pkgenv$indicators, indicator)
}


#' Register or list resources in mapme.biodiversity
#'
#' `available_resources()` returns a tibble of registered resources with basic
#' information such as the source and the licence.
#'
#' @param resources If \code{NULL} returns a list of all resources (default).
#'   Otherwise only the ones specified.
#'
#' @return `available_resources()` returns a tibble listing available resources.
#' @name resources
#' @export
#' @include register.R
#' @examples
#' available_resources()
available_resources <- function(resources = NULL) {
  all_resources <- .pkgenv$resources

  if (is.null(resources)) {
    return(all_resources[order(all_resources[["name"]]), ])
  } else {
    if (any(!resources %in% all_resources[["name"]])) {
      not_avail <- which(!resources %in% names(all_resources))
      not_avail <- resources[not_avail]
      msg <- sprintf(
        "The following resources are not available:\n%s",
        paste(not_avail, collpase = " ")
      )
      stop(msg)
    }
    all_resources[all_resources[["name"]] %in% resources, ]
  }
}



#' Register or list indicators in mapme.biodiversity
#'
#' `available_indicators()` returns a tibble of registered indicators with basic
#' information such as the required resources.
#'
#' @param indicators If \code{NULL} returns a list of all registered indicators
#' (default). Otherwise only the ones specified.
#'
#' @return `available_resources()` returns a tibble listing available indicators.
#' @name indicators
#' @export
#' @include register.R
#' @examples
#' available_indicators()
available_indicators <- function(indicators = NULL) {
  all_indicators <- .pkgenv$indicators
  resources <- lapply(all_indicators[["resources"]], function(x) available_resources(x))
  all_indicators[["resources"]] <- resources

  if (is.null(indicators)) {
    return(all_indicators[order(all_indicators[["name"]]), ])
  } else {
    if (any(!indicators %in% all_indicators[["name"]])) {
      not_avail <- which(!indicators %in% names(all_indicators))
      not_avail <- indicators[not_avail]
      msg <- sprintf(
        "The following indicators are not available:\n%s",
        paste(not_avail, collpase = " ")
      )
      stop(msg)
    }
    all_indicators[all_indicators[["name"]] %in% indicators, ]
  }
}


.add_resource <- function(resource) {
  stopifnot(inherits(resource, "list"))
  stopifnot(!is.null(names(resource)))
  if (length(.pkgenv$avail_resources) == 0) {
    .pkgenv$avail_resources <- resource
  } else {
    name <- names(resource)
    .pkgenv$avail_resources[name] <- resource
  }
}

.avail_resources <- function(name) {
  return(.pkgenv$avail_resources)
}

.clear_resources <- function() {
  .pkgenv$avail_resources <- list()
}

.check_system_requirements <- function() {
  chk <- mapply(.compare_sf_version, c("GDAL", "PROJ"), c("3.7.0", "4.8.0"))
  invisible()
}

.compare_sf_version <- function(lib, target) {
  installed_version <- sf::sf_extSoftVersion()[[lib]]
  if (utils::compareVersion(installed_version, target) < 0) {
    msg <- sprintf("%s system library version (%s) < %s", lib, installed_version, target)
    warning(msg, call. = FALSE)
  }
}

test_that("gsw change works", {
  x <- read_sf(
    system.file("extdata", "shell_beach_protected_area_41057_B.gpkg",
      package = "mapme.biodiversity"
    )
  )

  .clear_resources()
  outdir <- file.path(tempdir(), "mapme.data")
  .copy_resource_dir(outdir)
  mapme_options(outdir = outdir, verbose = FALSE)
  get_resources(x, get_global_surface_water_change(version = "v1_4_2021"))
  gsw_change <- prep_resources(x)[["global_surface_water_change"]][[1]]

  gswc <- calc_gsw_change()
  expect_true(is.null(gswc(x, NULL)))
  chg <- gswc(x, gsw_change)
  expect_silent(.check_single_asset(chg))
  expect_equal(chg$value, 100, tolerance = 1e-4)
})

test_that("get_gsw_time_series works", {
  aoi <- read_sf(
    system.file("extdata", "shell_beach_protected_area_41057_B.gpkg",
                package = "mapme.biodiversity"
    )
  )

  .clear_resources()
  outdir <- file.path(tempdir(), "mapme.data")
  .copy_resource_dir(outdir)
  mapme_options(outdir = outdir, verbose = FALSE)
  outdir_gsw <- file.path(outdir, "gsw_time_series")

  gsw_fnames_short <- dir(outdir_gsw, pattern = ".tif$")
  gsw_fnames_long <- sub("v5_", "VER5-0_yearlyClassification", gsw_fnames_short)
  file.copy(
    file.path(outdir_gsw, gsw_fnames_short),
    file.path(outdir_gsw, gsw_fnames_long)
  )
  years <- 2000:2001

  expect_error(
    get_gsw_time_series(years = years, version = ""),
    "version %in% available_versions is not TRUE"
  )
  expect_error(
    get_gsw_time_series(years = 0),
    "The target years do not intersect with the availability of gsw_time_series."
  )
  expect_warning(
    get_gsw_time_series(years = c(NA, NULL, ".", 2000)),
    "NAs introduced by coercion"
  )
  expect_message(
    get_gsw_time_series(years = c(1900, 2000)),
    "Some target years are not available for gsw_time_series."
  )
  expect_silent(
    get_gsw_time_series(years = c(2000, 2000))
  )

  expect_silent(.check_footprints(get_gsw_time_series(2000:2001)(aoi)))
  expect_silent(.check_footprints(get_gsw_time_series(2000)(aoi)))

  aoi_shift <- aoi
  gsw <- get_gsw_time_series(2000:2001)
  st_geometry(aoi_shift) <- st_geometry(aoi) + c(0, -90)
  st_crs(aoi_shift) <- 4326
  expect_error(
    gsw(aoi_shift),
    "The extent of the portfolio does not intersect with the GSW grid."
  )
})

test_that("treecover area and emissions works", {
  x <- read_sf(system.file("extdata", "gfw_sample.gpkg",
    package = "mapme.biodiversity"
  ))

  .clear_resources()
  outdir <- file.path(tempdir(), "mapme.data")
  .copy_resource_dir(outdir)
  mapme_options(outdir = outdir, verbose = FALSE)
  get_resources(
    x,
    get_gfw_treecover(version = "GFC-2023-v1.11"),
    get_gfw_lossyear(version = "GFC-2023-v1.11"),
    get_gfw_emissions()
  )
  gfw_treecover <- prep_resources(x, resources = "gfw_treecover")[[1]]
  gfw_lossyear <- prep_resources(x, resources = "gfw_lossyear")[[1]]
  gfw_emissions <- prep_resources(x, resources = "gfw_emissions")[[1]]

  years <- 2000:2005
  min_size <- 1
  min_cover <- 10
  tae <- calc_treecover_area_and_emissions(years, min_size, min_cover)
  expect_true(is.null(tae(x, gfw_treecover, gfw_lossyear, NULL)))

  result <- tae(x, gfw_treecover, gfw_lossyear, gfw_emissions)

  expect_equal(unique(result$variable), c("emissions", "treecover"))
  expect_equal(unique(format(result$datetime, "%Y")), as.character(c(2000:2005)))
  expect_snapshot(result$value)

  te <- calc_treecoverloss_emissions(years, min_size, min_cover)
  stats_emissions <- te(x, gfw_treecover, gfw_lossyear, gfw_emissions)
  expect_equal(result$value[result$variable == "emissions"], stats_emissions$value, tolerance = 1e-4)

  # test that emissions and forest loss are returned as 0 if no loss occurs
  gfw_lossyear[gfw_lossyear == 3] <- 1
  stats_treeloss <- tae(x, gfw_treecover, gfw_lossyear, gfw_emissions)
  stats_treeloss <- stats_treeloss[stats_treeloss$variable == "emissions", ]
  expect_equal(stats_treeloss$value[stats_treeloss$datetime == "2003-01-01"], 0)

  years <- 2023:2024
  te <- calc_treecoverloss_emissions(years, min_size, min_cover)
  expect_equal(nrow(te(x, gfw_treecover, gfw_lossyear, gfw_emissions)), 1)

  years <- 2024:2025
  te <- calc_treecoverloss_emissions(years, min_size, min_cover)
  expect_true(is.null(te(x, gfw_treecover, gfw_lossyear, gfw_emissions)))
})

test_that("biome computation works", {
  x <- read_sf(
    system.file("extdata", "sierra_de_neiba_478140.gpkg",
      package = "mapme.biodiversity"
    )
  )
  .clear_resources()
  outdir <- file.path(tempdir(), "mapme.data")
  .copy_resource_dir(outdir)
  mapme_options(outdir = outdir, verbose = FALSE)
  get_resources(x, get_teow("test.zip"))
  teow <- prep_resources(x)[["teow"]]

  cb <- calc_biome()
  result <- cb(x, teow)
  expect_silent(.check_single_asset(result))
  expect_equal(result$value, 18352.24, tolerance = 1e-4)
  # check NULL is returned for 0-length tibbles
  st_geometry(x) <- st_geometry(x) + 5
  st_crs(x) <- st_crs(4326)
  expect_equal(cb(x, teow), NULL)
})

test_that("calc_species_richness works", {
  x <- read_sf(system.file("extdata", "sierra_de_neiba_478140_2.gpkg",
    package = "mapme.biodiversity"
  ))
  .clear_resources()
  outdir <- file.path(tempdir(), "mapme.data")
  .copy_resource_dir(outdir)
  mapme_options(outdir = outdir, verbose = FALSE)

  iucn_dir <- system.file("res", "iucn", package = "mapme.biodiversity")
  iucn_files <- list.files(iucn_dir, pattern = "_SR_", full.names = TRUE)

  get_resources(x, get_iucn(iucn_files))
  iucn <- prep_resources(x)[["iucn"]]

  expect_error(calc_species_richness(engine = "other"))
  expect_error(calc_species_richness(stats = "other"))

  csr <- calc_species_richness(stats = "median")
  expect_equal(csr(x, NULL), NULL)
  iucnNA <- iucn
  iucnNA[] <- NA
  expect_equal(csr(x, iucnNA), NULL)

  expect_silent(result <- csr(x, iucn))
  expect_silent(.check_single_asset(result))
  expect_equal(nrow(result), 2)
  vars <- c("amphibians_sr_2023_median", "birds_thr_sr_2023_median")
  expect_equal(unique(result$variable), vars)
  vals <- c(15, 27)
  expect_equal(result$value, vals)
})

test_that("calc_key_biodiversity_areas works", {
  sample_path <- system.file("extdata", "shell_beach_protected_area_41057_B.gpkg",
              package = "mapme.biodiversity")
  x <- read_sf(sample_path)
  x_area <- as.numeric(st_area(x))
  x_area <- x_area / 10000

  .clear_resources()
  outdir <- tempfile()
  dir.create(outdir)
  mapme_options(outdir = outdir, verbose = FALSE)

  get_resources(x, get_key_biodiversity_areas(path = sample_path))
  kbas <- prep_resources(x)[["key_biodiversity_areas"]]

  kb <- calc_key_biodiversity_area()

  expect_null(kb(x, key_biodiversity_areas = NULL))
  expect_null(kb(x, key_biodiversity_areas = head(x, 0)))

  result <- kb(x, kbas)
  expect_silent(.check_single_asset(result))
  expect_equal(result$value, x_area, tolerance = 0.01)

  st_geometry(x) <- st_geometry(x) + 5
  st_crs(x) <- st_crs(4326)
  expect_equal(kb(x, kbas), NULL)

  mapme_options(chunk_size = 10000)
  x <- read_sf(sample_path)
  fname_kba <- system.file("res", "key_biodiversity_areas/kbas.gpkg",
                           package = "mapme.biodiversity")
  res <- get_resources(x, get_key_biodiversity_areas(path = fname_kba))
  kbas <- calc_indicators(res, calc_key_biodiversity_area())
  kbas <- portfolio_long(kbas, drop_geoms = TRUE)

  area_kba <- st_area(read_sf(fname_kba))
  area_kba <- as.numeric(area_kba)
  area_kba <- area_kba / 10000

  expect_equal(
    kbas$value,
    area_kba,
    tolerance = 0.01
  )
})

test_that("calc_burned_area works", {
  skip_on_cran()
  x <- read_sf(
    system.file("extdata", "sierra_de_neiba_478140_2.gpkg",
      package = "mapme.biodiversity"
    )
  )
  .clear_resources()
  outdir <- file.path(tempdir(), "mapme.data")
  .copy_resource_dir(outdir)
  mapme_options(outdir = outdir, verbose = FALSE)
  get_resources(x, get_mcd64a1(years = 2010))
  mcd64a1 <- prep_resources(x)[["mcd64a1"]]

  ba <- calc_burned_area()
  expect_true(is.null(ba(x, NULL)))
  result <- ba(x, mcd64a1)
  ba <- calc_burned_area(engine = "zonal")
  result_zonal <- ba(x, mcd64a1)
  ba <- calc_burned_area(engine = "extract")
  result_extract <- ba(x, mcd64a1)
  ba <- calc_burned_area(engine = "exactextract")
  result_exact <- ba(x, mcd64a1)

  expect_silent(.check_single_asset(result))
  expect_silent(.check_single_asset(result_zonal))
  expect_silent(.check_single_asset(result_extract))
  expect_silent(.check_single_asset(result_exact))

  expect_equal(result_zonal$value, result_zonal$value, tolerance = 1e-4)
  expect_snapshot(result_exact$value)
})

test_that("esa global landcover works", {
  skip_on_cran()
  x <- read_sf(
    system.file("extdata", "sierra_de_neiba_478140.gpkg",
      package = "mapme.biodiversity"
    )
  )
  .clear_resources()
  outdir <- file.path(tempdir(), "mapme.data")
  .copy_resource_dir(outdir)
  mapme_options(outdir = outdir, verbose = FALSE)
  get_resources(x, get_esalandcover(years = 2015))
  esalandcover <- prep_resources(x)[["esalandcover"]][[1]]

  cl <- calc_landcover()
  expect_true(is.null(cl(x, NULL)))
  result <- cl(x, esalandcover)
  expect_silent(.check_single_asset(result))
})

test_that("gsw occurrence works", {
  x <- read_sf(
    system.file("extdata", "shell_beach_protected_area_41057_B.gpkg",
      package = "mapme.biodiversity"
    )
  )

  .clear_resources()
  outdir <- file.path(tempdir(), "mapme.data")
  .copy_resource_dir(outdir)
  mapme_options(outdir = outdir, verbose = FALSE)
  get_resources(x, get_global_surface_water_occurrence(version = "v1_4_2021"))
  gsw_occurrence <- prep_resources(x)[["global_surface_water_occurrence"]][[1]]

  gswo <- calc_gsw_occurrence(min_occurrence = 10)
  expect_true(is.null(gswo(x, NULL)))
  occ <- gswo(x, gsw_occurrence)
  expect_equal(occ$value, 694.159, tolerance = 1e-4)
})

test_that(".calc_population_count works", {
  x <- read_sf(
    system.file("extdata", "sierra_de_neiba_478140.gpkg",
      package = "mapme.biodiversity"
    )
  )
  .clear_resources()
  outdir <- file.path(tempdir(), "mapme.data")
  .copy_resource_dir(outdir)
  mapme_options(outdir = outdir, verbose = FALSE)
  get_resources(x, get_worldpop(years = 2010:2020))
  worldpop <- prep_resources(x)[["worldpop"]]

  cpc <- calc_population_count()
  result <- cpc(x, worldpop)
  cpc <- calc_population_count(stats = c("mean", "median", "sd"))
  result_multi_stat <- cpc(x, worldpop)
  cpc <- calc_population_count(engine = "zonal")
  result_zonal <- cpc(x, worldpop)
  cpc <- calc_population_count(engine = "extract")
  result_extract <- cpc(x, worldpop)
  cpc <- calc_population_count(engine = "exactextract")
  result_exact <- cpc(x, worldpop)

  expect_silent(.check_single_asset(result))
  expect_silent(.check_single_asset(result_multi_stat))
  expect_silent(.check_single_asset(result_zonal))
  expect_silent(.check_single_asset(result_extract))
  expect_silent(.check_single_asset(result_exact))

  vars <- c("population_mean", "population_median", "population_sd")
  expect_equal(unique(result_multi_stat$variable), vars)
  expect_equal(result_zonal$value, result_extract$value, tolerance = 1e-4)
  expect_snapshot(result_exact$value)
})

test_that("mapme_options works", {
  opts <- mapme_options()
  names <- c("outdir", "chunk_size", "retries", "delay", "verbose", "log_dir")
  expect_equal(names(opts), names)

  expect_error(mapme_options(outdir = 1))
  expect_error(mapme_options(outdir = "/does-not-exist"), "not writeable via GDAL")
  expect_error(mapme_options(verbose = 1))
  expect_error(mapme_options(chunk_size = "a"))
  expect_error(mapme_options(retries = "a"))
  expect_silent(mapme_options(outdir = NULL))
})

test_that("test register_resource works", {
  name <- "sample"
  description <- "simple description"
  type <- "raster"
  source <- "sample_source"
  licence <- "CC-BY"

  expect_error(
    register_resource(),
    "neither name, description, licence, source, nor type can be NULL"
  )

  expect_error(
    register_resource(
      name = 1, description = description, licence = licence, source = source, type = type
    ),
    "name needs to be a single charachter string"
  )

  expect_error(
    register_resource(
      name = name, description = 1, licence = licence, source = source, type = type
    ),
    "description needs to be a single charachter string"
  )

  expect_error(
    register_resource(
      name = name, description = description, licence = 1, source = source, type = type
    ),
    "licence needs to be a single charachter string"
  )

  expect_error(
    register_resource(
      name = name, description = description, licence = licence, source = 1, type = type
    ),
    "source needs to be a single charachter string"
  )


  expect_error(
    register_resource(
      name = name, description = description, licence = licence, source = source, type = "unknown"
    ),
    "type needs to be one of 'vector' or 'raster'"
  )


  expect_silent(register_resource(name, description, licence, source, type))
  res <- available_resources(name)
  expect_equal(names(res), c("name", "description", "licence", "source", "type"))
  expect_equal(res[["name"]], name)
  expect_equal(res[["description"]], description)
  expect_equal(res[["licence"]], licence)
  expect_equal(res[["source"]], source)
  expect_equal(res[["type"]], type)
})

test_that("test register_indicator works", {
  name <- "sample"
  description <- "sample desc"
  resources <- c("gfw_treecover", "gmw")

  expect_error(
    register_indicator(),
    "neither name, description nor resources can be NULL"
  )

  expect_error(
    register_indicator(
      name = 1, description = description, resources = resources
    ),
    "name needs to be a single charachter string"
  )

  expect_error(
    register_indicator(
      name = name, description = 1, resources = resources
    ),
    "description needs to be a single charachter string"
  )

  expect_error(
    register_indicator(
      name = name,
      description = description,
      resources = 1
    ),
    "resources needs to be a charachter vector"
  )

  expect_silent(register_indicator(name, description, resources))
  ind <- available_indicators(name)
  expect_equal(names(ind), c("name", "description", "resources"))
  res <- ind[["resources"]]
  expect_true(inherits(res, "list"))
  res <- res[[1]]
  expect_true(inherits(res, "tbl_df"))
  expect_equal(nrow(res), 2)
  expect_equal(res[["name"]], resources)
})

test_that("test available_resources works", {
  res <- available_resources()
  expect_true(inherits(res, "tbl_df"))
  expect_true(length(res) > 1)
  expect_error(available_resources("not-available"))
})

test_that("test available_indicators works", {
  ind <- available_indicators()
  expect_true(inherits(ind, "tbl_df"))
  expect_true(nrow(ind) > 1)
  expect_error(available_indicators("not-available"))
})


test_that("add / clear resource works", {
  .clear_resources()
  add <- list(1:10)
  names(add) <- "test_res"
  res <- .avail_resources()
  expect_true(class(res) == "list")
  expect_length(res, 0)
  .add_resource(add)
  res <- .avail_resources()
  expect_true(class(res) == "list")
  expect_length(res, 1)
  expect_equal(names(res), "test_res")
  expect_equal(res[["test_res"]], 1:10)
  .clear_resources()
  res <- .avail_resources()
  expect_length(res, 0)
})

test_that("precipitation indicator works", {
  x <- read_sf(
    system.file("extdata", "gfw_sample.gpkg",
      package = "mapme.biodiversity"
    )
  )
  .clear_resources()
  outdir <- file.path(tempdir(), "mapme.data")
  .copy_resource_dir(outdir)
  mapme_options(outdir = outdir, verbose = FALSE)
  get_resources(x, get_chelsa(years = 2010))
  chelsa <- prep_resources(x)[["chelsa"]]

  expect_message(cpc <- calc_precipitation_chelsa(1978:1982))
  expect_error(calc_precipitation_chelsa(years = 1970:1971))
  expect_error(calc_precipitation_chelsa(years = 1981:1982, engine = "not-av"))
  expect_true(is.null(cpc(x, NULL)))
  cpc <- calc_precipitation_chelsa(years = 2010)
  result <- cpc(x, chelsa)
  cpc <- calc_precipitation_chelsa(years = 2010, engine = "zonal")
  result_zonal <- cpc(x, chelsa)
  cpc <- calc_precipitation_chelsa(years = 2010, engine = "extract")
  result_extract <- cpc(x, chelsa)
  cpc <- calc_precipitation_chelsa(years = 2010, engine = "exactextract")
  result_exact <- cpc(x, chelsa)

  expect_silent(.check_single_asset(result))
  expect_silent(.check_single_asset(result_zonal))
  expect_silent(.check_single_asset(result_extract))
  expect_silent(.check_single_asset(result_exact))

  expect_equal(unique(result$variable), "precipitation")
  expect_equal(result_zonal$value, result_extract$value,
    tolerance = 1e-4
  )
  expect_snapshot(result$value)
})

test_that(".calc_fatalities_ucpd works", {
  x <- read_sf(
    system.file("extdata", "burundi.gpkg",
      package = "mapme.biodiversity"
    )
  )
  .clear_resources()
  outdir <- file.path(tempdir(), "mapme.data")
  .copy_resource_dir(outdir)
  mapme_options(outdir = outdir, verbose = FALSE)
  get_resources(x, get_ucdp_ged(version = "22.1"))
  ucdp_ged <- prep_resources(x)[["ucdp_ged"]][[1]]

  expect_error(
    calc_fatalities_ucdp(years = 1980),
    "The target years do not intersect with the availability of ucdp_ged."
  )
  expect_error(
    calc_fatalities_ucdp(precision_location = 8),
    "Argument precision_location must be a single numeric between 1 and 7."
  )
  expect_error(
    calc_fatalities_ucdp(precision_time = 6),
    "Argument precision_time must be a single numeric between 1 and 5."
  )

  cf <- calc_fatalities_ucdp(years = 1991:1992)
  result_default <- cf(x, list(ucdp_ged))
  cf <- calc_fatalities_ucdp(years = 1991:1992, precision_location = 7, precision_time = 5)
  result_all <- cf(x, list(ucdp_ged))

  conf_types <- c("state_based_conflict", "non_state_conflict", "one_sided_violence")
  death_types <- c("deaths_civilians", "deaths_unknown", "deaths_total")
  vars <- paste("fatalities", rep(conf_types, each = length(death_types)), death_types, sep = "_")

  expect_silent(.check_single_asset(result_default))
  expect_equal(unique(result_default$variable), vars[7:9])
  expect_equal(nrow(result_default), 3)
  expect_equal(sum(result_default$value), 40)

  expect_equal(unique(result_all$variable), vars[-c(4:6)])
  expect_equal(nrow(result_all), 9)
  expect_equal(sum(result_all$value), 592)
})

test_that("get_ipbes_biomes works", {
  skip_on_cran()
  skip_if_not(Sys.getenv("USER") == "darius")
  gib <- get_ipbes_biomes()
  expect_silent(fps <- gib())
  expect_silent(.check_footprints(fps))
  expect_equal(fps$filename, "IPBES_UoA_biomes_JK.tif")
})

test_that("calc_ipbes_biomes works", {
  skip_on_cran()
  x <- read_sf(system.file("extdata", "shell_beach_protected_area_41057_B.gpkg",
    package = "mapme.biodiversity"
  ))
  .clear_resources()
  outdir <- file.path(tempdir(), "mapme.data")
  .copy_resource_dir(outdir)
  mapme_options(outdir = outdir, verbose = FALSE)

  f <- file.path(outdir, "res", "ipbes_biomes", "IPBES_UoA_biomes_JK.tif")
  ipbes <- rast(f)
  ib <- calc_ipbes_biomes()

  expect_equal(ib(x, NULL), NULL)
  expect_silent(result <- ib(x, ipbes))
  expect_true(inherits(result, "list"))
  result <- result[[1]]
  expect_silent(.check_single_asset(result))
  expect_equal(result$value, c(1184.2016, 1988.4815), tolerance = 0.01)
  vars <- c("tropical_and_subtropical_dry_and_humid_forests", "shelf_ecosystems")
  expect_equal(result$variable, vars)
})

test_that("traveltime works", {
  x <- read_sf(
    system.file("extdata", "sierra_de_neiba_478140.gpkg",
      package = "mapme.biodiversity"
    )
  )
  .clear_resources()
  outdir <- file.path(tempdir(), "mapme.data")
  .copy_resource_dir(outdir)
  mapme_options(outdir = outdir, verbose = FALSE)
  get_resources(x, get_nelson_et_al(ranges = "100k_200k"))
  nelson_et_al <- prep_resources(x)[["nelson_et_al"]]

  ctt <- calc_traveltime()
  expect_true(is.null(ctt(x, NULL)))
  result <- ctt(x, nelson_et_al)
  ctt <- calc_traveltime(stats = c("mean", "median", "sd"))
  result_multi_stat <- ctt(x, nelson_et_al)
  ctt <- calc_traveltime(engine = "zonal")
  result_zonal <- ctt(x, nelson_et_al)
  ctt <- calc_traveltime(engine = "extract")
  result_extract <- ctt(x, nelson_et_al)
  ctt <- calc_traveltime(engine = "exactextract")
  result_exact <- ctt(x, nelson_et_al)

  expect_silent(.check_single_asset(result))
  expect_silent(.check_single_asset(result_multi_stat))
  expect_silent(.check_single_asset(result_zonal))
  expect_silent(.check_single_asset(result_extract))
  expect_silent(.check_single_asset(result_exact))

  vars <- c(
    "100k_200k_traveltime_mean", "100k_200k_traveltime_median",
    "100k_200k_traveltime_sd"
  )
  expect_equal(unique(result_multi_stat$variable)[1:3], vars)
  expect_equal(result_zonal$value, result_zonal$value, tolerance = 1e-4)
  expect_snapshot(result_exact$value)
})

test_that("traveltime_2000 works", {
  x <- read_sf(
    system.file("extdata", "sierra_de_neiba_478140.gpkg",
                package = "mapme.biodiversity"
    )
  )
  .clear_resources()
  outdir <- file.path(tempdir(), "mapme-data")
  .copy_resource_dir(outdir)
  mapme_options(outdir = outdir, verbose = FALSE)
  get_resources(x, get_accessibility_2000())
  accessibility_2000 <- prep_resources(x)[["accessibility_2000"]]

  ctt_2000 <- calc_traveltime_2000()
  expect_true(is.null(ctt_2000(x, NULL)))
  result <- ctt_2000(x, accessibility_2000)
  ctt_2000 <- calc_traveltime_2000(stats = c("mean", "median", "sd"))
  result_multi_stat <- ctt_2000(x, accessibility_2000)
  ctt_2000 <- calc_traveltime_2000(engine = "zonal")
  result_zonal <- ctt_2000(x, accessibility_2000)
  ctt_2000 <- calc_traveltime_2000(engine = "extract")
  result_extract <- ctt_2000(x, accessibility_2000)
  ctt_2000 <- calc_traveltime_2000(engine = "exactextract")
  result_exact <- ctt_2000(x, accessibility_2000)

  expect_silent(.check_single_asset(result))
  expect_silent(.check_single_asset(result_multi_stat))
  expect_silent(.check_single_asset(result_zonal))
  expect_silent(.check_single_asset(result_extract))
  expect_silent(.check_single_asset(result_exact))

  vars <- c("traveltime_2000_mean", "traveltime_2000_median", "traveltime_2000_sd")
  expect_equal(unique(result_multi_stat$variable), vars)
  expect_equal(result_zonal$value, result_extract$value, tolerance = 1e-4)
  expect_snapshot(result_exact$value)
})

test_that("get_resources works", {
  skip_on_cran()
  aoi <- read_sf(
    system.file("extdata", "sierra_de_neiba_478140.gpkg",
      package = "mapme.biodiversity"
    )
  )
  aoi <- suppressWarnings(st_cast(aoi, to = "POLYGON")[1, ])

  temp_loc <- file.path(tempdir(), "mapme.biodiversity")
  dir.create(temp_loc, showWarnings = FALSE)
  resource_dir <- system.file("res", package = "mapme.biodiversity")
  file.copy(resource_dir, temp_loc, recursive = TRUE)
  outdir <- file.path(tempdir(), "mapme.biodiversity", "res")
  tmpdir <- tempdir()

  mapme_options(
    outdir = outdir,
    tmpdir = tmpdir,
    verbose = FALSE,
    testing = FALSE
  )


  expect_error(get_resources(aoi, 12))
  expect_error(get_resources(aoi, mean))
  .clear_resources()
  aoi <- get_resources(
    aoi,
    get_gfw_treecover(version = "GFC-2023-v1.11"),
    get_gfw_lossyear(version = "GFC-2023-v1.11"),
    get_gfw_emissions()
  )

  resources <- .avail_resources()
  expect_equal(names(resources), c("gfw_treecover", "gfw_lossyear", "gfw_emissions"))
  expect_message(
    get_resources(aoi, get_gfw_treecover(version = "GFC-2023-v1.11")),
    "Resource 'gfw_treecover' is already available."
  )
})

test_that("test .check_resource_fun works", {
  expect_error(.check_resource_fun("a"))
  fun <- function(x, name, type, outdir, verbose) {}
  expect_error(.check_resource_fun(fun))
  fun <- function(x, name = "new_res", type, outdir, verbose) {}
  expect_silent(f <- .check_resource_fun(fun))
  expect_true(class(f) == "function")
})

test_that("test .make_path works", {
  outdir <- tempfile()
  dir.create(outdir)
  out <- .make_path(outdir, "test")
  expect_true(file.exists(file.path(outdir, "test")))
  expect_equal(basename(out), "test")
})

test_that("test make_footprints works", {
  temp_loc <- file.path(tempdir(), "mapme.biodiversity")
  dir.create(temp_loc, showWarnings = FALSE)
  resource_dir <- system.file("res", package = "mapme.biodiversity")
  file.copy(resource_dir, temp_loc, recursive = TRUE)
  outdir <- file.path(tempdir(), "mapme.biodiversity", "res")

  tifs <- normalizePath(list.files(file.path(outdir, "worldpop"),
    pattern = ".tif$",
    recursive = TRUE, full.names = TRUE
  ), mustWork = FALSE)

  fps <- make_footprints(tifs, what = "raster")
  expect_true(inherits(fps, "sf"))
  expect_equal(nrow(fps), length(tifs))
  expect_equal(fps$location, tifs)
})

test_that("get_hfp works", {
  .clear_resources()
  outdir <- file.path(tempdir(), "mapme.data")
  .copy_resource_dir(outdir)
  mapme_options(outdir = NULL)
  expect_warning(get_humanfootprint(years = 2010), "outdir")
  mapme_options(outdir = outdir)
  expect_message(get_humanfootprint(years = 1999:2000), "target years")

  skip_on_cran()
  skip_if_not(Sys.getenv("USER") == "darius")
  ghfp <- get_humanfootprint(years = 2010)
  fps <- ghfp(outdir = file.path(outdir, "humanfootprint"))
  expect_silent(.check_footprints(fps))
  expect_equal(nrow(fps), 1)
  expect_equal(fps$filename, "hfp2010.tif")
})

test_that("biodiversity intactness index works", {
  x <- read_sf(system.file("extdata", "shell_beach_protected_area_41057_B.gpkg",
                           package = "mapme.biodiversity"))
  .clear_resources()
  outdir <- file.path(tempdir(), "mapme.data")
  .copy_resource_dir(outdir)
  mapme_options(outdir = outdir, verbose = FALSE)

  lbii <- system.file("res", "biodiversity_intactness_index", "lbii.asc",
                      package = "mapme.biodiversity")
  get_resources(x, get_biodiversity_intactness_index(lbii))
  bii <- prep_resources(x)[["biodiversity_intactness_index"]]
  bi <- calc_biodiversity_intactness_index()

  expect_null(
    bi(x, biodiversity_intactness_index = NULL)
  )
  expect_silent(result <- bi(x, bii))
  expect_silent(.check_single_asset(result))
  expect_equal(nrow(result), 1)
  expect_equal(result$variable, "biodiversity_intactness_index")
  expect_equal(result$value, 0.91, tolerance = 0.01)
})

test_that("mangrove extent works", {
  x <- read_sf(
    system.file("extdata", "shell_beach_protected_area_41057_B.gpkg",
      package = "mapme.biodiversity"
    )
  )
  .clear_resources()
  outdir <- file.path(tempdir(), "mapme.data")
  .copy_resource_dir(outdir)
  mapme_options(outdir = outdir, verbose = FALSE)
  get_resources(x, get_gmw(years = c(1996, 2016)))
  gmw <- prep_resources(x)[["gmw"]]

  ma <- calc_mangroves_area()
  expect_true(is.null(ma(x, NULL)))
  result <- ma(x, gmw)
  expect_equal(unique(result$variable), "mangroves")
  expect_equal(result$value, c(1214.88, 1206.61), tolerance = 1e-4)
  expect_equal(format(result$datetime, "%Y"), c("1996", "2016"))
})

test_that("drought indicator works", {
  x <- read_sf(
    system.file("extdata", "sierra_de_neiba_478140.gpkg",
      package = "mapme.biodiversity"
    )
  )

  .clear_resources()
  outdir <- file.path(tempdir(), "mapme.data")
  .copy_resource_dir(outdir)
  mapme_options(outdir = outdir, verbose = FALSE)
  get_resources(x, get_nasa_grace(years = 2022))
  nasa_grace <- prep_resources(x)[["nasa_grace"]]

  cdi <- calc_drought_indicator()
  result <- cdi(x, nasa_grace)
  cdi <- calc_drought_indicator(stats = c("mean", "median", "sd"))
  result_multi_stats <- cdi(x, nasa_grace)
  cdi <- calc_drought_indicator(engine = "zonal")
  result_zonal <- cdi(x, nasa_grace)
  cdi <- calc_drought_indicator(engine = "extract")
  result_extract <- cdi(x, nasa_grace)
  cdi <- calc_drought_indicator(engine = "exactextract")
  result_exact <- cdi(x, nasa_grace)

  expect_silent(.check_single_asset(result[[1]]))
  expect_equal(
    unique(result_multi_stats[[1]]$variable),
    c("wetness_mean", "wetness_median", "wetness_sd")
  )
  expect_equal(
    names(result_zonal),
    names(result_extract)
  )
  expect_equal(
    names(result_zonal),
    names(result_exact)
  )
  expect_equal(
    result[[1]]$value,
    result_zonal[[1]]$value,
    tolerance = 1e-4
  )
  expect_equal(
    result_zonal[[1]]$value,
    result_extract[[1]]$value,
    tolerance = 1e-4
  )
  expect_snapshot(
    result_exact[[1]]$value
  )
})

test_that("worldclim precipitation works", {
  x <- read_sf(
    system.file("extdata", "sierra_de_neiba_478140.gpkg",
      package = "mapme.biodiversity"
    )
  )

  .clear_resources()
  outdir <- file.path(tempdir(), "mapme.data")
  .copy_resource_dir(outdir)
  mapme_options(outdir = outdir, verbose = FALSE)
  get_resources(x, get_worldclim_precipitation(years = 2018))
  worldclim_precipitation <- prep_resources(x)[["worldclim_precipitation"]]

  cpwc <- calc_precipitation_wc()
  result <- cpwc(x, worldclim_precipitation)
  cpwc <- calc_precipitation_wc(stats = c("mean", "median", "sd"))
  result_multi_stat <- cpwc(x, worldclim_precipitation, )
  cpwc <- calc_precipitation_wc(engine = "zonal")
  result_zonal <- cpwc(x, worldclim_precipitation)
  cpwc <- calc_precipitation_wc(engine = "extract")
  result_extract <- cpwc(x, worldclim_precipitation)
  cpwc <- calc_precipitation_wc(engine = "exactextract")
  result_exact <- cpwc(x, worldclim_precipitation)

  expect_silent(.check_single_asset(result))
  expect_silent(.check_single_asset(result_multi_stat))
  expect_silent(.check_single_asset(result_zonal))
  expect_silent(.check_single_asset(result_extract))
  expect_silent(.check_single_asset(result_exact))

  vars <- c("worldclim_prec_mean", "worldclim_prec_median", "worldclim_prec_sd")
  expect_equal(unique(result_multi_stat$variable), vars)
  expect_equal(result_zonal$value, result_extract$value, tolerance = 1e-4)
  expect_snapshot(result_exact$value)
})

test_that("gsw transitions works", {
  x <- read_sf(
    system.file("extdata", "shell_beach_protected_area_41057_B.gpkg",
      package = "mapme.biodiversity"
    )
  )

  .clear_resources()
  outdir <- file.path(tempdir(), "mapme.data")
  .copy_resource_dir(outdir)
  mapme_options(outdir = outdir, verbose = FALSE)
  get_resources(x, get_global_surface_water_transitions(version = "v1_4_2021"))
  gsw_transitions <- prep_resources(x)[["global_surface_water_transitions"]][[1]]

  gswt <- calc_gsw_transitions()
  expect_true(is.null(gswt(x, NULL)))
  transitions <- gswt(x, gsw_transitions)
  expect_silent(.check_single_asset(transitions))

  transitions_expected <- tibble(
    class = c(
      "Permanent", "New Permanent", "Lost Permanent", "Seasonal",
      "New Seasonal", "Seasonal to Permanent", "Permanent to Seasonal",
      "Ephemeral Permanent", "Ephemeral Seasonal"
    ),
    area = c(
      199.40, 388.84, 0.38,
      1.90, 83.90, 28.859,
      1.06, 0.60, 21.62
    )
  )
  expect_equal(transitions$value, transitions_expected$area, tolerance = 1e-4)
})

test_that("calc_gsw_time_series works", {
  aoi <- read_sf(
    system.file("extdata", "shell_beach_protected_area_41057_B.gpkg",
                package = "mapme.biodiversity"
    )
  )

  .clear_resources()
  outdir <- file.path(tempdir(), "mapme.data")
  .copy_resource_dir(outdir)
  mapme_options(outdir = outdir, verbose = FALSE)
  outdir_gsw <- file.path(outdir, "gsw_time_series")

  gsw_fnames_short <- dir(outdir_gsw, pattern = ".tif$")
  gsw_fnames_long <- sub("v5_", "VER5-0_yearlyClassification", gsw_fnames_short)
  file.copy(
    file.path(outdir_gsw, gsw_fnames_short),
    file.path(outdir_gsw, gsw_fnames_long)
  )

  res <- calc_indicators(
    get_resources(
      aoi, get_gsw_time_series(years = 2000)
      ),
    calc_gsw_time_series()
  )
  expect_equal(
    nrow(res$gsw_timeseries [[1]]),
    4
  )

  years <- 2000:2001
  aoi <- get_resources(aoi, get_gsw_time_series(years))
  res <- calc_indicators(aoi, calc_gsw_time_series())
  res <- res$gsw_timeseries [[1]]

  file.remove(
    file.path(outdir_gsw, gsw_fnames_long)
  )

  expect_equal(
    res$datetime,
    rep(as.POSIXct(c("2000-01-01T00:00:00Z", "2001-01-01T00:00:00Z")), 4)
  )

  expect_equal(
    res$variable,
    rep(c("no_observations", "not_water", "seasonal_water", "permanent_water"), each = 2)
  )

  expect_equal(
    res$unit,
    rep("ha", 8)
  )

  expect_equal(
    res$value,
    c(2684.78, 2635.35, 13.99, 0.00, 337.11, 327.85, 129.96, 202.64),
    tolerance = 0.1
  )
})

test_that("conflict_exposure_ucdp works", {
  x <- read_sf(
    system.file("extdata", "burundi.gpkg",
      package = "mapme.biodiversity"
    )
  )
  .clear_resources()
  worldpop <- rast(system.file("extdata", "burundi_worldpop.tif",
    package = "mapme.biodiversity"
  ))
  ucdp_ged <- list(read_sf(system.file("res", "ucdp_ged", "ged221-csv.gpkg",
    package = "mapme.biodiversity"
  )))

  expect_silent(calc_exposed_population_ucdp(distance = 1))
  expect_error(calc_exposed_population_ucdp(distance = c(1, 2)), "distance")
  expect_silent(calc_exposed_population_ucdp(violence_types = 1:2, distance = c(1, 2)))
  expect_error(calc_exposed_population_ucdp(violence_types = 4, "violence_type"))
  expect_error(calc_exposed_population_ucdp(years = 1988), "years")
  expect_error(calc_exposed_population_ucdp(precision_location = 8), "precision_location")
  expect_error(calc_exposed_population_ucdp(precision_time = 8), "precision_time")
  expect_error(calc_exposed_population_ucdp(engine = "NA"), "engine")

  cce <- calc_exposed_population_ucdp(
    distance = 10000,
    violence_types = 1:3,
    years = 2000,
    precision_location = 4,
    precision_time = 4
  )

  expect_equal(cce(x, NULL, worldpop), NULL)
  expect_equal(cce(x, ucdp_ged, NULL), NULL)
  expect_equal(cce(x, ucdp_ged, worldpop), NULL)

  cce <- calc_exposed_population_ucdp(
    distance = 10000,
    violence_types = 1:3,
    years = 1991:1992,
    precision_location = 4,
    precision_time = 4
  )


  ucdp2 <- list(ucdp_ged[[1]][12:16, ])
  expect_equal(cce(x, ucdp2, worldpop), NULL)
  result <- cce(x, ucdp_ged, worldpop)
  expect_silent(.check_single_asset(result))
  expect_equal(nrow(result), 5)
  expect_equal(result$value[c(3, 5)], c(873211, 123397), tolerance = 1e-4)

  ucdp_ged[[1]]$date_start <- as.POSIXct(ucdp_ged[[1]]$date_start) + 14 * 365 * 24 * 60 * 60

  worldpop <- c(worldpop, round(worldpop * 1.15))
  names(worldpop) <- c(
    "ppp_2004_1km_Aggregated",
    "ppp_2005_1km_Aggregated"
  )

  cce <- calc_exposed_population_ucdp(
    distance = 10000,
    years = 2005:2006,
    precision_location = 4,
    precision_time = 4
  )
  result <- cce(x, ucdp_ged, worldpop)
  expect_silent(.check_single_asset(result))
  expect_equal(nrow(result), 5)
  expect_equal(result$value[c(3, 5)], c(1004191, 141910), tolerance = 1e-4)

  cce <- calc_exposed_population_ucdp(
    distance = c(10000, 10000),
    violence_types = c(1, 3),
    years = 2005:2006,
    precision_location = 4,
    precision_time = 4
  )

  result <- cce(x, ucdp_ged, worldpop)
  expect_silent(.check_single_asset(result))
  expect_equal(nrow(result), 5)
  expect_equal(result$value[c(3, 5)], c(1004191, 141910), tolerance = 1e-4)
})

test_that("calc_conflict_exposure_acled works", {
  x <- read_sf(
    system.file("extdata", "burundi.gpkg",
      package = "mapme.biodiversity"
    )
  )
  .clear_resources()
  worldpop <- rast(system.file("extdata", "burundi_worldpop.tif",
    package = "mapme.biodiversity"
  ))
  acled <- list(read_sf(system.file("res", "acled", "acled_events_2000.gpkg",
    package = "mapme.biodiversity"
  )))

  expect_silent(calc_exposed_population_acled(distance = 1))
  expect_error(calc_exposed_population_acled(distance = c(1, 2)), "Wrong number")
  expect_silent(calc_exposed_population_acled(distance = 1:6))
  expect_error(calc_exposed_population_acled(filter_category = "other"), "arg")
  expect_error(
    calc_exposed_population_acled(
      filter_category = "event_type", filter_types = "other"
    ),
    "filter_types"
  )
  expect_silent(calc_exposed_population_acled(filter_types = c("battles", "riots")))
  expect_silent(calc_exposed_population_acled(distance = 1:2, filter_types = c("battles", "riots")))
  expect_error(
    calc_exposed_population_acled(
      distance = 1:3, filter_types = c("battles", "riots")
    ),
    "Total number"
  )
  expect_error(calc_exposed_population_acled(precision_location = 4), "precision_location")
  expect_error(calc_exposed_population_acled(precision_time = 4), "precision_time")
  expect_error(calc_exposed_population_acled(years = 1996), "years")
  expect_error(calc_exposed_population_acled(engine = "NA"), "engine")

  cce <- calc_exposed_population_acled(
    distance = 10000,
    years = 2001,
    precision_location = 3,
    precision_time = 3
  )

  expect_equal(cce(x, NULL, worldpop), NULL)
  expect_equal(cce(x, acled, NULL), NULL)
  expect_equal(cce(x, acled, worldpop), NULL)

  cce <- calc_exposed_population_acled(
    distance = 10000,
    years = 2000,
    precision_location = 3,
    precision_time = 3
  )
  result_default <- cce(x = x, acled, worldpop)
  expect_silent(.check_single_asset(result_default))
  expect_equal(nrow(result_default), 7)
  expect_equal(result_default$value[7], 1788220, tolerance = 1e-4)

  acled2 <- list(dplyr::filter(acled[[1]], event_type == "Battles"))
  cce <- calc_exposed_population_acled(
    distance = 10000,
    filter_category = "event_type",
    filter_types = "riots",
    years = 2000,
    precision_location = 3,
    precision_time = 3
  )
  expect_equal(cce(x, acled2, worldpop), NULL)

  cce <- calc_exposed_population_acled(
    distance = 10000,
    filter_category = "event_type",
    filter_types = "battles",
    years = 2000,
    precision_location = 3,
    precision_time = 3
  )
  result_battles <- cce(x, acled2, worldpop)
  expect_silent(.check_single_asset(result_battles))
  expect_equal(nrow(result_battles), 2)
  expect_equal(unique(result_battles$value), 227035, tolerance = 1e-4)

  acled2 <- acled[[1]]
  acled2$year <- 2001
  acled <- list(acled[[1]], acled2)

  cce <- calc_exposed_population_acled(
    distance = 10000,
    years = 2000:2001,
    precision_location = 3,
    precision_time = 3
  )

  result <- cce(x, acled, worldpop)
  expect_silent(.check_single_asset(result))
  expect_equal(nrow(result), 14)
  expect_identical(result$value[1], result$value[8])

  cce <- calc_exposed_population_acled(filter_category = "sub_event_type")
  expect_silent(result <- cce(x, acled[1], worldpop))
  expect_equal(nrow(result), 6)

  cce <- calc_exposed_population_acled(filter_category = "disorder_type")
  expect_silent(result <- cce(x, acled[1], worldpop))
  expect_equal(nrow(result), 4)

  cce <- calc_exposed_population_acled(
    distance = c(5000, 10000),
    filter_category = "event_type",
    filter_types = c("riots", "explosions/remote_violence")
  )
  expect_silent(result <- cce(x, acled[1], worldpop))
  expect_equal(nrow(result), 3)
})

# This test was deactivated because calls to the CHIRPS website would fail.
# TODO: reactivate this test
# test_that("precipitation indicator works", {
#   skip_on_cran()
#   x <- read_sf(
#     system.file("extdata", "gfw_sample.gpkg",
#       package = "mapme.biodiversity"
#     )
#   )
#   .clear_resources()
#   outdir <- file.path(tempdir(), "mapme.data")
#   .copy_resource_dir(outdir)
#   mapme_options(outdir = outdir, verbose = FALSE)
#   get_resources(x, get_chirps(years = 2010))
#   chirps <- prep_resources(x)[["chirps"]]
#
#   expect_message(cpc <- calc_precipitation_chirps(1980:1982))
#   expect_error(calc_precipitation_chirps(years = 1970:1971))
#   expect_error(calc_precipitation_chirps(years = 1981:1982, engine = "not-av"))
#   expect_true(is.null(cpc(x, NULL)))
#   cpc <- calc_precipitation_chirps(years = 2010)
#   result <- cpc(x, chirps)
#   cpc <- calc_precipitation_chirps(years = 2010, engine = "zonal")
#   result_zonal <- cpc(x, chirps)
#   cpc <- calc_precipitation_chirps(years = 2010, engine = "extract")
#   result_extract <- cpc(x, chirps)
#   cpc <- calc_precipitation_chirps(years = 2010, engine = "exactextract")
#   result_exact <- cpc(x, chirps)
#
#   expect_silent(.check_single_asset(result[[1]]))
#   expect_silent(.check_single_asset(result_zonal[[1]]))
#   expect_silent(.check_single_asset(result_extract[[1]]))
#   expect_silent(.check_single_asset(result_exact[[1]]))
#
#   expect_equal(unique(result[[1]]$variable), "precipitation")
#   expect_equal(result_zonal[[1]]$value, result_extract[[1]]$value,
#     tolerance = 1e-4
#   )
#   expect_snapshot(result[[1]]$value)
# })

test_that("gsw recurrence works", {
  x <- read_sf(
    system.file("extdata", "shell_beach_protected_area_41057_B.gpkg",
      package = "mapme.biodiversity"
    )
  )

  .clear_resources()
  outdir <- file.path(tempdir(), "mapme.data")
  .copy_resource_dir(outdir)
  mapme_options(outdir = outdir, verbose = FALSE)
  get_resources(x, get_global_surface_water_recurrence(version = "v1_4_2021"))
  gsw_recurrence <- prep_resources(x)[["global_surface_water_recurrence"]][[1]]

  gswr <- calc_gsw_recurrence(min_recurrence = 10)
  expect_true(is.null(gswr(x, NULL)))
  rec <- gswr(x, gsw_recurrence)
  expect_equal(rec$value, 719.590, tolerance = 1e-4)
})

test_that("worldclim minimum temperature works", {
  x <- read_sf(
    system.file("extdata", "sierra_de_neiba_478140.gpkg",
      package = "mapme.biodiversity"
    )
  )

  .clear_resources()
  outdir <- file.path(tempdir(), "mapme.data")
  .copy_resource_dir(outdir)
  mapme_options(outdir = outdir, verbose = FALSE)
  get_resources(x, get_worldclim_min_temperature(years = 2018))
  worldclim_min_temperature <- prep_resources(x)[["worldclim_min_temperature"]]

  cmin <- calc_temperature_min_wc()
  expect_true(is.null(cmin(x, NULL)))
  result <- cmin(x, worldclim_min_temperature)
  cmin <- calc_temperature_min_wc(stats = c("mean", "median", "sd"))
  result_multi_stat <- cmin(x, worldclim_min_temperature)
  cmin <- calc_temperature_min_wc(engine = "zonal")
  result_zonal <- cmin(x, worldclim_min_temperature)
  cmin <- calc_temperature_min_wc(engine = "extract")
  result_extract <- cmin(x, worldclim_min_temperature)
  cmin <- calc_temperature_min_wc(engine = "exactextract")
  result_exact <- cmin(x, worldclim_min_temperature)

  expect_silent(.check_single_asset(result))
  expect_silent(.check_single_asset(result_multi_stat))
  expect_silent(.check_single_asset(result_zonal))
  expect_silent(.check_single_asset(result_extract))
  expect_silent(.check_single_asset(result_exact))


  vars <- c("worldclim_tmin_mean", "worldclim_tmin_median", "worldclim_tmin_sd")
  expect_equal(unique(result_multi_stat$variable), vars)
  expect_equal(result_zonal$value, result_extract$value, tolerance = 1e-4)
  expect_snapshot(result_exact$value)
})

test_that("deforestation drivers works", {
  skip_on_cran()
  x <- read_sf(
    system.file("extdata", "sierra_de_neiba_478140.gpkg",
      package = "mapme.biodiversity"
    )
  )
  .clear_resources()
  outdir <- file.path(tempdir(), "mapme.data")
  .copy_resource_dir(outdir)
  mapme_options(outdir = outdir, verbose = FALSE)
  suppressWarnings(get_resources(x, get_fritz_et_al(resolution = 100)))
  drivers <- prep_resources(x)[["fritz_et_al"]]

  cdf <- calc_deforestation_drivers()
  result <- cdf(x, drivers)
  expect_silent(.check_single_asset(result))
  expect_equal(
    unique(result$variable),
    c(
      "commercial_agriculture", "commercial_oil_palm", "managed_forests",
      "mining", "natural_disturbances", "pasture", "roads", "wildfire",
      "other_subsistance_agriculture", "shifting_cultivation"
    )
  )
  expect_equal(sum(result$value), 16809, tolerance = 1e-4)
})

test_that(".get_worldpop works", {
  skip_on_cran()
  .clear_resources()
  outdir <- file.path(tempdir(), "mapme.data")
  .copy_resource_dir(outdir)
  mapme_options(outdir = outdir, verbose = FALSE)

  expect_error(get_worldpop(1999))
  gwp <- get_worldpop(years = 2010)
  expect_silent(.check_resource_fun(gwp))
  fps <- gwp(outdir = file.path(outdir, "worldpop"))
  expect_silent(.check_footprints(fps))
  expect_equal(fps$filename, "ppp_2010_1km_Aggregated.tif")
})

test_that("calc_fatalitites_acled works", {
  x <- read_sf(
    system.file("extdata", "burundi.gpkg",
      package = "mapme.biodiversity"
    )
  )
  .clear_resources()
  outdir <- file.path(tempdir(), "mapme.data")
  .copy_resource_dir(outdir)
  mapme_options(outdir = outdir, verbose = FALSE)

  get_resources(x, get_acled(
    years = 2000,
    email = "myemail@com",
    password = "mypassword",
    accept_terms = TRUE
  ))

  acled <- prep_resources(x)[["acled"]][[1]]

  expect_error(
    calc_fatalities_acled(years = 1980),
    "The target years do not intersect with the availability of acled"
  )
  expect_error(
    calc_fatalities_acled(precision_location = 4),
    "Argument precision_location must be a single numeric between 1 and 3."
  )
  expect_error(
    calc_fatalities_acled(precision_time = 4),
    "Argument precision_time must be a single numeric between 1 and 3."
  )

  expect_error(
    calc_fatalities_acled(stratum = "other"),
    'should be one of "event_type", "sub_event_type", "disorder_type"'
  )

  cf <- calc_fatalities_acled(years = 2000)
  expect_equal(cf(x, NULL), NULL)
  cf <- calc_fatalities_acled(years = 2001)
  expect_equal(cf(x, list(acled)), NULL)
  cf <- calc_fatalities_acled(years = 2000, stratum = "event_type")
  result_event <- cf(x, list(acled))
  cf <- calc_fatalities_acled(years = 2000, stratum = "sub_event_type")
  result_sub <- cf(x, list(acled))
  cf <- calc_fatalities_acled(years = 2000, stratum = "disorder_type")
  result_disorder <- cf(x, list(acled))

  expect_silent(.check_single_asset(result_event))
  expect_silent(.check_single_asset(result_sub))
  expect_silent(.check_single_asset(result_disorder))

  expect_equal(sum(result_event$value), sum(result_sub$value))
  expect_equal(sum(result_event$value), sum(result_disorder$value))

  events <- c(
    "fatalities_explosions/remote_violence",
    "fatalities_riots",
    "fatalities_violence_against_civilians",
    "fatalities_total"
  )
  expect_equal(unique(result_event$variable), events)

  events <- c(
    "fatalities_abduction/forced_disappearance",
    "fatalities_air/drone_strike",
    "fatalities_attack",
    "fatalities_government_regains_territory",
    "fatalities_peaceful_protest",
    "fatalities_total"
  )
  expect_equal(unique(result_sub$variable), events)

  events <- c(
    "fatalities_demonstrations",
    "fatalities_political_violence",
    "fatalities_strategic_developments",
    "fatalities_total"
  )
  expect_equal(unique(result_disorder$variable), events)

  cf <- calc_fatalities_acled(years = 2000, precision_location = 3, precision_time = 3)
  result_all <- cf(x, list(acled))
  expect_equal(nrow(result_all), 7)
  expect_equal(sum(result_all$value), 791 * 2)
  events <- c(
    "fatalities_battles",
    "fatalities_explosions/remote_violence",
    "fatalities_protests",
    "fatalities_riots",
    "fatalities_strategic_developments",
    "fatalities_violence_against_civilians",
    "fatalities_total"
  )
  expect_equal(unique(result_all$variable), events)
})

test_that("gsw seasonality works", {
  x <- read_sf(
    system.file("extdata", "shell_beach_protected_area_41057_B.gpkg",
      package = "mapme.biodiversity"
    )
  )

  .clear_resources()
  outdir <- file.path(tempdir(), "mapme.data")
  .copy_resource_dir(outdir)
  mapme_options(outdir = outdir, verbose = FALSE)
  get_resources(x, get_global_surface_water_seasonality(version = "v1_4_2021"))
  gsw_seasonality <- prep_resources(x)[["global_surface_water_seasonality"]][[1]]

  gsws <- calc_gsw_seasonality()
  expect_true(is.null(gsws(x, NULL)))
  season <- gsws(x, gsw_seasonality)
  expect_equal(season$value,
    c(2514.759, 22.461, 18.045, 23.831, 19.872, 2.512, 0.152, 0, 0, 0, 0, 0, 617.104),
    tolerance = 1e-3
  )
})

#' Climatologies at High resolution for the Earth Land Surface Areas (CHELSA)
#'
#' The CHELSA data (Karger et al. 2017) consists of downscaled model output
#' temperature and precipitation estimates at a horizontal resolution of 30 arc
#' sec. The precipitation algorithm incorporates orographic predictors including
#' wind fields, valley exposition, and boundary layer height, with a subsequent
#' bias correction. The spatial resolution is about 1-arc second (~1km at the
#' equator). This resource makes V2 available.
#'
#'
#' @name chelsa
#' @param years A numeric vector of the years to make CHELSA monthly precipitation
#'   layers available for. Must be greater 1979, defaults to `c(1979:2019)`.
#' @keywords resource
#' @returns A function that returns an `sf` footprint object.
#' @source \url{https://envicloud.wsl.ch/#/?prefix=chelsa/chelsa_V2/GLOBAL/}
#' @references Karger, D.N., Conrad, O., Böhner, J., Kawohl, T., Kreft, H.,
#'   Soria-Auza, R.W., Zimmermann, N.E., Linder, H.P. & Kessler, M. (2021)
#'   Climatologies at high resolution for the earth’s land surface areas.
#'   EnviDat. \doi{doi:10.16904/envidat.228.v2.1}
#'
#'   Karger, D.N., Conrad, O., Böhner, J., Kawohl, T., Kreft, H., Soria-Auza,
#'   R.W., Zimmermann, N.E., Linder, P., Kessler, M. (2017): Climatologies at high
#'   resolution for the Earth land surface areas. Scientific Data. 4
#'   170122. \doi{doi:10.1038/sdata.2017.122}
#' @include register.R
#' @export
get_chelsa <- function(years = 1979:2019) {
  years <- check_available_years(years, 1979:2019, "chelsa")

  function(x,
           name = "chelsa",
           type = "raster",
           outdir = mapme_options()[["outdir"]],
           verbose = mapme_options()[["verbose"]]) {
    urls <- .get_chelsa_urls(years)
    co <- c("-ot", "Int32", "-co", "COMPRESS=DEFLATE", "-co", "PREDICTOR=2", "-a_nodata", "-2147483647")
    bbox <- c(xmin = -180., ymin = -90., xmax = 180., ymax = 84.)
    fps <- st_as_sfc(st_bbox(bbox, crs = "EPSG:4326"))
    fps <- st_as_sf(rep(fps, length(urls)))
    fps[["source"]] <- urls

    make_footprints(fps, what = "raster", co = co)
  }
}

.get_chelsa_urls <- function(years = 1979:2019) {
  url <- "https://os.zhdk.cloud.switch.ch/chelsav2/GLOBAL/monthly/pr/"
  files <- paste(sprintf("%02d", 1:12), rep(years, each = 12), sep = "_")
  files <- sprintf("CHELSA_pr_%s_V.2.1.tif", files)
  paste("/vsicurl/", url, files, sep = "")
}

register_resource(
  name = "chelsa",
  description = "Climatologies at High resolution for the Earth Land Surface Areas (CHELSA)",
  licence = "Unknown - Must cite!",
  source = "https://chelsa-climate.org/",
  type = "raster"
)

#' Population Count layer for year 2000-2020
#'
#' This resource is published by open spatial demographic data and research
#' organization called WorldPop. This resource represents the population
#' count, 1 km spatial resolution layers available to download from the year
#' 2000 to 2020. The dataset is called as WorldPop Unconstrained Global Mosaics.
#' The encoded cell value represents the total number of people in that particular
#' grid cell.
#'
#' It may be required to increase the timeout option to successfully download
#' theses WorldPop layers from their source location via e.g.
#' `options(timeout = 600)`.
#'
#'
#' @name worldpop
#' @param years A numeric vector indicating the years for which to make the
#'   resource available.
#' @docType data
#' @keywords resource
#' @returns A function that returns an `sf` footprint object.
#' @source \url{https://www.worldpop.org/}
#' @include register.R
#' @export
get_worldpop <- function(years = 2000) {
  years <- check_available_years(years, c(2000:2020), "worldpop")

  if (is.null(mapme_options()[["outdir"]])) {
    warning(paste(
      "Worldpop layers must be downloaded from the source location",
      "irrespective if `outdir` was specified or not."
    ))
  }

  function(x,
           name = "worldpop",
           type = "raster",
           outdir = mapme_options()[["outdir"]],
           verbose = mapme_options()[["verbose"]]) {
    srcs <- unlist(sapply(years, function(year) .get_worldpop_url(year)))
    has_outdir <- !is.null(outdir)

    if (has_outdir) {
      dsts <- file.path(outdir, basename(srcs))
    } else {
      tmpdir <- tempfile()
      dir.create(tmpdir)
      dsts <- file.path(tmpdir, basename(srcs))
    }

    is_available <- purrr::map_lgl(dsts, spds_exists, what = "raster")
    if (all(is_available)) {
      return(make_footprints(dsts, what = "raster"))
    }

    purrr::walk2(srcs, dsts, function(src, dst) {
      if (!spds_exists(dst, what = "raster")) {
        # download original worldpop layer
        tmp <- tempfile(fileext = ".tif")
        download.file(src, tmp, mode = ifelse(Sys.info()["sysname"] == "Windows", "wb", "w"))

        # translate to regular grid
        sf::gdal_utils(
          util = "translate",
          source = tmp,
          destination = dst,
          options = c(
            "-co", "COMPRESS=LZW",
            "-co", "PREDICTOR=2",
            "-ot", "Float32",
            "-a_ullr", "-180.0", "84.0", "180.0", "-72.0"
          )
        )
        file.remove(tmp)
      }
    })

    make_footprints(dsts, what = "raster")
  }
}

#' Helper function to construct population layer urls
#'
#' @param target_year A numeric indicating the target year
#'
#' @return A character vector.
#' @keywords internal
#' @noRd
.get_worldpop_url <- function(target_year) {
  available_years <- c(2000:2020)
  if (target_year %in% available_years) {
    paste0(
      "https://data.worldpop.org/GIS/Population/Global_2000_2020/",
      target_year, "/0_Mosaicked/ppp_", target_year, "_1km_Aggregated.tif"
    )
  } else {
    warning(
      sprintf(
        "Population count not available for target year %s", target_year
      )
    )
    NULL
  }
}


register_resource(
  name = "worldpop",
  description = "WorldPop - Unconstrained Global Mosaics 2000 - 2020",
  licence = "CC-BY 4.0",
  source = "https://www.worldpop.org/",
  type = "raster"
)

#' Calculate population exposed to violent conflict from ACLED
#'
#' The indicator calculates the population exposed to conflict events within a
#' specified buffer distance around events in ACLED. Per default,
#' the first available WorldPop layer is used to estimate exposed populations
#' for years before the respective year, while the most recent layer is
#' used for years after.
#'
#' The indicator is inspired by the Conflict Exposure tool from ACLED (see
#' citation below), but differs in the regard that we simply flatten our
#' buffered event layer instead of applying voronoi tessellation.
#'
#' The required resources for this indicator are:
#'  - [acled]
#'  - [worldpop]
#'
#' Events in ACLED are classified according to the schema described extensively
#' in their codebook.
#' You may filter for certain types of events. The categories for which a
#' filter can be applied are either "event_type", "event_sub_type", or
#' "disorder_type". These are translated into the following categories:
#'
#' - event_type:
#'   - battles
#'   - protests
#'   - riots
#'   - explosions/remote_violence
#'   - violence_against_civilians
#'   - strategic_developments
#'
#' - event_sub_type:
#'   - government_regains_territory
#'   - non-state_actor_overtakes_territory
#'   - armed_clash
#'   - excessive_force_against_protesters
#'   - protest_with_intervention
#'   - peaceful_protest
#'   - violent_demonstration
#'   - mob_violence
#'   - chemical_weapon
#'   - air/drone_strike
#'   - suicide_bomb
#'   - shelling/artillery/missile_attack
#'   - remote_explosive/landmine/ied
#'   - grenade
#'   - sexual_violence
#'   - attack
#'   - abduction/forced_disappearance
#'   - agreement
#'   - arrests
#'   - change_to_group/activity
#'   - disrupted_weapons_use
#'   - headquarters_or_base_established
#'   - looting/property_destruction
#'   - non-violent_transfer_of_territory
#'   - other
#'
#' - disorder_type:
#'   - political_violence
#'   - political_violence;_demonstrations
#'   - demonstrations
#'   - political_violence
#'   - strategic_developments
#'
#' You may supply buffer distances for each of the event categories. Custom
#' buffers will then be drawn per category. Supply a single value if you
#' do not wish do differentiate between categories. Otherwise, supply a
#' vector of distances equal to the length of included categories.
#'
#' You may apply quality filters based on the precision of the geolocation
#' of events and the temporal precision. By default, these are set to only
#' include events with the highest precision scores.
#'
#' For geo-precision there are levels 1 to 3 with decreasing accuracy:
#' - value 1: the source reporting indicates a particular town, and coordinates
#'   are available for that town
#' - value 2: the source material indicates that activity took place in a
#'   small part of a region, and mentions a general area or if an activity
#'   occurs near a town or a city, the event is coded to a town with
#'   geo-referenced coordinates to represent that area
#' - value 3: a larger region is mentioned, the closest natural location noted
#'   in reporting (like “border area,” “forest,” or “sea,” among others) – or a
#'   provincial capital is used if no other information at all is available
#'
#' For temporal precision there are levels 1 to 3 with decreasing precision:
#' - value 1: the source material includes an actual date of an event
#' - value 2: the source material indicates that an event happened sometime
#'   during the week or within a similar period of time
#' - value 3: the source material only indicates that an event took place
#'   sometime during a month (i.e. in the past two or three weeks, or in
#'   January), without reference to the particular date, the month mid-point
#'   is chosen
#'
#' @name calc_exposed_population_acled
#' @param distance A numeric vector indicating the buffer radius in meters.
#'   If length is 1, the same buffer size around included conflict events is
#'   drawn. Otherwise, it must be equal to the length of included categories
#'   selected with \code{filter_types}.
#' @param filter_category A character indicating the categories to be used
#'   to calculate the exposed population by. Defaults to `event_type` meaning
#'   one estimation per event type will be returned.
#' @param filter_types A character vector of event types of the respective
#'   category specified in `filter_category` to retain. Defaults to NULL,
#'   meaning that no filter is applied and all types are retained.
#' @param years A numeric vector indicating for which years to calculate
#'   the exposed population. Restricted to available years for ACLED.
#'   For years not intersecting with available WorldPop layers, the first layer
#'   is used for earlier years and the last layer to more recent years.
#' @param precision_location A numeric indicating precision value for the
#'   geolocation up to which events are included. Defaults to 1.
#' @param precision_time A numeric indicating the precision value of the
#'   temporal coding up to which events are included. Defaults to 1.
#' @keywords indicator
#' @returns A function that returns an indicator tibble with conflict exposure
#'   as variable and percentage of the population as its value.
#' @references Raleigh, C; C Dowd; A Tatem; A Linke; N Tejedor-Garavito; M
#'   Bondarenko and K Kishi. 2023. Assessing and Mapping Global and Local
#'   Conflict Exposure. Working Paper.
#' @include register.R
#' @export
#' @examples
#' \dontshow{
#' mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
#' }
#' \dontrun{
#' if (FALSE) {
#'   library(sf)
#'   library(mapme.biodiversity)
#'
#'   outdir <- file.path(tempdir(), "mapme-data")
#'   dir.create(outdir, showWarnings = FALSE)
#'
#'   mapme_options(
#'     outdir = outdir,
#'     verbose = FALSE,
#'     chunk_size = 1e8
#'   )
#'
#'   aoi <- system.file("extdata", "burundi.gpkg",
#'     package = "mapme.biodiversity"
#'   ) %>%
#'     read_sf() %>%
#'     get_resources(
#'       get_acled(year = 2000),
#'       get_worldpop(years = 2000)
#'     ) %>%
#'     calc_indicators(
#'       calc_exposed_population_acled(
#'         distance = 5000,
#'         years = 2000,
#'         precision_location = 1,
#'         precision_time = 1
#'       )
#'     ) %>%
#'     portfolio_long()
#'
#'   aoi
#' }
#' }
calc_exposed_population_acled <- function(
    distance = 5000,
    filter_category = c("event_type", "sub_event_type", "disorder_type"),
    filter_types = NULL,
    years = c(1997:2024),
    precision_location = 1,
    precision_time = 1) {
  check_namespace("exactextractr")
  filter_category <- match.arg(filter_category)
  if (filter_category == "event_type") {
    types <- .acled_event_types
  } else if (filter_category == "sub_event_type") {
    types <- .acled_sub_event_types
  } else {
    types <- .acled_disorder_types
  }
  if (!is.null(filter_types)) {
    stopifnot(all(filter_types %in% types))
  }

  if (!precision_location %in% 1:3) {
    stop("Argument precision_location must be a single numeric between 1 and 3.")
  }
  if (!precision_time %in% 1:3) {
    stop("Argument precision_time must be a single numeric between 1 and 3.")
  }

  if (length(distance) > 1) {
    if (is.null(filter_category)) {
      stop("filter_category cannot be NULL if per-class distances are specified.")
    }
    if (is.null(filter_types)) {
      if (length(types) != length(distance)) {
        msg <- c(
          "Wrong number of distances specified for filter_category `%s`!\n",
          "Total number of categories is %s."
        )
        stop(sprintf(msg, c(filter_category, length(types))))
      }
    } else {
      if (length(filter_types) != length(distance)) {
        msg <- c(
          "Wrong number of distances specified for filter_types!\n",
          "Total number of selected categories is %d."
        )
        stop(sprintf(msg, length(filter_types)))
      }
    }
  }

  years <- check_available_years(years, c(1997:as.integer(format(Sys.Date(), "%Y"))), "exposed_population_acled")

  function(x,
           acled = NULL,
           worldpop = NULL,
           name = "exposed_population_acled",
           mode = "asset",
           aggregation = "sum",
           verbose = mapme_options()[["verbose"]]) {
    time_precision <- geo_precision <- event_date <- year <- stratum <- NULL

    if (is.null(acled) | is.null(worldpop)) {
      return(NULL)
    }

    acled <- st_sf(purrr::list_rbind(acled))
    acled <- acled[unlist(st_contains(x, acled)), ]
    acled <- dplyr::filter(
      acled,
      year %in% years,
      geo_precision <= precision_location,
      time_precision <= precision_time
    )

    if (nrow(acled) == 0) {
      return(NULL)
    }

    acled <- dplyr::select(
      acled,
      year,
      stratum = !!filter_category
    )

    acled <- dplyr::mutate(
      acled,
      stratum = gsub(" ", "_", tolower(stratum))
    )

    if (is.null(filter_types)) {
      acled$distance <- distance
    } else {
      acled <- dplyr::filter(acled, stratum %in% filter_types)
      distance_df <- data.frame(
        stratum = filter_types,
        distance = distance
      )
      acled <- dplyr::left_join(acled, distance_df, by = "stratum")
    }

    if (nrow(acled) == 0) {
      return(NULL)
    }

    acled <- st_buffer(acled, acled$distance)

    # one multi-polygon per year and stratum
    acled_yearly <- dplyr::select(acled, year, stratum) %>%
      dplyr::group_by(year, stratum) %>%
      dplyr::summarise(geom = st_intersection(st_make_valid(st_union(geom)), x))

    # one multi-polygon per year, irrespective of stratum (total exposed pop)
    acled_yearly_total <- dplyr::summarise(acled_yearly, geom = st_intersection(st_make_valid(st_union(geom)), x))
    acled_yearly_total$stratum <- "total"
    acled_yearly <- rbind(acled_yearly, acled_yearly_total)

    .calc_exp_pop(
      acled_yearly,
      worldpop,
      years
    )
  }
}

.calc_exp_pop <- function(polys, popr, years) {
  year <- NULL
  stopifnot("year" %in% names(polys))
  wpop_years <- as.numeric(substr(names(popr), 5, 8))

  exposures <- purrr::map(years, function(y) {
    polys_year <- dplyr::filter(polys, year == y)
    if (nrow(polys_year) == 0) {
      return(NULL)
    }
    if (y %in% wpop_years) {
      index <- which(wpop_years == y)
    } else {
      index <- ifelse(y < wpop_years[1], 1, length(wpop_years))
    }

    pop <- popr[[index]]
    exposed_pop <- exactextractr::exact_extract(
      pop, polys_year,
      fun = "sum", progress = FALSE
    )

    tibble::tibble(
      datetime = as.POSIXct(as.Date(paste0(y, "-01-01"), "%Y-%m-%d")),
      variable = paste0("exposed_population_", polys_year$stratum),
      unit = "count",
      value = exposed_pop
    )
  })

  if (all(sapply(exposures, is.null))) {
    return(NULL)
  }
  purrr::list_rbind(exposures)
}

.acled_categories <- c(
  "event_type",
  "sub_event_type",
  "disorder_type"
)

.acled_event_types <- c(
  "battles",
  "protests",
  "riots",
  "explosions/remote_violence",
  "violence_against_civilians",
  "strategic_developments"
)

.acled_disorder_types <- c(
  "political_violence",
  "political_violence;_demonstrations",
  "demonstrations",
  "political_violence",
  "strategic_developments"
)

.acled_sub_event_types <- c(
  "government_regains_territory",
  "non-state_actor_overtakes_territory",
  "armed_clash",
  "excessive_force_against_protesters",
  "protest_with_intervention",
  "peaceful_protest",
  "violent_demonstration",
  "mob_violence",
  "chemical_weapon",
  "air/drone_strike",
  "suicide_bomb",
  "shelling/artillery/missile_attack",
  "remote_explosive/landmine/ied",
  "grenade",
  "sexual_violence",
  "attack",
  "abduction/forced_disappearance",
  "agreement",
  "arrests",
  "change_to_group/activity",
  "disrupted_weapons_use",
  "headquarters_or_base_established",
  "looting/property_destruction",
  "non-violent_transfer_of_territory",
  "other"
)


register_indicator(
  name = "exposed_population_acled",
  description = "Number of people exposed to conflicts based on ACLED",
  resources = c("acled", "worldpop")
)

.res_defaults <- c("x", "name", "type", "outdir", "verbose")
#' Download biodiversity resources
#'
#' `get_resources()` data sets required for the
#' calculation of indicators can be made available. The function supports the
#' specification of several resource functions. To determine the output path,
#' temporary directory and verbosity, the output of `mapme_options()` is used.
#'
#' @param x An `sf` object with features of type `"POLYGON"`
#' @param ... One or more functions for resources/indicators
#' @return `get_resources()` is called for its side effect of making resources
#'  available in the package environment. Returns `x`, invisibly.
#' @name mapme
#' @export
get_resources <- function(x, ...) {
  x <- .check_portfolio(x)
  if (!.has_internet()) {
    return(invisible(x))
  }
  # initiate GDAL environment variables
  .init_gdal_http()
  funs <- purrr::map(list(...), function(fun) .check_resource_fun(fun))
  for (fun in funs) .get_single_resource(x, fun)
  invisible(x)
}

.check_resource_fun <- function(fun) {
  if (!inherits(fun, "function")) {
    stop("get_resources() expects you supply one or more functions.")
  }

  args <- formals(fun)
  if (any(!names(args) %in% .res_defaults) || any(!.res_defaults %in% names(args))) {
    msg <- paste(
      "Resource functions are required to have the following default arguments:\n",
      paste(.res_defaults, sep = "", collapse = ", ")
    )
    stop(msg)
  }

  resource_name <- args[["name"]]

  if (resource_name %in% names(.avail_resources())) {
    msg <- sprintf("Resource '%s' is already available.", resource_name)
    message(msg)
  }

  invisible(fun)
}

#' @keywords internal
#' @noRd
.get_single_resource <- function(x = NULL,
                                 fun = NULL,
                                 opts = mapme_options()) {
  force(x)
  args <- formals(fun)
  resource_name <- args[["name"]]
  outdir <- .make_path(opts[["outdir"]], resource_name)
  # attach required objects to args list
  args[["x"]] <- x
  args[["outdir"]] <- outdir

  resource <- try(do.call(fun, args = as.list(args)))
  resource <- .check_footprints(resource, resource_name)

  resource <- .fetch_resource(
    resource = resource,
    name = resource_name,
    type = args[["type"]],
    outdir = outdir,
    verbose = opts[["verbose"]],
    retries = opts[["retries"]],
    delay = opts[["delay"]]
  )

  resource <- .set_precision(resource)

  resource <- list(resource)
  names(resource) <- resource_name
  .add_resource(resource)

  x
}

.make_path <- function(outdir, name) {
  if (is.null(outdir)) {
    return(outdir)
  }
  path <- file.path(outdir, name)
  dir.create(path, showWarnings = FALSE)
  path
}

.resource_cols <- c("filename", "location", "type", "oo", "co", "source", "geometry")

.check_footprints <- function(resource, name) {
  if (inherits(resource, "try-error")) {
    stop(paste0(
      "Download for resource ", name, " failed.\n",
      "Returning unmodified portfolio."
    ))
  }

  if (!inherits(resource, "sf")) {
    msg <- paste("Resource functions are expected to return sf objects.")
    stop(msg)
  }

  if (any(!names(resource) %in% .resource_cols)) {
    msg <- paste(
      "Resource functions are expected to return sf objects with",
      "columns:", paste(.resource_cols, collapse = ", ")
    )
    stop(msg)
  }

  invisible(resource)
}

.fetch_resource <- function(resource,
                            name = NULL,
                            type = NULL,
                            outdir = mapme_options()[["outdir"]],
                            verbose = mapme_options()[["verbose"]],
                            retries = mapme_options()[["retries"]],
                            delay = mapme_options()[["delay"]]) {

  stopifnot(inherits(resource, "sf"), type %in% c("raster", "vector"))

  if (is.null(outdir)) {
    resource[["location"]] <- resource[["source"]]
  } else {
    if (verbose) {
      has_progressr <- check_namespace("progressr", error = FALSE)
      if (has_progressr) {
        n <- nrow(resource)
        s <- 1L
        if (n > 100L) {
          s <- round(n * 0.01)
          n <- 100L
        }
        p <- progressr::progressor(n)
      }
    }

    resource[["destination"]] <- file.path(outdir, resource[["filename"]])
    resource <- lapply(seq_len(nrow(resource)), function(i) resource[i, ])

    furrr::future_iwalk(resource, function(x, i) {
      attempts <- 0L

      while (attempts <= retries) {
        attempts <- attempts + 1L

        is_available <- .get_spds(
          source = x[["source"]],
          destination = x[["destination"]],
          opts = unlist(c(x[["co"]], x[["oo"]])),
          what = x[["type"]]
        )
        if (is_available) {
          attempts <- retries + 1L
        } else {
          Sys.sleep(delay)
        }
      }

      if (verbose && has_progressr && (i %% s == 0L)) {
        p(message = sprintf("Fetching resource '%s'.", name))
      }
    }, .options = furrr::furrr_options(seed = TRUE, chunk_size = 1L))

    resource <- sf::st_as_sf(do.call(rbind, resource))
    resource[["location"]] <- resource[["destination"]]
    is_available <- sapply(resource[["location"]], spds_exists, what = type)
    if (isFALSE(all(is_available))) {
      not_available_resource <- resource[!is_available, ]
      msg <- sprintf("%d out of %d resources are not available for '%s'",
                     nrow(not_available_resource), nrow(resource), name)
      log_dir <- mapme_options()[["log_dir"]]
      if (!is.null(log_dir)) {
        dsn <- file.path(log_dir, paste0(Sys.Date(), "_", name,
                                         "_mapme-error-resources.gpkg"))
        # drop list columns to write GPKG
        not_available_resource[["opts"]] <- apply(not_available_resource, 1, FUN = function(x) {
          paste(unlist(c(x[["co"]], x[["oo"]])), collapse = "|")
        })
        not_available_resource <- not_available_resource[,
                                  setdiff(names(not_available_resource), c("oo", "co"))]
        sf::st_write(not_available_resource, dsn, append = TRUE, quiet = TRUE)
        msg <- paste(msg, sprintf("The list can be found in '%s'", dsn), sep = "\n")
      }
      msg <- paste("***** WARNING *****", msg, sep = "\n")
      message(msg)
      resource <- resource[is_available, ]
    }
  }
  return(resource)
}

test_that(".get_gfw_emissions works", {
  skip_on_cran()

  x <- read_sf(
    system.file("extdata", "sierra_de_neiba_478140.gpkg",
      package = "mapme.biodiversity"
    )
  )
  .clear_resources()
  ge <- get_gfw_emissions()
  expect_silent(.check_resource_fun(ge))
  expect_silent(fps <- ge(x))
  expect_silent(.check_footprints(fps))
  expect_equal(fps$filename, "gfw_forest_carbon_gross_emissions_Mg_CO2e_px_20N_080W.tif")
})

test_that(".get_droughtind works", {
  skip_on_cran()

  expect_error(get_nasa_grace(2000))
  gng <- get_nasa_grace(years = 2004:2010)
  expect_silent(.check_resource_fun(gng))
  expect_silent(fps <- gng())
  expect_silent(.check_footprints(fps))
  expect_equal(nrow(fps), 365)
})

test_that(".get_teow works", {
  skip_on_cran()

  gteow <- get_teow("test.zip")
  expect_silent(.check_resource_fun(gteow))
  fps <- gteow()
  expect_silent(.check_footprints(fps))
  expect_equal(fps$filename, "wwf_terr_ecos.gpkg")
})

test_that("get_chelsa works", {
  skip_on_cran()

  expect_error(get_chelsa(1970))
  gc <- get_chelsa(years = 2010)
  expect_silent(.check_resource_fun(gc))
  fps <- gc(aoi)
  expect_silent(fps <- .check_footprints(fps))
  expect_equal(nrow(fps), 12)
})

# This test was deactivated because calls to the CHIRPS website would fail.
# TODO: reactivate this test
# test_that(".get_chirps works", {
#   skip_on_cran()
#
#   expect_error(get_chirps(1970))
#   gc <- get_chirps(years = 2010)
#   expect_silent(.check_resource_fun(gc))
#   fps <- gc(aoi)
#   expect_silent(fps <- .check_footprints(fps))
#   expect_equal(nrow(fps), 12)
# })

test_that(".get_gmw works", {
  skip_on_cran()

  expect_error(get_gmw(2011))
  expect_silent(gmw <- get_gmw(years = 2007:2008))
  expect_silent(.check_resource_fun(gmw))
  expect_silent(fps <- gmw())
  expect_silent(.check_footprints(fps))
  expect_equal(fps$filename, c("gmw_v3_2007_vec.gpkg", "gmw_v3_2008_vec.gpkg"))
})

test_that(".get_nasa_srtm works", {
  skip_on_cran()

  x <- read_sf(
    system.file("extdata", "sierra_de_neiba_478140.gpkg",
      package = "mapme.biodiversity"
    )
  )

  gns <- get_nasa_srtm()
  expect_silent(.check_resource_fun(gns))
  fps <- gns(x)
  expect_silent(.check_footprints(fps))
  expect_equal(fps$filename, "NASADEM_HGT_n18w072.tif")
  x_split <- st_as_sf(st_make_grid(x, n = 2))
  fps <- gns(x_split)
  expect_equal(nrow(fps), 1)
})

test_that(".get_fritz_et_al works", {
  skip_on_cran()

  expect_error(get_fritz_et_al(resolution = 1))
  gf <- get_fritz_et_al(resolution = 100)
  expect_silent(.check_resource_fun(gf))
  suppressWarnings(fps <- gf())
  expect_silent(.check_footprints(fps))
  gf <- get_fritz_et_al(resolution = 1000)
  suppressWarnings(fps2 <- gf())
  expect_silent(.check_footprints(fps2))
  expect_equal(fps$filename, "geo_fritz_et_al_100m.tif")
  expect_equal(fps2$filename, "geo_fritz_et_al_1000m.tif")
})

test_that(".get_esalandcover works", {
  skip_on_cran()
  x <- read_sf(
    system.file("extdata", "sierra_de_neiba_478140.gpkg",
      package = "mapme.biodiversity"
    )
  )

  # Add testing attribute in order to skip downloads
  expect_error(get_esalandcover(years = 2000))
  gel <- get_esalandcover(years = 2015)
  expect_silent(.check_resource_fun(gel))
  fps <- gel(x)
  expect_silent(.check_footprints(fps))
  # adds test to check for multiple polygons in the same tile
  x_split <- st_as_sf(st_make_grid(x, n = 2))
  fps2 <- gel(x_split)
  expect_true(identical(fps, fps2))
})

# test-get_ucdp_ged.R
test_that(".get_ucdp_ged works", {
  skip_on_cran()

  expect_error(get_ucdp_ged(version = "not-available"))

  guc <- get_ucdp_ged(version = "latest")
  expect_silent(.check_resource_fun(guc))
  fps <- guc()
  expect_silent(.check_footprints(fps))
  expect_equal(fps$filename, "ged241-csv.gpkg")

  guc <- get_ucdp_ged(version = "19.1")
  fps <- guc()
  expect_equal(fps$filename, "ged191-csv.gpkg")

  expect_equal(
    .ucdp_versions(),
    c(
      "5.0", "17.1", "17.2", "18.1", "19.1",
      "20.1", "21.1", "22.1", "23.1", "24.1"
    )
  )
})

test_that("get_soilgrids works", {
  skip_on_cran()
  skip_if_not(Sys.getenv("USER") == "darius")

  expect_error(get_soilgrids(layers = "na", depths = .sg_depths, stats = .sg_stats))
  expect_error(get_soilgrids(layers = names(.sg_layers), depths = "na", stats = .sg_stats))
  expect_error(get_soilgrids(layers = names(.sg_layers), depths = .sg_depths, stats = "na"))

  gsg <- get_soilgrids(layers = names(.sg_layers)[1], depths = .sg_depths[1], stats = .sg_stats[1])
  expect_silent(.check_resource_fun(gsg))
  suppressWarnings(fps <- gsg())
  expect_silent(.check_footprints(fps))
  expect_equal(fps$filename, "bdod_0-5cm_Q0.05.vrt")
})

test_that("get_mcd64a1 works", {
  skip_on_cran()

  x <- read_sf(
    system.file("extdata", "sierra_de_neiba_478140_2.gpkg",
      package = "mapme.biodiversity"
    )
  )

  gmc <- get_mcd64a1(years = 2010)
  expect_silent(.check_resource_fun(gmc))
  fps <- gmc(x)
  expect_silent(.check_footprints(fps))
  expect_equal(fps$filename[1], "MCD64A1.A2010335.h11v07.061.2021309002413_Burn_Date.tif")
  x_split <- st_as_sf(st_make_grid(x, n = 2))
  fps <- gmc(x_split)
  expect_equal(nrow(fps), 12)
})

test_that(".get_gfw_lossyear works", {
  skip_on_cran()
  .clear_resources()

  x <- read_sf(
    system.file("extdata", "sierra_de_neiba_478140.gpkg",
      package = "mapme.biodiversity"
    )
  )
  expect_error(get_gfw_lossyear("na"))
  gl <- get_gfw_lossyear()
  expect_silent(.check_resource_fun(gl))
  expect_silent(fps <- gl(x))
  expect_silent(.check_footprints(fps))
  expect_equal(fps$filename, "Hansen_GFC-2024-v1.12_lossyear_20N_080W.tif")
})

test_that(".get_accessibility_2000 works", {

  # Run the function and check that it returns a valid function
  ga2000 <- get_accessibility_2000()
  expect_silent(.check_resource_fun(ga2000))

  # Execute the returned function to get the footprint
  fps <- ga2000()

  # Check that the footprint is a valid sf object
  expect_silent(.check_footprints(fps))

  # Verify the filename in the returned footprint
  expect_equal(fps$filename, "acc_50k.tif")
})

test_that(".get_nelson_et_al works", {
  skip_on_cran()

  expect_error(get_nelson_et_al(ranges = "na"))
  gne <- get_nelson_et_al(ranges = "20k_50k")
  expect_silent(.check_resource_fun(gne))
  fps <- gne()
  expect_silent(.check_footprints(fps))
  expect_equal(fps$filename, "traveltime-20k_50k.tif")
})

#' Calculate Key Biodiversity Areas
#'
#' This function calculates the total area of key biodiversity areas for a given
#' input polygon.
#'
#' The required resources for this indicator are:
#'  - [key_biodiversity_areas_resource]
#'
#' @name key_biodiversity_areas_indicator
#' @docType data
#' @keywords indicator
#' @format A function returning an indicator tibble with `key_biodiversity_area`
#'   as variable and the total overlap area (in ha) as value.
#' @include register.R
#' @export
#' @examples
#' \dontshow{
#' mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
#' }
#' \dontrun{
#' library(sf)
#' library(mapme.biodiversity)
#'
#' outdir <- file.path(tempdir(), "mapme-data")
#' dir.create(outdir, showWarnings = FALSE)
#'
#' mapme_options(
#'   outdir = outdir,
#'   verbose = FALSE
#' )
#'
#' aoi <- read_sf(
#'   system.file("extdata", "shell_beach_protected_area_41057_B.gpkg",
#'               package = "mapme.biodiversity"
#' ))
#' kbas <- system.file("res", "key_biodiversity_areas", "kbas.gpkg",
#'                     package = "mapme.biodiversity")
#' aoi <- get_resources(aoi, get_key_biodiversity_areas(kbas))
#' aoi <- calc_indicators(aoi, calc_key_biodiversity_area())
#' aoi <- portfolio_long(aoi)
#'
#' aoi
#' }
calc_key_biodiversity_area <- function() {
  function(x = NULL,
           key_biodiversity_areas,
           name = "key_biodiversity_areas",
           mode = "asset",
           aggregation = "sum",
           verbose = mapme_options()[["verbose"]]) {

    key_biodiversity_areas <- key_biodiversity_areas[[1]]
    if (is.null(key_biodiversity_areas)) {
      return(NULL)
    }
    if (length(key_biodiversity_areas) == 0) {
      return(NULL)
    }

    int_area <- suppressWarnings(st_intersection(x, key_biodiversity_areas))
    if (nrow(int_area) == 0) return(NULL)
    int_area_ha <- as.numeric(sum(st_area(int_area), na.rm = TRUE) / 10000)

    results <- tibble::tibble(
      datetime = as.POSIXct("2024-01-01T00:00:00Z"),
      variable = "key_biodiversity_area",
      unit = "ha",
      value = int_area_ha
    )

    return(results)
  }
}

register_indicator(
  name = "key_biodiversity_areas",
  description = "Area estimation of intersection with key biodiversity areas.",
  resources = "key_biodiversity_areas"
)

#' Calculate areal statistics for IBPES Biomes
#'
#' This indicator calculates the areal distribution of different biome classes
#' within an asset based on the IBPES biomes dataset.
#'
#' The required resources for this indicator are:
#'  - [ipbes_biomes]
#'
#' @name ipbes_biome_stats
#' @keywords indicator
#' @returns A function that returns an indicator tibble with the biome class
#' as variable and the respective area (in ha) as value.
#' @export
#' @examples
#' \dontshow{
#' mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
#' }
#' \dontrun{
#' library(sf)
#' library(mapme.biodiversity)
#'
#' outdir <- file.path(tempdir(), "mapme-data")
#' dir.create(outdir, showWarnings = FALSE)
#'
#' mapme_options(
#'   outdir = outdir,
#'   verbose = FALSE
#' )
#'
#' aoi <- system.file("extdata", "shell_beach_protected_area_41057_B.gpkg",
#'   package = "mapme.biodiversity"
#' ) %>%
#'   read_sf() %>%
#'   get_resources(get_ipbes_biomes()) %>%
#'   calc_indicators(calc_ipbes_biomes()) %>%
#'   portfolio_long()
#'
#' aoi
#' }
calc_ipbes_biomes <- function() {
  function(
      x,
      ipbes_biomes,
      name = "ipbes_biomes",
      mode = "portfolio",
      aggregation = "sum",
      verbose = mapme_options()[["verbose"]]) {
    if (is.null(ipbes_biomes)) {
      return(NULL)
    }
    if (st_crs(x) != st_crs(ipbes_biomes)) {
      x <- st_transform(x, st_crs(ipbes_biomes))
    }

    results <- lapply(1:nrow(x), function(i) {
      exactextractr::exact_extract(ipbes_biomes, x[i, ], fun = function(data, class_df) {
        data[["coverage_area"]] <- data[["coverage_area"]] / 10000
        class_area <- by(data[["coverage_area"]], data[["value"]], sum)
        result <- tibble::tibble(
          variable = as.numeric(names(class_area)),
          unit = "ha",
          value = as.numeric(class_area)
        )
        result[["variable"]] <- class_df[["name"]][result[["variable"]]]
        result[["variable"]] <- gsub(" ", "_", result[["variable"]])
        result[["datetime"]] <- as.POSIXct("2019-01-01T00:00:00Z")
        result[, c("datetime", "variable", "unit", "value")]
      }, class_df = .ipbes_biome_classes, coverage_area = TRUE, summarize_df = TRUE)
    })

    results
  }
}

register_indicator(
  name = "ipbes_biomes",
  description = "Area distribution of IBPES biomes within a polygon.",
  resources = "ipbes_biomes"
)

.ipbes_biome_classes <- data.frame(
  class = c(1:15),
  name = c(
    "tropical and subtropical dry and humid forests",
    "temperate and boreal forests and woodland",
    "mediterranean forests woodlands and scrub",
    "tundra and high mountain habitats",
    "tropical and subtropical savannas and grasslands",
    "temperate grassland",
    "deserts and xeric shrubland",
    "wetlands",
    "urban and semi-urban areas",
    "cultivated areas",
    "cryosphere",
    "aquaculture areas",
    "inland surface waters and water bodies",
    "shelf ecosystems",
    "open ocean pelagic systems"
  )
)

#' Calculate drought indicator statistics
#'
#' This function allows to efficiently calculate the relative wetness in the
#' shallow groundwater section with regard to the the 1948-2012 reference period.
#' The values represent the wetness percentile a given area achieves at a given
#' point in time in regard to the reference period.
#' For each polygon, the desired statistic/s (mean, median or sd) is/are
#' returned.
#'
#' The required resources for this indicator are:
#'  - [nasa_grace]
#'
#' @name drought_indicator
#' @param stats Function to be applied to compute statistics for polygons
#'    either one or multiple inputs as character "mean", "median" or "sd".
#' @param engine The preferred processing functions from either one of "zonal",
#'   "extract" or "exactextract" as character.
#' @keywords indicator
#' @returns A function that returns an indicator tibble with specified drought
#'   indicator statistics as variable and corresponding values as value.
#' @include register.R
#' @export
#' @examples
#' \dontshow{
#' mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
#' }
#' \dontrun{
#' library(sf)
#' library(mapme.biodiversity)
#'
#' outdir <- file.path(tempdir(), "mapme-data")
#' dir.create(outdir, showWarnings = FALSE)
#'
#' mapme_options(
#'   outdir = outdir,
#'   verbose = FALSE
#' )
#'
#' aoi <- system.file("extdata", "sierra_de_neiba_478140_2.gpkg",
#'   package = "mapme.biodiversity"
#' ) %>%
#'   read_sf() %>%
#'   get_resources(get_nasa_grace(years = 2022)) %>%
#'   calc_indicators(
#'     calc_drought_indicator(
#'       engine = "extract",
#'       stats = c("mean", "median")
#'     )
#'   ) %>%
#'   portfolio_long()
#'
#' aoi
#' }
calc_drought_indicator <- function(engine = "extract", stats = "mean") {
  engine <- check_engine(engine)
  stats <- check_stats(stats)

  function(x,
           nasa_grace = NULL,
           name = "drought_indicator",
           mode = "portfolio",
           aggregation = "stat",
           verbose = mapme_options()[["verbose"]]) {
    # check if input engines are correct
    if (is.null(nasa_grace)) {
      return(NULL)
    }
    results <- select_engine(
      x = x,
      raster = nasa_grace,
      stats = stats,
      engine = engine,
      name = "wetness",
      mode = mode
    )

    dates <- sub(".*(\\d{8}).*", "\\1", names(nasa_grace))
    dates <- as.POSIXct(paste0(as.Date(dates, "%Y%m%d"), "T00:00:00Z"))

    prep_results <- function(result, datetimes) {
      result %>%
        dplyr::mutate(datetime = datetimes, unit = "percentage") %>%
        tidyr::pivot_longer(cols = -c(datetime, unit), names_to = "variable") %>%
        dplyr::select(datetime, variable, unit, value)
    }

    if (mode == "portfolio") {
      results <- purrr::map(results, prep_results, datetimes = dates)
    } else {
      results <- prep_results(results, date)
    }
    results
  }
}

register_indicator(
  name = "drought_indicator",
  description = "Relative wetness statistics based on NASA GRACE",
  resources = "nasa_grace"
)

#' Calculate accessibility statistics
#'
#' Accessibility is the ease with which larger cities can be reached from a
#' certain location. This function allows to efficiently calculate accessibility
#' statistics (i.e. travel time to nearby major cities) for polygons. For each
#' polygon, the desired statistic/s (mean, median or sd) is/are returned.
#'
#' The required resources for this indicator are:
#'  - [nelson_et_al]
#'
#' @name traveltime
#' @param engine The preferred processing functions from either one of "zonal",
#'   "extract" or "exactextract" as character.
#' @param stats Function to be applied to compute statistics for polygons either
#'   single or multiple inputs as character. Supported statistics are: "mean",
#'   "median", "sd", "min", "max", "sum" "var".
#' @keywords indicator
#' @returns A function that returns an indicator tibble with city ranges and
#'   statistics as variable and corresponding values (in minutes) as value.
#' @include register.R
#' @export
#' @examples
#' \dontshow{
#' mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
#' }
#' \dontrun{
#' library(sf)
#' library(mapme.biodiversity)
#'
#' outdir <- file.path(tempdir(), "mapme-data")
#' dir.create(outdir, showWarnings = FALSE)
#'
#' mapme_options(
#'   outdir = outdir,
#'   verbose = FALSE
#' )
#'
#' aoi <- system.file("extdata", "sierra_de_neiba_478140_2.gpkg",
#'   package = "mapme.biodiversity"
#' ) %>%
#'   read_sf() %>%
#'   get_resources(get_nelson_et_al(ranges = "100k_200k")) %>%
#'   calc_indicators(
#'     calc_traveltime(engine = "extract", stats = c("min", "max"))
#'   ) %>%
#'   portfolio_long()
#'
#' aoi
#' }
calc_traveltime <- function(engine = "extract", stats = "mean") {
  engine <- check_engine(engine)
  stats <- check_stats(stats)

  function(x,
           nelson_et_al = NULL,
           name = "traveltime",
           mode = "asset",
           aggregation = "stat",
           verbose = mapme_options()[["verbose"]]) {
    if (is.null(nelson_et_al)) {
      return(NULL)
    }
    # set max value of 65535 to NA
    nelson_et_al <- clamp(nelson_et_al, lower = -Inf, upper = 65534, values = FALSE)
    results <- select_engine(
      x = x,
      raster = nelson_et_al,
      stats = stats,
      engine = engine,
      name = "traveltime",
      mode = "asset"
    )

    distances <- unlist(
      lapply(
        names(nelson_et_al),
        function(x) strsplit(x, "-|.tif")[[1]][2]
      )
    )

    results %>%
      dplyr::mutate(distances = distances) %>%
      tidyr::pivot_longer(-distances, names_to = "variable") %>%
      dplyr::mutate(
        datetime = as.POSIXct("2015-01-01T00:00:00Z"),
        variable = paste0(distances, "_", variable),
        unit = "minutes"
      ) %>%
      dplyr::select(datetime, variable, unit, value)
  }
}

register_indicator(
  name = "traveltime",
  description = "Statistics of traveltime to the closests city grouped by city category",
  resources = "nelson_et_al"
)

#' Year of forest loss occurrence
#'
#' This resource is part of the publication by Hansen et al. (2013)
#' "High-Resolution Global Maps of 21st-Century Forest Cover Change". It
#' represents "Forest loss during the period 2000–2021, defined as a
#' stand-replacement disturbance, or a change from a forest to non-forest state.
#' Encoded as either 0 (no loss) or else a value in the range 1–20, representing
#' loss detected primarily in the year 2001–2021, respectively." Due to changes
#' in the satellites products used in the compilation of the tree loss product,
#' results before the year 2011 and afterwards are not directly comparable
#' until reprocessing has finished. Users should be aware of this limitation,
#'  especially when the timeframe of the analysis spans over the two periods
#'  delimited by the year 2011.
#'
#' @name gfw_lossyear
#' @param version The version of the dataset to download. Defaults to
#'   "GFC-2024-v1.12". Check \code{mapme.biodiversity:::.available_gfw_versions()}
#'   to get a list of available versions
#' @keywords resource
#' @returns A function that returns an `sf` footprint object.
#' @references Hansen, M. C., P. V. Potapov, R. Moore, M. Hancher, S. A.
#' Turubanova, A. Tyukavina, D. Thau, S. V. Stehman, S. J. Goetz, T. R.
#' Loveland, A. Kommareddy, A. Egorov, L. Chini, C. O. Justice, and J. R. G.
#' Townshend. 2013. “High-Resolution Global Maps of 21st-Century Forest Cover
#' Change.” Science 342 (15 November): 850–53.
#' @source \url{https://data.globalforestwatch.org/documents/tree-cover-loss/explore}
#' @include register.R
#' @export
get_gfw_lossyear <- function(version = "GFC-2024-v1.12") {
  version <- .check_gfw_version(version)

  function(x,
           name = "gfw_lossyear",
           type = "raster",
           outdir = mapme_options()[["outdir"]],
           verbose = mapme_options()[["verbose"]]) {
    # make the GFW grid and construct urls for intersecting tiles
    baseurl <- sprintf(
      "/vsicurl/https://storage.googleapis.com/earthenginepartners-hansen/%s/",
      version
    )
    grid_gfc <- make_global_grid(
      xmin = -180, xmax = 170, dx = 10,
      ymin = -50, ymax = 80, dy = 10
    )
    tile_ids <- unique(unlist(st_intersects(x, grid_gfc)))
    if (length(tile_ids) == 0) {
      stop("The extent of the portfolio does not intersect with the GFW grid.",
        call. = FALSE
      )
    }
    ids <- sapply(tile_ids, function(n) .get_gfw_tile_id(grid_gfc[n, ]))
    urls <- sprintf("%sHansen_%s_lossyear_%s.tif", baseurl, version, ids)
    fps <- grid_gfc[tile_ids, ]
    fps[["source"]] <- urls
    make_footprints(fps,
      what = "raster",
      co = c("-co", "INTERLEAVE=BAND", "-co", "COMPRESS=LZW", "-ot", "Byte")
    )
  }
}

register_resource(
  name = "gfw_lossyear",
  description = "Global Forest Watch - Year of forest cover loss occurrence",
  licence = "CC-BY 4.0",
  source = "https://data.globalforestwatch.org/documents/tree-cover-loss/explore",
  type = "raster"
)

#' MODIS Burned Area Monthly (MCD64A1)
#'
#' The Terra and Aqua combined MCD64A1 Version 6.1 Burned Area data product is
#' a monthly, global gridded 500 meter (m) product containing per-pixel
#' burned-area and quality information. The MCD64A1 burned-area mapping approach
#' employs 500 m Moderate Resolution Imaging Spectroradiometer (MODIS) Surface
#' Reflectance imagery coupled with 1 kilometre (km) MODIS active fire observations.
#'
#' The algorithm uses a burn sensitive Vegetation Index (VI) to create dynamic
#' thresholds that are applied to the composite data. The VI is derived from
#' MODIS shortwave infrared atmospherically corrected surface reflectance bands
#' 5 and 7 with a measure of temporal texture. The algorithm identifies the date
#' of burn for the 500 m grid cells within each individual MODIS tile. The date
#' is encoded in a single data layer as the ordinal day of the calendar year on
#' which the burn occurred with values assigned to unburned land pixels and
#' additional special values reserved for missing data and water grid cells.
#'
#' @name mcd64a1
#' @param years Numeric vector of years to make the MCD64A1 product available
#'   for. Must be greater than the year 2000.
#' @keywords resource
#' @returns A function that returns an `sf` footprint object.
#' @references Giglio, L., C. Justice, L. Boschetti, D. Roy. MODIS/Terra+Aqua
#'   Burned Area Monthly L3 Global 500m SIN Grid V061. 2021, distributed by
#'   NASA EOSDIS Land Processes Distributed Active Archive Center.
#'   \doi{doi:10.5067/MODIS/MCD64A1.061}
#' @source \url{https://planetarycomputer.microsoft.com/dataset/modis-64A1-061}
#' @include register.R
#' @export
get_mcd64a1 <- function(years = 2000:2022) {
  check_namespace("rstac")
  now <- as.numeric(format(Sys.Date(), "%Y"))
  years <- check_available_years(years, c(2000:now), "mcd64a1")

  function(x,
           name = "mcd64a1",
           type = "raster",
           outdir = mapme_options()[["outdir"]],
           verbose = mapme_options()[["verbose"]]) {
    dt <- paste0(min(years), "-01-01/", max(years), "-12-31")
    items <- try(rstac::stac("https://planetarycomputer.microsoft.com/api/stac/v1/") %>%
      rstac::stac_search(
        collection = "modis-64A1-061",
        bbox = as.numeric(st_bbox(x)),
        datetime = dt,
        limit = NULL
      ) %>%
      rstac::post_request() %>%
      rstac::items_fetch())

    if (inherits(items, "try-error")) {
      stop("Download for MCD64A1 resource was unsuccesfull")
    }
    # "secret" option to revert to the old method of MPC URL signing
    if (getOption("mapme_biodiversity_MPC_sign_old", default = FALSE)) {
      urls <- paste0("/vsicurl?pc_url_signing=yes&url=",
                     rstac::assets_url(items, asset_names = "Burn_Date"))
      file_names <- basename(urls)
    } else {
      signed_urls <- rstac::items_sign_planetary_computer(items,
                                                          subscription_key = Sys.getenv("PC_SDK_SUBSCRIPTION_KEY")) %>%
        rstac::assets_url(asset_names = "Burn_Date")
      urls <- paste0("/vsicurl/", signed_urls)
      file_names <- basename(rstac::assets_url(items, asset_names = "Burn_Date"))
    }

    # urls <- rstac::assets_url(items, asset_names = "Burn_Date")
    if (length(urls) == 0) {
      stop("The extent of the portfolio does not intersect with the SRTM grid.")
    }

    bboxs <- rstac::items_bbox(items)
    fps <- purrr::map(bboxs, function(x) {
      names(x) <- c("xmin", "ymin", "xmax", "ymax")
      bbox <- st_bbox(x, crs = "EPSG:4326")
      st_as_sf(st_as_sfc(bbox))
    })
    fps <- st_as_sf(purrr::list_rbind(fps))
    # fps[["source"]] <- paste0("/vsicurl?pc_url_signing=yes&url=", urls)
    fps[["source"]] <- urls
    fps <- st_transform(fps, "+proj=sinu +lon_0=0 +x_0=0 +y_0=0 +R=6371007.181 +units=m +no_defs")
    make_footprints(
      fps,
      filenames = file_names,
      what = "raster",
      co = c("-co", "COMPRESS=DEFLATE", "-of", "COG"),
      precision = 1e3
    )
  }
}

register_resource(
  name = "mcd64a1",
  description = "MODIS Burned Area Monthly Product (Aqua and Terra)",
  licence = "https://www.earthdata.nasa.gov/engage/open-data-services-software-policies/data-use-guidance",
  source = "https://planetarycomputer.microsoft.com/dataset/modis-64A1-061",
  type = "raster"
)

#' Portfolio methods
#'
#' `write_portfolio()` writes a processed biodiversity portfolio to disk.
#' Portfolio data will only be serialized to disk as a GeoPackage including
#' two tables: `metadata` and `indicators`. The `metadata` tables
#' includes, among other simple variables the geometries and a primary
#' key called `assetid`. The 'indicators' tables includes the foreign key
#' `assetid`, a column called `indicator` giving the name of the original
#' indicator as well as the standard indicator columns `datetime`, `variable`,
#' `unit`, and `value`. For convenience, use `read_portfolio()` to read
#' such a portfolio GeoPackage back into R.
#'
#' @param x A portfolio object processed with `mapme.biodiversity`.
#' @param dsn A file path for the output file (must end with `gpkg`).
#' @param ... Additional arguments supplied to `write_sf()` or `read_sf()`
#' @return `write_portfolio()` returns `dsn`, invisibly.
#' @name portfolio
#' @export
write_portfolio <- function(x,
                            dsn,
                            ...) {
  assetid <- NULL
  x <- .check_portfolio(x)
  inds_cols <- .indicators_col(x)

  if (length(inds_cols) == 0) {
    stop("No calculated indicators have been found")
  }

  if (tools::file_ext(dsn) != "gpkg") {
    warning("Can serialze portoflio only to GeoPackage.")
    dsn <- gsub(tools::file_ext(dsn), "gpkg", dsn)
  }

  metadata <- dplyr::select(x, -dplyr::all_of(names(inds_cols)))
  write_sf(metadata, dsn, "metadata", ...)

  data <- st_drop_geometry(dplyr::select(x, assetid, dplyr::all_of(names(inds_cols))))
  purrr::walk(names(inds_cols), function(ind) {
    tmp <- dplyr::select(data, assetid, dplyr::all_of(ind))
    tmp <- tidyr::unnest(tmp, dplyr::all_of(ind), keep_empty = FALSE)
    tmp[["indicator"]] <- ind
    vars <- c("assetid", "indicator", "datetime", "variable", "unit", "value")
    if (!all(vars %in% names(tmp))) {
      msg <- sprintf("Indicator '%s' contained no valid values. Dropping it from the output.", ind)
      warning(msg)
      return()
    }
    write_sf(tmp[, vars], dsn, "indicators", append = TRUE, ...)
  })

  return(invisible(dsn))
}

#' `read_portfolio()` is used to read a portfolio object that was previously
#' written to disk via `write_portfolio()` back into R as an `sf` object.
#' It should be directed against a GeoPackage which was the output
#' of `write_portfolio()`, otherwise the function is very likely to fail.
#' All available indicators will be read back into R as nested list columns
#' reflecting the output once `calc_indicators()` has been called.
#'
#' @param src A character vector pointing to a GeoPackage that has been
#'   previously written to disk via `write_portfolio()`
#' @return `read_portfolio()` returns an `sf` object object with nested list
#'   columns for every indicator found in the GeoPackage source file.
#' @name portfolio
#' @export
#'
read_portfolio <- function(src, ...) {
  assetid <- indicator <- NULL
  all_layers <- st_layers(src)
  if (!identical(all_layers[["name"]], c("metadata", "indicators"))) {
    stop(sprintf(
      "Input file at '%s' does not seem to be a proper portfolio file written with 'write_portfolio()'",
      src
    ))
  }

  metadata <- read_sf(src, layer = "metadata")
  data <- read_sf(src, layer = "indicators")

  data <- tidyr::nest(data, data = c(-assetid, -indicator))
  is_null <- purrr::map_lgl(data[["data"]], function(x) all(is.na(x)))
  data[["data"]][is_null] <- list(NULL)
  data <- tidyr::pivot_wider(data, id_cols = assetid, names_from = indicator, values_from = data)

  metadata <- dplyr::left_join(metadata, data, by = "assetid")
  .geom_last(st_as_sf(tibble::as_tibble(metadata)))
}

#' Transform portfolio to long
#'
#' `portfolio_long()` transforms a portfolio to long-format, potentially
#' dropping geometries in the process.
#'
#' @param indicators If NULL (the default), all indicator columns will be detected
#'   and transformed automatically. If a character vector is supplied, only
#'   those indicators will be transformed.
#' @param drop_geoms A logical, indicating if geometries should be dropped.
#'
#' @return `portfolio_long()` returns the portfolio object in long-format.
#' @name portfolio
#' @export
#'
portfolio_long <- function(x, indicators = NULL, drop_geoms = FALSE) {
  .check_portfolio(x)
  if (is.null(indicators)) {
    indicators <- names(.indicators_col(x))
  }
  stopifnot(all(indicators %in% names(x)))

  is_null <- sapply(indicators, function(ind) {
    all(sapply(x[[ind]], is.null))
  })

  if (all(is_null)) {
    warning("All indicator columns contained 'NULL'.")
    return(x[, -which(names(x) %in% indicators)])
  }

  if (any(is_null)) {
    x <- x[, -.indicators_col(x)[is_null]]
    indicators <- names(.indicators_col(x))
  }

  if (drop_geoms) x <- st_drop_geometry(x)

  ind_cols <- c("indicator", "datetime", "variable", "unit", "value")
  all_cols <- names(x)
  other_cols <- all_cols[-which(all_cols %in% indicators)]

  x_long <- purrr::map(indicators, function(ind) {
    indicator <- tidyr::unnest(x, cols = {{ ind }})
    indicator["indicator"] <- ind
    indicator <- indicator[, c(other_cols, ind_cols)]
  }) %>%
    purrr::list_rbind()

  if (!drop_geoms) {
    x_long <- .geom_last(st_as_sf(x_long))
  }
  x_long
}

#' Transform portfolio to wide
#'
#' `portfolio_wide()` transforms a portfolio to wide-format, potentially
#' dropping geometries in the process.
#'
#' @return `portfolio_wide()` returns the portfolio object in wide-format.
#' @name portfolio
#' @export
#'
portfolio_wide <- function(x, indicators = NULL, drop_geoms = FALSE) {
  assetid <- NULL
  .check_portfolio(x)
  if (is.null(indicators)) {
    indicators <- names(.indicators_col(x))
  }
  stopifnot(all(indicators %in% names(x)))

  is_null <- sapply(indicators, function(ind) {
    all(sapply(x[[ind]], is.null))
  })

  if (all(is_null)) {
    warning("All indicator columns contained 'NULL'.")
    return(x[, -which(names(x) %in% indicators)])
  }

  if (any(is_null)) {
    x <- x[, -.indicators_col(x)[is_null]]
    indicators <- names(.indicators_col(x))
  }

  if (drop_geoms) x <- st_drop_geometry(x)

  indicators_wide <- purrr::map(indicators, function(indicator) {
    indicator_assets <- x[[indicator]]
    indicator_assets <- purrr::map(indicator_assets, function(asset) {
      if (is.null(asset)) {
        return(NULL)
      }
      asset[["indicator"]] <- indicator
      tidyr::pivot_wider(
        asset,
        names_from = c(indicator, datetime, variable, unit),
        names_sep = "_", values_from = value
      )
    })
    names(indicator_assets) <- x[["assetid"]]
    purrr::list_rbind(indicator_assets, names_to = "assetid")
  })

  indicators_wide <- purrr::reduce(indicators_wide, dplyr::left_join, by = "assetid")
  indicators_wide[["assetid"]] <- as.numeric(indicators_wide[["assetid"]])

  x_wide <- x %>%
    dplyr::select(-{{ indicators }}) %>%
    dplyr::left_join(indicators_wide, by = "assetid")

  if (!drop_geoms) {
    x_wide <- .geom_last(st_as_sf(x_wide))
  }
  x_wide
}

.check_portfolio <- function(x, verbose = mapme_options()[["verbose"]]) {
  stopifnot(inherits(x, "sf"))

  sf_col <- attr(x, "sf_column")
  if (st_crs(x) != st_crs(4326)) {
    message("CRS of x is not EPSG:4326. Attempting to transform.")
    x <- st_transform(x, 4326)
  }
  if (any(!unique(st_geometry_type(x)) %in% c("POLYGON", "MULTIPOLYGON"))) {
    stop("Only assets of type 'POLYGON' and 'MULTIPOLYGON' are supported.")
  }
  if (!inherits(x, "tbl_df")) {
    x <- st_as_sf(tibble::as_tibble(x))
  }
  has_assetid <- "assetid" %in% names(x)
  if (!has_assetid) x[["assetid"]] <- 1:nrow(x)
  is_unique <- length(unique(x[["assetid"]])) == nrow(x)
  if (!is_unique) {
    msg <- paste("Found a column named 'assetid' with non-unique identifiers.",
      "Overwritting its values.",
      sep = " "
    )
    message(msg)
    x[["assetid"]] <- 1:nrow(x)
  }
  st_geometry(x) <- sf_col
  x
}

.indicators_col <- function(x) {
  inds <- which(sapply(x, is.list))
  is_sf <- inherits(x, "sf")
  if (is_sf && length(inds) == 1) {
    msg <- paste("No calculated indicators have been found.",
      "Cannot write as a portfolio.",
      sep = " "
    )
    stop(msg)
  }
  if (is_sf) {
    inds[-which(names(inds) == attributes(x)[["sf_column"]])]
  } else {
    inds
  }
}

test_that("portoflio I/O works as expected", {
  mapme_options(verbose = F)
  x <- read_sf(
    system.file("extdata", "sierra_de_neiba_478140.gpkg",
      package = "mapme.biodiversity"
    )
  )
  x <- suppressWarnings(st_cast(x, to = "POLYGON")[1, ])

  x[["assetid"]] <- 1
  dsn <- tempfile(fileext = ".gpkg")

  expect_error(
    write_portfolio(x, dsn),
    "No calculated indicators have been found"
  )

  indicator <- list(tibble(
    datetime = "2000-01-01",
    variable = "biome",
    unit = "ha",
    value = 1
  ))

  x[["biome"]] <- indicator

  expect_invisible(out <- write_portfolio(x, dsn, quiet = TRUE))
  expect_equal(out, dsn)
  expect_silent(write_portfolio(x, dsn))
  expect_equal(st_layers(dsn)[["name"]], c("metadata", "indicators"))

  meta <- st_read(dsn, layer = "metadata", quiet = TRUE)
  inds <- st_read(dsn, layer = "indicators", quiet = TRUE)

  expect_true(inherits(meta, "sf"))
  expect_true(inherits(inds, "data.frame"))

  vars <- c("assetid", "indicator", "datetime", "variable", "unit", "value")
  expect_true(all(vars %in% names(inds)))

  data <- read_portfolio(dsn, quiet = TRUE)
  vars <- c("WDPAID", "ISO3", "assetid", "biome", "geom")
  expect_true(all(vars %in% names(data)))
  expect_true(inherits(data[["biome"]], "list"))

  file.remove(dsn)

  x[["biome2"]] <- indicator
  write_portfolio(x, dsn, quiet = TRUE)
  data <- read_portfolio(dsn, quiet = TRUE)
  vars <- c("biome", "biome2")
  expect_true(all(vars %in% names(data)))

  file.remove(dsn)

  x2 <- x
  x2[["biome"]] <- list(NULL)
  x2[["biome2"]] <- list(NULL)
  x <- do.call(rbind, list(x, x2))
  x$assetid <- 1:2

  write_portfolio(x, dsn)
  data <- read_portfolio(dsn)
  expect_true(!is.null(data[["biome"]][[1]]))
  expect_true(!is.null(data[["biome2"]][[1]]))
  expect_true(is.null(data[["biome"]][[2]]))
  expect_true(is.null(data[["biome2"]][[2]]))

  file.remove(dsn)

  x$biome2 <- list(c(NULL, NULL))
  expect_warning(write_portfolio(x, dsn), "Dropping")
  data <- read_portfolio(dsn)
  expect_true(!"biome2" %in% names(data))
  suppressWarnings(inds <- read_sf(dsn, "indicators"))
  # check NULL was not writte
  expect_equal(nrow(inds), 1)
})

test_that(".check_portfolio works as expected", {
  x <- read_sf(
    system.file("extdata", "sierra_de_neiba_478140.gpkg",
      package = "mapme.biodiversity"
    )
  )
  p <- suppressWarnings(st_centroid(x))
  expect_error(.check_portfolio(p), "Only assets of type 'POLYGON' and 'MULTIPOLYGON' are supported.")
  expect_silent(.check_portfolio(x))

  center <- as.numeric(suppressWarnings(st_coordinates(st_centroid(x))))
  srs <- sprintf(
    "+proj=laea +x_0=0 +y_0=0 +lon_0=%s +lat_0=%s",
    center[1], center[2]
  )
  x <- st_transform(x, srs)

  expect_message(
    x <- .check_portfolio(x),
    "CRS of x is not EPSG:4326. Attempting to transform."
  )
  mapme_options(verbose = TRUE)
  expect_silent(.check_portfolio(x))
  x$assetid <- NULL
  expect_silent(x <- .check_portfolio(x))
  expect_true("assetid" %in% names(x))
  x <- do.call(rbind, lapply(1:2, function(y) x))
  expect_message(.check_portfolio(x), "Found a column named 'assetid' with non-unique identifiers")
})

test_that("portfolio helpers work as expected", {
  mapme_options(verbose = FALSE)
  x <- read_sf(
    system.file("extdata", "sierra_de_neiba_478140.gpkg",
      package = "mapme.biodiversity"
    )
  )
  x <- suppressWarnings(st_cast(x, to = "POLYGON")[1, ])
  x[["assetid"]] <- 1

  expect_error(.indicators_col(x))

  indicator <- list(tibble(
    datetime = "2000-01-01",
    variable = "biome",
    unit = "ha",
    value = 1
  ))

  x[["biome"]] <- indicator
  x[["biome2"]] <- indicator

  cols <- .indicators_col(x)
  expected <- 5:6
  names(expected) <- c("biome", "biome2")
  expect_equal(cols, expected)

  vars <- c("indicator", "datetime", "variable", "unit", "value")
  out <- portfolio_long(x, names(cols))
  expect_equal(nrow(out), 2)
  expect_true(inherits(out, "sf"))
  expect_true(all(vars %in% names(out)))
  out <- portfolio_long(x, names(cols)[1], drop_geoms = TRUE)
  expect_equal(nrow(out), 1)
  expect_false(inherits(out, "sf"))
  expect_true(all(vars %in% names(out)))

  out <- portfolio_wide(x, names(cols))
  expect_equal(nrow(out), 1)
  vars <- c("assetid", "biome_2000-01-01_biome_ha", "biome2_2000-01-01_biome_ha")
  expect_true(all(vars %in% names(out)))
  expect_true(inherits(out, "sf"))
  out <- portfolio_wide(x, names(cols), drop_geoms = TRUE)
  expect_equal(nrow(out), 1)
  expect_true(all(vars %in% names(out)))
  expect_false(inherits(out, "sf"))
  out <- portfolio_wide(x, names(cols)[1])
  expect_equal(nrow(out), 1)
  expect_true(inherits(out, "sf"))
  expect_true(all(c(vars[1:2], "biome2") %in% names(out)))

  x <- rbind(x, x)
  x$assetid <- 1:2
  x$biome2 <- list(x$biome2[[1]], NULL)
  expect_silent(out <- portfolio_wide(x))
  expect_equal(nrow(out), 2)
  expect_equal(out$`biome_2000-01-01_biome_ha`[2], 1)
  expect_true(is.na(out$`biome2_2000-01-01_biome_ha`[2]))
  expect_silent(out <- portfolio_long(x))
  expect_equal(nrow(out), 3)
  expect_equal(out$value, c(1, 1, 1))

  x$biome <- list(NULL)
  expect_silent(portfolio_long(x))
  expect_silent(portfolio_wide(x))
  x$biome2 <- list(NULL)
  expect_warning(portfolio_long(x), "All indicator columns contained 'NULL'.")
  expect_warning(portfolio_wide(x), "All indicator columns contained 'NULL'.")
})

#' UCDP Georeferenced Event Dataset (UCDP GED)
#'
#' This resource distributed by the Uppsala Conflict Data Program (UCDP) constitutes
#' its most disaggregated dataset on individual events of organized violence.
#' It encodes the different actors involved, is spatially disaggregated down
#' to village levels anc currently covers the time period of 1989 to 2021.
#' Older versions of the data set can be downloaded, but users are recommended
#' to download the latest data set.
#'
#' The following versions are available:
#' - 5.0
#' - 17.1
#' - 17.2
#' - 18.1
#' - 19.1
#' - 20.1
#' - 21.1
#' - 22.1
#' - 23.1
#' - 24.1
#' - latest
#'
#' @name ucdp_ged
#' @param version A character vector specifying
#'   the version to download. Defaults to "latest".
#' @keywords resource
#' @returns A function that returns an `sf` footprint object.
#' @references Davies, Shawn, Therese Pettersson & Magnus Öberg (2022).
#' Organized violence 1989-2021 and drone warfare.
#' Journal of Peace Research 59(4).
#' \doi{10.1177/00223433221108428}
#' @source \url{https://ucdp.uu.se/downloads/}
#' @include register.R
#' @export
get_ucdp_ged <- function(version = "latest") {
  check_namespace("rvest")
  try(versions <- .ucdp_versions())
  if (inherits(versions, "try-error")) {
    stop("Available versions of UCDP GED could not be fetched")
  }
  if (version == "latest") version <- versions[length(versions)]
  if (!version %in% versions) {
    msg <- paste(versions, collapse = ", ")
    stop(paste0("Valid versions for UCDP GED: ", msg, "."))
  }

  function(x,
           name = "ucdp_ged",
           type = "vector",
           outdir = mapme_options()[["outdir"]],
           verbose = mapme_options()[["verbose"]]) {
    version_ged <- paste0("ged", gsub("\\.", "", version), "-csv.zip")

    base_url <- "/vsizip//vsicurl/https://ucdp.uu.se/downloads/ged/"
    url <- paste0(base_url, version_ged)
    switch(version,
      "19.1" = {
        url <- paste0(url, "/ged191.csv")
      },
      "5.0" = {
        url <- paste0(url, "/ged50.csv")
      }
    )

    bbox <- c(xmin = -180.0, ymin = -90.0, xmax = 180.0, ymax = 90.0)
    fps <- st_as_sf(st_as_sfc(st_bbox(bbox, crs = "EPSG:4326")))
    fps[["source"]] <- url

    make_footprints(
      fps,
      filenames = gsub("zip", "gpkg", version_ged),
      what = "vector", oo = c("-oo", "GEOM_POSSIBLE_NAMES=geom_wkt", "-s_srs", "EPSG:4326", "-t_srs", "EPSG:4326")
    )
  }
}


.ucdp_versions <- function() {
  c("5.0", "17.1", "17.2", "18.1", "19.1", "20.1", "21.1", "22.1", "23.1", "24.1")
}

register_resource(
  name = "ucdp_ged",
  description = "UCDP Georeferenced Event Dataset (UCDP GED)",
  licence = "CC-BY 4.0",
  source = "https://ucdp.uu.se/downloads/",
  type = "vector"
)

#' Global Surface Water Change
#'
#' The Global Surface Water dataset was developed by the European Commission's
#' Joint Research Centre in the framework of the Copernicus Programme. It maps
#' the location and temporal distribution of water surfaces at the global scale
#' over the past 3.8 decades and provides statistics on their extent and change.
#' It is provisioned as a global tiled raster resource available for all land
#' areas. The reported data represent aggregated observations between 1984 - 2021.
#'
#' The change in water occurrence intensity between the two periods is derived
#' from homologous pairs of months (i.e. same months containing valid
#' observations in both periods). The difference in the occurrence of surface
#' water was calculated for each homologous pair of months. The average of all
#' of these differences constitutes the Surface Water Occurrence change
#' intensity. The raster files have integer cell values between \code{[0, 200]}
#' where 0 represents surface water loss and 200 represents surface water gain.
#'
#' @name global_surface_water_change
#' @param version A character vector indicating the version of the GSW data set
#'   to make available.
#' @keywords resource
#' @returns A function that returns an `sf` footprint object.
#' @references Pekel, JF., Cottam, A., Gorelick, N. et al. High-resolution
#' mapping of global surface water and its long-term changes. Nature 540,
#' 418–422 (2016). \doi{doi:10.1038/nature20584}
#' @source \url{https://global-surface-water.appspot.com/}
#' @include register.R
#' @export
get_global_surface_water_change <- function(version = "v1_4_2021") {
  stopifnot(version %in% .gsw_versions)

  function(x,
           name = "global_surface_water_change",
           type = "raster",
           outdir = mapme_options()[["outdir"]],
           verbose = mapme_options()[["verbose"]]) {
    .get_gsw(x, statistic = "change", version = version)
  }
}
#' Global Surface Water Transitions
#'
#' The Global Surface Water dataset was developed by the European Commission's
#' Joint Research Centre in the framework of the Copernicus Programme. It maps
#' the location and temporal distribution of water surfaces at the global scale
#' over the past 3.8 decades and provides statistics on their extent and change.
#' It is provisioned as a global tiled raster resource available for all land
#' areas. The reported data represent aggregated observations between 1984 - 2021.
#'
#' GSW transition data contains information about the type of surface water
#' change for each pixel. The raster files have integer cell values between
#' \code{[0, 10]} that code for different transition classes:
#'
#' | Value | Transition Class      |
#' |-------|-----------------------|
#' | 1     | Permanent             |
#' | 2     | New Permanent         |
#' | 3     | Lost Permanent        |
#' | 4     | Seasonal              |
#' | 5     | New Seasonal          |
#' | 6     | Lost Seasonal         |
#' | 7     | Seasonal to Permanent |
#' | 8     | Permanent to Seasonal |
#' | 9     | Ephemeral Permanent   |
#' | 10    | Ephemeral Seasonal    |
#'
#' @name global_surface_water_transitions
#' @param version A character vector indicating the version of the GSW data set
#'   to make available.
#' @keywords resource
#' @returns  A character of file paths.
#' @references Pekel, JF., Cottam, A., Gorelick, N. et al. High-resolution
#' mapping of global surface water and its long-term changes. Nature 540,
#' 418–422 (2016). \doi{doi:10.1038/nature20584}
#' @source \url{https://global-surface-water.appspot.com/}
#' @include register.R
#' @export
get_global_surface_water_transitions <- function(version = "v1_4_2021") {
  stopifnot(version %in% .gsw_versions)

  function(x,
           name = "global_surface_water_transitions",
           type = "raster",
           outdir = mapme_options()[["outdir"]],
           verbose = mapme_options()[["verbose"]]) {
    .get_gsw(x, statistic = "transitions", version = version)
  }
}

#' Global Surface Water Seasonality
#'
#' The Global Surface Water dataset was developed by the European Commission's
#' Joint Research Centre in the framework of the Copernicus Programme. It maps
#' the location and temporal distribution of water surfaces at the global scale
#' over the past 3.8 decades and provides statistics on their extent and change.
#' It is provisioned as a global tiled raster resource available for all land
#' areas. The reported data represent aggregated observations between 1984 - 2021.
#'
#' GSW seasonality describes the intra-annual distribution of surface water for
#' each pixel. The raster files have integer cell values between \code{[0, 12]},
#' indicating how many months per year the pixel was classified as water.
#'
#' @name global_surface_water_seasonality
#' @param version A character vector indicating the version of the GSW data set
#'   to make available.
#' @keywords resource
#' @returns  A character of file paths.
#' @references Pekel, JF., Cottam, A., Gorelick, N. et al. High-resolution
#' mapping of global surface water and its long-term changes. Nature 540,
#' 418–422 (2016). \doi{doi:10.1038/nature20584}
#' @source \url{https://global-surface-water.appspot.com/}
#' @include register.R
#' @export
get_global_surface_water_seasonality <- function(version = "v1_4_2021") {
  stopifnot(version %in% .gsw_versions)

  function(x,
           name = "global_surface_water_seasonality",
           type = "raster",
           outdir = mapme_options()[["outdir"]],
           verbose = mapme_options()[["verbose"]]) {
    .get_gsw(x, statistic = "seasonality", version = version)
  }
}

#' Global Surface Water Recurrence
#'
#' The Global Surface Water dataset was developed by the European Commission's
#' Joint Research Centre in the framework of the Copernicus Programme. It maps
#' the location and temporal distribution of water surfaces at the global scale
#' over the past 3.8 decades and provides statistics on their extent and change.
#' It is provisioned as a global tiled raster resource available for all land
#' areas. The reported data represent aggregated observations between 1984 - 2021.
#'
#' Water Recurrence is a measurement of the degree of variability in the
#' presence of water from year to year. It describes the frequency with which
#' water returned to a particular location from one year to another, and is
#' expressed as a percentage. The raster files have integer cell values between
#' \code{[0, 100]}, where 100 represents that water reoccurs predictably every
#' year, whereas lower values indicate that water only occurs episodically.
#'
#' @name global_surface_water_recurrence
#' @param version A character vector indicating the version of the GSW data set
#'   to make available.
#' @keywords resource
#' @returns  A character of file paths.
#' @references Pekel, JF., Cottam, A., Gorelick, N. et al. High-resolution
#' mapping of global surface water and its long-term changes. Nature 540,
#' 418–422 (2016). \doi{doi:10.1038/nature20584}
#' @source \url{https://global-surface-water.appspot.com/}
#' @include register.R
#' @export
get_global_surface_water_recurrence <- function(version = "v1_4_2021") {
  stopifnot(version %in% .gsw_versions)

  function(x,
           name = "global_surface_water_recurrence",
           type = "raster",
           outdir = mapme_options()[["outdir"]],
           verbose = mapme_options()[["verbose"]]) {
    .get_gsw(x, statistic = "recurrence", version = version)
  }
}

#' Global Surface Water Occurrence
#'
#' The Global Surface Water dataset was developed by the European Commission's
#' Joint Research Centre in the framework of the Copernicus Programme. It maps
#' the location and temporal distribution of water surfaces at the global scale
#' over the past 3.8 decades and provides statistics on their extent and change.
#' It is provisioned as a global tiled raster resource available for all land
#' areas. The reported data represent aggregated observations between 1984 - 2021.
#'
#' GSW occurrence raw data comes in raster files with integer cell values
#' between \code{[0, 100]}. This value gives the percentage of the time that a
#' given pixel was classified as water during the entire observation period. So
#' a 0 denotes a pixel that was never classified as water, 100 denotes a pixel
#' with permanent water.
#'
#' @name global_surface_water_occurrence
#' @param version A character vector indicating the version of the GSW data set
#'   to make available.
#' @keywords resource
#' @returns  A character of file paths.
#' @references Pekel, JF., Cottam, A., Gorelick, N. et al. High-resolution
#' mapping of global surface water and its long-term changes. Nature 540,
#' 418–422 (2016). \doi{doi:10.1038/nature20584}
#' @source \url{https://global-surface-water.appspot.com/}
#' @include register.R
#' @export
get_global_surface_water_occurrence <- function(version = "v1_4_2021") {
  stopifnot(version %in% .gsw_versions)

  function(x,
           name = "global_surface_water_occurrence",
           type = "raster",
           outdir = mapme_options()[["outdir"]],
           verbose = mapme_options()[["verbose"]]) {
    .get_gsw(x, statistic = "occurrence", version = version)
  }
}

.get_gsw <- function(x, statistic = "occurrence", version = "v1_4_2021") {
  stopifnot(
    statistic %in% .gsw_statistics,
    version %in% .gsw_versions
  )

  # make the gsw grid and construct urls for intersecting tiles
  baseurl <- sprintf(
    "/vsicurl/https://storage.googleapis.com/global-surface-water/downloads2021/%s/%s",
    statistic, statistic
  )
  grid_gfc <- make_global_grid(
    xmin = -180, xmax = 170, dx = 10,
    ymin = -50, ymax = 80, dy = 10
  )
  tile_ids <- unique(unlist(st_intersects(x, grid_gfc)))
  if (length(tile_ids) == 0) {
    stop("The extent of the portfolio does not intersect with the GSW grid.",
      call. = FALSE
    )
  }
  ids <- sapply(tile_ids, function(n) .get_gsw_tile_id(grid_gfc[n, ]))
  urls <- sprintf("%s_%s%s.tif", baseurl, ids, version)

  fps <- grid_gfc[tile_ids, ]
  fps[["source"]] <- urls

  make_footprints(fps, what = "raster", co = c("-co", "COMPRESS=LZW", "-ot", "Byte"))
}

.get_gsw_tile_id <- function(tile) {
  min_x <- st_bbox(tile)[1]
  max_y <- st_bbox(tile)[4]
  ndigits_x <- nchar(abs(min_x))
  ndigits_y <- nchar(abs(max_y))
  formatstr_x <- paste0("%0", ndigits_x, "i")
  formatstr_y <- paste0("%0", ndigits_y, "i")
  if (min_x < 0) {
    min_x <- paste0(sprintf(formatstr_x, abs(min_x)), "W")
  } else {
    min_x <- paste0(sprintf(formatstr_x, abs(min_x)), "E")
  }
  if (max_y < 0) {
    max_y <- paste0(sprintf(formatstr_y, abs(max_y)), "S")
  } else {
    max_y <- paste0(sprintf(formatstr_y, abs(max_y)), "N")
  }
  paste0(min_x, "_", max_y)
}

.gsw_statistics <- c(
  "change",
  "occurrence",
  "recurrence",
  "seasonality",
  "transitions"
)

.gsw_versions <- c(
  "v1_4_2021"
)

.gsw_descr <- data.frame(
  name = .gsw_statistics,
  desc = c(
    "Global Surface Water - Change of water occurrence intensity",
    "Global Surface Water - Percentage of water occurrence",
    "Global Surface Water - Percentage of water recurrence",
    "Global Surface Water - Seasonality of water occurrence",
    "Global Surface Water - Transition classes"
  )
)

for (gsw_statistic in .gsw_statistics) {
  resource_name <- paste0("global_surface_water_", gsw_statistic)
  register_resource(
    name = resource_name,
    description = .gsw_descr$desc[which(.gsw_descr$name == gsw_statistic)],
    licence = "https://www.copernicus.eu/en/access-data",
    source = "https://global-surface-water.appspot.com/download",
    type = "raster"
  )
}

test_that(".get_gsw works", {
  skip_on_cran()

  x <- read_sf(
    system.file("extdata", "sierra_de_neiba_478140.gpkg",
      package = "mapme.biodiversity"
    )
  )

  expect_error(.get_gsw(x, statistic = "not-available"))
  expect_error(.get_gsw(x, version = "not-available"))

  gsw <- get_global_surface_water_change()
  expect_silent(.check_resource_fun(gsw))
  expect_silent(fps <- gsw(x))
  expect_silent(.check_footprints(fps))
  expect_equal(fps$filename, "change_80W_20Nv1_4_2021.tif")

  gsw <- get_global_surface_water_transitions()
  expect_silent(.check_resource_fun(gsw))
  expect_silent(fps <- gsw(x))
  expect_silent(.check_footprints(fps))
  expect_equal(fps$filename, "transitions_80W_20Nv1_4_2021.tif")

  gsw <- get_global_surface_water_seasonality()
  expect_silent(.check_resource_fun(gsw))
  expect_silent(fps <- gsw(x))
  expect_silent(.check_footprints(fps))
  expect_equal(fps$filename, "seasonality_80W_20Nv1_4_2021.tif")

  gsw <- get_global_surface_water_recurrence()
  expect_silent(.check_resource_fun(gsw))
  expect_silent(fps <- gsw(x))
  expect_silent(.check_footprints(fps))
  expect_equal(fps$filename, "recurrence_80W_20Nv1_4_2021.tif")

  gsw <- get_global_surface_water_occurrence()
  expect_silent(.check_resource_fun(gsw))
  expect_silent(fps <- gsw(x))
  expect_silent(.check_footprints(fps))
  expect_equal(fps$filename, "occurrence_80W_20Nv1_4_2021.tif")

  # adds test to check for multiple polygons in the same tile
  x_split <- st_as_sf(st_make_grid(x, n = 2))
  expect_silent(fps <- .get_gsw(x_split, statistic = "occurrence"))
  expect_equal(nrow(fps), 1)
})

#' Calculate precipitation average based on CHELSA
#'
#' This functions allows to calculate averaged precipitation
#' from the CHELSA downscaled precipitation layers. Based on
#' user-selected years, monthly averages of precipitation are calculated.
#'
#' The required resources for this indicator are:
#'  - [chelsa]
#'
#' @name precipitation_chelsa
#' @param years A numeric vector indicating the years for which to calculate
#'   precipitation statistics.
#' @param engine The preferred processing functions from either one of "zonal",
#'   "extract" or "exactextract" as character.
#' @keywords indicator
#' @returns A function that returns an indicator tibble with variable
#'   precipitation and sum of precipitation (in mm/m^2) as value.
#' @include register.R
#' @export
#' @examples
#' \dontshow{
#' mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
#' }
#' \dontrun{
#' library(sf)
#' library(mapme.biodiversity)
#'
#' outdir <- file.path(tempdir(), "mapme-data")
#' dir.create(outdir, showWarnings = FALSE)
#'
#' mapme_options(
#'   outdir = outdir,
#'   verbose = FALSE
#' )
#'
#' aoi <- system.file("extdata", "sierra_de_neiba_478140_2.gpkg",
#'   package = "mapme.biodiversity"
#' ) %>%
#'   read_sf() %>%
#'   get_resources(get_chelsa(years = 2010)) %>%
#'   calc_indicators(
#'     calc_precipitation_chelsa(
#'       years = 2010,
#'       engine = "extract"
#'     )
#'   ) %>%
#'   portfolio_long()
#'
#' aoi
#' }
calc_precipitation_chelsa <- function(years = 1979:2018,
                                      engine = "extract") {
  engine <- check_engine(engine)
  avail_years <- 1979:2018
  years <- check_available_years(years, avail_years, "precipitation_chelsa")

  function(x,
           chelsa,
           name = "precipitation_chelsa",
           mode = "asset",
           aggregation = "mean",
           verbose = mapme_options()[["verbose"]]) {
    if (is.null(chelsa)) {
      return(NULL)
    }

    if (length(years) == 0) {
      return(NULL)
    }

    dates <- substr(names(chelsa), 11, 17)
    dates <- as.Date(paste0("01_", dates), "%d_%m_%Y")
    datetime <- as.POSIXct(paste0(dates, "T00:00:00Z"))

    layer_years <- as.numeric(substr(names(chelsa), 14, 17))
    chelsa <- chelsa[[which(layer_years %in% years)]]

    # extract zonal statistics
    stats <- select_engine(
      x = x,
      raster = chelsa / 100,
      stats = "mean",
      engine = engine,
      mode = "asset"
    )

    tibble(
      datetime = datetime,
      variable = "precipitation",
      unit = "mm/m^2",
      value = as.numeric(stats[["mean"]])
    )
  }
}


register_indicator(
  name = "precipitation_chelsa",
  description = "Statistics of CHELSA precipitation layer",
  resources = "chelsa"
)

#' Helper to create a grid of regular resolution and CRS
#'
#' Use this function to create a regular grid in a custom CRS. This is used
#' e.g. to create the tile grid for Global Forest Watch in order to retrieve
#' the intersecting tiles with a given portfolio.
#'
#' @param xmin minimum longitude value (E/W)
#' @param xmax maximum longitude value (E/W)
#' @param ymin minimum latitude value (S/N)
#' @param ymax maximum latitude value (E/W)
#' @param dx difference in longitude value per grid
#' @param dy difference in latitude value per grid
#' @param proj projection system
#' @returns An sf object with a defined grid.
#' @keywords utils
#' @export
make_global_grid <- function(xmin = -180, xmax = 170, dx = 10,
                             ymin = -50, ymax = 80, dy = 10,
                             proj = NULL) {
  if (is.null(proj)) proj <- st_crs(4326)
  ncells <- c(
    (xmax - xmin) / dx,
    (ymax - ymin) / dy
  )

  bbox <- st_bbox(c(xmin = xmin, xmax = xmax, ymax = ymax, ymin = ymin))
  bbox <- st_as_sfc(bbox, crs = proj)
  st_as_sf(st_make_grid(bbox, n = ncells, crs = proj, what = "polygons"))
}

#' Helper to check yearly availability
#'
#' Use this function to check if a specified vector of years intersects
#' with the yearly availability of a resource.
#'
#' @param target_years Numeric indicating the target year.
#' @param available_years Numeric indicating the available years.
#' @param indicator A character vector with target resource/indicator name.
#' @keywords utils
#' @export
check_available_years <- function(target_years,
                                  available_years,
                                  indicator) {
  if (any(!target_years %in% available_years)) {
    target_years <- target_years[target_years %in% available_years]
    if (length(target_years) > 0) {
      message(sprintf("Some target years are not available for %s.", indicator))
    } else {
      stop(
        sprintf(
          "The target years do not intersect with the availability of %s.",
          indicator
        )
      )
    }
  }
  target_years
}

#' Checks if namespace is available
#'
#' Use this function if your resource/indicator function requires the
#' namespace of a certain package to be available. An informative error/warning
#' message is printed if that is not the case.
#'
#' @param pkg A character vector of length one indicating a package name
#'   for which the namespace is tested
#' @param error A logical indicating whether or not to promote missing namespace
#'   to error. If FALSE, a warning is emitted.
#'
#' @return TRUE, invisible, if the namespace is available. An error message
#'   if `error = TRUE`, FALSE and a warning otherwise.
#' @keywords utils
#' @export
check_namespace <- function(pkg, error = TRUE) {
  verb <- ifelse(error, "required", "recommended")
  if (!requireNamespace(pkg, quietly = TRUE)) {
    msg <- paste("R package '%s' is %s.\n",
      "Please install via `install.packages('%s')`",
      sep = ""
    )
    msg <- sprintf(msg, pkg, verb, pkg)
    if (error) {
      stop(msg, call. = FALSE)
    } else {
      message(msg)
      return(invisible(FALSE))
    }
  }
  invisible(TRUE)
}

#' @importFrom curl has_internet
.has_internet <- function() {
  if (Sys.getenv("mapme_check_connection", unset = "TRUE") == "FALSE") {
    return(TRUE)
  }
  # The has_internet function tests for internet connectivity by performing a dns lookup.
  # If a proxy server is detected, it will also check for connectivity by connecting via the proxy.
  #
  # don't use this ad-hoc code
  # rsp <- httr2::req_perform(httr2::request("www.google.com"))
  # has_internet <- !httr2::resp_is_error(rsp)
  # delegate to curl::has_internet
  has_internet <- curl::has_internet()
  if (!has_internet) {
    message("There seems to be no internet connection. Cannot download resources.")
  }
  return(has_internet)
}

.geom_last <- function(data) {
  stopifnot(inherits(data, "sf"))
  sf_col <- attr(data, "sf_column")
  dplyr::relocate(data, !!sf_col,
    .after = dplyr::last_col()
  )
}

test_that("make_global_grid works", {
  standard_grid <- make_global_grid()
  nrows <- nrow(standard_grid)
  bbox <- st_bbox(standard_grid)
  points <- st_multipoint(matrix(c(-180, -50, -180, 80, 170, 80, 170, -50), ncol = 2, byrow = TRUE))
  bbox_c <- st_bbox(points)
  st_crs(bbox_c) <- st_crs("EPSG:4326")
  expect_equal(nrows, 455)
  expect_equal(bbox, bbox_c)
})

test_that(".check_available_years works", {
  expect_equal(
    check_available_years(2000:2020, 2000:2020, indicator = "treecover_area"),
    2000:2020
  )
  expect_equal(
    check_available_years(2000:2010, 2000:2020, indicator = "treecover_area"),
    2000:2010
  )
  expect_error(
    check_available_years(2000:2010, 2011:2020, indicator = "treecover_area"),
    "The target years do not intersect with the availability of treecover."
  )
  expect_message(
    out <- check_available_years(2000:2011, 2011:2020, indicator = "treecover_area"),
    "Some target years are not available for treecover."
  )
  expect_equal(out, 2011)
})

test_that(".check_namespace works", {
  expect_error(check_namespace("not-a-package"))
  expect_message(check_namespace("not-a-package", error = FALSE))
  expect_silent(check_namespace("base"))
})

#' Terrestrial Ecoregions of the World (TEOW) Polygon
#'
#' This resource is part of the publication by Olson et al. (2004)
#' "Terrestrial Ecosystems of the World (TEOW) from WWF-US (Olson)". It
#' depicts 867 terrestrial ecoregions around the world classified into 14
#' different terrestrial biomes such as forests, grasslands, or deserts.
#' The polygons represent the ecoregions, defined as relatively large units of
#' land or inland water sharing a large majority of biodiversity. The datasets
#' is made available from World Wildlife Fund (WWF) for the year 2001.
#'
#' @name teow
#' @keywords resource
#' @param path A character vector to the Terrestrial Ecoregions of the World (TEOW)
#'   zip file. Note, that the file has to be downloaded manually.
#' @returns  A function that returns an `sf` footprint object.
#' @references Olson, D. M., Dinerstein, E., Wikramanayake, E. D., Burgess, N. D.,
#' Powell, G. V. N., Underwood, E. C., D’Amico, J. A., Itoua, I., Strand, H. E.,
#' Morrison, J. C., Loucks, C. J., Allnutt, T. F., Ricketts, T. H., Kura, Y., Lamoreux,
#' J. F., Wettengel, W. W., Hedao, P., Kassem, K. R. 2001. Terrestrial ecoregions of
#' the world: a new map of life on Earth. Bioscience 51(11):933-938.
#' \doi{doi:10.1641/0006-3568(2001)051[0933:TEOTWA]2.0.CO;2}
#' @source \url{https://files.worldwildlife.org/wwfcmsprod/files/Publication/file/6kcchn7e3u_official_teow.zip}
#' @include register.R
#' @export
# get_teow <- function() {
#   function(x,
#            name = "teow",
#            type = "vector",
#            outdir = mapme_options()[["outdir"]],
#            verbose = mapme_options()[["verbose"]]) {
#     url <- paste(
#       "/vsizip//vsicurl/",
#       "https://files.worldwildlife.org/wwfcmsprod/files/",
#       "Publication/file/6kcchn7e3u_official_teow.zip/",
#       "official/wwf_terr_ecos.shp",
#       sep = ""
#     )
#     bbox <- c(xmin = -180.0, ymin = -90.0, xmax = 180.0, ymax = 84.0)
#     tile <- st_as_sf(st_as_sfc(st_bbox(bbox, crs = "EPSG:4326")))
#     tile[["source"]] <- url
#     make_footprints(tile, filenames = "wwf_terr_ecos.gpkg", what = "vector")
#   }
# }

get_teow <- function(path = NULL) {
  if (is.null(path) || !endsWith(tolower(path), ".zip")) {
    stop("Expecting path to point towards an existing '.zip' file.")
  }

  function(x,
           name = "teow",
           type = "vector",
           outdir = mapme_options()[["outdir"]],
           verbose = mapme_options()[["verbose"]]) {
    url <- paste0(
      "/vsizip/",
      path,
      "/official/wwf_terr_ecos.shp"
    )
    bbox <- c(xmin = -180.0, ymin = -90.0, xmax = 180.0, ymax = 84.0)
    tile <- sf::st_as_sf(sf::st_as_sfc(sf::st_bbox(bbox, crs = "EPSG:4326")))
    tile[["source"]] <- url
    make_footprints(tile, filenames = "wwf_terr_ecos.gpkg", what = "vector")
  }
}

register_resource(
  name = "teow",
  description = "Terrestrial Ecosystems of the World (TEOW) from WWF-US",
  licence = "unknown",
  source = "https://www.worldwildlife.org/publications/terrestrial-ecoregions-of-the-world",
  type = "vector"
)

#' ESA Copernicus Global Land Cover layer
#'
#' This 100 meter spatial resolution land cover resource is published by Buchhorn
#' et al. (2020) "Copernicus Global Land Cover Layers—Collection 2". The resource
#' represents the actual surface cover of ground available annually for the period
#' 2015 to 2019. The cell values range from 0 to 200, representing total of 23
#' discrete classifications from ESA.
#'
#' @name esalandcover
#' @param years A numeric vector indicating the years for which to make the
#'   resource available.
#' @keywords resource
#' @returns A function that returns an `sf` footprint object.
#' @details
#' This function mimics the behavior of the original `get_esalandcover()` function by
#' downloading the parts of the global raster that correspond to the 20° x 20° tiles used
#' previously. The files are named the same as before. This allows other code that uses
#' this resource to remain unchanged.
#' @references Buchhorn, M., Lesiv, M., Tsendbazar, N.-E., Herold, M., Bertels, L.,
#' & Smets, B. (2020). Copernicus Global Land Cover Layers - Collection 2.
#' Remote Sensing, 12(6), 1044.
#' \doi{doi:10.3390/rs12061044}
#' @source \url{https://zenodo.org/records/3939050}
#' @include register.R
#' @export
get_esalandcover <- function(years = 2015L:2019L) {
  years <- check_available_years(years, c(2015L:2019L), "esalandcover")

  function(x,
           name = "esalandcover",
           type = "raster",
           outdir = mapme_options()[["outdir"]],
           verbose = mapme_options()[["verbose"]]) {

    # make the ESA grid and construct urls for intersecting tiles
    grid_esa <- make_global_grid(
      xmin = -180.0, xmax = 180.0, dx = 20.0,
      ymin = -60.0, ymax = 80.0, dy = 20.0
    )
    # find requoired (intersecting) tiles
    tile_ids <- unique(unlist(sf::st_intersects(x, grid_esa)))
    if (length(tile_ids) == 0L) {
      stop(paste("The extent of the portfolio does not ",
        "intersect with the Land Cover grid.",
        sep = ""
      ))
    }
    # create all urls for target years and per tile
    fps <- purrr::map(tile_ids, function(id) {
      urls <- purrr::map_chr(years, .get_esa_url)
      filenames <- purrr::map_chr(years, .get_esa_filename, tile = grid_esa[id, ])
      # options will define the part of global raster to retrieve
      # options <- purrr::map(years, .get_esa_option, tile = grid_esa[id, ])
      options <- rep(.get_esa_option(tile = grid_esa[id, ]), length.out = length(urls))
      fp <- sf::st_as_sf(rep(sf::st_as_sfc(grid_esa[id, ]), length(urls)))
      fp[["source"]] <- urls
      fp[["filename"]] <- filenames
      fp[["oo"]] <- options
      fp
    })

    fps <- sf::st_as_sf(purrr::list_rbind(fps))
    co <- c("-of", "COG", "-co", "COMPRESS=DEFLATE")

    fps <- make_footprints(fps, filenames = fps[["filename"]], what = "raster", co = co, oo = fps[["oo"]])
    if (length(fps) == 0L) {
      return(NULL)
    }
    fps
  }
}

#' Helper function to create ESA land cover urls
#'
#' @param year A single numeric value indicating the target year
#'
#' @return A character vector
#' @keywords internal
#' @noRd
.get_esa_url <- function(year) {
  urls <- c(
    "/vsicurl/https://zenodo.org/records/3939038/files/PROBAV_LC100_global_v3.0.1_2015-base_Discrete-Classification-map_EPSG-4326.tif",
    "/vsicurl/https://zenodo.org/records/3518026/files/PROBAV_LC100_global_v3.0.1_2016-conso_Discrete-Classification-map_EPSG-4326.tif",
    "/vsicurl/https://zenodo.org/records/3518036/files/PROBAV_LC100_global_v3.0.1_2017-conso_Discrete-Classification-map_EPSG-4326.tif",
    "/vsicurl/https://zenodo.org/records/3518038/files/PROBAV_LC100_global_v3.0.1_2018-conso_Discrete-Classification-map_EPSG-4326.tif",
    "/vsicurl/https://zenodo.org/records/3939050/files/PROBAV_LC100_global_v3.0.1_2019-nrt_Discrete-Classification-map_EPSG-4326.tif"
  )
  return(urls[match(year, 2015L:2019L)])
}

#' Helper function to create ESA land cover file names
#'
#' @param tile An sf object representing the spatial extent of the a tile
#' @param year A single numeric value indicating the target year
#'
#' @return A character vector
#' @keywords internal
#' @noRd
.get_esa_filename <- function(tile, year) {
  min_x <- st_bbox(tile)[1]
  max_y <- st_bbox(tile)[4]
  # prepare tile names
  if (min_x < 0) {
    min_x <- paste0("W", sprintf("%03i", abs(min_x)))
  } else {
    min_x <- paste0("E", sprintf("%03i", min_x))
  }
  if (max_y < 0) {
    max_y <- paste0("S", sprintf("%02i", abs(max_y)))
  } else {
    max_y <- paste0("N", sprintf("%02i", max_y))
  }

  grid <- paste0(min_x, max_y)
  filename <- sprintf("%s_LC100_v3.0.1_%s.tif", grid, year)
  return(filename)
}

#' Helper function to create ESA land cover GDAL options
#'
#' @param tile An sf object representing the spatial extent of the a tile
#'
#' @return A character vector
#' @keywords internal
#' @noRd
.get_esa_option <- function(tile) {
  bb <- sf::st_bbox(tile)
  ext <- sprintf("%f", as.numeric(bb[c("xmin", "ymax", "xmax", "ymin")]))
  opt <- c("-projwin", ext)
  return(list(opt))
}

register_resource(
  name = "esalandcover",
  licence = "CC-BY 4.0",
  description = "Copernicus Land Monitoring Service (CLMS) 100 meter land cover product",
  source = "https://registry.opendata.aws/esa-worldcover-vito/",
  type = "raster"
)

#' Accessibility to Cities layer
#'
#' This resource is published by Weiss et al. (2018) "A global map of travel
#' time to cities to assess inequalities in accessibility in 2015" on journal
#' nature. Accessibility is the ease with which larger cities can be reached
#' from a certain location. This resource represents the travel time to major
#' cities in the year 2015. Encoded as minutes, representing the time needed
#' to reach that particular cell from nearby city of target population range.
#' The following ranges to nearby cities are available:
#' - "5k_10k"
#' - "10k_20k"
#' - "20k_50k"
#' - "50k_100k"
#' - "100k_200k"
#' - "200k_500k"
#' - "500k_1mio"
#' - "1mio_5mio"
#' - "50k_50mio"
#' - "5k_110mio"
#' - "20k_110mio"
#' - "5mio_50mio"
#'
#' @note Note, that the 'figshare' server applies a rather restrictive rate limit
#'   thus frequently resulting in opaque error codes (see \url{https://github.com/mapme-initiative/mapme.biodiversity/issues/308}).
#'   Please set GDAL configuration options to sensible values in case
#'   you are running into this issue, e.g.: \code{Sys.setenv("GDAL_HTTP_MAX_RETRY" = "5", "GDAL_HTTP_RETRY_DELAY" = "15")}.
#'
#' @name nelson_et_al
#' @param ranges A character vector indicating one or more ranges
#'   to download.
#' @keywords resource
#' @returns A function that returns an `sf` footprint object.
#' @references Weiss, D. J., Nelson, A., Gibson, H. S., Temperley, W., Peedell,
#' S., Lieber, A., … & Gething, P. W. (2018). A global map of travel time to cities
#' to assess inequalities in accessibility in 2015. Nature, 553(7688), 333-336.
#' @source \url{https://figshare.com/articles/dataset/Travel_time_to_cities_and_ports_in_the_year_2015/7638134/3}
#' @include register.R
#' @export
get_nelson_et_al <- function(ranges = "20k_50k") {
  if (any(!ranges %in% .nelson_df$range)) {
    index <- which(!ranges %in% .nelson_df$range)
    ranges <- ranges[-index]
    if (length(ranges) == 0) {
      stop("No supoorted ranges have been specified.")
    }
  }

  function(x,
           name = "nelson_et_al",
           type = "raster",
           outdir = mapme_options()[["outdir"]],
           verbose = mapme_options()[["verbose"]]) {
    # get url for accessibility layer
    fps <- .get_traveltime_url(ranges, paste0("traveltime-", ranges, ".tif"),
      verbose = verbose
    )

    make_footprints(fps,
      filenames = fps[["filename"]], what = "raster",
      co = c("-co", "COMPRESS=LZW", "-ot", "UInt16", "-a_nodata", "65535")
    )
  }
}


.nelson_df <- data.frame(
  range = c(
    "5k_10k", "10k_20k", "20k_50k", "50k_100k", "100k_200k", "200k_500k",
    "500k_1mio", "1mio_5mio", "50k_50mio", "5k_110mio", "20k_110mio", "5mio_50mio"
  ),
  index = c(
    14189840, 14189837, 14189831, 14189825, 14189819, 14189816, 14189810,
    14189807, 14189849, 14189852, 14189843, 14189804
  )
)

.get_traveltime_url <- function(range, filenames, verbose = TRUE) {
  urls <- unlist(lapply(range, function(x) {
    index <- .nelson_df$index[.nelson_df$range == x]
    # paste0("/vsicurl/https://figshare.com/ndownloader/files/", index)
    paste0("/vsicurl/https://api.figshare.com/v2/file/download/", index)
  }))

  bbox <- c(xmin = -180., ymin = -60., xmax = 180., ymax = 85.)
  fps <- st_as_sfc(st_bbox(bbox, crs = "EPSG:4326"))
  fps <- st_as_sf(rep(fps, length(urls)))
  fps[["source"]] <- urls
  fps[["filename"]] <- filenames
  fps
}

register_resource(
  name = "nelson_et_al",
  description = "Global maps of traveltime to cities",
  licence = "CC-BY 4.0",
  source = "https://figshare.com/articles/dataset/Travel_time_to_cities_and_ports_in_the_year_2015/7638134/3",
  type = "raster"
)

#' Armed Conflict Location & Event Data (ACLED)
#'
#' From ACLED's homepage: *The Armed Conflict Location & Event Data Project
#' (ACLED) is a disaggregated data collection, analysis, and crisis mapping
#' project. ACLED collects information on the dates, actors, locations,
#' fatalities, and types of all reported political violence and protest events
#' around the world. The ACLED team conducts analysis to describe, explore, and
#' test conflict scenarios, and makes both data and analysis open for free use
#' by the public.*
#'
#' In order to access data from the ACLED API, you first must register
#' an account. Make sure you register with your institutional domain
#' (e.g., organization, university, or company) email address rather than
#' your personal email address, in order to be able to use the API,
#' as explained [here](https://acleddata.com/myacled-faqs).
#' Note that the ACLED API provides a *living database*
#' with single events being altered or removed altogether over time.
#'
#' @name acled
#' @param years A numeric vector specifying the years for which to make
#'   ACLED data available (between 1997 and today).
#' @param email Email addressed used to register with ACLED (see Details).
#' @param password Password used to register with ACLED (see Details).
#' @param accept_terms A logical indicating if you agree to abide by ACLED's terms
#'   of use. Defaults to FALSE, thus must be manually set to TRUE.
#' @keywords resource
#' @returns A function that returns an `sf` footprint object.
#' @references Raleigh, C., Kishi, R. & Linke, A. Political instability patterns
#'   are obscured by conflict dataset scope conditions, sources, and coding
#'   choices. Humanit Soc Sci Commun 10, 74 (2023).
#'   \doi{doi:10.1057/s41599-023-01559-4}
#' @source Armed Conflict Location & Event Data Project (ACLED).
#' @include register.R
#' @importFrom httr2 request req_perform resp_body_json
#' @export
get_acled <- function(
    years = 1997,
    email = Sys.getenv("ACLED_EMAIL"),
    password = Sys.getenv("ACLED_PASSWORD"),
    accept_terms = FALSE) {

  if (is.null(mapme_options()[["outdir"]])) {
    warning(paste(
      "ACLED layers must be downloaded from the source location",
      "irrespective if `outdir` was specified or not."
    ))
  }

  # check input arguments
  if (!accept_terms) {
    msg <- "Please read and agree to ACLED's Terms of Use here:\nhttps://acleddata.com/terms-and-conditions"
    stop(msg)
  } else {
    if (isTRUE(mapme_options()[["verbose"]])) {
      msg <- "You agreed to abide to ACLED's Terms of Use (https://acleddata.com/terms-and-conditions)."
      message(msg)
    }
  }
  if (is.null(email) | email == "") {
    msg <- "Please specify your email registered with ACLED."
    stop(msg)
  }
  if (is.null(password) | password == "") {
    msg <- "Please specify your password registered with ACLED."
    stop(msg)
  }
  years <- check_available_years(years, 1997:as.integer(format(Sys.Date(), "%Y")))

  function(x,
           name = "acled",
           type = "vector",
           outdir = mapme_options()[["outdir"]],
           verbose = mapme_options()[["verbose"]]) {
    # token authorization url
    .token_url <- "https://acleddata.com/oauth/token"
    acled_yearly <- purrr::map_chr(years, function(year) {
      filename <- sprintf("acled_events_%s.gpkg", year)
      # make full path to output file
      if (is.null(outdir)) {
        filename <- file.path(tempdir(), filename)
      } else {
        filename <- file.path(outdir, filename)
      }
      # if already available just return the file name
      if (spds_exists(filename)) {
        return(filename)
      }
      # start building the request
      base_url <- "https://acleddata.com/api/acled/read"
      req <- httr2::request(base_url)
      # add year as query parameter
      req <- httr2::req_url_query(req, year = year)
      # authenticate the request
      req <- httr2::req_oauth_password(
        req,
        client = httr2::oauth_client("acled", .token_url),
        username = email,
        password = password
      )
      # prepare for page iteration
      next_page <- TRUE
      page <- 1
      data_lst <- NULL
      running_count <- 0L
      # iterate pages
      while (next_page) {
        req <- httr2::req_url_query(req, page = page)
        resp <- httr2::req_perform(req)
        cnt <- httr2::resp_body_json(resp, simplifyVector = TRUE, flatten = TRUE)
        # check if we still have some data
        count <- cnt$count
        if (count == 0L) {
          next_page <- FALSE
          next
        }
        # increment running count
        running_count <- running_count + count
        # get data as data.frame
        events <- cnt$data
        # store events data.frame in list
        if (page == 1) {
          data_lst <- list(events)
          total_count <- cnt$total_count
        } else {
          data_lst[[page]] <- events
        }
        # increment page number
        page <- page + 1
      }
      # check everything OK
      if (running_count != total_count) {
        msg <- sprintf("ACLED API returned only %d events out of %d.", running_count, total_count)
        stop(msg)
      }
      if (is.null(data_lst)) {
        stop("ACLED API returned 0 events.")
      }
      # rbind list into global data.frame
      data <- purrr::list_rbind(data_lst)
      # convert to sf object
      data <- sf::st_as_sf(data,
                           coords = c("longitude", "latitude"),
                           crs = sf::st_crs("EPSG:4326"))
      # convert all non-geometry columns to character to be compatible with previous version
      # temporarily extract geometry
      geom <- sf::st_geometry(data)
      # convert non-geometry columns
      data_df <- sf::st_drop_geometry(data)
      data_df[] <- lapply(data_df, as.character)
      # reattach geometry
      data <- sf::st_sf(data_df, geom)
      # write the file and return file name
      sf::write_sf(data, filename)
      return(filename)
    })
    # make footprint
    bbox <- c(xmin = -180.0, ymin = -90.0, xmax = 180.0, ymax = 90.0)
    fps <- sf::st_as_sfc(sf::st_bbox(bbox, crs = "EPSG:4326"))
    fps <- sf::st_as_sf(rep(fps, length(acled_yearly)))
    fps[["source"]] <- acled_yearly
    fps <- make_footprints(fps, what = "vector")
  }
}

register_resource(
  name = "acled",
  description = "Armed Conflict Location & Event Data (ACLED)",
  licence = "Visit acleddata.com",
  source = "Visit acleddata.com",
  type = "vector"
)

test_that("get_acled works", {
  expect_error(get_acled(accept_terms = FALSE), "Terms of Use")
  expect_error(expect_message(get_acled(accept_terms = TRUE, email = "")), "email")
  expect_error(expect_message(get_acled(accept_terms = TRUE, email = "mymail")), "password")
  Sys.setenv(ACLED_EMAIL = "my-mail")
  Sys.setenv(ACLED_PASSWORD = "my-key")
  expect_no_error(get_acled(accept_terms = TRUE))
  Sys.setenv(ACLED_EMAIL = "")
  Sys.setenv(ACLED_PASSWORD = "")
})

available_stats <- c("mean", "median", "sd", "min", "max", "sum", "var")
available_engines <- c("zonal", "extract", "exactextract")

#' Function to select processing engines
#'
#' `check_engine()` checks if an extraction engine for zonal vector-raster
#' operations is supported by the backend.
#'
#' @param queried_engine A character vector of length one indicating the engine
#'   to check for.
#'
#' @return `check_engine()` returns the character of the queried engine,
#'  if supported. Throws an error otherwise.
#' @keywords utils
#' @name engine
#' @export
check_engine <- function(queried_engine) {
  if (length(queried_engine) > 1) {
    stop(sprintf(
      "Please specify only one engine of: %s.",
      paste(available_engines, collapse = ", ")
    ))
  }
  if (!queried_engine %in% available_engines) {
    stop(sprintf(
      paste("Engine '%s' is not an available engine.",
        "Please choose one of: %s",
        collapse = " "
      ),
      queried_engine, paste(available_engines, collapse = ", ")
    ))
  }
  queried_engine
}

#' Checks if queried statistcs are available
#'
#' `check_stats` checks if one or multiple statistics are supported
#' for zonal vector-raster extraction by the backend.
#'
#' @param queried_stats A character vector with statistic names to be checked
#'   if they are supported by the backend
#'
#' @return `check_stats` returns a character vector of supported statistics.
#'  Throws an error if any of the queried statistics is not supported.
#' @keywords utils
#' @name engine
#' @export
check_stats <- function(queried_stats) {
  if (any(!queried_stats %in% available_stats)) {
    not_available <- queried_stats[which(!queried_stats %in% available_stats)]
    msg_body <- "%s '%s' %s not supported. Please choose one of: %s"
    if (length(not_available) == 1) {
      stat <- "Statistic"
      verb <- "is"
    } else {
      stat <- "Statistics"
      verb <- "are"
    }
    msg <- sprintf(
      msg_body, stat,
      paste(not_available, collapse = "', '"),
      verb,
      paste(available_stats, collapse = ", ")
    )
    stop(msg)
  }
  queried_stats
}

#' Processes statistics with an engine
#'
#' `select_engine` extracts zonal vector-raster statistics for supported engine
#' and for one or more statistics. Columns are named according to the argument
#' `name` plus the respective stat. Both `portfolio` and `asset` modes are supported.
#'
#' @param x An sf object representing a portfolio.
#' @param raster An terra SpatRaster from which values are to be extracted.
#' @param stats A character vector of statistics to aggregate the raster values
#'   with.
#' @param engine A character vector of length one specifying the engine to be
#'   used for the extraction.
#' @param name A character vector indicating the name to append to the columns
#'   names.
#' @param mode A character vector indicating in which mode to conduct the
#'   extraction (e.g. `asset`-wise or for the whole `portfolio` at once).
#' @return `select_engine` returns a tibble.
#' @keywords utils
#' @name engine
#' @export
select_engine <- function(x,
                          raster,
                          stats,
                          engine,
                          name = NULL,
                          mode = "asset") {
  check_stats(stats)
  check_engine(engine)

  engine <- switch(engine,
    "extract" = .engine_extract,
    "exactextract" = .engine_exact_extract,
    "zonal" = .engine_zonal
  )

  if (mode == "asset") {
    result <- engine(x, raster, stats, name)
  } else {
    result <- purrr::map(1:nrow(x), function(i) {
      engine(x[i, ], raster, stats, name)
    })
  }
  result
}

available_stats <- c("mean", "median", "sd", "min", "max", "sum", "var")
available_engines <- c("zonal", "extract", "exactextract")

.engine_zonal <- function(x, raster, stats, name = NULL) {
  results <- purrr::map_dfc(stats, function(stat) {
    out <- terra::zonal(
      raster,
      vect(x),
      fun = get(stat),
      na.rm = TRUE
    )
    out <- tibble(as.numeric(out))
    names(out) <- ifelse(is.null(name), stat, paste(name, "_", stat, sep = ""))
    out
  })
  results
}

.engine_extract <- function(x, raster, stats, name = NULL) {
  results <- purrr::map_dfc(stats, function(stat) {
    out <- terra::extract(
      raster,
      x,
      fun = get(stat),
      na.rm = TRUE,
      ID = FALSE
    )
    out <- tibble(as.numeric(out))
    names(out) <- ifelse(is.null(name), stat, paste(name, "_", stat, sep = ""))
    out
  })
  results
}

.engine_exact_extract <- function(x, raster, stats, name = NULL) {
  if (!requireNamespace("exactextractr", quietly = TRUE)) {
    stop(paste(
      "Needs package 'exactextractr' to be installed.",
      "Consider installing with 'install.packages('exactextractr')"
    ))
  }

  results <- purrr::map_dfc(stats, function(stat) {
    org_stat <- stat
    if (stat %in% c("sd", "var")) {
      stat <- ifelse(stat == "sd", "stdev", "variance")
    }

    out <- exactextractr::exact_extract(
      raster,
      x,
      fun = stat
    )
    out <- tibble(as.numeric(out))
    names(out) <- ifelse(is.null(name), stat, paste(name, "_", org_stat, sep = ""))
    out
  })
  results
}

test_that("check_engine works", {
  expect_error(
    check_engine(c("not_implemented", "not_implemented2")),
    "Please specify only one engine of:"
  )
  expect_error(
    check_engine("not_implemented"),
    "Engine 'not_implemented' is not an available engine. Please choose one of:"
  )
})

test_that("check_stats works", {
  expect_error(
    check_stats(c("mean", "min", "other")),
    "Statistic 'other' is not supported. Please choose one of: "
  )
  expect_error(
    check_stats(c("mean", "other", "other2")),
    "Statistics 'other', 'other2' are not supported. Please choose one of"
  )
})


test_that("select_engine works", {
  shp <- read_sf(
    system.file("extdata", "sierra_de_neiba_478140.gpkg",
      package = "mapme.biodiversity"
    )
  )
  nasa_grace <- list.files(system.file("res", "nasa_grace",
    package = "mapme.biodiversity"
  ), pattern = ".tif$", full.names = TRUE)
  nasa_grace <- rast(nasa_grace)
  attributes(shp)$years <- 2003:2022

  expect_error(
    select_engine(
      shp,
      nasa_grace,
      stats = c("mean", "sd"),
      engine = "not-implemented",
      name = NULL,
      mode = "asset"
    )
  )

  expect_error(
    select_engine(
      shp,
      nasa_grace,
      stats = "not_implemented",
      engine = "zonal",
      name = NULL,
      mode = "asset"
    )
  )

  result_asset <- select_engine(
    shp, nasa_grace,
    stats = c("mean", "sd"), engine = "zonal",
    name = NULL, mode = "asset"
  )
  result_portfolio <- select_engine(
    shp, nasa_grace,
    stats = c("mean", "sd"), engine = "zonal",
    name = NULL, mode = "portfolio"
  )
  result_name <- select_engine(
    shp, nasa_grace,
    stats = c("mean", "sd"), engine = "zonal",
    name = "drought", mode = "asset"
  )

  expect_true(inherits(result_asset, "tbl_df"))
  expect_true(inherits(result_portfolio, "list"))
  expect_equal(names(result_asset), c("mean", "sd"))
  expect_equal(names(result_name), c("drought_mean", "drought_sd"))
})

#' Calculate carbon statistics
#'
#' These functions allow to calculated statistics based on the harmonized
#' carbon layers for 2010 and 2018 by Noon et al. (2022).
#'
#' The required resources for these indicators are:
#'  - [carbon_resources]
#'
#' @details
#' Irrecoverable carbon is the amount of carbon that, if lost today, could not
#' be recovered until 2050. It can be calculated for above- and below-ground
#' carbon, the total amount of carbon, or for all layers.
#'
#' @name carbon_indicators
#' @param type One of "total", "soil", "biomass", "all". Determines
#'     for which data layer the statistics are calculated.
#' @param engine The preferred processing functions from either one of "zonal",
#'   "extract" or "exactextract" as character.
#' @param stats Function to be applied to compute statistics for polygons either
#'   one or multiple inputs as character. Supported statistics are: "mean",
#'   "median", "sd", "min", "max", "sum", and "var".
#' @keywords indicator
#' @returns A function that returns an indicator tibble with `(type)_carbon_(stat)`
#'   as variable and the respective statistic (in Mg) as value.
#' @export
#' @examples
#' \dontshow{
#' mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
#' }
#' \dontrun{
#' library(sf)
#' library(mapme.biodiversity)
#'
#' outdir <- file.path(tempdir(), "mapme-data")
#' dir.create(outdir, showWarnings = FALSE)
#'
#' mapme_options(
#'   outdir = outdir,
#'   verbose = FALSE
#' )
#'
#' aoi <- system.file("extdata", "shell_beach_protected_area_41057_B.gpkg",
#'   package = "mapme.biodiversity"
#' ) %>%
#'   read_sf() %>%
#'   get_resources(
#'     get_man_carbon(),
#'     get_vul_carbon(),
#'     get_irr_carbon()
#'   ) %>%
#'   calc_indicators(
#'     calc_man_carbon(stats = "sum"),
#'     calc_vul_carbon(stats = "sum"),
#'     calc_irr_carbon(stats = "sum")
#'   ) %>%
#'   portfolio_long()
#'
#' aoi
#' }
calc_irr_carbon <- function(type = c("total", "soil", "biomass", "all"),
                            engine = "extract",
                            stats = "mean") {
  type <- match.arg(type)
  engine <- check_engine(engine)
  stats <- check_stats(stats)

  function(
      x,
      irr_carbon,
      name = "irr_carbon",
      mode = "asset",
      aggregation = "stat",
      verbose = mapme_options()[["verbose"]]) {
    .calc_carbon_stats(
      x,
      layer = irr_carbon,
      which_layer = type,
      stats = stats,
      engine = engine,
      name = "irr_carbon",
      mode = "asset"
    )
  }
}

register_indicator(
  name = "irr_carbon",
  description = "Statistics of irrecoverable carbon per polygon.",
  resources = "irr_carbon"
)

#' @details
#' Manageable carbon is the amount of carbon that, in principle, is manageable
#' by human activities, e.g. its release to the atmosphere can be prevented.
#' It can be calculated for above- and below-ground carbon, the total amount
#' of carbon, or for all layers.
#'
#' @name carbon_indicators
#' @keywords indicator
#' @export
calc_man_carbon <- function(type = c("total", "soil", "biomass", "all"),
                            engine = "extract",
                            stats = "mean") {
  type <- match.arg(type)
  engine <- check_engine(engine)
  stats <- check_stats(stats)

  function(
      x,
      man_carbon,
      name = "man_carbon",
      mode = "asset",
      aggregation = "stat",
      verbose = mapme_options()[["verbose"]]) {
    .calc_carbon_stats(
      x,
      layer = man_carbon,
      which_layer = type,
      stats = stats,
      engine = engine,
      name = "man_carbon",
      mode = "asset"
    )
  }
}

register_indicator(
  name = "man_carbon",
  description = "Statistics of manageable carbon per polygon.",
  resources = "man_carbon"
)

#' @details
#' Vulnerable carbon is the amount of carbon that would be released in a typical
#' land conversion activity.
#' It can be calculated for above- and below-ground carbon, the total amount
#' of carbon, or for all layers.
#'
#' @name carbon_indicators
#' @keywords indicator
#' @export
calc_vul_carbon <- function(type = c("total", "soil", "biomass", "all"),
                            engine = "extract",
                            stats = "mean") {
  type <- match.arg(type)
  engine <- check_engine(engine)
  stats <- check_stats(stats)

  function(
      x,
      vul_carbon,
      name = "vul_carbon",
      mode = "asset",
      aggregation = "stat",
      verbose = mapme_options()[["verbose"]]) {
    .calc_carbon_stats(
      x,
      layer = vul_carbon,
      which_layer = type,
      stats = stats,
      engine = engine,
      name = "vul_carbon",
      mode = "asset"
    )
  }
}

register_indicator(
  name = "vul_carbon",
  description = "Statistics of vulnerable carbon per polygon.",
  resources = "vul_carbon"
)

.calc_carbon_stats <- function(
    x,
    layer,
    which_layer = c("total", "soil", "biomass", "all"),
    stats = "mean",
    engine = "extract",
    name = NULL,
    mode = "asset") {
  stat <- NULL
  if (is.null(layer)) {
    return(NULL)
  }

  if (which_layer == "all") which_layer <- c("total", "soil", "biomass")
  names(layer) <- tolower(names(layer))
  layer <- layer[[grep(paste(which_layer, collapse = "|"), names(layer))]]

  if (all(unlist(global(noNA(layer), fun = "sum")) == 0)) {
    return(NULL)
  }

  area_r <- terra::cellSize(layer, unit = "ha")
  result <- select_engine(
    x = x,
    raster = layer * area_r,
    stats = stats,
    engine = engine,
    name = name,
    mode = mode
  )

  type <- sapply(strsplit(names(layer), "_"), function(x) x[3])
  year <- as.numeric(gsub("\\D", "", names(layer)))

  result[["type"]] <- type
  result[["year"]] <- year
  result <- tidyr::pivot_longer(result, cols = -c(type, year), names_to = "variable")
  result <- dplyr::mutate(result,
    stat = strsplit(variable, "_")[[1]][3],
    variable = paste(name, type, stats, sep = "_"),
    datetime = as.POSIXct(paste0(year, "-01-01T00:00:00Z")),
    unit = "Mg",
    value = sapply(value, function(x) ifelse(is.infinite(x) || is.nan(x), NA, x))
  )
  result[, c("datetime", "variable", "unit", "value")]
}

#' Calculate slope statistics
#'
#' This function allows to calculate slope statistics for
#' polygons. For each polygon, the desired statistic(s)
#' are returned.
#'
#' The required resource for this indicator is:
#'  - [nasa_srtm]
#'
#' @name slope
#' @param engine The preferred processing function from either one of "zonal",
#'    "extract" or "exactextract" as a character string.
#' @param stats Function to be applied to compute statistics for polygons.
#'    Accepts either a single string or a vector of strings, such as "mean",
#'    "median", "sd", "min", "max", "sum", or "var".
#' @keywords indicator
#' @return A function that returns an indicator tibble with specified slope
#'    statistics as variables and corresponding values (in degrees).
#' @include register.R
#' @export
#' @examples
#' \dontshow{
#' mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
#' }
#' \dontrun{
#' library(sf)
#' library(mapme.biodiversity)
#'
#' outdir <- file.path(tempdir(), "mapme-data")
#' dir.create(outdir, showWarnings = FALSE)
#'
#' mapme_options(
#'   outdir = outdir,
#'   verbose = FALSE
#' )
#'
#' aoi <- system.file("extdata", "sierra_de_neiba_478140_2.gpkg",
#'   package = "mapme.biodiversity"
#' ) %>%
#'   read_sf() %>%
#'   get_resources(get_nasa_srtm()) %>%
#'   calc_indicators(
#'     calc_slope(stats = c("mean", "median", "sd", "var"), engine = "extract")
#'   ) %>%
#'   portfolio_long()
#'
#' aoi
#' }
calc_slope <- function(engine = "exactextract", stats = "mean") {
  engine <- check_engine(engine)
  stats <- check_stats(stats)

  function(x,
           nasa_srtm = NULL,
           name = "slope",
           mode = "asset",
           aggregation = "stat",
           verbose = mapme_options()[["verbose"]]) {
    if (is.null(nasa_srtm)) {
      return(NULL)
    }

    # Calculate the slope using terra::terrain
    slope_raster <- terra::terrain(
      nasa_srtm,
      v = "slope",       # Slope calculation
      unit = "degrees",  # Slope in degrees
      neighbors = 8      # Default to 8 neighbors
    )

    result <- select_engine(
      x = x,
      raster = slope_raster,
      stats = stats,
      engine = engine,
      name = "slope",
      mode = mode
    )

    result %>%
      tidyr::pivot_longer(cols = dplyr::everything(), names_to = "variable") %>%
      dplyr::mutate(
        datetime = as.POSIXct("2000-02-01T00:00:00Z"),
        unit = "degrees"
      ) %>%
      dplyr::select(datetime, variable, unit, value)
  }
}

register_indicator(
  name = "slope",
  description = "Statistics of slope based on NASA SRTM",
  resources = "nasa_srtm"
)

test_that("slope calculation works", {
  # Load the polygon data for testing
  x <- read_sf(
    system.file("extdata", "sierra_de_neiba_478140.gpkg",
                package = "mapme.biodiversity"
    )
  )

  # Load the NASA SRTM data required for slope calculation
  nasa_srtm <- list.files(system.file("res", "nasa_srtm",
                                      package = "mapme.biodiversity"
  ), pattern = ".tif$", full.names = TRUE)
  nasa_srtm <- rast(nasa_srtm)

  # Test calc_slope function with default settings
  cslope <- calc_slope()
  expect_true(is.null(cslope(x, NULL)))
  result <- cslope(x, nasa_srtm)

  # Test calc_slope function with multiple statistics
  cslope <- calc_slope(stats = c("mean", "median", "sd", "var"))
  result_multi_stat <- cslope(x, nasa_srtm)

  # Test calc_slope function with "zonal" engine
  cslope <- calc_slope(engine = "zonal")
  result_zonal <- cslope(x, nasa_srtm)

  # Test calc_slope function with "extract" engine
  cslope <- calc_slope(engine = "extract")
  result_extract <- cslope(x, nasa_srtm)

  # Test calc_slope function with "exactextract" engine
  cslope <- calc_slope(engine = "exactextract")
  result_exact <- cslope(x, nasa_srtm)

  # Ensure no errors and correct structure with various engines and settings
  expect_silent(.check_single_asset(result))
  expect_silent(.check_single_asset(result_multi_stat))
  expect_silent(.check_single_asset(result_zonal))
  expect_silent(.check_single_asset(result_extract))
  expect_silent(.check_single_asset(result_exact))

  # Check that the correct variables are returned for multiple statistics
  vars <- c("slope_mean", "slope_median", "slope_sd", "slope_var")
  expect_equal(unique(result_multi_stat$variable), vars)

  # Check that zonal and extract engines produce similar results
  expect_equal(result_zonal$value, result_extract$value, tolerance = 1e-4)

  # Capture a snapshot of the exact extract values for future comparison
  expect_snapshot(result_exact$value)
})

#' Calculate terrestrial ecoregions statistics (TEOW) based on WWF
#'
#' This function allows to efficiently retrieve the name of the ecoregions
#' and compute the corresponding area from Terrestrial Ecoregions of the
#' World (TEOW) - World Wildlife Fund (WWF) for polygons. For each polygon,
#' the name and area of the ecoregions (in hectare) is returned.
#' The required resources for this indicator are:
#'  - [teow]
#'
#' @name ecoregion
#' @keywords indicator
#' @returns A function that returns an indicator tibble with ecoregion type as
#'   variable and corresponding area (in ha) as value.
#' @include register.R
#' @export
#' @examples
#' \dontshow{
#' mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
#' }
#' \dontrun{
#' library(sf)
#' library(mapme.biodiversity)
#'
#' outdir <- file.path(tempdir(), "mapme-data")
#' dir.create(outdir, showWarnings = FALSE)
#'
#' mapme_options(
#'   outdir = outdir,
#'   verbose = FALSE
#' )
#'
#' aoi <- system.file("extdata", "sierra_de_neiba_478140_2.gpkg",
#'   package = "mapme.biodiversity"
#' ) %>%
#'   read_sf() %>%
#'   get_resources(get_teow()) %>%
#'   calc_indicators(calc_ecoregion()) %>%
#'   portfolio_long()
#'
#' aoi
#' }
calc_ecoregion <- function() {
  function(x,
           teow = NULL,
           name = "ecoregion",
           mode = "asset",
           aggregation = "sum",
           verbose = mapme_options()[["verbose"]]) {
    ECO_NAME <- NULL

    if (nrow(teow[[1]]) == 0) {
      return(NULL)
    }

    .comp_teow(
      x = x,
      teow = teow,
      var = ECO_NAME,
      verbose = verbose
    )
  }
}

#' Helper function to intersect polygons and add biome names
#'
#' @param teow terrestrial ecoregions vector from which to compute statistics
#'
#' @return A data-frame
#' @keywords internal
#' @noRd

.comp_teow <- function(x,
                       teow,
                       var,
                       verbose = TRUE,
                       ...) {
  teow <- teow[[1]]
  teow <- st_make_valid(teow)
  teow[["ECO_NAME"]] <- gsub("[-/&'() ]+", "_", tolower(teow[["ECO_NAME"]]))

  intersected <- suppressWarnings(st_intersection(x, teow))
  biome_and_name <- data.frame(
    BIOME = c(1:14, 98, 99),
    BIOME_NAME = c(
      "tropical_subtropical_moist_broadleaf_forests",
      "tropical_subtropical_dry_broadleaf_forests",
      "tropical_subtropical_coniferous_forests",
      "temperate_broadleaf_mixed_forests",
      "temperate_conifer_forests",
      "boreal_forests_taiga",
      "tropical_subtropical_grasslands_savannas_shrublands",
      "temperate_grasslands_savannas_shrublands",
      "flooded_grasslands_savannas",
      "montane_grasslands_shrublands",
      "tundra",
      "mediterranea_forests_woodlands_scrub",
      "deserts_xeric_shrublands",
      "mangroves",
      "lake",
      "rock_and_ice"
    )
  )

  intersected <- merge(intersected, biome_and_name)
  intersected$value <- as.numeric((st_area(intersected))) / 10000

  if (nrow(intersected) == 0) {
    return(NULL)
  }

  intersected %>%
    sf::st_drop_geometry() %>%
    dplyr::select({{ var }}, value) %>%
    dplyr::mutate(
      datetime = as.POSIXct("2001-01-01T00:00:00Z"),
      variable = {{ var }},
      unit = "ha"
    ) %>%
    dplyr::select(datetime, variable, unit, value) %>%
    dplyr::group_by(datetime, variable, unit) %>%
    dplyr::summarise(value = sum(as.numeric(value))) %>%
    dplyr::ungroup()
}

register_indicator(
  name = "ecoregion",
  description = "Areal statistics of ecoregions based on TEOW",
  resources = "teow"
)

#' Calculate Global Surface Water Time Series
#'
#' This function calculates the total area of the global surface water time
#' series data, separated by the following classes:
#'
#' - No Observation: It was not possible to determine whether a pixel was water
#' (this may be the case for frozen areas or during the polar night in extreme
#' latitudes).
#' - Permanent Water: Water was detected in twelve months per year or in a
#' combination of permanent and no observation.
#' - Seasonal Water: Water and no water was detected.
#' - No Water: No Water was detected.
#'
#' The required resources for this indicator are:
#'  - [gsw_time_series_resource]
#'
#' @name gsw_time_series_indicator
#' @keywords indicator
#' @format A function returning a tibble with time series of global surface
#' water data classes.
#' @include register.R
#' @export
#' @examples
#' \dontshow{
#' mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
#' outdir <- file.path(tempdir(), "mapme-data")
#' path_gsw_ts <- file.path(outdir, "gsw_time_series")
#' gsw_fnames_short <- dir(path_gsw_ts, pattern = ".tif$")
#' gsw_fnames_long <- sub("v5_", "VER5-0_yearlyClassification", gsw_fnames_short)
#'
#' invisible(file.copy(
#'    file.path(path_gsw_ts, gsw_fnames_short),
#'    file.path(path_gsw_ts, gsw_fnames_long)
#' ))
#' }
#' \dontrun{
#' library(mapme.biodiversity)
#' library(sf)
#'
#' outdir <- file.path(tempdir(), "mapme-data")
#' dir.create(outdir, showWarnings = FALSE)
#'
#' mapme_options(
#'   outdir = outdir,
#'   verbose = FALSE
#' )
#'
#' aoi <- read_sf(
#'   system.file("extdata", "shell_beach_protected_area_41057_B.gpkg",
#'               package = "mapme.biodiversity"
#' ))
#' aoi <- get_resources(aoi, get_gsw_time_series (years = 2000:2001))
#' aoi <- calc_indicators(aoi, calc_gsw_time_series())
#' aoi <- portfolio_long(aoi)
#'
#' aoi
#' }
calc_gsw_time_series <- function() {
  check_namespace("exactextractr")

  function(x = NULL,
           gsw_time_series,
           name = "gsw_timeseries",
           mode = "asset",
           aggregation = "sum",
           verbose = mapme_options()[["verbose"]]) {

    if (is.null(gsw_time_series)) {
      return(NULL)
    }

    gsw_time_series <- mask(
      gsw_time_series,
      x
    )

    gsw_time_series <- clamp(
      gsw_time_series,
      lower = 0,
      upper = 3,
      values = FALSE
    )

    idx_yr <- as.numeric(gregexpr("yearlyClassification", names(gsw_time_series))) + 20
    years_resource <- substr(names(gsw_time_series), idx_yr, idx_yr + 3)

    names(gsw_time_series) <- years_resource
    coverage_fractions <- exactextractr::exact_extract(gsw_time_series, x, "frac", coverage_area = TRUE)
    if(nlyr(gsw_time_series) == 1) {
      names(coverage_fractions) <- paste(names(coverage_fractions), years_resource, sep = ".")
    }
    x_total_area <- as.numeric(st_area(x)) / 10000

    results <- purrr::map_df(names(coverage_fractions), function(colname) {
      year <- substr(colname, nchar(colname) - 3, nchar(colname))
      variable <- switch(
        substr(colname, 6, 6),
        "0" = "no_observations",
        "1" = "not_water",
        "2" = "seasonal_water",
        "3" = "permanent_water"
      )
      tibble::tibble(
        datetime = as.POSIXct(paste0(year, "-01-01T00:00:00Z")),
        variable = variable,
        unit = "ha",
        value = coverage_fractions[[colname]] * x_total_area
      )
    })

    return(results)
  }
}

register_indicator(
  name = "gsw_time_series",
  description = "Global Surface Water - Yearly Time Series area estimation of water classes.",
  resources = "gsw_time_series"
)

.ind_defaults <- c("x", "name", "mode", "aggregation", "verbose")
.ind_cols <- c("datetime", "variable", "unit", "value")
#' Compute specific indicators
#'
#' `calc_indicators()` calculates specific biodiversity indicators. A
#' requirement is that the resources that are mandatory inputs for the requested
#' indicators are available locally. Multiple indicators and their respective
#' additional arguments can be supplied.
#'
#' @param  x An `sf` object with features of type `"POLYGON"`
#' @param ... One or more functions for resources/indicators
#' @return `calc_indicators()` returns `x`, invisibly, with an additional nested
#'   list column per requested indicator.
#' @name mapme
#' @export
#' @include get_resources.R
calc_indicators <- function(x, ...) {
  x <- .check_portfolio(x)
  funs <- list(...)
  funs <- purrr::map(funs, function(fun) .check_indicator_fun(fun))
  req_resources <- purrr::map(funs, function(fun) .get_req_resources(fun))
  .check_avail_resources(names(.avail_resources()), unique(unlist(req_resources)))
  for (fun in funs) x <- .get_single_indicator(x, fun)
  x
}

.check_indicator_fun <- function(fun) {
  if (!inherits(fun, "function")) {
    msg <- "calc_indicators() expects you to supply one or more indicator functions."
    stop(msg)
  }
  args <- names(formals(fun))
  if (any(!.ind_defaults %in% args)) {
    msg <- "Indicator functions are required to have the following default arguments:\n"
    msg <- paste(msg, paste(.ind_defaults, sep = "", collapse = ", "))
    stop(msg)
  }
  invisible(fun)
}

.get_req_resources <- function(fun) {
  if (!inherits(fun, "function")) {
    stop("calc_indicators() expects you to supply one or more indicator functions.")
  }
  args <- names(formals(fun))
  req_resources <- setdiff(args, .ind_defaults)
  if (length(req_resources) == 0) {
    stop("The indicator functions lacks required resources.")
  }
  req_resources
}

.check_avail_resources <- function(avail_resources, req_resources) {
  if (any(!req_resources %in% avail_resources)) {
    not_avail <- req_resources[which(!req_resources %in% avail_resources)]
    msg <- "The following requested %s not available: %s."
    msg <- sprintf(
      msg,
      ifelse(length(not_avail) == 1, "resource is", "resources are"),
      paste(not_avail, collapse = ", ")
    )
    stop(msg)
  }
}

.get_single_indicator <- function(x = NULL,
                                  fun = NULL,
                                  opts = mapme_options()) {
  args <- formals(fun)
  indicator_name <- args[["name"]]
  processor <- switch(args[["mode"]],
    asset = .asset_processor,
    portfolio = .portfolio_processor,
    stop("Processing mode not supported.")
  )

  results <- processor(
    x = x,
    fun = fun,
    avail_resources = .avail_resources(),
    req_resources = .get_req_resources(fun),
    chunk_size = opts[["chunk_size"]],
    aggregation = args[["aggregation"]],
    verbose = opts[["verbose"]]
  )

  x <- .add_indicator_column(x, results, indicator_name)
  x
}

.asset_processor <- function(x,
                             fun,
                             avail_resources,
                             req_resources,
                             chunk_size,
                             aggregation,
                             verbose) {
  assetid <- NULL
  x_chunk <- .chunk(x, chunk_size)

  n <- nrow(x_chunk)
  s <- 1L
  if (n > 100L) {
    s <- round(n * 0.01)
    n <- 100L
  }

  if (verbose) {
    has_progressr <- check_namespace("progressr", error = FALSE)
    if (has_progressr) {
      p <- progressr::progressor(n)
    }
  }
  # results <- furrr::future_map(1:nrow(x_chunk), function(i) {
  results <- furrr::future_map(seq_len(nrow(x_chunk)), function(i) {
    chunk <- x_chunk[i, ]
    resources <- prep_resources(chunk, avail_resources, req_resources, mode = "asset")
    result <- .compute(chunk, resources, fun, verbose)

    if (verbose && has_progressr) {
      if (i %% s == 0L) {
        p()
      }
    }

    .check_single_asset(result, chunk)
  }, .options = furrr::furrr_options(seed = TRUE, chunk_size = s))

  results <- split(results, x_chunk[["assetid"]])
  results <- purrr::map(results, .combine_chunks, aggregation)
  results <- results[as.character(x[["assetid"]])]
  unname(results)
}

.portfolio_processor <- function(x,
                                 fun,
                                 avail_resources,
                                 req_resources,
                                 chunk_size,
                                 aggregation,
                                 verbose) {
  x_bbox <- st_as_sf(st_as_sfc(st_bbox(x)))
  resources <- prep_resources(x_bbox, avail_resources, req_resources, mode = "portfolio")
  results <- .compute(x, resources, fun, verbose)
  if (!inherits(results, "list")) {
    stop("Expected output for processing mode 'portfolio' is a list.")
  }
  results <- purrr::map(1:length(results), function(i) {
    .check_single_asset(results[[i]], x[i, ])
  })
  results
}

.process <- function(x, fun, avail_resources, req_resources, verbose) {
  resources <- prep_resources(x, avail_resources, req_resources)
  .compute(x, resources, fun, verbose)
}

.compute <- function(x, resources, fun, verbose) {
  args <- list(verbose = verbose)
  args <- append(args, resources)
  args[["x"]] <- x
  try(do.call(what = fun, args = args), silent = TRUE)
}

.check_single_asset <- function(obj,
                                asset = NULL,
                                log_dir = mapme_options()[["log_dir"]]) {
  obj_names <- names(obj)
  if (!inherits(obj, "tbl_df") || nrow(obj) == 0 || !identical(obj_names, .ind_cols)) {
    warning(obj)
    if (!is.null(log_dir) && !is.null(asset)) {
      dsn <- file.path(log_dir, paste0(Sys.Date(), "_mapme-error-assets.gpkg"))
      st_write(asset, dsn, append = TRUE, quiet = TRUE)
    }
    return(NULL)
  }
  obj
}

.add_indicator_column <- function(x, results, name) {
  if (name %in% names(x)) {
    msg <- "Indicator column '%s' is already present. Overwriting now."
    warning(sprintf(msg, name))
  }
  x[name] <- list(results)
  .geom_last(x)
}

test_that("calc_indicator works", {
  .clear_resources()
  x <- read_sf(
    system.file("extdata", "gfw_sample.gpkg",
      package = "mapme.biodiversity"
    )
  )

  outdir <- file.path(tempdir(), "mapme.data")
  .copy_resource_dir(outdir)
  mapme_options(outdir = outdir, verbose = FALSE)

  x <- get_resources(
    x,
    get_gfw_treecover(version = "GFC-2023-v1.11"),
    get_gfw_lossyear(version = "GFC-2023-v1.11")
  )

  stat <- calc_indicators(
    x,
    calc_treecover_area(years = 2000:2005, min_size = 5, min_cover = 30)
  )$treecover_area[[1]]

  expect_equal(names(stat), c("datetime", "variable", "unit", "value"))
  expect_equal(format(stat$datetime, "%Y"), as.character(2000:2005))
  expect_equal(stat$value,
    c(1997.659, 1997.145, 1995.457, 1961.845, 1952.464, 1948.261),
    tolerance = 1e-3
  )
})

test_that("calc_indicator works with MULTIPOLYGON and chunking", {
  x <- read_sf(
    system.file("extdata", "gfw_sample.gpkg",
      package = "mapme.biodiversity"
    )
  )
  x <- st_cast(x, "MULTIPOLYGON")
  area_ha <- as.numeric(st_area(x)) / 10000 / 2

  expect_true(st_geometry_type(x) == "MULTIPOLYGON")

  outdir <- file.path(tempdir(), "mapme.data")
  .copy_resource_dir(outdir)
  mapme_options(outdir = outdir, chunk_size = area_ha, verbose = FALSE)

  x <- get_resources(
    x,
    get_gfw_treecover(version = "GFC-2023-v1.11"),
    get_gfw_lossyear(version = "GFC-2023-v1.11")
  )

  stat <- calc_indicators(
    x,
    calc_treecover_area(years = 2000:2005, min_size = 5, min_cover = 30)
  )

  expect_true(inherits(stat$treecover_area[[1]], "tbl_df"))
})

test_that("Parallelization works", {
  .clear_resources()
  x <- read_sf(
    system.file("extdata", "gfw_sample.gpkg",
      package = "mapme.biodiversity"
    )
  )
  x <- st_as_sf(st_make_grid(x, n = 3))

  outdir <- file.path(tempdir(), "mapme.data")
  .copy_resource_dir(outdir)
  mapme_options(outdir = outdir, verbose = FALSE)

  # aoi <- get_resources(
  #   aoi,
  #   get_gfw_treecover(version = "GFC-2023-v1.11"),
  #   get_gfw_lossyear(version="GFC-2023-v1.11"))

  library(future)
  plan(multisession, workers = 2)
  stat <- get_resources(
    x,
    get_gfw_treecover(version = "GFC-2023-v1.11"),
    get_gfw_lossyear(version = "GFC-2023-v1.11")
  ) %>%
    calc_indicators(
      calc_treecover_area(years = 2000:2005, min_size = 5, min_cover = 30)
    )
  plan(sequential)

  expect_equal(names(stat)[1:2], c("assetid", "treecover_area"))
  expect_equal(nrow(stat), 9)

  stat <- lapply(stat$treecover_area, function(x) x$value)
  names(stat) <- 1:length(stat)
  stat <- rowSums(as.data.frame(stat))

  expect_equal(
    stat,
    c(2612.269, 2609.14, 2604.875, 2565.622, 2548.602, 2540.92),
    tolerance = 1e-3
  )
})

test_that(".check_indicator_fun works correctly", {
  expect_error(.check_indicator_fun("a"))
  fun <- function(x, name, mode, aggregation) {}
  expect_error(.check_indicator_fun(fun))
  fun <- function(x, name, mode, aggregation, verbose) {}
  expect_silent(.check_indicator_fun(fun))
  fun <- function(x, some_resource, name, mode, aggregation, verbose) {}
  expect_silent(f <- .check_indicator_fun(fun))
  expect_true(inherits(f, "function"))
})

test_that(".get_req_resources works", {
  expect_error(.get_req_resources("a"))
  fun <- function(x, name, mode) {}
  expect_error(.get_req_resources(fun))
  fun <- function(x, some_resource, other_resource, name, mode, verbose) {}
  expect_silent(f <- .get_req_resources(fun))
  expect_equal(f, c("some_resource", "other_resource"))
})

test_that(".check_avail_resources works correctly", {
  expect_silent(.check_avail_resources("some_resource", "some_resource"))
  expect_error(.check_avail_resources("some_resource", c("some_resource", "other_resource")))
  expect_error(.check_avail_resources(list(), "some_resource"))
})

test_that(".add_indicator_column works correctly", {
  .clear_resources()
  x <- read_sf(
    system.file("extdata", "gfw_sample.gpkg",
      package = "mapme.biodiversity"
    )
  )

  indicator <- lapply(1:nrow(x), function(i) tibble(A = 1, b = 2))
  expect_silent(x <- .add_indicator_column(x, indicator, name = "new_indicator"))
  expect_warning(.add_indicator_column(x, indicator, name = "new_indicator"))
  expect_true("new_indicator" %in% names(x))
})

test_that(".check_single_asset works correctly", {
  asset <- read_sf(system.file("extdata", "burundi.gpkg", package = "mapme.biodiversity"))
  expect_warning(out <- .check_single_asset(NULL, asset))
  expect_equal(out, NULL)
  expect_warning(out <- .check_single_asset(try("a" + 1, silent = TRUE), asset))
  expect_equal(out, NULL)
  expect_warning(out <- .check_single_asset(c(1:10), asset))
  expect_equal(out, NULL)
  expect_warning(out <- .check_single_asset(tibble(), asset))
  expect_equal(out, NULL)
  expect_warning(out <- .check_single_asset(tibble(a = 1), asset))
  expect_equal(out, NULL)
  obj <- tibble(datetime = 1, variable = 1, unit = 1, value = 1)
  expect_identical(.check_single_asset(obj, asset), obj)
})

test_that("calc_hfp works", {
  x <- read_sf(system.file("extdata", "shell_beach_protected_area_41057_B.gpkg",
    package = "mapme.biodiversity"
  ))
  hfp <- rast(system.file("res", "humanfootprint", "hfp2010.tif",
    package = "mapme.biodiversity"
  ))
  hf <- calc_humanfootprint(stats = c("mean", "median"))
  expect_silent(result <- hf(x, hfp))
  expect_silent(.check_single_asset(result))
  expect_equal(nrow(result), 2)
  expect_equal(result$variable, c("humanfootprint_mean", "humanfootprint_median"))
  expect_equal(result$value, c(2.87, 2.62), tolerance = 0.01)
})

test_that("srtm elevation works", {
  x <- read_sf(
    system.file("extdata", "sierra_de_neiba_478140.gpkg",
      package = "mapme.biodiversity"
    )
  )
  nasa_srtm <- list.files(system.file("res", "nasa_srtm",
    package = "mapme.biodiversity"
  ), pattern = ".tif$", full.names = TRUE)
  nasa_srtm <- rast(nasa_srtm)
  ce <- calc_elevation()
  result <- ce(x, nasa_srtm)
  ce <- calc_elevation(stats = c("mean", "median", "sd"))
  result_multi_stat <- ce(x, nasa_srtm)
  ce <- calc_elevation(engine = "zonal")
  result_zonal <- ce(x, nasa_srtm)
  ce <- calc_elevation(engine = "extract")
  result_extract <- ce(x, nasa_srtm)
  ce <- calc_elevation(engine = "exactextract")
  result_exact <- ce(x, nasa_srtm)

  expect_silent(.check_single_asset(result))
  expect_equal(
    unique(result_multi_stat$variable),
    c("elevation_mean", "elevation_median", "elevation_sd")
  )
  expect_equal(
    names(result_zonal),
    names(result_extract)
  )
  expect_equal(
    names(result_zonal),
    names(result_exact)
  )
  expect_equal(
    result_zonal$value,
    result_extract$value
  )
  expect_snapshot(
    result_exact$value
  )
})

#' Helper function to download Global Surface Water (GSW) yearly time series
#' data
#'
#' This function constructs  the necessary data URLs for a given data set,
#' version and polygon and downloads them for further processing with the
#' mapme.biodiversity package.
#'
#' @details
#' The available surface water classes for a given pixel are the following:
#'
#' - No Observation: It was not possible to determine whether a pixel was water
#' (this may be the case for frozen areas or during the polar night in extreme
#' latitudes).
#' - Permanent Water: Water was detected in twelve months per year or in a
#' combination of permanent and no observation.
#' - Seasonal Water: Water and no water was detected.
#' - No Water: No Water was detected.
#'
#' @name gsw_time_series_resource
#' @keywords resource
#' @returns A function that returns a character vector of file paths.
#' @references
#' * Global Surface Water Explorer: \url{https://global-surface-water.appspot.com/}
#' * Data Users Guide: \url{https://storage.cloud.google.com/global-surface-water/downloads_ancillary/DataUsersGuidev2021.pdf}
#' * Research Article: \url{https://www.nature.com/articles/nature20584}
#' @source Raw Data: \url{https://jeodpp.jrc.ec.europa.eu/ftp/jrc-opendata/GSWE/YearlyClassification/LATEST/tiles/}
#' @param years Numeric vector of years to process between 1984 and 2021.
#' Default: `1984:2021`.
#' @param version Version of the data set to process. Available options are
#' (`VER1-0`, `VER2-0`, `VER3-0`, `VER4-0`, `VER5-0` and `LATEST`) Default:
#' `LATEST`. Choosing `LATEST` will result in the latest available version.
#' @export
get_gsw_time_series <- function(years, version = "LATEST") {
  version <- unique(version)
  available_versions = c("VER1-0", "VER2-0", "VER3-0", "VER4-0", "VER5-0",
                         "LATEST")
  stopifnot(version %in% available_versions)
  if(version == "LATEST") {
    version <- "VER5-0"
  }

  years <- unique(years)
  years <- years[!is.na(as.numeric(years))]
  available_years <- 1984:2021
  years <- check_available_years(years, available_years, "gsw_time_series")

  function(x,
           name = "gsw_time_series",
           type = "raster",
           outdir = mapme_options()[["outdir"]],
           verbose = mapme_options()[["verbose"]]) {
    # make the gsw grid and construct urls for intersecting tiles
    baseurl <- sprintf(
      "/vsicurl/https://jeodpp.jrc.ec.europa.eu/ftp/jrc-opendata/GSWE/YearlyClassification/%s/tiles/",
      version
    )
    grid_gsw <- make_global_grid(
      xmin = -180, xmax = 180, dx = 10,
      ymin = -60, ymax = 80, dy = 10
    )
    tile_ids <- unique(unlist(sf::st_intersects(x, grid_gsw)))
    if (length(tile_ids) == 0) {
      stop("The extent of the portfolio does not intersect with the GSW grid.",
           call. = FALSE
      )
    }
    ids <- sapply(tile_ids, function(n) .get_gsw_ts_tile_id(grid_gsw[n, ]))

    source_combinations <- expand.grid(years = years, ids = ids)
    urls <- purrr::map2_chr(source_combinations$years, source_combinations$ids, function(year, tile_url_id) {
      # Warning: file naming system is different for 2021
      separator <- ifelse(year == 2021, "_", "-")
      sprintf(
        "%syearlyClassification%s/yearlyClassification%s%s%s.tif",
        baseurl, year, year, separator, tile_url_id
      )
    })

    fps <- grid_gsw[expand.grid(years, tile_ids = tile_ids)$tile_ids, ]
    fps[["source"]] <- urls
    make_footprints(fps,
                    filenames = paste0(version, "_", basename(fps$source)),
                    what = "raster",
                    co = c("-co", "INTERLEAVE=BAND", "-co", "COMPRESS=LZW", "-ot", "Byte")
    )
  }
}

.get_gsw_ts_tile_id <- function(tile) {
  min_x <- st_bbox(tile)[1]
  max_y <- st_bbox(tile)[4]

  x_idx <- which(seq(-180, 170, by = 10) == min_x) - 1
  y_idx <- which(seq(80, -50, by = -10) == max_y) - 1

  tile_id <- sprintf(
    "%010d-%010d",
    y_idx * 40000,
    x_idx * 40000
  )

  return(tile_id)
}

register_resource(
  name = "gsw_time_series",
  description = "Global Surface Water - Yearly Time Series",
  licence = "https://global-surface-water.appspot.com/download",
  source = "https://jeodpp.jrc.ec.europa.eu/ftp/jrc-opendata/GSWE/YearlyClassification/LATEST/tiles/",
  type = "raster"
)

#' Accessibility to Cities in 2000
#'
#' This resource provides global maps of travel time to cities of 50,000 or more
#' people in year 2000. Accessibility refers to the ease with which larger cities can be reached from
#' a certain location. This dataset represents travel time to major cities globally
#' as of the year 2000, encoded in minutes. The data is essential for historical
#' analyses, such as understanding the impact of accessibility on land use and
#' socio-economic outcomes during this period.
#'
#' @name accessibility_2000
#' @keywords resource
#' @return A function that returns an `sf` footprint object.
#' @references European Commission, Joint Research Centre (JRC), Global Accessibility Maps (GAM), 2000.
#' @source https://forobs.jrc.ec.europa.eu/gam
#' @include register.R
#' @export
get_accessibility_2000 <- function() {
  function(x,
           name = "accessibility_2000",
           type = "raster",
           outdir = mapme_options()[["outdir"]],
           verbose = mapme_options()[["verbose"]]) {
    url <- paste(
      "/vsizip//vsicurl/",
      "https://forobs.jrc.ec.europa.eu/data/products/gam/access_50k.zip/",
      "acc_50k.tif",
      sep = ""
    )

    bbox <- c(xmin = -180.0, ymin = -90.0, xmax = 180.0, ymax = 90.0)
    tile <- st_as_sf(st_as_sfc(st_bbox(bbox, crs = "EPSG:4326")))
    tile[["source"]] <- url
    make_footprints(tile, filenames = "acc_50k.tif", what = "raster",
                    co = c("-co", "COMPRESS=LZW",
                           "-co", "BLOCKXSIZE=128",
                           "-co", "BLOCKYSIZE=128",
                           "-co", "TILED=YES",
                           "-ot", "Int32",
                           "-a_nodata", "-2147483647"))
  }
}

# Registering the resource as done for nelson_et_al
register_resource(
  name = "accessibility_2000",
  description = "Accessibility data for the year 2000 from the Global Accessibility Map project",
  licence = "See JRC data policy: https://joint-research-centre.ec.europa.eu/about-us/service-standards-and-principles/data-policy_en",
  source = "https://forobs.jrc.ec.europa.eu/gam",
  type = "raster"
)

#' Calculate Global Surface Water (GSW) Change
#'
#' The change in water occurrence intensity between the two periods is derived
#' from homologous pairs of months (i.e. same months containing valid
#' observations in both periods). The difference in the occurrence of surface
#' water was calculated for each homologous pair of months. The average of all
#' of these differences constitutes the Surface Water Occurrence change
#' intensity. The raster files have integer cell values between `[0, 200]`
#' where 0 represents surface water loss and 200 represents surface water gain.
#'
#' The pixel values are aggregated using method provided via the
#' `stats` parameter using the specified `engine`.
#'
#' The required resources for this indicator are:
#'  - [global_surface_water_change]
#'
#' @name gsw_change
#' @param engine The preferred processing functions from either one of "zonal",
#' "extract" or "exactextract". Default: "extract".
#' @param stats Aggregation function with which the data are combined.
#' Default: "mean".
#' @keywords indicator
#' @returns A function that returns an indicator tibble with change intensity
#'   as variable and corresponding (unitless) values as value.
#' @include register.R
#' @export
#' @examples
#' \dontshow{
#' mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
#' }
#' \dontrun{
#' library(sf)
#' library(mapme.biodiversity)
#'
#' outdir <- file.path(tempdir(), "mapme-data")
#' dir.create(outdir, showWarnings = FALSE)
#'
#' mapme_options(
#'   outdir = outdir,
#'   verbose = FALSE
#' )
#'
#' aoi <- system.file("extdata", "shell_beach_protected_area_41057_B.gpkg",
#'   package = "mapme.biodiversity"
#' ) %>%
#'   read_sf() %>%
#'   get_resources(get_global_surface_water_change()) %>%
#'   calc_indicators(
#'     calc_gsw_change(engine = "extract", stats = "mean")
#'   ) %>%
#'   portfolio_long()
#'
#' aoi
#' }
calc_gsw_change <- function(engine = "extract", stats = "mean") {
  engine <- check_engine(engine)
  stats <- check_stats(stats)

  function(x,
           global_surface_water_change = NULL,
           name = "gsw_change",
           mode = "asset",
           aggregation = "stat",
           verbose = mapme_options()[["verbose"]]) {
    if (is.null(global_surface_water_change)) {
      return(NULL)
    }

    global_surface_water_change <- terra::clamp(
      global_surface_water_change,
      lower = 0,
      upper = 200,
      values = FALSE
    )

    results <- select_engine(
      x = x,
      raster = global_surface_water_change,
      stats = stats,
      engine = engine,
      name = "gsw_change",
      mode = "asset"
    )

    results %>%
      tidyr::pivot_longer(cols = dplyr::everything(), names_to = "variable") %>%
      dplyr::mutate(
        datetime = as.POSIXct("2021-01-01T00:00:00Z"),
        unit = "unitless"
      ) %>%
      dplyr::select(datetime, variable, unit, value)
  }
}

register_indicator(
  name = "gsw_change",
  description = "Statistics of the surface water change layer by JRC",
  resources = "global_surface_water_change"
)

#' NASA GRACE-based Drought Indicator layer
#'
#' The resource is published by NASA GRACE Tellus. This data set
#' reflects on potential drought conditions in the shallow groundwater section
#' relative to a reference period spanning from 1948 to 2012.
#' It is available as a global raster with a weekly temporal resolution starting
#' with the year 2003. The value indicates the wetness percentile of a given
#' pixel with regard to the reference period.
#'
#' @name nasa_grace
#' @param years A numeric vector indicating the years for which to make the
#'   resource available.
#' @keywords resource
#' @returns A function that returns an `sf` footprint object.
#' @include register.R
#' @export
get_nasa_grace <- function(years = 2003:2022) {
  years <- check_available_years(years, c(2003:2022), "nasa_grace")

  function(x,
           name = "nasa_grace",
           type = "raster",
           outdir = mapme_options()[["outdir"]],
           verbose = mapme_options()[["verbose"]]) {
    urls <- unlist(sapply(years, function(year) .get_nasagrace_url(year)))
    bbox <- c(xmin = -180.0, ymin = -60, xmax = 180, ymax = 90)
    tiles <- st_as_sfc(st_bbox(bbox, crs = "EPSG:4326"))
    tiles <- st_as_sf(rep(tiles, length(urls)))
    tiles[["source"]] <- urls
    make_footprints(tiles, what = "raster", co = c("-co", "COMPRESS=LZW"))
  }
}


#' Helper function to construct GRACE URLs
#'
#' @param target_year
#'
#' @return A character vector
#' @keywords internal
#' @noRd
.get_nasagrace_url <- function(target_year) {
  available_years <- c(2003:2022)
  dates <- seq.Date(as.Date("2003/02/03"), as.Date("2022/05/16"), by = "week")
  available_dates <- as.integer(format(dates, "%Y%m%d"))
  if (target_year %in% available_years) {
    target_dates <- subset(
      available_dates, substr(available_dates, 1, 4) == target_year
    )
    paste0(
      "/vsicurl/https://nasagrace.unl.edu/globaldata/", target_dates,
      "/gws_perc_025deg_GL_", target_dates, ".tif"
    )
  } else {
    warning(sprintf(
      "Drought indicator not available for target year %s",
      target_year
    ))
    NULL
  }
}

register_resource(
  name = "nasa_grace",
  description = "NASA Gravity Recovery And Climate Experiment (GRACE) - Measurments of Earth's mass and water changes",
  licence = "https://nasagrace.unl.edu/About.aspx",
  source = "https://nasagrace.unl.edu/globaldata/",
  type = "raster"
)

#' Calculate number of fatalities of violent conflict from UCDP GED
#'
#' The indicator aggregated the number of fatalities within a given asset
#' on a monthly cadence stratified by the type of conflict. The different types
#' of conflicts encoded in the UCDP GED database are:
#' - state-based conflict
#' - non-state conflict
#' - one-sided violence
#'
#' The required resources for this indicator are:
#'  - [ucdp_ged]
#'
#' You may apply quality filters based on the precision of the geolocation
#' of events and the temporal precision. By default, these are set to only
#' include events with the highest precision scores.
#'
#' For geo-precision there are levels 1 to 7 with decreasing accuracy:
#' - value 1: the location information corresponds exactly to the geographical
#'   coordinates available
#' - value 2: the location information refers to a limited area around a
#'   specified location
#' - value 3: the source refers to or can be specified to a larger location at
#'   the level of second order administrative divisions (ADM2), such as district
#'   or municipality, the GED uses centroid point coordinates for that ADM2.
#' - value 4: the location information refers to a first order administrative
#'   division, such as a province (ADM1), the GED uses the coordinates for the
#'   centroid point of ADM1
#' - value 5: is used in different cases if the source refers to parts of a
#'   country which are larger than ADM1, but smaller than the entire country;
#'   if two locations are mentioned a representative point in between is
#'   selected; if the location mentioned is an non-independent island; if
#'   the location is not very specifically mentioned or in relation to another
#'   location
#' - value 6: the location mentioned refers to an entire country and its
#'   centroid is used
#' - value 7: If the event takes place over water or in international airspace,
#'   the geographical coordinates in the dataset either represent the centroid
#'   point of a certain water area or estimated coordinates
#'
#' For temporal precision there are levels 1 to 5 with decreasing precision:
#' - value 1: if the exact date of an event is known
#' - value 2: if start and end dates for events are of unspecified character,
#'   spanning more than one calendar day though no longer than six days
#' - value 3: if when start and end dates for events are specified to a certain
#'   week, but specific dates are not provided
#' - value 4: if start and end dates for events are specified to a certain month
#' - value 5: if start and end dates for events are specified to a certain year,
#'   but specific dates are not provided
#'
#'
#' @name fatalities_ucpd
#' @param years A numeric vector indicating the years for which to summarize
#'    fatalities.
#' @param precision_location A numeric indicating precision value for the
#'   geolocation up to which events are included. Defaults to 1.
#' @param precision_time A numeric indicating the precision value of the
#'   temporal coding up to which events are included. Defaults to 1.
#' @keywords indicator
#' @returns A function that returns an indicator tibble with the type of
#'   violence as variable and counts of civilian fatalities as value.
#' @references Sundberg, Ralph, and Erik Melander, 2013, “Introducing the UCDP
#'   Georeferenced Event Dataset”, Journal of Peace Research, vol.50, no.4, 523-532
#' @include register.R
#' @export
#' @examples
#' \dontshow{
#' mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
#' }
#' \dontrun{
#' library(sf)
#' library(mapme.biodiversity)
#'
#' outdir <- file.path(tempdir(), "mapme-data")
#' dir.create(outdir, showWarnings = FALSE)
#'
#' mapme_options(
#'   outdir = outdir,
#'   verbose = FALSE,
#'   chunk_size = 1e8
#' )
#'
#' aoi <- system.file("extdata", "burundi.gpkg",
#'   package = "mapme.biodiversity"
#' ) %>%
#'   read_sf() %>%
#'   get_resources(get_ucdp_ged(version = "22.1")) %>%
#'   calc_indicators(
#'     calc_fatalities(
#'       years = 1991:1992,
#'       precision_location = 1,
#'       precision_time = 1
#'     )
#'   ) %>%
#'   portfolio_long()
#'
#' aoi
#' }
calc_fatalities_ucdp <- function(years = 1989:2023,
                                 precision_location = 1,
                                 precision_time = 1) {
  years <- check_available_years(years, c(1989:2023), "ucdp_ged")

  if (!precision_location %in% 1:7) {
    stop("Argument precision_location must be a single numeric between 1 and 7.")
  }

  if (!precision_time %in% 1:5) {
    stop("Argument precision_time must be a single numeric between 1 and 5.")
  }

  function(x,
           ucdp_ged = NULL,
           name = "fatalities_ucdp",
           mode = "asset",
           aggregation = "sum",
           verbose = mapme_options()[["verbose"]]) {
    date_prec <- where_prec <- date_start <- type_of_violence <- NULL
    year <- month <- deaths_a <- deaths_b <- event_count <- type_of_death <- NULL

    if (is.null(ucdp_ged)) {
      return(NULL)
    }
    ucdp_ged <- ucdp_ged[[1]]

    ucdp_ged <- ucdp_ged[unlist(st_contains(x, ucdp_ged)), ]
    ucdp_ged <- dplyr::select(
      st_drop_geometry(ucdp_ged),
      tidyr::starts_with("deaths_"),
      type_of_violence,
      date_prec,
      where_prec,
      date_start
    )

    ucdp_ged <- dplyr::filter(
      ucdp_ged,
      where_prec <= precision_location,
      date_prec <= precision_time
    )

    ucdp_ged <- dplyr::mutate(
      ucdp_ged,
      date_start = as.Date(date_start),
      year = format(date_start, "%Y")
    )
    ucdp_ged <- dplyr::filter(ucdp_ged, year %in% years)

    fatalities <- dplyr::summarise(
      ucdp_ged,
      dplyr::across(
        dplyr::starts_with("deaths_"),
        ~ sum(as.numeric(.x))
      ),
      event_count = dplyr::n(),
      .by = c(year, type_of_violence)
    )

    fatalities <- dplyr::mutate(
      fatalities,
      deaths_total = rowSums(dplyr::across(dplyr::starts_with("deaths_"))),
      year = as.Date(paste0(year, "-01-01"))
    ) %>%
      dplyr::select(-deaths_a, -deaths_b) %>%
      dplyr::relocate(event_count, .after = dplyr::last_col()) %>%
      dplyr::arrange(year, type_of_violence) %>%
      dplyr::mutate(type_of_violence = dplyr::case_when(
        type_of_violence == 1 ~ "state_based_conflict",
        type_of_violence == 2 ~ "non_state_conflict",
        type_of_violence == 3 ~ "one_sided_violence"
      )) %>%
      tibble::as_tibble() %>%
      tidyr::pivot_longer(cols = tidyr::starts_with("death"), names_to = "type_of_death") %>%
      dplyr::mutate(
        datetime = as.POSIXct(paste0(year, "T00:00:00Z")),
        variable = paste0("fatalities_", type_of_violence, "_", type_of_death),
        unit = "count",
        value = value
      ) %>%
      dplyr::select(datetime, variable, unit, value)

    fatalities
  }
}


register_indicator(
  name = "fatalities_ucdp",
  description = "Number of fatalities by group of conflict based on UCDP GED",
  resources = "ucdp_ged"
)

#' Calculate deforestation drivers
#'
#' This function extracts areal statistics for the drivers of deforestation
#' based on the data source produced by Fritz et al (2022).
#'
#' The required resource for this indicator is:
#'  - [fritz_et_al]
#'
#' @name deforestation_drivers
#' @keywords indicator
#' @returns A function that returns an indicator tibble with deforestation
#'   drivers as variable and corresponding area (in ha) as value.
#' @include register.R
#' @export
#' @examples
#' \dontshow{
#' mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
#' }
#' \dontrun{
#' library(sf)
#' library(mapme.biodiversity)
#'
#' outdir <- file.path(tempdir(), "mapme-data")
#' dir.create(outdir, showWarnings = FALSE)
#'
#' mapme_options(
#'   outdir = outdir,
#'   verbose = FALSE
#' )
#'
#' aoi <- system.file("extdata", "sierra_de_neiba_478140_2.gpkg",
#'   package = "mapme.biodiversity"
#' ) %>%
#'   read_sf() %>%
#'   get_resources(get_fritz_et_al(resolution = 100)) %>%
#'   calc_indicators(calc_deforestation_drivers()) %>%
#'   portfolio_long()
#'
#' aoi
#' }
calc_deforestation_drivers <- function() {
  function(x,
           fritz_et_al = NULL,
           name = "deforestation_drivers",
           mode = "asset",
           aggregation = "sum",
           verbose = mapme_options()[["verbose"]]) {
    if (is.null(fritz_et_al)) {
      return(NULL)
    }
    x <- st_transform(x, st_crs(fritz_et_al))
    classes <- data.frame(
      class = c(
        "commercial_agriculture", "commercial_oil_palm",
        "managed_forests", "mining", "natural_disturbances", "pasture",
        "roads", "wildfire", "other_subsistance_agriculture",
        "shifting_cultivation"
      ),
      code = c(1:7, 9, 80, 81)
    )

    fritz_et_al <- terra::mask(fritz_et_al, x)
    names(fritz_et_al) <- "code"
    arearaster <- cellSize(fritz_et_al, unit = "ha")
    zonal_stat <- zonal(arearaster, fritz_et_al, fun = "sum")

    dplyr::left_join(classes, zonal_stat, by = "code") %>%
      tidyr::replace_na(list(area = 0)) %>%
      dplyr::mutate(
        datetime = as.POSIXct("2008-01-01T00:00:00Z"),
        variable = class,
        unit = "ha",
        value = area
      ) %>%
      dplyr::select(datetime, variable, unit, value) %>%
      tibble::as_tibble()
  }
}

register_indicator(
  name = "deforestation_drivers",
  description = "Areal statistics of deforestation drivers",
  resources = "fritz_et_al"
)

#' Calculate biomes statistics (TEOW) based on WWF
#'
#' This function allows to efficiently retrieve the name of the biomes
#' and compute the corresponding area from Terrestrial Ecoregions of the
#' World (TEOW) - World Wildlife Fund (WWF) for polygons. For each polygon,
#' the name and area of the biomes (in hectare) is returned.
#' The required resources for this indicator are:
#'  - [teow]
#'
#' @name biome
#' @keywords indicator
#' @returns A function that returns an indicator tibble with variable biome type
#'   and corresponding area (in ha) as values.
#' @include register.R
#' @export
#' @examples
#' \dontshow{
#' mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
#' }
#' \dontrun{
#' library(sf)
#' library(mapme.biodiversity)
#'
#' outdir <- file.path(tempdir(), "mapme-data")
#' dir.create(outdir, showWarnings = FALSE)
#'
#' mapme_options(
#'   outdir = outdir,
#'   verbose = FALSE
#' )
#'
#' aoi <- system.file("extdata", "sierra_de_neiba_478140_2.gpkg",
#'   package = "mapme.biodiversity"
#' ) %>%
#'   read_sf() %>%
#'   get_resources(get_teow()) %>%
#'   calc_indicators(calc_biome()) %>%
#'   portfolio_long()
#'
#' aoi
#' }
calc_biome <- function() {
  function(x,
           teow = NULL,
           name = "biome",
           mode = "asset",
           aggregation = "sum",
           verbose = mapme_options()[["verbose"]]) {
    BIOME_NAME <- NULL

    if (nrow(teow[[1]]) == 0) {
      return(NULL)
    }

    .comp_teow(
      x = x,
      teow = teow,
      var = BIOME_NAME,
      verbose = verbose
    )
  }
}

register_indicator(
  name = "biome",
  description = "Areal statistics of biomes from TEOW",
  resources = "teow"
)

#' SoilGrids data layers
#'
#' SoilGrids is a project combining global observation data with
#' machine learning to map the spatial distribution of soil properties across
#' the globe. It is produced at a spatial resolution of 250 meters and each
#' parameter is mapped at different depths. In order to be able to assess
#' prediction uncertainty, besides the mean and median prediction, the 0.05 and
#' 0.95 percentile predictions are available.
#' The following parameters are available:
#' \describe{
#'   \item{bdod}{Bulk density of the fine earth fraction (kg/dm3)}
#'   \item{cec}{Cation Exchange Capacity of the soil (cmol(c)/kg)}
#'   \item{cfvo}{Volumetric fraction of coarse fragments > 2 mm (cm3/100cm3 (volPerc))}
#'   \item{clay}{Proportion of clay particles < 0.002 mm in the fine earth fraction (g/100g)}
#'   \item{nitrogen}{Total nitrogen (g/kg)}
#'   \item{phh2o}{Soil pH (pH)}
#'   \item{sand}{Proportion of sand particles > 0.05 mm in the fine earth fraction (g/100g)}
#'   \item{silt}{Proportion of silt particles >= 0.002 mm and <= 0.05 mm in the fine earth fraction (g/100g)}
#'   \item{soc}{Soil organic carbon content in the fine earth fraction (g/kg)}
#'   \item{ocd}{Organic carbon density (kg/m3)}
#'   \item{ocs}{Organic carbon stocks (kg/m²)}
#' }
#'
#' Except for \code{ocs}, which is only available for a depth of \code{"0-30cm"},
#' all other parameters are available at the following depths:
#' - "0-5cm"
#' - "5-15cm"
#' - "15-30cm"
#' - "30-60cm"
#' - "60-100cm"
#' - "100-200cm"
#'
#' Each parameter and depth is available for the following statistics:
#' - "Q0.05"
#' - "Q0.50"
#' - "mean"
#' - "Q0.95"
#' @name soilgrids
#' @param layers A character vector indicating the layers to download from
#'   soilgrids
#' @param depths A character vector indicating the depths to download
#' @param stats A character vector indicating the statistics to download.
#' @keywords resource
#' @returns A function that returns an `sf` footprint object.
#' @references Poggio, L., de Sousa, L. M., Batjes, N. H., Heuvelink, G. B. M.,
#'   Kempen, B., Ribeiro, E., and Rossiter, D.: SoilGrids 2.0: producing soil
#'   information for the globe with quantified spatial uncertainty, SOIL, 7,
#'   217–240, 2021. \doi{doi:10.5194/soil-7-217-2021}
#' @source \url{https://isric.org/explore/soilgrids}
#' @include register.R
#' @export
get_soilgrids <- function(layers, depths, stats) {
  if (any(missing(layers), missing(depths), missing(stats))) {
    stop(
      paste("For downloading data from soilgrid a valid layer, a valid ",
        "depth range and a valid statistic have to be specified.",
        sep = ""
      ),
      call. = FALSE
    )
  }

  if (any(!layers %in% names(.sg_layers))) {
    na_layers <- layers[which(!layers %in% .sg_layers)]
    stop(sprintf(
      paste("The selected layer(s) '%s' is/are not available. ",
        "Please choose one of: %s.",
        sep = ""
      ),
      paste(na_layers, sep = ", "), paste(names(.sg_layers), collapse = ", ")
    ), call. = FALSE)
  }

  if (any(!depths %in% .sg_depths)) {
    na_depths <- depths[which(!depths %in% .sg_depths)]
    stop(
      sprintf(
        paste("The selected depth range(s) '%s' is/are not available. ",
          "Please choose one of: %s.",
          sep = ""
        ),
        na_depths, paste(.sg_depths, collapse = ", ")
      ),
      call. = FALSE
    )
  }

  if (any(!stats %in% .sg_stats)) {
    na_stats <- stats[which(!stats %in% .sg_stats)]
    stop(
      sprintf(
        paste("The selected predictions statistic(s) '%s' is not available. ", "
              Please choose one of: %s.", sep = ""),
        na_stats, paste(.sg_stats, collapse = ", ")
      ),
      call. = FALSE
    )
  }

  function(x,
           name = "soilgrids",
           type = "raster",
           outdir = mapme_options()[["outdir"]],
           verbose = mapme_options()[["verbose"]]) {
    grid <- expand.grid(layers, depths, stats)
    names(grid) <- c("layer", "depth", "stat")

    urls <- purrr::pmap_chr(grid, function(layer, depth, stat) {
      if (layer != "ocs" & depth == "0-30cm") {
        message("Depth '0-30cm' is only available of layer 'ocs'.")
        return(NA)
      }

      if (layer == "ocs" & depth != "0-30cm") {
        message("Layer 'ocs' is only available at depth '0-30cm'.")
        return(NA)
      }
      baseurl <- "/vsicurl/https://files.isric.org/soilgrids/latest/data/"
      datalayer <- sprintf("%s/%s_%s_%s.vrt", layer, layer, depth, stat)
      paste0(baseurl, datalayer)
    })

    urls <- na.omit(urls)
    filenames <- basename(urls)
    bbox <- c(xmin = -19949750, ymin = -6147500, xmax = 19861750, ymax = 8361000)
    fps <- st_as_sf(st_as_sfc(st_bbox(bbox, crs = .sg_wkt)))
    fps[["source"]] <- urls
    make_footprints(fps, filenames = filenames, what = "raster")
  }
}


.sg_layers <- list(
  bdod = list(
    description = "Bulk density of the fine earth fraction",
    mapped_units = "cg/cm3",
    conversion_factor = 100,
    conventional_units = "kg/dm3"
  ),
  cec = list(
    description = "Cation Exchange Capacity of the soil",
    mapped_units = "mmol(c)/kg",
    conversion_factor = 10,
    conventional_units = "cmol(c)/kg"
  ),
  cfvo = list(
    description = "Volumetric fraction of coarse fragments (> 2 mm)",
    mapped_units = "cm3/dm3 (volPerc)",
    conversion_factor = 10,
    conventional_units = "cm3/100cm3 (volPerc)"
  ),
  clay = list(
    description = paste("Proportion of clay particles (< 0.002 mm) ",
      "in the fine earth fraction",
      sep = ""
    ),
    mapped_units = "g/kg",
    conversion_factor = 10,
    conventional_units = "g/100g (Perc)"
  ),
  nitrogen = list(
    description = "Total nitrogen (N)",
    mapped_units = "cg/kg",
    conversion_factor = 100,
    conventional_units = "g/kg"
  ),
  phh2o = list(
    description = "Soil pH",
    mapped_units = "pHx10",
    conversion_factor = 10,
    conventional_units = "pH"
  ),
  sand = list(
    description = paste("Proportion of sand particles (> 0.05 mm) ",
      "in the fine earth fraction",
      sep = ""
    ),
    mapped_units = "g/kg",
    conversion_factor = 10,
    conventional_units = "g/100g (Perc)"
  ),
  silt = list(
    description = paste("Proportion of silt particles (>= 0.002 mm ",
      "and <= 0.05 mm) in the fine earth fraction",
      sep = ""
    ),
    mapped_units = "g/kg",
    conversion_factor = 10,
    conventional_units = "g/100g (Perc)"
  ),
  soc = list(
    description = "Soil organic carbon content in the fine earth fraction",
    mapped_units = "dg/kg",
    conversion_factor = 10,
    conventional_units = "g/kg"
  ),
  ocd = list(
    description = "Organic carbon density",
    mapped_units = "hg/m3",
    conversion_factor = 10,
    conventional_units = "kg/m3"
  ),
  ocs = list(
    description = "Organic carbon stocks",
    mapped_units = "t/ha",
    conversion_factor = 10,
    conventional_units = "kg/m2"
  )
)

.sg_depths <- c(
  "0-5cm", "5-15cm", "15-30cm", "30-60cm",
  "60-100cm", "100-200cm", "0-30cm"
)
.sg_stats <- c("Q0.05", "Q0.5", "mean", "Q0.95")

.sg_wkt <- 'PROJCRS["Interrupted_Goode_Homolosine",
    BASEGEOGCRS["GCS_WGS_1984 ellipse",
        DATUM["World Geodetic System 1984",
            ELLIPSOID["WGS 84",6378137,298.257223563,
                LENGTHUNIT["metre",1]],
            ID["EPSG",6326]],
        PRIMEM["Greenwich",0,
            ANGLEUNIT["Degree",0.0174532925199433]]],
    CONVERSION["unnamed",
        METHOD["Interrupted Goode Homolosine"]],
    CS[Cartesian,2],
        AXIS["(E)",east,
            ORDER[1],
            LENGTHUNIT["metre",1,
                ID["EPSG",9001]]],
        AXIS["(N)",north,
            ORDER[2],
            LENGTHUNIT["metre",1,
                ID["EPSG",9001]]]]'

register_resource(
  name = "soilgrids",
  description = "ISRIC - Modelled global soil property layers",
  licence = "CC-BY 4.0",
  source = "https://www.isric.org/explore/soilgrids",
  type = "raster"
)

#' Calculate precipitation sums based on CHIRPS
#'
#' This functions allows to calculate precipitation sums based on the
#' CHIRPS rainfall estimates. Corresponding to the time-frame of the analysis
#' of the portfolio, monthly precipitation sums are calculated.
#'
#' The required resources for this indicator are:
#'  - [chirps]
#'
#' @name precipitation_chirps
#' @param years A numeric vector indicating the years for which to calculate
#'   precipitation statistics.
#' @param engine The preferred processing functions from either one of "zonal",
#'   "extract" or "exactextract" as character.
#' @keywords indicator
#' @returns A function that returns an indicator tibble with variable
#'   precipitation and sum of precipitation (in mm) as value.
#' @include register.R
#' @export
#' @examples
#' \dontshow{
#' mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
#' }
#' \dontrun{
#' library(sf)
#' library(mapme.biodiversity)
#'
#' outdir <- file.path(tempdir(), "mapme-data")
#' dir.create(outdir, showWarnings = FALSE)
#'
#' mapme_options(
#'   outdir = outdir,
#'   verbose = FALSE
#' )
#'
#' aoi <- system.file("extdata", "sierra_de_neiba_478140_2.gpkg",
#'   package = "mapme.biodiversity"
#' ) %>%
#'   read_sf() %>%
#'   get_resources(get_chirps(years = 2010)) %>%
#'   calc_indicators(
#'     calc_precipitation_chirps(
#'       years = 2010,
#'       engine = "extract"
#'     )
#'   ) %>%
#'   portfolio_long()
#'
#' aoi
#' }
calc_precipitation_chirps <- function(years = 1981:2020,
                                      engine = "extract") {
  engine <- check_engine(engine)
  avail_years <- seq(1981, format(Sys.Date(), "%Y"))
  years <- check_available_years(years, avail_years, "precipitation_chirps")

  function(x,
           chirps,
           name = "precipitation_chirps",
           mode = "portfolio",
           aggregation = "sum",
           verbose = mapme_options()[["verbose"]]) {
    if (is.null(chirps)) {
      return(NULL)
    }

    if (length(years) == 0) {
      return(NULL)
    }

    src_names <- names(chirps)
    # set values smaller 0 to NA
    chirps <- clamp(
      chirps,
      lower = 0,
      upper = Inf,
      values = FALSE
    )

    layer_years <- as.numeric(substr(src_names, 13, 17))
    target_chirps <- chirps[[which(layer_years %in% years)]]
    dates <- as.Date(paste0(substr(names(target_chirps), 13, 19), ".01"), "%Y.%m.%d")
    datetime <- as.POSIXct(paste0(dates, "T00:00:00Z"))

    # extract zonal statistics
    results <- select_engine(
      x = x,
      raster = target_chirps,
      stats = "sum",
      engine = engine,
      mode = "portfolio"
    )

    if (mode == "portfolio") {
      results <- purrr::map(1:length(results), function(i) {
        tibble(
          datetime = datetime,
          variable = "precipitation",
          unit = "mm",
          value = as.numeric(results[[i]][["sum"]])
        )
      })
    } else {
      results <- tibble(
        datetime = datetime,
        variable = "precipitation",
        unit = "mm",
        value = as.numeric(results[[1]][["sum"]])
      )
    }
    results
  }
}


register_indicator(
  name = "precipitation_chirps",
  description = "Statistics of CHIRPS precipitation layer",
  resources = "chirps"
)

#' Calculate area of different landcover classes
#'
#' The land cover data shows us how much of the region is covered by forests,
#' rivers, wetlands, barren land, or urban infrastructure thus allowing the
#' observation of land cover dynamics over a period of time. This function
#' allows to efficiently calculate area of different landcover classes for
#' polygons. For each polygon, the area of the classes in hectare(ha) is
#' returned.
#'
#' The required resources for this indicator are:
#'  - [esalandcover]
#'
#' @name landcover
#' @keywords indicator
#' @returns A function that returns an indicator tibble with landcover classes
#'   as variables and corresponding areas (in ha) as value.
#' @include register.R
#' @export
#' @examples
#' \dontshow{
#' mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
#' }
#' \dontrun{
#' library(sf)
#' library(mapme.biodiversity)
#'
#' outdir <- file.path(tempdir(), "mapme-data")
#' dir.create(outdir, showWarnings = FALSE)
#'
#' mapme_options(
#'   outdir = outdir,
#'   verbose = FALSE
#' )
#'
#' aoi <- system.file("extdata", "sierra_de_neiba_478140_2.gpkg",
#'   package = "mapme.biodiversity"
#' ) %>%
#'   read_sf() %>%
#'   get_resources(get_esalandcover(years = 2016:2017)) %>%
#'   calc_indicators(calc_landcover()) %>%
#'   portfolio_long()
#'
#' aoi
#' }
calc_landcover <- function() {
  function(x,
           esalandcover = NULL,
           name = "landcover",
           mode = "asset",
           aggregation = "sum",
           verbose = mapme_options()[["verbose"]]) {
    percentage <- NULL
    year <- NULL
    classes <- NULL

    if (is.null(esalandcover)) {
      return(NULL)
    }

    x_v <- vect(x)
    esa_mask <- terra::mask(esalandcover, x_v)
    arearaster <- cellSize(esa_mask, mask = TRUE, unit = "ha")
    total_size <- as.numeric(global(arearaster, fun = sum, na.rm = TRUE))

    purrr::map_dfr(1:nlyr(esa_mask), function(i) {
      zonal(arearaster, esa_mask[[i]], sum) %>%
        tidyr::pivot_longer(cols = -area, names_to = "datetime", values_to = "code") %>%
        dplyr::left_join(.esa_landcover_classes, by = "code") %>%
        dplyr::mutate(
          datetime = regmatches(datetime, regexpr("\\d{4}", datetime)),
          datetime = as.POSIXct(paste0(datetime, "-01-01T00:00:00Z")),
          variable = variable,
          unit = "ha"
        ) %>%
        dplyr::select(datetime, variable, unit, value = area)
    })
  }
}

.esa_landcover_classes <- data.frame(
  code = c(0, 111:116, 121:126, seq(20, 100, 10), 200),
  variable = c(
    "no_data", "closed_forest_evergreen_needle_leaf", "closed_forest_evergreen_broad_leaf", "closed_forest_deciduous_needle_leaf",
    "closed_forest_deciduous_broad_leaf", "closed_forest_mixed", "closed_forest_unknown", "open_forest_evergreen_needle_leaf",
    "open_forest_evergreen_broad_leaf", "open_forest_deciduous_needle_leaf", "open_forest_deciduous_broad_leaf",
    "open_forest_mixed", "open_forest_unknown", "shrubs", "herbaceous_vegetation", "cropland", "built_up", "bare_vegetation",
    "snow_and_ice", "permanent_water_bodies", "herbaceous_wetland", "moss_and_lichen", "open_sea"
  )
)

register_indicator(
  name = "landcover",
  description = "Areal statistics grouped by landcover class",
  resources = "esalandcover"
)

#' Calculate human footprint statistics
#'
#' Human footprint data measures the pressure imposed on the natural environment
#' by different dimensions of human actions. The theoretical maximum value,
#' representing the highest level of human pressure, is 50. This routine allows
#' to extract zonal statistics of the human footprint data.
#'
#' The required resources for this indicator are:
#'  - [humanfootprint_resource]
#'
#' @name humanfootprint_indicator
#' @param engine The preferred processing functions from either one of "zonal",
#'   "extract" or "exactextract" as character.
#' @param stats Function to be applied to compute statistics for polygons either
#'   one or multiple inputs as character. Supported statistics are: "mean",
#'   "median", "sd", "min", "max", "sum" "var".
#' @keywords indicator
#' @returns A function that returns an indicator tibble the `humanfootprint`
#' as variable and the associated value (unitless) per year.
#' @importFrom mapme.biodiversity check_engine check_stats select_engine
#' @export
#' @examples
#' \dontshow{
#' mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
#' }
#' \dontrun{
#' library(sf)
#' library(mapme.biodiversity)
#'
#' outdir <- file.path(tempdir(), "mapme-data")
#' dir.create(outdir, showWarnings = FALSE)
#'
#' mapme_options(
#'   outdir = outdir,
#'   verbose = FALSE
#' )
#'
#' aoi <- system.file("extdata", "shell_beach_protected_area_41057_B.gpkg",
#'   package = "mapme.biodiversity"
#' ) %>%
#'   read_sf() %>%
#'   get_resources(get_humanfootprint(years = 2010)) %>%
#'   calc_indicators(calc_humanfootprint(stats = "median")) %>%
#'   portfolio_long()
#'
#' aoi
#' }
calc_humanfootprint <- function(engine = "extract", stats = "mean") {
  engine <- check_engine(engine)
  stats <- check_stats(stats)

  function(
      x,
      humanfootprint,
      name = "humanfootprint",
      mode = "asset",
      aggregation = "stat",
      verbose = mapme_options()[["verbose"]]) {
    if (is.null(humanfootprint)) {
      return(NULL)
    }

    if (all(unlist(global(noNA(humanfootprint), fun = "sum")) == 0)) {
      return(NULL)
    }

    x <- st_transform(x, st_crs(humanfootprint))
    result <- select_engine(
      x = x,
      raster = humanfootprint,
      stats = stats,
      engine = engine,
      name = "humanfootprint",
      mode = "asset"
    )
    years <- as.numeric(substring(names(humanfootprint), 4, 7))
    result[["datetime"]] <- as.POSIXct(paste0(years, "-01-01T00:00:00Z"))
    result <- tidyr::pivot_longer(result, cols = -datetime, names_to = "variable")
    result[["value"]] <- sapply(result[["value"]], function(value) {
      ifelse(is.infinite(value) || is.nan(value), NA, value)
    })
    result[["unit"]] <- "unitless"
    result[, c("datetime", "variable", "unit", "value")]
  }
}

register_indicator(
  name = "humanfootprint",
  description = "Statistics of the human footprint data set per polygon.",
  resources = "humanfootprint"
)

#' Calculate population count statistics
#'
#' WorldPop, which was initiated in 2013, offers easy access to spatial demographic
#' datasets, claiming to use peer-reviewed and fully transparent methods to create
#' global mosaics for the years 2000 to 2020. This function allows to efficiently
#' calculate population count statistics (e.g. total number of population) for
#' polygons. For each polygon, the desired statistic/s (min, max, sum, mean,
#' median, sd or var) is/are returned.
#'
#' The required resources for this indicator are:
#'  - [worldpop]
#'
#' @name population_count
#' @param engine The preferred processing functions from either one of "zonal",
#'   "extract" or "exactextract" as character.
#' @param stats Function to be applied to compute statistics for polygons
#'    either one or multiple inputs as character "min", "max", "sum", "mean", "median"
#'    "sd" or "var".
#' @keywords indicator
#' @returns A function that returns an indicator tibble with the specified
#'   populations statistics as variable and the corresponding values as value.
#' @include register.R
#' @export
#' @examples
#' \dontshow{
#' mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
#' }
#' \dontrun{
#' library(sf)
#' library(mapme.biodiversity)
#'
#' outdir <- file.path(tempdir(), "mapme-data")
#' dir.create(outdir, showWarnings = FALSE)
#'
#' mapme_options(
#'   outdir = outdir,
#'   verbose = FALSE
#' )
#'
#' aoi <- system.file("extdata", "sierra_de_neiba_478140_2.gpkg",
#'   package = "mapme.biodiversity"
#' ) %>%
#'   read_sf() %>%
#'   get_resources(get_worldpop(years = 2010:2020)) %>%
#'   calc_indicators(
#'     calc_population_count(engine = "extract", stats = c("sum", "median"))
#'   ) %>%
#'   portfolio_long()
#'
#' aoi
#' }
calc_population_count <- function(engine = "extract", stats = "sum") {
  engine <- check_engine(engine)
  stats <- check_stats(stats)

  function(x,
           worldpop = NULL,
           name = "population_count",
           mode = "asset",
           aggregation = "stat",
           verbose = mapme_options()[["verbose"]]) {
    if (is.null(worldpop)) {
      return(NULL)
    }

    # set max value of 65535 to NA
    worldpop <- clamp(
      worldpop,
      lower = -Inf,
      upper = 65534,
      values = FALSE
    )

    results <- select_engine(
      x = x,
      raster = worldpop,
      stats = stats,
      engine = engine,
      name = "population",
      mode = "asset"
    )

    years <- unlist(lapply(names(worldpop), function(x) strsplit(x, "_")[[1]][2]))
    results[["datetime"]] <- as.POSIXct(paste0(years, "-01-01T00:00:00Z"))
    results[["unit"]] <- "count"
    results %>%
      tidyr::pivot_longer(-c(datetime, unit), names_to = "variable", values_to = "value") %>%
      dplyr::select(datetime, variable, unit, value)
  }
}

register_indicator(
  name = "population_count",
  description = "Statistic of population counts",
  resources = "worldpop"
)

#' Species richness based on IUCN raster data
#'
#' Species richness counts the number of potential species intersecting with a
#' polygon grouped by the IUCN threat categorization. Note, that
#' this indicator function requires the manual download of the respective
#' raster files.
#'
#' The specific meaning of the species richness indicator depends on the
#' supplied raster file.
#'
#' The required resources for this indicator are:
#'  - [iucn]
#'
#' @name species_richness
#' @param engine The preferred processing functions from either one of "zonal",
#'   "extract" or "exactextract" as character.
#' @param stats Function to be applied to compute statistics for polygons either
#'   one or multiple inputs as character. Supported statistics are: "mean",
#'   "median", "sd", "min", "max", "sum" "var".
#' @keywords indicator
#' @returns A function that returns an indicator tibble with IUCN layers with
#'   specified statistics as variable and respective species richness (count)
#'   as value.
#' @export
#' @examples
#' \dontshow{
#' mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
#' }
#' \dontrun{
#' library(sf)
#' library(mapme.biodiversity)
#'
#' outdir <- file.path(tempdir(), "mapme-data")
#' dir.create(outdir, showWarnings = FALSE)
#'
#' mapme_options(
#'   outdir = outdir,
#'   verbose = FALSE
#' )
#'
#' iucn_dir <- system.file("res", "iucn", package = "mapme.biodiversity")
#' sr_rasters <- list.files(iucn_dir, pattern = "*_SR_*", full.names = TRUE)
#'
#' aoi <- system.file("extdata", "sierra_de_neiba_478140_2.gpkg",
#'   package = "mapme.biodiversity"
#' ) %>%
#'   read_sf() %>%
#'   get_resources(get_iucn(sr_rasters)) %>%
#'   calc_indicators(calc_species_richness(stats = "median")) %>%
#'   portfolio_long()
#'
#' aoi
#' }
calc_species_richness <- function(engine = "extract", stats = "mean") {
  engine <- check_engine(engine)
  stats <- check_stats(stats)

  function(
      x,
      iucn,
      name = "species_richness",
      mode = "asset",
      aggregation = "stat",
      verbose = mapme_options()[["verbose"]]) {
    if (is.null(iucn)) {
      return(NULL)
    }

    if (all(unlist(global(noNA(iucn), fun = "sum")) == 0)) {
      return(NULL)
    }

    x <- st_transform(x, st_crs(iucn))

    result <- select_engine(
      x = x,
      raster = iucn,
      stats = stats,
      engine = engine,
      name = "",
      mode = "asset"
    )

    result[["variable"]] <- gsub(".tif", "", names(iucn))
    result <- tidyr::pivot_longer(result, -variable, names_to = "stats")
    result[["variable"]] <- paste0(tolower(result[["variable"]]), result[["stats"]])
    result[["value"]][is.na(result[["value"]])] <- 0
    year <- as.numeric(gsub("\\D", "", names(iucn)))
    result[["datetime"]] <- as.POSIXct(paste0(year, "-01-01T00:00:00Z"))
    result[["unit"]] <- "count"
    result[, c("datetime", "variable", "unit", "value")]
  }
}

register_indicator(
  name = "species_richness",
  description = "Species richness statistics based on user-specified raster files.",
  resources = "iucn"
)

#' Calculate Biodiversity Intactness Index
#'
#' This function calculates the mean biodiversity intactness index for a region.
#'
#' The required resources for this indicator are:
#'  - [biodiversity_intactness_index_resource]
#'
#' @name biodiversity_intactness_index_indicator
#' @docType data
#' @keywords indicator
#' @format  A function that returns an indicator tibble with variable
#'   biodiversity_intactness_index and corresponding values (unitless) as value.
#' @include register.R
#' @export
#' @examples
#' \dontshow{
#' mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
#' }
#' \dontrun{
#' library(sf)
#' library(mapme.biodiversity)
#'
#' outdir <- file.path(tempdir(), "mapme-data")
#' dir.create(outdir, showWarnings = FALSE)
#'
#' mapme_options(
#'   outdir = outdir,
#'   verbose = FALSE
#' )
#'
#' lbii <- system.file("res", "biodiversity_intactness_index", "lbii.asc",
#'                     package = "mapme.biodiversity")
#'
#' aoi <- read_sf(
#'   system.file("extdata", "shell_beach_protected_area_41057_B.gpkg",
#'               package = "mapme.biodiversity"
#' ))
#' aoi <- get_resources(aoi, get_biodiversity_intactness_index(lbii))
#' aoi <- calc_indicators(aoi, calc_biodiversity_intactness_index())
#' aoi <- portfolio_long(aoi)
#'
#' aoi
#' }
calc_biodiversity_intactness_index <- function() {

  function(x = NULL,
           biodiversity_intactness_index,
           name = "biodiversity_intactness_index",
           mode = "asset",
           aggregation = "mean",
           verbose = mapme_options()[["verbose"]]) {

    if (is.null(biodiversity_intactness_index)) {
      return(NULL)
    }

    mean_bii <- exactextractr::exact_extract(biodiversity_intactness_index, x, fun = "mean")

    results <- tibble::tibble(
      datetime = as.POSIXct("2015-01-01T00:00:00Z"),
      variable = "biodiversity_intactness_index",
      unit = "unitless",
      value = mean_bii
    )

    return(results)
  }
}

register_indicator(
  name = "biodiversity_intactness_index",
  description = "Averaged biodiversity intactness index.",
  resources = "biodiversity_intactness_index"
)

#' Calculate minimum temperature statistics based on WorldClim
#'
#' This function allows to efficiently calculate minimum temperature statistics
#' from Worldclim for polygons. For each polygon, the desired statistic/s (min,
#' max, sum, mean, median, sd or var) is/are returned.
#'
#' The required resources for this indicator are:
#'  - minimum temperature layer from [worldclim_min_temperature]
#'
#' @name temperature_min_wc
#' @param engine The preferred processing functions from either one of "zonal",
#'   "extract" or "exactextract" as character.
#' @param stats Function to be applied to compute statistics for polygons either
#'   single or multiple inputs as character. Supported statistics are: "mean",
#'   "median", "sd", "min", "max", "sum" "var".
#' @keywords indicator
#' @returns A function that returns an indicator tibble with minimum temperature
#'   statistics as variables and corresponding values as value.
#' @include register.R
#' @export
#' @examples
#' \dontshow{
#' mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
#' }
#' \dontrun{
#' library(sf)
#' library(mapme.biodiversity)
#'
#' outdir <- file.path(tempdir(), "mapme-data")
#' dir.create(outdir, showWarnings = FALSE)
#'
#' mapme_options(
#'   outdir = outdir,
#'   verbose = FALSE
#' )
#'
#' aoi <- system.file("extdata", "sierra_de_neiba_478140_2.gpkg",
#'   package = "mapme.biodiversity"
#' ) %>%
#'   read_sf() %>%
#'   get_resources(get_worldclim_min_temperature(years = 2018)) %>%
#'   calc_indicators(
#'     calc_temperature_min_wc(
#'       engine = "extract",
#'       stats = c("mean", "median")
#'     )
#'   ) %>%
#'   portfolio_long()
#'
#' aoi
#' }
calc_temperature_min_wc <- function(engine = "extract", stats = "mean") {
  engine <- check_engine(engine)
  stats <- check_stats(stats)

  function(x,
           worldclim_min_temperature = NULL,
           name = "temperature_min_wc",
           mode = "asset",
           aggregation = "stat",
           verbose = mapme_options()[["verbose"]]) {
    if (is.null(worldclim_min_temperature)) {
      return(NULL)
    }

    results <- .calc_worldclim(
      x = x,
      worldclim = worldclim_min_temperature,
      engine = engine,
      stats = stats,
      unit = "C"
    )
    results
  }
}

register_indicator(
  name = "temperature_min_wc",
  description = "Statistics of WorldClim minimum temperature layer",
  resources = "worldclim_min_temperature"
)

#' Helper function to compute worldclim statistics
#'
#' @param worldclim worldclim raster from which to compute statistics
#' @param x A single polygon for which to calculate the climatic statistic
#' @param stats Function to be applied to compute statistics for polygons
#'    either one or multiple inputs as character "min", "max", "sum", "mean", "median"
#'    "sd" or "var".
#' @param engine The preferred processing functions from either one of "zonal",
#'   "extract" or "exactextract" as character.
#'
#' @return A data-frame
#' @keywords internal
#' @include register.R
#' @noRd
.calc_worldclim <- function(x,
                            worldclim,
                            engine = "extract",
                            stats = "mean",
                            unit = NULL) {
  stopifnot(!is.null(unit))
  if (is.null(worldclim)) {
    return(NULL)
  }
  # set max value of 65535 to NA
  worldclim <- clamp(
    worldclim,
    lower = -Inf,
    upper = 65534,
    values = FALSE
  )

  # Cache band names once
  band_names <- names(worldclim)

  # Extract variable name either for 4.06 or 4.09
  var <- regmatches(band_names[1], regexpr("(prec|tmin|tmax)", band_names[1]))
  # give informative message if fails
  if (length(var) == 0) {
    stop("Cannot parse variable from band name: ", band_names[1])
  }
  # Layer name
  layer <- paste0("worldclim_", var)
                             # CHANGED: was hard-coded split on "_" token

  # Extract YYYY-MM and convert to POSIXct
  bn <- basename(band_names)
  dates <- sub(".*(\\d{4}-\\d{2}).*", "\\1", bn)
  if (any(!grepl("^\\d{4}-\\d{2}$", dates))) {
    stop("Unexpected WorldClim band names: ", paste(unique(bn), collapse = ", "))
  }
  iso_dates <- paste0(dates, "-01T00:00:00Z")
  datetime <- as.POSIXct(iso_dates, format = "%Y-%m-%dT%H:%M:%SZ", tz = "UTC")


  results <- select_engine(
    x = x,
    raster = worldclim,
    stats = stats,
    engine = engine,
    name = layer,
    mode = "asset"
  )

  results[["datetime"]] <- datetime

  results %>%
    tidyr::pivot_longer(-datetime, names_to = "variable", values_to = "value") %>%
    dplyr::mutate(unit = unit) %>%
    dplyr::select(datetime, variable, unit, value)
}

#' NASADEM HGT v001
#'
#' This resource is processed by the Land Processes Distributed Active Archive
#' Center (LP DAAC) and made available at the Microsoft Planetery Computer.
#' NASADEM are distributed in 1 degree latitude by 1 degree longitude tiles and
#' consist of all land between 60° N and 56° S latitude. This accounts for about
#' 80% of Earth’s total landmass.
#'
#' @name nasa_srtm
#' @keywords resource
#' @returns A function that returns an `sf` footprint object.
#' @references NASA JPL (2020). NASADEM Merged DEM Global 1 arc second V001.
#'   NASA EOSDIS Land Processes DAAC. Accessed 2023-07-01 from
#'   \doi{doi:10.5067/MEaSUREs/NASADEM/NASADEM_HGT.001}
#' @source \url{https://planetarycomputer.microsoft.com/dataset/nasadem}
#' @include register.R
#' @export
get_nasa_srtm <- function() {
  check_namespace("rstac")

  function(x,
           name = "nasa_srtm",
           type = "raster",
           outdir = mapme_options()[["outdir"]],
           verbose = mapme_options()[["verbose"]]) {
    items <- try(rstac::stac("https://planetarycomputer.microsoft.com/api/stac/v1/") %>%
      rstac::stac_search(
        collection = "nasadem",
        bbox = as.numeric(st_bbox(x)),
        limit = NULL
      ) %>%
      rstac::post_request() %>%
      rstac::items_fetch())

    if (inherits(items, "try-error")) {
      stop("Download for NASA SRTM resource was unsuccesfull")
    }

    urls <- rstac::assets_url(items, asset_names = "elevation")
    if (length(urls) == 0) {
      stop("The extent of the portfolio does not intersect with the SRTM grid.")
    }

    bboxs <- rstac::items_bbox(items)
    fps <- purrr::map(bboxs, function(x) {
      names(x) <- c("xmin", "ymin", "xmax", "ymax")
      bbox <- st_bbox(x, crs = "EPSG:4326")
      st_as_sf(st_as_sfc(bbox))
    })
    fps <- st_as_sf(purrr::list_rbind(fps))
    fps[["source"]] <- paste0("/vsicurl?pc_url_signing=yes&url=", urls)
    make_footprints(fps, what = "raster", co = c("-co", "COMPRESS=DEFLATE"))
  }
}

register_resource(
  name = "nasa_srtm",
  description = "NASA Shuttle Radar Topography Mission (SRTM) Digital Elevation Model (DEM)",
  licence = "https://www.earthdata.nasa.gov/engage/open-data-services-software-policies/data-use-guidance",
  source = "https://planetarycomputer.microsoft.com/dataset/nasadem",
  type = "raster"
)

#' Calculate Global Surface Water (GSW) Seasonality
#'
#' GSW seasonality describes the intra-annual distribution of surface water for
#' each pixel. The raster files have integer cell values between \code{[0, 12]},
#' indicating how many months per year the pixel was classified as water.
#'
#' The pixel values are aggregated using method provided via the
#' \code{stats} parameter.
#'
#' The required resources for this indicator are:
#'  - [global_surface_water_seasonality]
#'
#' @name gsw_seasonality
#' @keywords indicator
#' @returns A function that returns an indicator tibble with seasonality
#'   categories as variables and corresponding areas (in ha) as value.
#' @include register.R
#' @export
#' @examples
#' \dontshow{
#' mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
#' }
#' \dontrun{
#' library(sf)
#' library(mapme.biodiversity)
#'
#' outdir <- file.path(tempdir(), "mapme-data")
#' dir.create(outdir, showWarnings = FALSE)
#'
#' mapme_options(
#'   outdir = outdir,
#'   verbose = FALSE
#' )
#'
#' aoi <- system.file("extdata", "shell_beach_protected_area_41057_B.gpkg",
#'   package = "mapme.biodiversity"
#' ) %>%
#'   read_sf() %>%
#'   get_resources(get_global_surface_water_seasonality()) %>%
#'   calc_indicators(calc_gsw_seasonality()) %>%
#'   portfolio_long()
#'
#' aoi
#' }
calc_gsw_seasonality <- function() {
  function(x,
           global_surface_water_seasonality = NULL,
           name = "gsw_seasonality",
           mode = "asset",
           aggregation = "sum",
           verbose = mapme_options()[["verbose"]]) {
    if (is.null(global_surface_water_seasonality)) {
      return(NULL)
    }

    global_surface_water_seasonality <- terra::mask(
      global_surface_water_seasonality,
      x
    )

    global_surface_water_seasonality <- terra::clamp(
      global_surface_water_seasonality,
      lower = 0,
      upper = 12,
      values = FALSE
    )

    pixel_areas <- terra::cellSize(
      global_surface_water_seasonality,
      mask = TRUE,
      unit = "ha"
    )

    res_zonal <- terra::zonal(
      pixel_areas,
      global_surface_water_seasonality,
      fun = "sum"
    )

    names(res_zonal) <- c("variable", "value")

    result <- tibble::tibble(
      variable = 0:12
    )

    result <- merge(
      result,
      res_zonal,
      all.x = TRUE
    )
    result[["value"]][is.na(result[["value"]])] <- 0

    result %>%
      dplyr::mutate(
        variable = paste0("gsw_seasonality_", sprintf("%02d", variable)),
        datetime = as.POSIXct("2021-01-01T00:00:00Z"),
        unit = "ha"
      ) %>%
      dplyr::select(datetime, variable, unit, value) %>%
      tibble::as_tibble()
  }
}

register_indicator(
  name = "gsw_seasonality",
  description = "Areal statistic of surface water by seasonality",
  resources = "global_surface_water_seasonality"
)

#' Calculate Global Surface Water (GSW) Transitions
#'
#' GSW transition data contains information about the type of surface water
#' change for each pixel. The raster files have integer cell values between
#' \code{[0, 10]} that code for different transition classes:
#'
#' | Value | Transition Class      |
#' |-------|-----------------------|
#' | 1     | Permanent             |
#' | 2     | New Permanent         |
#' | 3     | Lost Permanent        |
#' | 4     | Seasonal              |
#' | 5     | New Seasonal          |
#' | 6     | Lost Seasonal         |
#' | 7     | Seasonal to Permanent |
#' | 8     | Permanent to Seasonal |
#' | 9     | Ephemeral Permanent   |
#' | 10    | Ephemeral Seasonal    |
#'
#' To aggregate, we sum up the area of each transition class for a given region.
#'
#' The required resources for this indicator are:
#'  - [global_surface_water_transitions]
#'
#' @name gsw_transitions
#' @keywords indicator
#' @returns A function that returns an indicator tibble with transition classes
#'   as variable and corresponding areas (in ha) as value.
#' @include register.R
#' @export
#' @examples
#' \dontshow{
#' mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
#' }
#' \dontrun{
#' library(sf)
#' library(mapme.biodiversity)
#'
#' outdir <- file.path(tempdir(), "mapme-data")
#' dir.create(outdir, showWarnings = FALSE)
#'
#' mapme_options(
#'   outdir = outdir,
#'   verbose = FALSE
#' )
#'
#' aoi <- system.file("extdata", "shell_beach_protected_area_41057_B.gpkg",
#'   package = "mapme.biodiversity"
#' ) %>%
#'   read_sf() %>%
#'   get_resources(get_global_surface_water_transitions()) %>%
#'   calc_indicators(calc_gsw_transitions()) %>%
#'   portfolio_long()
#'
#' aoi
#' }
calc_gsw_transitions <- function() {
  function(x,
           global_surface_water_transitions = NULL,
           name = "gsw_transitions",
           mode = "asset",
           aggregation = "sum",
           verbose = mapme_options()[["verbose"]]) {
    if (is.null(global_surface_water_transitions)) {
      return(NULL)
    }

    global_surface_water_transitions <- terra::clamp(
      global_surface_water_transitions,
      lower = 1,
      upper = 10,
      values = FALSE
    )

    x_v <- terra::vect(x)
    transition_mask <- terra::mask(global_surface_water_transitions, x_v)
    arearaster <- terra::cellSize(transition_mask, mask = TRUE, unit = "ha")

    purrr::map_dfr(seq_len(terra::nlyr(transition_mask)), function(i) {
      terra::zonal(arearaster, transition_mask[[i]], sum) %>%
        stats::setNames(c("code", "value")) %>%
        dplyr::left_join(.gsw_transition_classes, by = "code") %>%
        dplyr::select(variable, value)
    }) %>%
      dplyr::mutate(
        variable = paste0("gsw_", variable),
        datetime = as.POSIXct("2021-01-01T00:00:00Z"),
        unit = "ha"
      ) %>%
      tibble::as_tibble() %>%
      dplyr::select(datetime, variable, unit, value)
  }
}

.gsw_transition_classes <- data.frame(
  code = 1:10,
  variable = c(
    "permanent", "new_permanent", "lost_permanent", "seasonal",
    "new_seasonal", "lost_seasonal", "seasonal_to_permanent",
    "permanent_to_seasonal", "ephemeral_permanent", "ephemeral_seasonal"
  )
)

register_indicator(
  name = "gsw_transitions",
  description = "Areal statistics of surface water grouped by transition class",
  resources = "global_surface_water_transitions"
)

.chunk <- function(x, chunk_size) {
  stopifnot(inherits(x, "sf") && "assetid" %in% names(x))
  metadata <- st_drop_geometry(x)
  x <- x[, "assetid"]
  x[["chunked"]] <- FALSE
  geom_org <- attr(x, "sf_column")

  x <- .split_dateline(x)
  if (!is.null(chunk_size)) {
    x <- .split_multipolygons(x, chunk_size)
    x <- .chunk_geoms(x, chunk_size)
  }
  names(x)[names(x) == attr(x, "sf_column")] <- geom_org
  st_geometry(x) <- geom_org

  .finalize_assets(x, metadata, geom_org)
}

.finalize_assets <- function(x, meta, geom_org) {
  stopifnot("assetid" %in% names(x) && "assetid" %in% names(meta))
  x <- st_sf(tibble::as_tibble(dplyr::left_join(meta, x, by = "assetid")),
    sf_column_name = geom_org
  )
  x$chunked <- NULL
  .geom_last(x)
}

.split_dateline <- function(x) {
  stopifnot(inherits(x, "sf"))
  crosses_dateline <- sapply(st_geometry(x), .crosses_dateline)
  if (any(crosses_dateline)) {
    fix_dt <- st_wrap_dateline(x[which(crosses_dateline), ], options = c("WRAPDATELINE=YES", "DATELINEOFFSET=10"))
    fix_dt <- .cast_to_polygon(fix_dt)
    x <- rbind(x[-which(crosses_dateline), ], fix_dt)
  }
  x
}

.crosses_dateline <- function(geom, offset = 10) {
  stopifnot(inherits(geom, "sfg"))
  bbox <- st_bbox(geom)
  sum <- abs(bbox[[1]]) + abs(bbox[[3]])
  diff <- bbox[[1]] < 0 && bbox[[3]] > 0
  diff && (360 - offset) <= sum
}

.cast_to_polygon <- function(geom) {
  stopifnot(inherits(geom, "sf") && "assetid" %in% names(geom))
  is_poly <- st_geometry_type(geom) == "POLYGON"
  polys <- geom[is_poly, ]
  casted <- suppressWarnings(st_cast(geom[!is_poly, ], "POLYGON"))
  polys <- st_sf(rbind(polys, casted))
  .try_make_valid(polys)
}

.split_multipolygons <- function(x, chunk_size) {
  stopifnot(inherits(x, "sf") && "assetid" %in% names(x) && "chunked" %in% names(x))
  is_smaller <- .calc_bbox_areas(x) < chunk_size
  x[["chunked"]][is_smaller] <- TRUE

  if (all(is_smaller)) {
    return(x)
  }

  x_split <- x[!is_smaller, ]
  x_split <- .cast_to_polygon(x_split)
  is_smaller2 <- .calc_bbox_areas(x_split) < chunk_size
  x_split[["chunked"]][is_smaller2] <- TRUE
  rbind(x[is_smaller, ], x_split)
}

.calc_bbox_areas <- function(geom) {
  bboxs <- lapply(st_geometry(geom), function(y) st_as_sfc(st_bbox(y)))
  bboxs <- st_sf(do.call("c", bboxs), crs = st_crs(geom))
  as.numeric(st_area(bboxs) / 10000)
}

.chunk_geoms <- function(x, chunk_size) {
  stopifnot(inherits(x, "sf") && "assetid" %in% names(x) && "chunked" %in% names(x))
  if (all(x[["chunked"]])) {
    return(x)
  }
  x_ok <- NULL
  x_grid <- x[!x[["chunked"]], ]
  if (any(x[["chunked"]])) {
    x_ok <- x[x[["chunked"]], ]
  }
  # only chunks if more than 2x2 cells
  n_cells <- ceiling(sqrt(.calc_bbox_areas(x_grid) / chunk_size))
  to_grid <- n_cells > 2
  if (any(!to_grid)) {
    x_ok <- rbind(x_ok, x_grid[!to_grid, ])
    x_grid <- x_grid[to_grid, ]
  }
  if (any(to_grid)) {
    x_grid <- purrr::map(1:nrow(x_grid), function(i) .make_grid(x_grid[i, ], chunk_size))
    x_grid <- st_sf(purrr::list_rbind(x_grid))
  }
  rbind(x_ok, x_grid)
}

.make_grid <- function(geom, chunk_size) {
  # stopifnot(inherits(geom, "sf") && "assetid" %in% names(geom))
  n <- ceiling(sqrt(.calc_bbox_areas(geom) / chunk_size))
  geom_grid <- st_make_grid(geom, n = n)
  geom_grid <- st_intersection(geom_grid, geom)
  geom_grid <- st_sf(geom_grid, assetid = geom[["assetid"]], chunked = TRUE)
  st_geometry(geom_grid) <- attr(geom, "sf_col")
  .try_make_valid(geom_grid)
}

#' @importFrom stats sd var
.aggregation_fun <- function(agg) {
  stopifnot(agg %in% available_stats)

  switch(agg,
    sum = sum,
    mean = mean,
    median = median,
    sd = sd,
    min = min,
    max = max,
    sum = sum,
    var = var
  )
}

.combine_chunks <- function(data, aggregation = "sum") {
  stat <- NULL
  is_null <- sapply(data, is.null)

  if (all(is_null)) {
    return(NULL)
  }

  data <- data[!is_null]
  if (length(data) == 1) {
    return(data[[1]])
  }

  if (aggregation == "stat") {
    data <- data %>%
      purrr::list_rbind() %>%
      dplyr::mutate(
        stat = purrr::map_chr(strsplit(variable, "_"), function(x) rev(x)[1])
      ) %>%
      dplyr::group_by(datetime, variable, unit) %>%
      dplyr::summarise(value = .aggregation_fun(stat[1])(value, na.rm = TRUE)) %>%
      dplyr::ungroup()
  } else {
    agg <- .aggregation_fun(aggregation)
    data <- data %>%
      purrr::list_rbind() %>%
      dplyr::group_by(datetime, variable, unit) %>%
      dplyr::summarise(value = agg(value, na.rm = TRUE)) %>%
      dplyr::ungroup()
  }
  data
}

test_that(".crosses_dateline and .split_dateline works", {
  bbox <- c(xmin = -175.1, ymin = -0.1, xmax = 175.1, ymax = 0.1)
  x <- st_as_sfc(st_bbox(bbox, crs = st_crs("EPSG:4326")))
  x <- st_sf(assetid = 1, geometry = x)
  expect_true(.crosses_dateline(st_geometry(x)[[1]], offset = 10))
  expect_false(.crosses_dateline(st_geometry(x)[[1]], offset = 9))
  expect_silent(x2 <- .split_dateline(x))
  expect_equal(nrow(x2), 2)
})

test_that(".calc_bbox_area works", {
  bbox <- c(xmin = 0.0, ymin = 0.0, xmax = 100.0, ymax = 100.0)
  x <- st_as_sf(st_as_sfc(st_bbox(bbox)))
  expect_equal(.calc_bbox_areas(x), 1)
  x <- st_make_grid(x, n = c(2, 2))
  expect_equal(.calc_bbox_areas(x), rep(0.25, 4))
})

test_that(".try_make_valid works", {
  bbox <- c(xmin = -10.0, ymin = -10.0, xmax = 10.0, ymax = 10.0)
  x <- st_as_sf(st_as_sfc(st_bbox(bbox)), crs = "EPSG:4326")
  grid <- st_sf(geometry = st_make_grid(x, n = 2))

  # invalid geom
  invalid <- st_sf(geometry = st_sfc(st_polygon(x = list(
    matrix(c(unlist(st_geometry(grid)[1]), 0, -10, -10, 0), ncol = 2, byrow = T)
  )), crs = "EPSG:4326"))
  expect_false(st_is_valid(invalid))

  # geom collection
  lst <- st_linestring(matrix(c(-10, -10, 10, 10), ncol = 2, byrow = T))
  col <- st_geometrycollection(x = list(lst, st_geometry(grid)[[2]]))
  col <- st_sf(geometry = st_sfc(col, crs = "EPSG:4326"))
  expect_equal(as.character(st_geometry_type(col)), "GEOMETRYCOLLECTION")
  # try make valid
  expect_silent(x2 <- .try_make_valid(rbind(grid, invalid, col)))
  expect_true(inherits(x2, "sf"))
  expect_equal(nrow(x2), 6)
  expect_equal(as.character(unique(st_geometry_type(x2))), "POLYGON")
})

test_that(".cast_to_polygon works", {
  bbox <- c(xmin = -1.0, ymin = -1., xmax = 1.0, ymax = 1.0)
  x <- st_as_sfc(st_bbox(bbox))
  x <- st_make_grid(x, n = c(2, 2))
  x <- st_sfc(st_multipolygon(x), crs = st_crs("EPSG:4326"))
  x <- st_sf(geometry = x, assetid = 1)
  expect_equal(nrow(x), 1)
  expect_true(st_geometry_type(x) == "MULTIPOLYGON")
  expect_silent(x2 <- .cast_to_polygon(x))
  expect_equal(nrow(x2), 4)
  expect_true(unique(st_geometry_type(x2)) == "POLYGON")
  expect_equal(st_bbox(x), st_bbox(x2))
  expect_equal(st_crs(x), st_crs(x2))
})

test_that(".split_multipolygons works", {
  bbox <- c(xmin = -10.0, ymin = -10.0, xmax = 10.0, ymax = 10.0)
  x <- st_as_sfc(st_bbox(bbox, crs = st_crs("EPSG:4326")))
  x <- st_make_grid(x, n = c(4, 4))
  x <- st_sf(
    assetid = 1, chunked = FALSE,
    geometry = st_sfc(st_multipolygon(x)), crs = "EPSG:4326"
  )
  expect_silent(x2 <- .split_multipolygons(x, chunk_size = .calc_bbox_areas(x) / 16))
  expect_equal(nrow(x2), 16)
  expect_equal(unique(x2$assetid), 1)
  expect_equal(st_bbox(x), st_bbox(x2))
  expect_equal(st_crs(x), st_crs(x2))
})

test_that(".make_grid works", {
  bbox <- c(xmin = -10.0, ymin = -10.0, xmax = 10.0, ymax = 10.0)
  x <- st_sf(
    my_geom = st_as_sfc(st_bbox(bbox)),
    crs = st_crs("EPSG:4326"),
    assetid = 1
  )
  x2 <- .make_grid(x, .calc_bbox_areas(x) / 16)
  expect_equal(nrow(x2), 16)
  expect_equal(unique(x2$assetid), 1)
  expect_equal(st_bbox(x), st_bbox(x2))
  expect_equal(st_crs(x), st_crs(x2))
  expect_equal(attr(x2, "sf_col"), "my_geom")
})

test_that(".chunk_geoms works", {
  bbox <- c(xmin = -10.0, ymin = -10.0, xmax = 10.0, ymax = 10.0)
  x <- st_sf(st_as_sfc(st_bbox(bbox)),
    crs = st_crs("EPSG:4326"),
    assetid = 1, chunked = TRUE
  )
  chunk_size <- .calc_bbox_areas(x) / 16
  expect_equal(x, .chunk_geoms(x, chunk_size))
  x$chunked <- FALSE
  expect_silent(x2 <- .chunk_geoms(x, chunk_size))
  expect_equal(nrow(x2), 16)
  expect_equal(st_bbox(x), st_bbox(x2))
  expect_equal(st_crs(x), st_crs(x2))
})

test_that(".finalize_assets works correctly", {
  bbox <- c(xmin = -10.0, ymin = -10.0, xmax = 10.0, ymax = 10.0)
  x <- st_sf(
    my_geom = st_as_sfc(st_bbox(bbox)),
    crs = st_crs("EPSG:4326"),
    assetid = 1, var = "variable"
  )
  meta <- st_drop_geometry(x)
  x2 <- x[, "assetid"]
  x2 <- .make_grid(x2, .calc_bbox_areas(x) / 4)
  expect_silent(x3 <- .finalize_assets(x2, meta, "my_geom"))
  expect_true(inherits(x3, "sf"))
  expect_equal(x3$assetid, rep(1, 4))
  expect_equal(x3$var, rep("variable", 4))
  expect_equal(attr(x3, "sf_col"), "my_geom")
})

test_that("chunking works correctly", {
  .clear_resources()
  x <- read_sf(
    system.file("extdata", "sierra_de_neiba_478140_2.gpkg",
      package = "mapme.biodiversity"
    )
  )
  x$assetid <- 1
  x <- .geom_last(x)
  st_geometry(x) <- "my_geom"
  x_chunked <- .chunk(x, chunk_size = .calc_bbox_areas(x) / 16)
  expect_equal(st_bbox(x), st_bbox(x_chunked), tolerance = 1e-4)
  expect_equal(st_area(x), sum(st_area(x_chunked)), tolerance = 1e-4)
  expect_equal(nrow(x_chunked), 12)
  expect_equal(attr(x_chunked, "sf_col"), "my_geom")
  expect_equal(x, .chunk(x, chunk_size = .calc_bbox_areas(x) * 10))
  expect_equal(x, .chunk(x, chunk_size = NULL))

  data <- tibble(
    datetime = "2000-01-01",
    variable = "test",
    unit = "ha",
    value = 1
  )

  data <- lapply(1:10, function(i) {
    if (i == 5) {
      return(NULL)
    }
    data
  })

  expect_silent(data2 <- .combine_chunks(data, aggregation = "sum"))
  expect_true(inherits(data2, "tbl_df"))
  expect_equal(nrow(data2), 1)
  expect_equal(data2[["value"]], 9)
  vals <- c()
  for (agg in available_stats) {
    expect_silent(data3 <- .combine_chunks(data, aggregation = agg))
    vals <- c(vals, data3[["value"]])
  }
  expect_equal(vals, c(1, 1, 0, 1, 1, 9, 0))
})

#' Calculate population exposed to violent conflict from UCDP GED
#'
#' The indicator calculates the population exposed to conflict events within a
#' specified buffer distance around violent events in UCDP GED. Per default,
#' the first available WorldPop layer is used to estimate exposed populations
#' for years before the respective year, while the most recent layer is
#' used for years after.
#'
#' The indicator is inspired by the Conflict Exposure tool from ACLED (see
#' citation below), but differs in the regard that we simply flatten our
#' buffered event layer instead of applying voronoi tessellation.
#'
#' The required resources for this indicator are:
#'  - [ucdp_ged]
#'  - [worldpop]
#'
#' You may filter for certain types of violence. The coded types according
#' to the UCDP codebook are:
#' value 1: state-based conflict
#' value 2: non-state conflict
#' value 3: one-sided conflict
#'
#' You may apply quality filters based on the precision of the geolocation
#' of events and the temporal precision. By default, these are set to only
#' include events with the highest precision scores.
#'
#' For geo-precision there are levels 1 to 7 with decreasing accuracy:
#' - value 1: the location information corresponds exactly to the geographical
#'   coordinates available
#' - value 2: the location information refers to a limited area around a
#'   specified location
#' - value 3: the source refers to or can be specified to a larger location at
#'   the level of second order administrative divisions (ADM2), such as district
#'   or municipality, the GED uses centroid point coordinates for that ADM2.
#' - value 4: the location information refers to a first order administrative
#'   division, such as a province (ADM1), the GED uses the coordinates for the
#'   centroid point of ADM1
#' - value 5: is used in different cases if the source refers to parts of a
#'   country which are larger than ADM1, but smaller than the entire country;
#'   if two locations are mentioned a representative point in between is
#'   selected; if the location mentioned is an non-independent island; if
#'   the location is not very specifically mentioned or in relation to another
#'   location
#' - value 6: the location mentioned refers to an entire country and its
#'   centroid is used
#' - value 7: If the event takes place over water or in international airspace,
#'   the geographical coordinates in the dataset either represent the centroid
#'   point of a certain water area or estimated coordinates
#'
#' For temporal precision there are levels 1 to 5 with decreasing precision:
#' - value 1: if the exact date of an event is known
#' - value 2: if start and end dates for events are of unspecified character,
#'   spanning more than one calendar day though no longer than six days
#' - value 3: if when start and end dates for events are specified to a certain
#'   week, but specific dates are not provided
#' - value 4: if start and end dates for events are specified to a certain month
#' - value 5: if start and end dates for events are specified to a certain year,
#'   but specific dates are not provided
#'
#'
#' @name exposed_population_ucdp
#' @param distance A numeric vector indicating the buffer size around
#'   included conflict events to calculate the exposed population. Either of
#'   length 1 to apply for all types of events, or discrete values for
#'   each category included in `violence_types`.
#' @param violence_types A numeric vector indicating the types of violence
#'  to be included (see Details).
#' @param years A numeric vector indicating for which years to calculate
#'   the exposed population. Restricted to available years for UCDP GED.
#'   For years not intersecting with available WorldPop layers, the first layer
#'   is used for earlier years and the last layer to more recent years.
#' @param precision_location A numeric indicating precision value for the
#'   geolocation up to which events are included. Defaults to 1.
#' @param precision_time A numeric indicating the precision value of the
#'   temporal coding up to which events are included. Defaults to 1.
#' @keywords indicator
#' @returns A function that returns an indicator tibble with conflict exposure
#'   as variable and percentage of the population as its value.
#' @references Raleigh, C; C Dowd; A Tatem; A Linke; N Tejedor-Garavito; M
#'   Bondarenko and K Kishi. 2023. Assessing and Mapping Global and Local
#'   Conflict Exposure. Working Paper.
#' @include register.R
#' @export
#' @examples
#' \dontshow{
#' mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
#' }
#' \dontrun{
#' if (FALSE) {
#'   library(sf)
#'   library(mapme.biodiversity)
#'
#'   outdir <- file.path(tempdir(), "mapme-data")
#'   dir.create(outdir, showWarnings = FALSE)
#'
#'   mapme_options(
#'     outdir = outdir,
#'     verbose = FALSE,
#'     chunk_size = 1e8
#'   )
#'
#'   aoi <- system.file("extdata", "burundi.gpkg",
#'     package = "mapme.biodiversity"
#'   ) %>%
#'     read_sf() %>%
#'     get_resources(
#'       get_ucdp_ged(version = "22.1"),
#'       get_worldpop(years = 2000)
#'     ) %>%
#'     calc_indicators(
#'       conflict_exposure(
#'         distance = 5000,
#'         violence_types = 1:3,
#'         years = 2000,
#'         precision_location = 1,
#'         precision_time = 1
#'       )
#'     ) %>%
#'     portfolio_long()
#'
#'   aoi
#' }
#' }
calc_exposed_population_ucdp <- function(distance = 5000,
                                         violence_types = 1:3,
                                         years = c(1989:2023),
                                         precision_location = 1,
                                         precision_time = 1) {
  check_namespace("exactextractr")
  if (!all(violence_types %in% 1:3)) {
    stop("Argument violence_types must be an numeric vector with values between 1 and 3.")
  }
  if (!precision_location %in% 1:7) {
    stop("Argument precision_location must be a single numeric between 1 and 7.")
  }
  if (!precision_time %in% 1:5) {
    stop("Argument precision_time must be a single numeric between 1 and 5.")
  }

  if (length(distance) > 1) {
    if (length(distance) != length(violence_types)) {
      stop("distance must be of equal length to `violence_types`.")
    }
  }
  years <- check_available_years(years, c(1989:2023), "exposed_population")

  function(x,
           ucdp_ged = NULL,
           worldpop = NULL,
           name = "exposed_population_ucdp",
           mode = "asset",
           aggregation = "sum",
           verbose = mapme_options()[["verbose"]]) {
    date_prec <- where_prec <- date_start <- NULL
    year <- type_of_violence <- stratum <- NULL

    if (is.null(ucdp_ged) | is.null(worldpop)) {
      return(NULL)
    }

    # filter ucdp
    ucdp_ged <- ucdp_ged[[1]]
    ucdp_ged <- ucdp_ged[unlist(st_contains(x, ucdp_ged)), ]

    if (nrow(ucdp_ged) == 0) {
      return(NULL)
    }

    ucdp_yearly <- ucdp_ged %>%
      dplyr::select(
        date_prec,
        where_prec,
        date_start,
        type_of_violence
      ) %>%
      dplyr::filter(
        where_prec <= precision_location,
        date_prec <= precision_time,
        type_of_violence %in% violence_types
      ) %>%
      dplyr::select(date_start, type_of_violence) %>%
      dplyr::mutate(
        year = format(as.Date(date_start), "%Y"),
        type_of_violence = as.integer(type_of_violence)
      )

    distance_df <- data.frame(
      type_of_violence = violence_types,
      distance = distance
    )
    ucdp_yearly <- dplyr::left_join(ucdp_yearly, distance_df, by = "type_of_violence")

    ucdp_yearly <- dplyr::mutate(
      ucdp_yearly,
      stratum = dplyr::case_when(
        type_of_violence == 1 ~ "state-based",
        type_of_violence == 2 ~ "non-state",
        type_of_violence == 3 ~ "one-sided"
      )
    ) %>%
      dplyr::select(year, distance, stratum)

    ucdp_yearly <- st_buffer(ucdp_yearly, ucdp_yearly$distance)

    # one multi-polygon per year and stratum
    ucdp_yearly <- dplyr::select(ucdp_yearly, year, stratum) %>%
      dplyr::group_by(year, stratum) %>%
      dplyr::summarise(geom = st_intersection(st_make_valid(st_union(geom)), x))

    # one multi-polygon per year, irrespective of stratum (total exposed pop)
    ucdp_yearly_total <- dplyr::summarise(ucdp_yearly, geom = st_intersection(st_make_valid(st_union(geom)), x))
    ucdp_yearly_total$stratum <- "total"
    ucdp_yearly <- rbind(ucdp_yearly, ucdp_yearly_total)

    .calc_exp_pop(
      ucdp_yearly,
      worldpop,
      years
    )
  }
}


register_indicator(
  name = "exposed_population_ucdp",
  description = "Number of people exposed to conflicts based on UCDP GED",
  resources = c("ucdp_ged", "worldpop")
)

#' Calculate number of fatalities of conflict events from ACLED
#'
#' The indicator aggregated the number of fatalities within a given asset
#' on a monthly cadence stratified either by event type, sub-event type or disorder
#' type. To learn about the different categorisation ACLED uses to encode events
#' please consult ACLED's codebook.
#'
#' The required resources for this indicator are:
#'  - [acled]
#'
#' You may apply quality filters based on the precision of the geolocation
#' of events and the temporal precision. By default, these are set to only
#' include events with the highest precision scores.
#'
#' For geo-precision there are levels 1 to 3 with decreasing accuracy:
#' - value 1: the source reporting indicates a particular town, and coordinates
#'   are available for that town
#' - value 2: the source material indicates that activity took place in a
#'   small part of a region, and mentions a general area or if an activity
#'   occurs near a town or a city, the event is coded to a town with
#'   geo-referenced coordinates to represent that area
#' - value 3: a larger region is mentioned, the closest natural location noted
#'   in reporting (like “border area,” “forest,” or “sea,” among others) – or a
#'   provincial capital is used if no other information at all is available
#'
#' For temporal precision there are levels 1 to 3 with decreasing precision:
#' - value 1: the source material includes an actual date of an event
#' - value 2: the source material indicates that an event happened sometime
#'   during the week or within a similar period of time
#' - value 3: the source material only indicates that an event took place
#'   sometime during a month (i.e. in the past two or three weeks, or in
#'   January), without reference to the particular date, the month mid-point
#'   is chosen
#'
#' @name fatalities_acled
#' @param years A numeric vector indicating the years for which to summarize
#'    fatalities.
#' @param stratum A character vector indicating the stratification to be applied.
#'   Should be one of "event_type", "sub_event_type", or "disorder_type".
#'   Defaults to "event_type".
#' @param precision_location A numeric indicating precision value for the
#'   geolocation up to which events are included. Defaults to 1.
#' @param precision_time A numeric indicating the precision value of the
#'   temporal coding up to which events are included. Defaults to 1.
#' @keywords indicator
#' @returns A function that returns an indicator tibble with the type of
#'   violence as variable and counts of civilian fatalities as value.
#' @references Raleigh, C., Kishi, R. & Linke, A. Political instability patterns
#'   are obscured by conflict dataset scope conditions, sources, and coding
#'   choices. Humanit Soc Sci Commun 10, 74 (2023).
#'   \doi{doi:10.1057/s41599-023-01559-4}
#' @include register.R
#' @export
#' @examples
#' \dontshow{
#' mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
#' }
#' \dontrun{
#' library(sf)
#' library(mapme.biodiversity)
#'
#' outdir <- file.path(tempdir(), "mapme-data")
#' dir.create(outdir, showWarnings = FALSE)
#'
#' mapme_options(
#'   outdir = outdir,
#'   verbose = FALSE,
#'   chunk_size = 1e8
#' )
#'
#' aoi <- system.file("extdata", "burundi.gpkg",
#'   package = "mapme.biodiversity"
#' ) %>%
#'   read_sf() %>%
#'   get_resources(get_acled(years = 2020)) %>%
#'   calc_indicators(
#'     calc_fatalities_acled(
#'       years = 2020,
#'       precision_location = 1,
#'       precision_time = 1
#'     )
#'   ) %>%
#'   portfolio_long()
#'
#' aoi
#' }
calc_fatalities_acled <- function(
    years = 2000,
    stratum = c("event_type", "sub_event_type", "disorder_type"),
    precision_location = 1,
    precision_time = 1) {
  years <- check_available_years(years, c(1997:as.integer(format(Sys.Date(), "%Y"))), "acled")
  stratum <- match.arg(stratum)

  if (!precision_location %in% 1:3) {
    stop("Argument precision_location must be a single numeric between 1 and 3.")
  }

  if (!precision_time %in% 1:3) {
    stop("Argument precision_time must be a single numeric between 1 and 3.")
  }

  function(x,
           acled = NULL,
           name = "fatalities_acled",
           mode = "asset",
           aggregation = "sum",
           verbose = mapme_options()[["verbose"]]) {
    time_precision <- geo_precision <- event_date <- year <- month <- NULL

    if (is.null(acled)) {
      return(NULL)
    }

    acled <- st_sf(purrr::list_rbind(acled))
    acled <- acled[unlist(st_contains(x, acled)), ]
    acled <- dplyr::filter(acled, year %in% years)

    if (nrow(acled) == 0) {
      return(NULL)
    }

    acled <- dplyr::select(
      st_drop_geometry(acled),
      event_date,
      time_precision,
      geo_precision,
      fatalities,
      stratum = !!stratum
    )

    acled <- dplyr::filter(
      acled,
      time_precision <= precision_location,
      geo_precision <= precision_time
    )

    acled <- dplyr::mutate(
      acled,
      event_date = as.Date(event_date),
      year = format(event_date, "%Y")
    )

    fatalities <- dplyr::ungroup(
      dplyr::summarise(
        dplyr::group_by(
          acled,
          year, stratum,
        ),
        fatalities = sum(as.numeric(fatalities), na.rm = TRUE)
      )
    )

    fatalities_total <- fatalities %>%
      dplyr::group_by(year) %>%
      dplyr::summarise(fatalities = sum(as.numeric(fatalities), na.rm = TRUE)) %>%
      dplyr::ungroup() %>%
      dplyr::mutate(stratum = "total")

    fatalities <- rbind(fatalities, fatalities_total)

    fatalities <- dplyr::mutate(
      fatalities,
      datetime = as.Date(paste0(year, "-01-01"), format = "%Y-%m-%d"),
      variable = paste0("fatalities_", gsub(" ", "_", tolower(stratum))),
      unit = "count",
      value = fatalities
    )

    dplyr::select(fatalities, datetime, variable, unit, value)
  }
}


register_indicator(
  name = "fatalities_acled",
  description = "Number of fatalities by event type based on ACLED.",
  resources = "acled"
)

#' Calculate Global Surface Water (GSW) Occurrence
#'
#' GSW occurrence raw data comes in raster files with integer cell values
#' between `[0, 100]`. This value gives the percentage of the time that a
#' given pixel was classified as water during the entire observation period. So
#' a 0 denotes a pixel that was never classified as water, 100 denotes a pixel
#' with permanent water.
#'
#' The raw data values are aggregated based on a provided threshold parameter
#' `min_occurrence`, the function returns the area covered by values
#' greater or equal than this threshold.
#'
#' The required resources for this indicator are:
#'  - [global_surface_water_occurrence]
#'
#' @name gsw_occurrence
#' @param engine The preferred processing functions from either one of "zonal",
#' "extract" or "exactextract". Default: "extract".
#' @param min_occurrence Threshold to define which pixels count towards the GSW
#' occurrence area `[0, 100]`.
#' @keywords indicator
#' @returns A function that returns an indicator tibble with occurrence as
#'   variable and the corresponding area (in ha) as value.
#' @include register.R
#' @export
#' @examples
#' \dontshow{
#' mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
#' }
#' \dontrun{
#' library(sf)
#' library(mapme.biodiversity)
#'
#' outdir <- file.path(tempdir(), "mapme-data")
#' dir.create(outdir, showWarnings = FALSE)
#'
#' mapme_options(
#'   outdir = outdir,
#'   verbose = FALSE
#' )
#'
#' aoi <- system.file("extdata", "shell_beach_protected_area_41057_B.gpkg",
#'   package = "mapme.biodiversity"
#' ) %>%
#'   read_sf() %>%
#'   get_resources(get_global_surface_water_occurrence()) %>%
#'   calc_indicators(
#'     calc_gsw_occurrence(engine = "extract", min_occurrence = 10)
#'   ) %>%
#'   portfolio_long()
#'
#' aoi
#' }
calc_gsw_occurrence <- function(engine = "extract", min_occurrence = NULL) {
  engine <- check_engine(engine)
  stopifnot(
    !is.null(min_occurrence),
    !is.na(min_occurrence),
    is.numeric(min_occurrence),
    min_occurrence >= 0 & min_occurrence <= 100
  )

  function(x,
           global_surface_water_occurrence = NULL,
           name = "gsw_occurrence",
           mode = "asset",
           aggregation = "sum",
           verbose = mapme_options()[["verbose"]]) {
    if (is.null(global_surface_water_occurrence)) {
      return(NULL)
    }

    rcl <- matrix(
      c(min_occurrence, 100, 1),
      ncol = 3
    )

    global_surface_water_occurrence <- terra::classify(
      x = global_surface_water_occurrence,
      rcl = rcl,
      include.lowest = TRUE,
      right = NA,
      others = NA
    )

    global_surface_water_occurrence <- terra::cellSize(
      global_surface_water_occurrence,
      mask = TRUE,
      unit = "ha"
    )

    results <- select_engine(
      x = x,
      raster = global_surface_water_occurrence,
      stats = "sum",
      engine = engine,
      name = "gsw_occurrence",
      mode = "asset"
    )

    results %>%
      tidyr::pivot_longer(cols = dplyr::everything(), names_to = "variable") %>%
      dplyr::mutate(
        variable = "gsw_occurrence",
        datetime = as.POSIXct("2021-01-01T00:00:00Z"),
        unit = "ha"
      ) %>%
      dplyr::select(datetime, variable, unit, value)
  }
}

register_indicator(
  name = "gsw_occurrence",
  description = "Areal statistic of surface water based on occurrence threshold",
  resources = "global_surface_water_occurrence"
)

#' Drivers of deforestation for tropical forests
#'
#' This resource is produced by a nearest-neighbour matching of a crowd-sourced
#' campaign to map dominant driver of forest loss based on visual interpretation
#' of VHR images matched with Global Forest Loss data by Hansen (2013) version
#' 1.7 The forest loss layer was re sampled to a resolution of 100 and 1.000
#' meters. Dominant drivers were determined for the period 2008 to 2009.
#'
#' It indicates 9 different classes:
#' - commercial agriculture
#' - commercial oil palm plantations
#' - managed forests
#' - mining
#' - natural disturbances
#' - pasture
#' - roads
#' - wildfire
#' - other subsistence agriculture
#' - shifting cultivation
#'
#' @name fritz_et_al
#' @param resolution An integer indicating the resolution
#'   to download. Defaults to 100.
#' @keywords resource
#' @returns A function that returns an `sf` footprint object.
#' @references Steffen, F., Carlos, J.C.L., See. L., Schepaschenko D.,
#' Hofhansl F., Jung M., Dürauer M., Georgieva I., Danylo O., Lesiv M.,
#' McCallum I. (2022) A Continental Assessment of the Drivers of Tropical
#' Deforestation With a Focus on Protected Areas. F.Cos.Sc.(3)
#' doi:10.3389/fcosc.2022.830248
#' @source \url{https://zenodo.org/record/7997885}
#' @include register.R
#' @export
get_fritz_et_al <- function(resolution = 100) {
  if (!resolution %in% c(100, 1000)) {
    stop("Fritz et al. resource is available only at resolutions 100 and 1.000.")
  }

  function(x,
           name = "fritz_et_al",
           type = "raster",
           outdir = mapme_options()[["outdir"]],
           verbose = mapme_options()[["verbose"]]) {
    urls <- c(
      "/vsizip//vsicurl/https://zenodo.org/record/7997885/files/Deforestation%20Drivers%20100m%20IIASA.zip/dr_han_hei_pr1.tif",
      "/vsizip//vsicurl/https://zenodo.org/record/7997945/files/Deforestation%20drivers%201km%20IIASA%20.zip/dr_heine_pr1.tif"
    )

    url <- ifelse(resolution == 100, urls[1], urls[2])
    filename <- paste0("geo_fritz_et_al_", resolution, "m.tif")
    co <- c("-of", "COG", "-ot", "Byte", "-co", "COMPRESS=LZW")
    bbox <- c(xmin = -20037507, ymin = -3341359, xmax = 20038000, ymax = 3340559)
    tile <- st_as_sf(st_as_sfc(st_bbox(bbox, crs = "ESRI:54052")))
    tile[["source"]] <- url
    make_footprints(tile, filename, what = "raster", co = co)
  }
}

register_resource(
  name = "fritz_et_al",
  licence = "CC-BY 4.0",
  description = "Drivers of deforestation in the tropics",
  source = "https://zenodo.org/record/7997885/",
  type = "raster"
)

#' Calculate Zonal Soil Properties
#'
#' This indicator allows the extraction of zonal statistics for resource layers
#' previously downloaded from SoilGrids, thus in total supporting the calculation
#' of zonal statistics for 10 different soil properties at 6 different depths for
#' a total of 4 different model outputs (stat). Zonal statistics will be calculated
#' for all SoilGrid layers that have been previously made available vie \code{get_resources()}.
#'
#' The required resource for this indicator is:
#'  - [soilgrids]
#'
#' @name soilproperties
#' @param engine The preferred processing functions from either one of "zonal",
#'   "extract" or "exactextract" as character.
#' @param stats Function to be applied to compute statistics for polygons either
#'   single or multiple inputs as character. Supported statistics are: "mean",
#'   "median", "sd", "min", "max", "sum" "var".
#' @keywords indicator
#' @returns A function that returns an indicator tibble with soilgrid layers and
#'   statistics as variables and the corresponding statistics as value.
#' @include register.R
#' @export
#' @examples
#' if (FALSE) {
#'   library(sf)
#'   library(mapme.biodiversity)
#'
#'   mapme_options(
#'     outdir = NULL,
#'     verbose = FALSE
#'   )
#'
#'   aoi <- system.file("extdata", "sierra_de_neiba_478140_2.gpkg",
#'     package = "mapme.biodiversity"
#'   ) %>%
#'     read_sf() %>%
#'     get_resources(
#'       get_soilgrids(
#'         layers = "clay",
#'         depths = "0-5cm",
#'         stats = "mean"
#'       )
#'     ) %>%
#'     calc_indicators(
#'       calc_soilproperties(engine = "extract", stats = c("mean", "median"))
#'     ) %>%
#'     portfolio_long()
#'
#'   aoi
#' }
calc_soilproperties <- function(engine = "extract", stats = "mean") {
  engine <- check_engine(engine)
  stats <- check_stats(stats)

  function(x,
           soilgrids = NULL,
           name = "soilproperties",
           mode = "asset",
           aggregation = "stat",
           verbose = mapme_options()[["verbose"]]) {
    layer <- stat <- conversion_factor <- conventional_units <- NULL
    # check if input engines are correct
    if (is.null(soilgrids)) {
      return(NULL)
    }
    results <- select_engine(
      x = x,
      raster = soilgrids,
      stats = stats,
      engine = engine,
      mode = "asset"
    )

    layers <- gsub("-", "_", gsub(".tif", "", names(soilgrids)))
    results[["variable"]] <- layers
    results[["layer"]] <- purrr::map_chr(layers, function(lyr) strsplit(lyr, "_")[[1]][1])
    results <- tidyr::pivot_longer(results, -c(variable, layer), names_to = "stat", values_to = "value")

    conv_df <- purrr::map(.sg_layers, tibble::as_tibble) %>% purrr::list_rbind()
    conv_df[["layer"]] <- names(.sg_layers)

    results <- tibble::tibble(merge(results, conv_df))

    results %>%
      dplyr::mutate(
        value = value / conversion_factor,
        variable = paste0(variable, "_", stat),
        datetime = as.POSIXct("2017-02-01T00:00:00Z")
      ) %>%
      dplyr::select(datetime, variable, unit = conventional_units, value = value)
  }
}

register_indicator(
  name = "soilproperties",
  description = "Statistics of SoilGrids layers",
  resources = "soilgrids"
)

#' Calculate precipitation statistics
#'
#' This function allows to efficiently calculate precipitation statistics
#' from Worldclim for polygons. For each polygon, the desired statistic/s (min,
#' max, sum, mean, median, sd or var) is/are returned.
#'
#' The required resources for this indicator are:
#'  - precipitation layer from [worldclim_precipitation]
#'
#' @name precipitation_wc
#' @param engine The preferred processing functions from either one of "zonal",
#'   "extract" or "exactextract" as character.
#' @param stats Function to be applied to compute statistics for polygons either
#'   single or multiple inputs as character. Supported statistics are: "mean",
#'   "median", "sd", "min", "max", "sum" "var".
#' @docType data
#' @keywords indicator
#' @returns A function that returns an indicator tibble with precipitation
#'   statistics as variable and corresponding values as value.
#' @include register.R
#' @export
#' @examples
#' \dontshow{
#' mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
#' }
#' \dontrun{
#' library(sf)
#' library(mapme.biodiversity)
#'
#' outdir <- file.path(tempdir(), "mapme-data")
#' dir.create(outdir, showWarnings = FALSE)
#'
#' mapme_options(
#'   outdir = outdir,
#'   verbose = FALSE
#' )
#'
#' aoi <- system.file("extdata", "sierra_de_neiba_478140_2.gpkg",
#'   package = "mapme.biodiversity"
#' ) %>%
#'   read_sf() %>%
#'   get_resources(get_worldclim_precipitation(years = 2018)) %>%
#'   calc_indicators(
#'     calc_precipitation_wc(
#'       engine = "extract",
#'       stats = c("mean", "median")
#'     )
#'   ) %>%
#'   portfolio_long()
#'
#' aoi
#' }
calc_precipitation_wc <- function(engine = "extract", stats = "mean") {
  engine <- check_engine(engine)
  stats <- check_stats(stats)

  function(x,
           worldclim_precipitation = NULL,
           name = "precipitation_wc",
           mode = "asset",
           aggregation = "stat",
           verbose = mapme_options()[["verbose"]]) {
    if (is.null(worldclim_precipitation)) {
      return(NULL)
    }

    .calc_worldclim(
      x = x,
      worldclim = worldclim_precipitation,
      engine = engine,
      stats = stats,
      unit = "mm"
    )
  }
}

register_indicator(
  name = "precipitation_wc",
  description = "Statistics of WorldClim precipitation layer",
  resources = "worldclim_precipitation"
)

#' Biodiversity Intactness Index
#'
#' The variable is the modelled average abundance of originally-present species,
#' relative to their abundance in an intact ecosystem. Please refer to
#' Newbold et al. (2016) for all details, and please cite it when using these
#' data.
#'
#' To use this data in mapme workflows, you will have to manually download the
#' global data set and point towards the file path on your local machine.
#' Please find the available data under the source link given below.
#'
#' @name biodiversity_intactness_index_resource
#' @keywords resource
#' @param path A character vector to the biodiversity intactness index ASCII file.
#' @returns A function that returns an `sf` footprints object.
#' @references Tim Newbold; Lawrence Hudson; Andy Arnell; Sara Contu et al.
#'   (2016). Global map of the Biodiversity Intactness Index, from Newbold et al.
#'   (2016) Science \[Data set\]. Natural History Museum.
#'   \doi{doi:10.5519/0009936}
#' @include register.R
#' @export
get_biodiversity_intactness_index <- function(path = NULL) {
  if (is.null(path) || !endsWith(path, ".asc") || !spds_exists(path, what = "raster")) {
    stop("Expecting path to point towards an existing '.asc' file.")
  }

  function(
      x,
      name = "biodiversity_intactness_index",
      type = "raster",
      outdir = mapme_options()[["outdir"]],
      verbose = mapme_options()[["verbose"]]) {
    filename <- gsub("asc", "tif", basename(path))
    fps <- make_footprints(
      path,
      filename,
      what = "raster",
      co = c("-co", "INTERLEAVE=BAND", "-co", "COMPRESS=LZW", "-ot", "Float32")
    )
    st_crs(fps) <- st_crs("EPSG:4326")
    fps
  }
}

register_resource(
  name = "biodiversity_intactness_index",
  description = "Biodiversity Intactness Index",
  licence = "CC-BY-4.0",
  source = "https://data.nhm.ac.uk/dataset/global-map-of-the-biodiversity-intactness-index-from-newbold-et-al-2016-science",
  type = "raster"
)

#' Calculate mangrove extent based on Global Mangrove Watch (GMW)
#'
#' This function allows to efficiently calculate area of mangrove from
#' Global Mangrove Watch - World Conservation Monitoring Centre (WCMC)
#' for polygons. For each polygon, the area of the mangrove (in hectare)
#' for desired year is returned.
#'
#' The required resources for this indicator are:
#'  - [gmw]
#'
#' @name mangroves_area
#' @docType data
#' @keywords indicator
#' @returns A function that returns an indicator tibble with mangroves as variable
#'   and corresponding areas (in ha) as value.
#' @include register.R
#' @export
#' @examples
#' \dontshow{
#' mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
#' }
#' \dontrun{
#' library(sf)
#' library(mapme.biodiversity)
#'
#' outdir <- file.path(tempdir(), "mapme-data")
#' dir.create(outdir, showWarnings = FALSE)
#'
#' mapme_options(
#'   outdir = outdir,
#'   verbose = FALSE
#' )
#'
#' aoi <- system.file("extdata", "shell_beach_protected_area_41057_B.gpkg",
#'   package = "mapme.biodiversity"
#' ) %>%
#'   read_sf() %>%
#'   get_resources(get_gmw(years = c(1996, 2016))) %>%
#'   calc_indicators(calc_mangroves_area()) %>%
#'   portfolio_long()
#'
#' aoi
#' }
calc_mangroves_area <- function() {
  function(x = NULL,
           gmw = NULL,
           name = "mangroves_area",
           mode = "asset",
           aggregation = "sum",
           verbose = mapme_options()[["verbose"]]) {
    if (is.null(gmw)) {
      return(NULL)
    }

    results <- purrr::map(1:length(gmw), function(j) {
      mangroves <- st_make_valid(gmw[[j]])
      intersected <- suppressWarnings(st_intersection(mangroves, x))
      intersected <- st_make_valid(intersected)
      intersected <- intersected[st_is_valid(intersected), ]
      area <- sum(as.numeric(st_area(intersected)), na.rm = TRUE) / 10000
      year <- strsplit(names(gmw[j]), "_|.gpkg")[[1]][3]

      tibble::tibble(
        datetime = as.POSIXct(paste0(year, "-01-01T00:00:00Z")),
        variable = "mangroves",
        unit = "ha",
        value = area
      )
    }) %>%
      purrr::list_rbind()
  }
}

register_indicator(
  name = "mangroves_area",
  description = "Area covered by mangroves",
  resources = "gmw"
)

#' Global Mangrove Extent Polygon
#'
#' This resource is part of the publication by Bunting et al. (2018)
#' "The Global Mangrove Watch—A New 2010 Global Baseline of Mangrove Extent".
#' The polygons represent the mangrove, which is tropical coastal vegetation
#' and considered the most significant part of the marine ecosystem. This
#' resource is available for the selected years in the period 1996- 2020 from
#' Global Mangrove Watch (GMW), providing geospatial information about global
#' mangrove extent.
#'
#'
#' @name gmw
#' @param years A numeric vector of the years for which to make GMW available.
#' @docType data
#' @keywords resource
#' @returns A function that returns an `sf` footprint object.
#' @references Bunting P., Rosenqvist A., Lucas R., Rebelo L-M., Hilarides L.,
#' Thomas N., Hardy A., Itoh T., Shimada M. and Finlayson C.M. (2018). The Global
#' Mangrove Watch – a New 2010 Global Baseline of Mangrove Extent. Remote Sensing
#' 10(10): 1669. doi:10.3390/rs10101669.
#' @source \url{https://habitats.oceanplus.org/}
#' @include register.R
#' @export
get_gmw <- function(years = c(1996, 2007:2010, 2015:2020)) {
  avail_years <- c(1996, 2007:2010, 2015:2020)
  years <- check_available_years(years, avail_years, "gmw")

  function(x,
           name = "gmw",
           type = "vector",
           outdir = mapme_options()[["outdir"]],
           verbose = mapme_options()[["verbose"]]) {
    urls <- unlist(sapply(years, function(year) .get_mangrove_url(year)))
    bbox <- c(xmin = -180., ymin = -38.85822, xmax = 180., ymax = 32.36822)
    fps <- st_as_sfc(st_bbox(bbox, crs = "EPSG:4326"))
    fps <- st_as_sf(rep(fps, length(urls)))
    fps[["source"]] <- urls
    fps <- make_footprints(
      fps,
      filenames = gsub(".shp", ".gpkg", basename(urls)),
      what = "vector"
    )

    if (2018 %in% years) {
      i <- which(years == 2018)
      fps[["co"]][i] <- list(c("-t_srs", "EPSG:4326"))
      fps[["filename"]][i] <- "gmw_v3_2018_vec.gpkg"
    }

    fps
  }
}

#' A helper function to construct correct mangrove layer urls
#'
#' @param target_year A numeric indicating the target year
#'
#' @return A character vector
#' @keywords internal
#' @noRd
.get_mangrove_url <- function(target_year) {
  base_url <- paste0("/vsizip//vsicurl/https://datadownload-production.s3.amazonaws.com/GMW_v3_", target_year, ".zip")
  path <- switch(as.character(target_year),
    "2018" = paste0("GMW_v3_2018/00_Data/gmw_v3_2018.shp"),
    paste0("gmw_v3_", target_year, "_vec.shp")
  )
  file.path(base_url, path)
}


register_resource(
  name = "gmw",
  description = "Global Mangrove Watch - Vector data of mangrove extent",
  licence = "CC BY 4.0",
  source = "https://data.unep-wcmc.org/datasets/45",
  type = "vector"
)

---
title: "Quickstart"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Quickstart}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  chunk_output_type: console
  markdown: 
    wrap: 72
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

# Introduction

In the following we will demonstrate an idealized workflow based on a
subset of the Global Forest Watch (GFW) data set that is delivered
together with this package. You can follow along the code snippets below
to reproduce the results. Please note that to reduce the time it takes
to process this vignette, we will not download any resources from the
internet. In a real use case, thus processing time might substantially
increase because resources have to be downloaded and real portfolios
might be larger than the one created in this example.

This vignette assumes that you have already followed the steps in
[Installation](https://mapme-initiative.github.io/mapme.biodiversity/articles/installation.html)
and have familiarized yourself with the terminology used in the package.
If you are unfamiliar with the terminology used here, please head over
to the
[Terminology](https://mapme-initiative.github.io/mapme.biodiversity/articles/terminology.html)
article to learn about the most important concepts.

The idealized workflow for using `{mapme.biodiversity}` consists of the
following steps:

-   prepare your sf-object containing only geometries of type
    `'POLYGON'` or `'MULTIPOLYGON'`
-   decide which indicator(s) you wish to calculate and make the
    required resource(s) available
-   conduct your indicator calculation, which adds a nested list column
    to your portfolio object
-   continue your analysis in R or decide to export your results to a
    spatial data format to use it with other geospatial software

# Getting started

First, we will load the `{mapme.biodiversity}` and the `{sf}` package
for handling spatial vector data. For tabular data handling, we will
also load the `{dplyr}` and `{tidyr}` packages. Then, we will read an
internal GeoPackage which includes part of the geometry of a protected
area in the Dominican Republic from the WDPA database.

```{r setup, message=FALSE}
library(mapme.biodiversity)
library(sf)
library(dplyr)
library(tidyr)

aoi_path <- system.file("extdata", "gfw_sample.gpkg", package = "mapme.biodiversity")
aoi <- st_read(aoi_path, quiet = TRUE)
aoi
```

```{r simplify-aoi, echo = FALSE}
aoi <- st_simplify(aoi, preserveTopology = TRUE, dTolerance = 500)
```

# Setting standard option

We use the `mapme_options()` function and set some arguments, such as
the output directory, that are important to govern the subsequent
processing. For this, we create a temporary directory. Internally, to
save time on downloading when building this vignette, we copied already
existing files to that output location (code not shown here).

```{r init-data-dir, echo=FALSE}
outdir <- file.path(tempdir(), "mapme-resources")
mapme.biodiversity:::.copy_resource_dir(outdir)
```

```{r portfolio-opts}
outdir <- file.path(tempdir(), "mapme-resources")
dir.create(outdir, showWarnings = FALSE)

mapme_options(
  outdir = outdir,
  verbose = TRUE
)
```

The `outdir` argument points towards a directory on the local file
system of your machine. All downloaded resources will be written to
respective directories nested within `outdir`.

Once you request a specific resource for your portfolio, only those
files will be downloaded that are missing to match its spatio-temporal
extent. This behaviour is beneficial, e.g. in case you share the `outdir`
between different projects to ensure that only resources matching your
current portfolio are returned.

The `verbose` logical controls whether or not the package will print
informative messages during the calculations. Note, that even if set to
`FALSE`, the package will inform users about any potential errors or
warnings.

# Getting the right resources

You can check which indicators are available via the
`available_indicators()` function:

```{r query-indicator}
available_indicators()
available_indicators("treecover_area")
```

Say, we are interested in the `treecover_area` indicator. We can learn
more about this indicator and its required resources by using either of
the commands below or, if you are viewing the online version, head over
to the
[treecover_area](https://mapme-initiative.github.io/mapme.biodiversity/reference/treecover_area.html)
documentation.

```{r help-indicator, eval = FALSE}
?treecover_area
help(treecover_area)
```

By inspecting the help page we learned that this indicator requires the
`gfw_treecover` and `gfw_lossyear` resources and it requires to specify
three extra arguments: the years for which to calculate treecover, the
minimum size of patches to be considered as forest and the minimum
canopy coverage of a single pixel to be considered as forested.

With that information at hand, we can start to retrieve the required
resource. We can learn about all available resources using the
`available_resources()` function:

```{r query-resources}
available_resources()
available_resources("gfw_treecover")
```

For the purpose of this vignette, we are going to download both, the
`gfw_treecover` and `gfw_lossyear` resources. We can get more detailed
information about a given resource, by using either of the commands
below to open up the help page. If you are viewing the online version of
this documentation, you can simply head over to the
[gfw_treecover](https://mapme-initiative.github.io/mapme.biodiversity/reference/gfw_treecover.html)
resource documentation.

```{r help-resource, eval = FALSE}
?gfw_treecover
help(gfw_treecover)
?gfw_lossyear
help(gfw_lossyear)
```

We can now make the required resources available for our portfolio. We
will use a common interface that is used for all resources, called
`get_resources()`. We have to specify our portfolio object and supply
one or more resource functions with their respective arguments. This
will then download the matching resources to the output directory
specified earlier.

```{r get-gfw}
aoi <- get_resources(
  x = aoi,
  get_gfw_treecover(version = "GFC-2023-v1.11"),
  get_gfw_lossyear(version = "GFC-2023-v1.11")
)
```

# Calculate specific indicators

The next step consists of calculating specific indicators. Note that
each indicator requires one or more resources that were made available
via the `get_resources()` function explained above. You will have to
re-run this function in every new R session, but note that data that is
already available will not be re-downloaded.

Here, we are going to calculate the `treecover_area` indicator which is
based on the resources from GFW. Since the resources have been made
available in the previous step, we can continue requesting the
calculation of our desired indicator. Note the command below would issue
an error in case a required resource has not been made available via
`get_resources()` beforehand.

```{r calc-indicator}
aoi <- calc_indicators(
  aoi,
  calc_treecover_area(years = 2000:2023, min_size = 1, min_cover = 30)
)
```

Now let's take a look at the results. In addition to the metadata we are
already familiar with, we see that there is an additional column called
`treecover_area` which contains a `tibble`.

```{r print-aoi}
aoi
```

The indicator is represented as a nested-list column in our `sf`-object
that is named alike the requested indicator. For our single asset, this
column contains a tibble with `r nrow(aoi$treecover_area[[1]])` rows and
four columns. Let's have a closer look at this object

```{r print-indicator}
aoi$treecover_area
```

The tibble follows a standard output format, which is the same for all
indicators. Each indicator is represented as a tibble with the four
columns `datetime`, `variable`, `unit`, and `value`. In case of the
`treecover_area` indicator, the variable is called `treecover` and is
expressed in `ha`.

Let's quickly visualize the results:

```{r plot-treecover, echo = FALSE, warning=FALSE}
mapme_options(verbose = FALSE)
data <- portfolio_long(aoi)

plot(value ~ datetime, data,
  main = "Treecover over time",
  xlab = "Year", ylab = "ha",
  pch = 16, col = "steelblue"
)
```

If you wish to change the layout of an portfolio, you can use
`portfolio_long()` and `portfolio_wide()` (see the respective [online
tutorial](https://mapme-initiative.github.io/mapme.biodiversity/articles/output-wide.html)).
Especially for large portfolios, it is usually a good idea to keep the
geometry information in a separated variable to keep the size of the
data object relatively small.

```{r long-drop-geoms}
geoms <- st_geometry(aoi)
portfolio_long(aoi, drop_geoms = TRUE)
```

## A note on parallel computing

`{mapme.biodiversity}` follows the parallel computing paradigm of the
[`{future}`](https://cran.r-project.org/package=future) package. That
means that you as a user are in the control if and how you would like to
set up parallel processing. Since `{mapme.biodiversity} v0.9`, we apply
pre-chunking to all assets in the portfolio. That means that assets are
split up into components of roughly the size of `chunk_size`. These
components can than be iterated over in parallel to speed up processing.
Indicator values will be aggregated automatically.

```{r parallel-1, eval = FALSE}
library(future)
plan(cluster, workers = 6)
```

As another example, with the code below one would apply parallel
processing of 2 assets, with each having 4 workers available to process
chunks, thus requiring a total of 8 available cores on the host machine.
Be sure to not request more workers than available on your machine.

```{r parallel, eval = FALSE}
library(progressr)

plan(cluster, workers = 2)

with_progress({
  aoi <- calc_indicators(
    aoi,
    calc_treecover_area_and_emissions(
      min_size = 1,
      min_cover = 30
    )
  )
})

plan(sequential) # close child processes
```

# Exporting an portfolio object

You can use the `write_portfolio()` function to save a processed
portfolio object to disk as a `GeoPackage`. This allows sharing your
data with contributors who might not be using R, but any other
geospatial software. Simply point towards a non-existing file on your
local disk to write the portfolio. You can use `read_portfolio()` to
read back a GeoPackage written in such a way into R:

```{r write-portfolio}
dsn <- tempfile(fileext = ".gpkg")
write_portfolio(x = aoi, dsn = dsn, quiet = TRUE)
from_disk <- read_portfolio(dsn, quiet = TRUE)
from_disk
```

```{r delete-dsn, echo=FALSE}
file.remove(dsn)
```

---
title: "Terminology"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Terminology}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 8,
  fig.height = 5
)
```

Here we present a quick introduction to the most important terminologies and concepts
used throughout this tutorial:

- **Portfolio**: We define a portfolio as
the collection of single assets in an `{sf}` object. The package
does only allow assets with geometries of type `'POLYGON'` or `'MULTIPOLYGON'` 
to be included in the object.
We expect the data to be in geodetic, i.e. unprojected, coordinates. 
In case the CRS of the portfolio is projected, we will attempt to transform to
`EPSG:4326`. Any metadata that is needed for your analysis is allowed
in the columns of the object and will be retained throughout the workflow so that
you can access it once you have calculated your indicators.
The package will add a *nested list columns* for each indicator to the portfolio
`sf`-object. Portfolio-wide arguments that govern the processing workflow, 
are set using the `mapme_options()` config functions.

- **Resources**: A resource is a supported dataset that can be downloaded via the 
`get_resources()` function. A resource represents either a raster or a 
vector data set. Vector datasets are saved to disk as
GeoPackages. Raster datasets can be tiled so that only tiles that cover the 
spatial extent of the portfolio will be downloaded. Global raster resources will
be downloaded completely. Additionally, some raster resources are time-invariant
(e.g. a `nasa_srtm`), others contain a temporal dimension (e.g. `esalandcover`). 
If a raster resource contains a temporal dimension, only those rasters
intersecting with user supplied time-frame will be downloaded. To learn about resources and
their arguments simply query the help-page for that resource (e.g. via `?gfw_treecover`).

- **Indicators**: An indicator calculates a tabular output for all
assets in a portfolio. Every indicator requires one or more resources to be
available locally and will inform you about any missing resources. 
Custom arguments might be set for some indicators, but sensible defaults 
have been set. The package will inform you about any misspecification of arguments. 
To learn more about a specific indicator and its required arguments, simply query 
the help-page (e.g. via `?treecover_area`). The output for indicators is 
standardized and is represented by a tibble with columns `datetime`, `variable`, 
`unit`, and `value`. Indicators are appended to the portfolio as
*nested list columns*. For assets where no sensible values can be calculated 
(e.g. for polygons over the ocean and the `treecover_area` indicator), `NULL` 
will be returned.

- **Engines**: Several indicators support different engines for the extraction of
the numeric outputs. Currently, these engines are based on either `terra::zonal()`,
`terra::extract()`, or `exactextractr::exact_extract()`. We took 
great care to set these engines to sensible defaults. However, to 
customize your workflow you can choose to use a different engine instead.

- **Processing Modes**: The processing mode is an important concept if you wish 
to extend the package with a new indicator, but not so much if you simply want to
use it for your statistical analysis. For most indicators the default processing 
mode is equal to `'asset'`, meaning that for each asset in the 
portfolio the required input resources are matched to the spatio-temporal extent 
of that specific asset and then the indicator calculation is conducted. For some 
resources/indicators this is not the most efficient approach. These are most 
commonly very coarse resolution raster resources where a cropping of many 
relatively small assets would lead to an increased overhead and as a result
very long processing times. For these indicators, the default processing mode is
set to `'portfolio'`, meaning that the indicator is first calculated for the
complete spatio-temporal extent of the portfolio and then the extraction is conducted
for every single asset. If you want to open a pull-request for a new indicator for
which neither of these two approaches yields satisfactory performance, please contact
the package maintainers to discuss your use-case in order to evaluate if another
processing mode can be added.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{check_available_years}
\alias{check_available_years}
\title{Helper to check yearly availability}
\usage{
check_available_years(target_years, available_years, indicator)
}
\arguments{
\item{target_years}{Numeric indicating the target year.}

\item{available_years}{Numeric indicating the available years.}

\item{indicator}{A character vector with target resource/indicator name.}
}
\description{
Use this function to check if a specified vector of years intersects
with the yearly availability of a resource.
}
\keyword{utils}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_nasa_srtm.R
\name{nasa_srtm}
\alias{nasa_srtm}
\alias{get_nasa_srtm}
\title{NASADEM HGT v001}
\source{
\url{https://planetarycomputer.microsoft.com/dataset/nasadem}
}
\usage{
get_nasa_srtm()
}
\value{
A function that returns an \code{sf} footprint object.
}
\description{
This resource is processed by the Land Processes Distributed Active Archive
Center (LP DAAC) and made available at the Microsoft Planetery Computer.
NASADEM are distributed in 1 degree latitude by 1 degree longitude tiles and
consist of all land between 60° N and 56° S latitude. This accounts for about
80\% of Earth’s total landmass.
}
\references{
NASA JPL (2020). NASADEM Merged DEM Global 1 arc second V001.
NASA EOSDIS Land Processes DAAC. Accessed 2023-07-01 from
\doi{doi:10.5067/MEaSUREs/NASADEM/NASADEM_HGT.001}
}
\keyword{resource}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_hfp.R
\name{humanfootprint_indicator}
\alias{humanfootprint_indicator}
\alias{calc_humanfootprint}
\title{Calculate human footprint statistics}
\usage{
calc_humanfootprint(engine = "extract", stats = "mean")
}
\arguments{
\item{engine}{The preferred processing functions from either one of "zonal",
"extract" or "exactextract" as character.}

\item{stats}{Function to be applied to compute statistics for polygons either
one or multiple inputs as character. Supported statistics are: "mean",
"median", "sd", "min", "max", "sum" "var".}
}
\value{
A function that returns an indicator tibble the \code{humanfootprint}
as variable and the associated value (unitless) per year.
}
\description{
Human footprint data measures the pressure imposed on the natural environment
by different dimensions of human actions. The theoretical maximum value,
representing the highest level of human pressure, is 50. This routine allows
to extract zonal statistics of the human footprint data.
}
\details{
The required resources for this indicator are:
\itemize{
\item \link{humanfootprint_resource}
}
}
\examples{
\dontshow{
mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
}
\dontrun{
library(sf)
library(mapme.biodiversity)

outdir <- file.path(tempdir(), "mapme-data")
dir.create(outdir, showWarnings = FALSE)

mapme_options(
  outdir = outdir,
  verbose = FALSE
)

aoi <- system.file("extdata", "shell_beach_protected_area_41057_B.gpkg",
  package = "mapme.biodiversity"
) \%>\%
  read_sf() \%>\%
  get_resources(get_humanfootprint(years = 2010)) \%>\%
  calc_indicators(calc_humanfootprint(stats = "median")) \%>\%
  portfolio_long()

aoi
}
}
\keyword{indicator}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_fritz_et_al.R
\name{fritz_et_al}
\alias{fritz_et_al}
\alias{get_fritz_et_al}
\title{Drivers of deforestation for tropical forests}
\source{
\url{https://zenodo.org/record/7997885}
}
\usage{
get_fritz_et_al(resolution = 100)
}
\arguments{
\item{resolution}{An integer indicating the resolution
to download. Defaults to 100.}
}
\value{
A function that returns an \code{sf} footprint object.
}
\description{
This resource is produced by a nearest-neighbour matching of a crowd-sourced
campaign to map dominant driver of forest loss based on visual interpretation
of VHR images matched with Global Forest Loss data by Hansen (2013) version
1.7 The forest loss layer was re sampled to a resolution of 100 and 1.000
meters. Dominant drivers were determined for the period 2008 to 2009.
}
\details{
It indicates 9 different classes:
\itemize{
\item commercial agriculture
\item commercial oil palm plantations
\item managed forests
\item mining
\item natural disturbances
\item pasture
\item roads
\item wildfire
\item other subsistence agriculture
\item shifting cultivation
}
}
\references{
Steffen, F., Carlos, J.C.L., See. L., Schepaschenko D.,
Hofhansl F., Jung M., Dürauer M., Georgieva I., Danylo O., Lesiv M.,
McCallum I. (2022) A Continental Assessment of the Drivers of Tropical
Deforestation With a Focus on Protected Areas. F.Cos.Sc.(3)
doi:10.3389/fcosc.2022.830248
}
\keyword{resource}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_carbon.R
\name{carbon_resources}
\alias{carbon_resources}
\alias{get_irr_carbon}
\alias{get_vul_carbon}
\alias{get_man_carbon}
\title{Carbon Layers}
\source{
\url{https://zenodo.org/records/4091029}
}
\usage{
get_irr_carbon()

get_vul_carbon()

get_man_carbon()
}
\value{
A function that returns an \code{sf} footprint object.
}
\description{
These resources are from the publication by Noon et al. (2022) "Mapping the
irrecoverable carbon in Earth’s ecosystems". This publication differentiates
between 3 different kinds of carbon with varying degrees of manageability
by humans. All three layers are available for above and below ground
carbon, as well as a layer combining the two.
}
\details{
It may be required to increase the timeout option to successfully download
theses layers from their source location via e.g. \code{options(timeout = 600)}.

Irrecoverable carbon is defined as the amount of carbon, that, if lost today,
cannot be recovered until mid 21st century (so within 30 years, considering
the publication date).

Vulnerable carbon is defined as the amount of carbon that would be lost in a
hypothetical but typical conversion event (without including information
of the probability of such an event to be actually occurring).

Manageable carbon is defined as all land areas, expect cyrosols, because
carbon loss is driven by direct land-use conversion which could be halted or
because climate change impacts affecting the area can potentially be directly
mitigated through adaptive management.
}
\references{
Noon, M.L., Goldstein, A., Ledezma, J.C. et al. Mapping the
irrecoverable carbon in Earth’s ecosystems. Nat Sustain 5, 37–46 (2022).
\doi{doi:10.1038/s41893-021-00803-6}
}
\keyword{resource}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_chirps.R
\name{chirps}
\alias{chirps}
\alias{get_chirps}
\title{Climate Hazards Group InfraRed Precipitation with Station data (CHIRPS)}
\source{
\url{https://data.chc.ucsb.edu/products/CHIRPS-2.0/global_monthly/cogs/}
}
\usage{
get_chirps(years = 1981:2020)
}
\arguments{
\item{years}{A numeric vector of the years to download CHIRPS precipitation
layers. Must be greater 1981, defaults to \code{c(1981:2020)}.}
}
\value{
A function that returns an \code{sf} footprint object.
}
\description{
This resource is published by Funk et al. (2015) and represents a quasi-global
(50°S-50°S) rainfall estimation at a monthly resolution starting with the year
1981 to the near-present. It has a spatial resolution of 0.05°. The data can
be used to retrieve information on the amount of rainfall. Due to the availability
of +30 years, anomaly detection and long-term average analysis is also possible.
The routine will download the complete archive in order to support long-term
average and anomaly calculations with respect to the 1981 - 2010 climate normal
period. Thus no additional arguments need to be specified.
}
\references{
Funk, C., Peterson, P., Landsfeld, M. et al. The climate hazards
infrared precipitation with stations—a new environmental record for
monitoring extremes. Sci Data 2, 150066 (2015).
\doi{10.1038/sdata.2015.66}
}
\keyword{resource}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_gsw_recurrence.R
\name{gsw_recurrence}
\alias{gsw_recurrence}
\alias{calc_gsw_recurrence}
\title{Calculate Global Surface Water (GSW) Recurrence}
\usage{
calc_gsw_recurrence(engine = "extract", min_recurrence = NULL)
}
\arguments{
\item{engine}{The preferred processing functions from either one of "zonal",
"extract" or "exactextract". Default: "extract".}

\item{min_recurrence}{Threshold to define which pixels count towards the GSW
recurrence area \verb{[0, 100]}.}
}
\value{
A function that returns an indicator tibble with recurrence as
variable and the corresponding area (in ha) as value.
}
\description{
Water Recurrence is a measurement of the degree of variability in the
presence of water from year to year. It describes the frequency with which
water returned to a particular location from one year to another, and is
expressed as a percentage. The raster files have integer cell values between
\verb{[0, 100]}, where 100 represents that water reoccurs predictably every year,
whereas lower values indicate that water only occurs episodically.
}
\details{
The raw data values are aggregated based on a provided threshold parameter
\code{min_recurrence}, the function returns the area covered by values
greater or equal than this threshold.

The required resources for this indicator are:
\itemize{
\item \link{global_surface_water_recurrence}
}
}
\examples{
\dontshow{
mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
}
\dontrun{
library(sf)
library(mapme.biodiversity)

outdir <- file.path(tempdir(), "mapme-data")
dir.create(outdir, showWarnings = FALSE)

mapme_options(
  outdir = outdir,
  verbose = FALSE
)

aoi <- system.file("extdata", "shell_beach_protected_area_41057_B.gpkg",
  package = "mapme.biodiversity"
) \%>\%
  read_sf() \%>\%
  get_resources(get_global_surface_water_recurrence()) \%>\%
  calc_indicators(
    calc_gsw_recurrence(engine = "extract", min_recurrence = 10)
  ) \%>\%
  portfolio_long()

aoi
}
}
\keyword{indicator}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/register.R
\name{indicators}
\alias{indicators}
\alias{register_indicator}
\alias{available_indicators}
\title{Register or list indicators in mapme.biodiversity}
\usage{
register_indicator(name = NULL, description = NULL, resources = NULL)

available_indicators(indicators = NULL)
}
\arguments{
\item{name}{A character vector indicating the name of the indicator.}

\item{description}{A character vector with a basic description}

\item{resources}{A character vector of the required resources
that need to be available to calculate the indicator. The names must
correspond with already registered resources.}

\item{indicators}{If \code{NULL} returns a list of all registered indicators
(default). Otherwise only the ones specified.}
}
\value{
\code{register_indicator()} is called for the side-effect of registering
an indicator

\code{available_resources()} returns a tibble listing available indicators.
}
\description{
\code{register_indicator()} is used to register a new indicator function with base
information to the package's internal environment used to inform users about
available indicators. Note, registering a custom indicator will
only have effect for the current R session.

\code{available_indicators()} returns a tibble of registered indicators with basic
information such as the required resources.
}
\examples{
\dontrun{
register_indicator(
  name = "treecover_area",
  description = "Area of forest cover by year",
  resources = c(
    "gfw_treecover",
    "gfw_lossyear"
  )
)
}
available_indicators()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_accessibility_2000.R
\name{accessibility_2000}
\alias{accessibility_2000}
\alias{get_accessibility_2000}
\title{Accessibility to Cities in 2000}
\source{
https://forobs.jrc.ec.europa.eu/gam
}
\usage{
get_accessibility_2000()
}
\value{
A function that returns an \code{sf} footprint object.
}
\description{
This resource provides global maps of travel time to cities of 50,000 or more
people in year 2000. Accessibility refers to the ease with which larger cities can be reached from
a certain location. This dataset represents travel time to major cities globally
as of the year 2000, encoded in minutes. The data is essential for historical
analyses, such as understanding the impact of accessibility on land use and
socio-economic outcomes during this period.
}
\references{
European Commission, Joint Research Centre (JRC), Global Accessibility Maps (GAM), 2000.
}
\keyword{resource}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_gsw_time_series.R
\name{gsw_time_series_resource}
\alias{gsw_time_series_resource}
\alias{get_gsw_time_series}
\title{Helper function to download Global Surface Water (GSW) yearly time series
data}
\source{
Raw Data: \url{https://jeodpp.jrc.ec.europa.eu/ftp/jrc-opendata/GSWE/YearlyClassification/LATEST/tiles/}
}
\usage{
get_gsw_time_series(years, version = "LATEST")
}
\arguments{
\item{years}{Numeric vector of years to process between 1984 and 2021.
Default: \code{1984:2021}.}

\item{version}{Version of the data set to process. Available options are
(\code{VER1-0}, \code{VER2-0}, \code{VER3-0}, \code{VER4-0}, \code{VER5-0} and \code{LATEST}) Default:
\code{LATEST}. Choosing \code{LATEST} will result in the latest available version.}
}
\value{
A function that returns a character vector of file paths.
}
\description{
This function constructs  the necessary data URLs for a given data set,
version and polygon and downloads them for further processing with the
mapme.biodiversity package.
}
\details{
The available surface water classes for a given pixel are the following:
\itemize{
\item No Observation: It was not possible to determine whether a pixel was water
(this may be the case for frozen areas or during the polar night in extreme
latitudes).
\item Permanent Water: Water was detected in twelve months per year or in a
combination of permanent and no observation.
\item Seasonal Water: Water and no water was detected.
\item No Water: No Water was detected.
}
}
\references{
\itemize{
\item Global Surface Water Explorer: \url{https://global-surface-water.appspot.com/}
\item Data Users Guide: \url{https://storage.cloud.google.com/global-surface-water/downloads_ancillary/DataUsersGuidev2021.pdf}
\item Research Article: \url{https://www.nature.com/articles/nature20584}
}
}
\keyword{resource}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_teow.R
\name{teow}
\alias{teow}
\alias{get_teow}
\title{Terrestrial Ecoregions of the World (TEOW) Polygon}
\source{
\url{https://files.worldwildlife.org/wwfcmsprod/files/Publication/file/6kcchn7e3u_official_teow.zip}
}
\usage{
get_teow(path = NULL)
}
\arguments{
\item{path}{A character vector to the Terrestrial Ecoregions of the World (TEOW)
zip file. Note, that the file has to be downloaded manually.}
}
\value{
A function that returns an \code{sf} footprint object.
}
\description{
This resource is part of the publication by Olson et al. (2004)
"Terrestrial Ecosystems of the World (TEOW) from WWF-US (Olson)". It
depicts 867 terrestrial ecoregions around the world classified into 14
different terrestrial biomes such as forests, grasslands, or deserts.
The polygons represent the ecoregions, defined as relatively large units of
land or inland water sharing a large majority of biodiversity. The datasets
is made available from World Wildlife Fund (WWF) for the year 2001.
}
\references{
Olson, D. M., Dinerstein, E., Wikramanayake, E. D., Burgess, N. D.,
Powell, G. V. N., Underwood, E. C., D’Amico, J. A., Itoua, I., Strand, H. E.,
Morrison, J. C., Loucks, C. J., Allnutt, T. F., Ricketts, T. H., Kura, Y., Lamoreux,
J. F., Wettengel, W. W., Hedao, P., Kassem, K. R. 2001. Terrestrial ecoregions of
the world: a new map of life on Earth. Bioscience 51(11):933-938.
\doi{doi:10.1641/0006-3568(2001)051[0933:TEOTWA]2.0.CO;2}
}
\keyword{resource}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_burned_area.R
\name{burned_area}
\alias{burned_area}
\alias{calc_burned_area}
\title{Calculate Monthly Burned Area based on MODIS (MCD64A1)}
\usage{
calc_burned_area(engine = "extract")
}
\arguments{
\item{engine}{The preferred processing functions from either one of "zonal",
"extract" or "exactextract" as character.}
}
\value{
A function that returns an indicator tibble with variable burned
area and corresponding area (in ha) as values.
}
\description{
Calculates Monthly Burned Area based on the Terra and Aqua combined
MCD64A1 Version 6.1. which s a monthly, global gridded 500 meter (m)
product containing per-pixel burned-area information.
}
\details{
The required resources for this indicator are:
\itemize{
\item \link{mcd64a1}
}
}
\examples{
\dontshow{
mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
}
\dontrun{
library(sf)
library(mapme.biodiversity)

outdir <- file.path(tempdir(), "mapme-data")
dir.create(outdir, showWarnings = FALSE)

mapme_options(
  outdir = outdir,
  verbose = FALSE
)

aoi <- system.file("extdata", "sierra_de_neiba_478140_2.gpkg",
  package = "mapme.biodiversity"
) \%>\%
  read_sf() \%>\%
  get_resources(get_mcd64a1(years = 2010)) \%>\%
  calc_indicators(calc_burned_area(engine = "extract")) \%>\%
  portfolio_long()

aoi
}
}
\references{
Giglio, L., C. Justice, L. Boschetti, D. Roy. MODIS/Terra+Aqua
Burned Area Monthly L3 Global 500m SIN Grid V061. 2021, distributed by
NASA EOSDIS Land Processes Distributed Active Archive Center.
\doi{doi:10.5067/MODIS/MCD64A1.061}
}
\keyword{indicator}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_ipbes_biomes.R
\name{ipbes_biome_stats}
\alias{ipbes_biome_stats}
\alias{calc_ipbes_biomes}
\title{Calculate areal statistics for IBPES Biomes}
\usage{
calc_ipbes_biomes()
}
\value{
A function that returns an indicator tibble with the biome class
as variable and the respective area (in ha) as value.
}
\description{
This indicator calculates the areal distribution of different biome classes
within an asset based on the IBPES biomes dataset.
}
\details{
The required resources for this indicator are:
\itemize{
\item \link{ipbes_biomes}
}
}
\examples{
\dontshow{
mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
}
\dontrun{
library(sf)
library(mapme.biodiversity)

outdir <- file.path(tempdir(), "mapme-data")
dir.create(outdir, showWarnings = FALSE)

mapme_options(
  outdir = outdir,
  verbose = FALSE
)

aoi <- system.file("extdata", "shell_beach_protected_area_41057_B.gpkg",
  package = "mapme.biodiversity"
) \%>\%
  read_sf() \%>\%
  get_resources(get_ipbes_biomes()) \%>\%
  calc_indicators(calc_ipbes_biomes()) \%>\%
  portfolio_long()

aoi
}
}
\keyword{indicator}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_treecover_area.R
\name{treecover_area}
\alias{treecover_area}
\alias{calc_treecover_area}
\title{Calculate treecover statistics}
\usage{
calc_treecover_area(years = 2000:2024, min_size = 10, min_cover = 35)
}
\arguments{
\item{years}{A numeric vector with the years for which to calculate treecover
area.}

\item{min_size}{The minimum size of a forest patch to be considered as forest in ha.}

\item{min_cover}{The minimum cover percentage per pixel to be considered as forest.}
}
\value{
A function that returns an indicator tibble with variable treecover
and corresponding area (in ha) as value.

If the \link{gfw_treecover} resource for an asset contains only zeros (no trees at all,
e.g. big water body like an ocean or a sea),
the \code{NULL} value is returned instead of the indicator tibble.
If it doesn't contain any pixels with value >=  \code{min_cover}, indicator tibble
reports area of zero hectares.
}
\description{
This functions allows to efficiently calculate treecover statistics for
polygons. For each year in the analysis timeframe, the forest losses in
preceding and the current years are subtracted from the treecover in the
year 2000 and actual treecover figures within the polygon are returned.
}
\details{
The required resources for this indicator are:
\itemize{
\item \link{gfw_treecover}
\item \link{gfw_lossyear}
}
}
\examples{
\dontshow{
mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
}
\dontrun{
library(sf)
library(mapme.biodiversity)

outdir <- file.path(tempdir(), "mapme-data")
dir.create(outdir, showWarnings = FALSE)

mapme_options(
  outdir = outdir,
  verbose = FALSE
)

aoi <- system.file("extdata", "sierra_de_neiba_478140_2.gpkg",
  package = "mapme.biodiversity"
) \%>\%
  read_sf() \%>\%
  get_resources(
    get_gfw_treecover(version = "GFC-2024-v1.12"),
    get_gfw_lossyear(version = "GFC-2024-v1.12")
  ) \%>\%
  calc_indicators(calc_treecover_area(years = 2016:2017, min_size = 1, min_cover = 30)) \%>\%
  portfolio_long()

aoi
}
}
\keyword{indicator}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_fatalities_acled.R
\name{fatalities_acled}
\alias{fatalities_acled}
\alias{calc_fatalities_acled}
\title{Calculate number of fatalities of conflict events from ACLED}
\usage{
calc_fatalities_acled(
  years = 2000,
  stratum = c("event_type", "sub_event_type", "disorder_type"),
  precision_location = 1,
  precision_time = 1
)
}
\arguments{
\item{years}{A numeric vector indicating the years for which to summarize
fatalities.}

\item{stratum}{A character vector indicating the stratification to be applied.
Should be one of "event_type", "sub_event_type", or "disorder_type".
Defaults to "event_type".}

\item{precision_location}{A numeric indicating precision value for the
geolocation up to which events are included. Defaults to 1.}

\item{precision_time}{A numeric indicating the precision value of the
temporal coding up to which events are included. Defaults to 1.}
}
\value{
A function that returns an indicator tibble with the type of
violence as variable and counts of civilian fatalities as value.
}
\description{
The indicator aggregated the number of fatalities within a given asset
on a monthly cadence stratified either by event type, sub-event type or disorder
type. To learn about the different categorisation ACLED uses to encode events
please consult ACLED's codebook.
}
\details{
The required resources for this indicator are:
\itemize{
\item \link{acled}
}

You may apply quality filters based on the precision of the geolocation
of events and the temporal precision. By default, these are set to only
include events with the highest precision scores.

For geo-precision there are levels 1 to 3 with decreasing accuracy:
\itemize{
\item value 1: the source reporting indicates a particular town, and coordinates
are available for that town
\item value 2: the source material indicates that activity took place in a
small part of a region, and mentions a general area or if an activity
occurs near a town or a city, the event is coded to a town with
geo-referenced coordinates to represent that area
\item value 3: a larger region is mentioned, the closest natural location noted
in reporting (like “border area,” “forest,” or “sea,” among others) – or a
provincial capital is used if no other information at all is available
}

For temporal precision there are levels 1 to 3 with decreasing precision:
\itemize{
\item value 1: the source material includes an actual date of an event
\item value 2: the source material indicates that an event happened sometime
during the week or within a similar period of time
\item value 3: the source material only indicates that an event took place
sometime during a month (i.e. in the past two or three weeks, or in
January), without reference to the particular date, the month mid-point
is chosen
}
}
\examples{
\dontshow{
mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
}
\dontrun{
library(sf)
library(mapme.biodiversity)

outdir <- file.path(tempdir(), "mapme-data")
dir.create(outdir, showWarnings = FALSE)

mapme_options(
  outdir = outdir,
  verbose = FALSE,
  chunk_size = 1e8
)

aoi <- system.file("extdata", "burundi.gpkg",
  package = "mapme.biodiversity"
) \%>\%
  read_sf() \%>\%
  get_resources(get_acled(years = 2020)) \%>\%
  calc_indicators(
    calc_fatalities_acled(
      years = 2020,
      precision_location = 1,
      precision_time = 1
    )
  ) \%>\%
  portfolio_long()

aoi
}
}
\references{
Raleigh, C., Kishi, R. & Linke, A. Political instability patterns
are obscured by conflict dataset scope conditions, sources, and coding
choices. Humanit Soc Sci Commun 10, 74 (2023).
\doi{doi:10.1057/s41599-023-01559-4}
}
\keyword{indicator}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_key_biodiversity_areas.R
\docType{data}
\name{key_biodiversity_areas_indicator}
\alias{key_biodiversity_areas_indicator}
\alias{calc_key_biodiversity_area}
\title{Calculate Key Biodiversity Areas}
\format{
A function returning an indicator tibble with \code{key_biodiversity_area}
as variable and the total overlap area (in ha) as value.
}
\usage{
calc_key_biodiversity_area()
}
\description{
This function calculates the total area of key biodiversity areas for a given
input polygon.
}
\details{
The required resources for this indicator are:
\itemize{
\item \link{key_biodiversity_areas_resource}
}
}
\examples{
\dontshow{
mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
}
\dontrun{
library(sf)
library(mapme.biodiversity)

outdir <- file.path(tempdir(), "mapme-data")
dir.create(outdir, showWarnings = FALSE)

mapme_options(
  outdir = outdir,
  verbose = FALSE
)

aoi <- read_sf(
  system.file("extdata", "shell_beach_protected_area_41057_B.gpkg",
              package = "mapme.biodiversity"
))
kbas <- system.file("res", "key_biodiversity_areas", "kbas.gpkg",
                    package = "mapme.biodiversity")
aoi <- get_resources(aoi, get_key_biodiversity_areas(kbas))
aoi <- calc_indicators(aoi, calc_key_biodiversity_area())
aoi <- portfolio_long(aoi)

aoi
}
}
\keyword{indicator}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/engines.R
\name{engine}
\alias{engine}
\alias{check_engine}
\alias{check_stats}
\alias{select_engine}
\title{Function to select processing engines}
\usage{
check_engine(queried_engine)

check_stats(queried_stats)

select_engine(x, raster, stats, engine, name = NULL, mode = "asset")
}
\arguments{
\item{queried_engine}{A character vector of length one indicating the engine
to check for.}

\item{queried_stats}{A character vector with statistic names to be checked
if they are supported by the backend}

\item{x}{An sf object representing a portfolio.}

\item{raster}{An terra SpatRaster from which values are to be extracted.}

\item{stats}{A character vector of statistics to aggregate the raster values
with.}

\item{engine}{A character vector of length one specifying the engine to be
used for the extraction.}

\item{name}{A character vector indicating the name to append to the columns
names.}

\item{mode}{A character vector indicating in which mode to conduct the
extraction (e.g. \code{asset}-wise or for the whole \code{portfolio} at once).}
}
\value{
\code{check_engine()} returns the character of the queried engine,
if supported. Throws an error otherwise.

\code{check_stats} returns a character vector of supported statistics.
Throws an error if any of the queried statistics is not supported.

\code{select_engine} returns a tibble.
}
\description{
\code{check_engine()} checks if an extraction engine for zonal vector-raster
operations is supported by the backend.

\code{check_stats} checks if one or multiple statistics are supported
for zonal vector-raster extraction by the backend.

\code{select_engine} extracts zonal vector-raster statistics for supported engine
and for one or more statistics. Columns are named according to the argument
\code{name} plus the respective stat. Both \code{portfolio} and \code{asset} modes are supported.
}
\keyword{utils}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_key_biodiversity_areas.R
\name{key_biodiversity_areas_resource}
\alias{key_biodiversity_areas_resource}
\alias{get_key_biodiversity_areas}
\title{Key Biodiversity Areas}
\source{
\url{https://www.keybiodiversityareas.org/request-gis-data}
}
\usage{
get_key_biodiversity_areas(path = NULL)
}
\arguments{
\item{path}{A character vector to the key biodiversity areas GPKG file.
Note, that the file has to be downloaded manually.}
}
\value{
A function that returns an \code{sf} footprints object.
}
\description{
This resource contains outlines of key biodiversity areas, which are areas
representing sites with specific importance for nature conservation.
}
\details{
To use this data in mapme workflows, you will have to manually download the
global data set and point towards its file path on your local machine.
Please find the available data under the source link given below.
}
\references{
BirdLife International (2024). The World Database of
Key Biodiversity Areas. Developed by the KBA Partnership: BirdLife
International, International Union for the Conservation of Nature,
Amphibian Survival Alliance, Conservation International, Critical Ecosystem
Partnership Fund, Global Environment Facility, Re:wild, NatureServe,
Rainforest Trust, Royal Society for the Protection of Birds, Wildlife
Conservation Society and World Wildlife Fund. Available at
www.keybiodiversityareas.org.
}
\keyword{resource}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_worldpop.R
\docType{data}
\name{worldpop}
\alias{worldpop}
\alias{get_worldpop}
\title{Population Count layer for year 2000-2020}
\source{
\url{https://www.worldpop.org/}
}
\usage{
get_worldpop(years = 2000)
}
\arguments{
\item{years}{A numeric vector indicating the years for which to make the
resource available.}
}
\value{
A function that returns an \code{sf} footprint object.
}
\description{
This resource is published by open spatial demographic data and research
organization called WorldPop. This resource represents the population
count, 1 km spatial resolution layers available to download from the year
2000 to 2020. The dataset is called as WorldPop Unconstrained Global Mosaics.
The encoded cell value represents the total number of people in that particular
grid cell.
}
\details{
It may be required to increase the timeout option to successfully download
theses WorldPop layers from their source location via e.g.
\code{options(timeout = 600)}.
}
\keyword{resource}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_ucdp_ged.R
\name{ucdp_ged}
\alias{ucdp_ged}
\alias{get_ucdp_ged}
\title{UCDP Georeferenced Event Dataset (UCDP GED)}
\source{
\url{https://ucdp.uu.se/downloads/}
}
\usage{
get_ucdp_ged(version = "latest")
}
\arguments{
\item{version}{A character vector specifying
the version to download. Defaults to "latest".}
}
\value{
A function that returns an \code{sf} footprint object.
}
\description{
This resource distributed by the Uppsala Conflict Data Program (UCDP) constitutes
its most disaggregated dataset on individual events of organized violence.
It encodes the different actors involved, is spatially disaggregated down
to village levels anc currently covers the time period of 1989 to 2021.
Older versions of the data set can be downloaded, but users are recommended
to download the latest data set.
}
\details{
The following versions are available:
\itemize{
\item 5.0
\item 17.1
\item 17.2
\item 18.1
\item 19.1
\item 20.1
\item 21.1
\item 22.1
\item 23.1
\item 24.1
\item latest
}
}
\references{
Davies, Shawn, Therese Pettersson & Magnus Öberg (2022).
Organized violence 1989-2021 and drone warfare.
Journal of Peace Research 59(4).
\doi{10.1177/00223433221108428}
}
\keyword{resource}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_traveltime_2000.R
\name{traveltime_2000}
\alias{traveltime_2000}
\alias{calc_traveltime_2000}
\title{Calculate accessibility statistics for the year 2000}
\usage{
calc_traveltime_2000(engine = "extract", stats = "mean")
}
\arguments{
\item{engine}{The preferred processing functions from either one of "zonal",
"extract" or "exactextract" as character.}

\item{stats}{Function to be applied to compute statistics for polygons either
single or multiple inputs as character. Supported statistics are: "mean",
"median", "sd", "min", "max", "sum", "var".}
}
\value{
A function that returns an indicator tibble with accessibility statistics
for the year 2000 as variables and corresponding values (in minutes) as values.
}
\description{
Accessibility refers to the ease with which cities can be reached from a
certain location. This function allows efficient calculation of accessibility
statistics (i.e., travel time to the nearest city) for polygons.  For each
polygon, the desired statistic/s (mean, median or sd) is/are returned.
}
\details{
The required resource for this indicator is:
\itemize{
\item \link{accessibility_2000}
}
}
\examples{
\dontshow{
mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
}
\dontrun{
library(sf)
library(mapme.biodiversity)

outdir <- file.path(tempdir(), "mapme-data")
dir.create(outdir, showWarnings = FALSE)

mapme_options(
  outdir = outdir,
  verbose = FALSE
)

aoi <- system.file("extdata", "sierra_de_neiba_478140_2.gpkg",
  package = "mapme.biodiversity"
) \%>\%
  read_sf() \%>\%
  get_resources(get_accessibility_2000()) \%>\%
  calc_indicators(
    calc_traveltime_2000(stats = c("mean", "median", "sd"), engine = "extract")
  ) \%>\%
  portfolio_long()

aoi
}
}
\keyword{indicator}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_gsw_time_series.R
\name{gsw_time_series_indicator}
\alias{gsw_time_series_indicator}
\alias{calc_gsw_time_series}
\title{Calculate Global Surface Water Time Series}
\format{
A function returning a tibble with time series of global surface
water data classes.
}
\usage{
calc_gsw_time_series()
}
\description{
This function calculates the total area of the global surface water time
series data, separated by the following classes:
}
\details{
\itemize{
\item No Observation: It was not possible to determine whether a pixel was water
(this may be the case for frozen areas or during the polar night in extreme
latitudes).
\item Permanent Water: Water was detected in twelve months per year or in a
combination of permanent and no observation.
\item Seasonal Water: Water and no water was detected.
\item No Water: No Water was detected.
}

The required resources for this indicator are:
\itemize{
\item \link{gsw_time_series_resource}
}
}
\examples{
\dontshow{
mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
outdir <- file.path(tempdir(), "mapme-data")
path_gsw_ts <- file.path(outdir, "gsw_time_series")
gsw_fnames_short <- dir(path_gsw_ts, pattern = ".tif$")
gsw_fnames_long <- sub("v5_", "VER5-0_yearlyClassification", gsw_fnames_short)

invisible(file.copy(
   file.path(path_gsw_ts, gsw_fnames_short),
   file.path(path_gsw_ts, gsw_fnames_long)
))
}
\dontrun{
library(mapme.biodiversity)
library(sf)

outdir <- file.path(tempdir(), "mapme-data")
dir.create(outdir, showWarnings = FALSE)

mapme_options(
  outdir = outdir,
  verbose = FALSE
)

aoi <- read_sf(
  system.file("extdata", "shell_beach_protected_area_41057_B.gpkg",
              package = "mapme.biodiversity"
))
aoi <- get_resources(aoi, get_gsw_time_series (years = 2000:2001))
aoi <- calc_indicators(aoi, calc_gsw_time_series())
aoi <- portfolio_long(aoi)

aoi
}
}
\keyword{indicator}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_drought_indicator.R
\name{drought_indicator}
\alias{drought_indicator}
\alias{calc_drought_indicator}
\title{Calculate drought indicator statistics}
\usage{
calc_drought_indicator(engine = "extract", stats = "mean")
}
\arguments{
\item{engine}{The preferred processing functions from either one of "zonal",
"extract" or "exactextract" as character.}

\item{stats}{Function to be applied to compute statistics for polygons
either one or multiple inputs as character "mean", "median" or "sd".}
}
\value{
A function that returns an indicator tibble with specified drought
indicator statistics as variable and corresponding values as value.
}
\description{
This function allows to efficiently calculate the relative wetness in the
shallow groundwater section with regard to the the 1948-2012 reference period.
The values represent the wetness percentile a given area achieves at a given
point in time in regard to the reference period.
For each polygon, the desired statistic/s (mean, median or sd) is/are
returned.
}
\details{
The required resources for this indicator are:
\itemize{
\item \link{nasa_grace}
}
}
\examples{
\dontshow{
mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
}
\dontrun{
library(sf)
library(mapme.biodiversity)

outdir <- file.path(tempdir(), "mapme-data")
dir.create(outdir, showWarnings = FALSE)

mapme_options(
  outdir = outdir,
  verbose = FALSE
)

aoi <- system.file("extdata", "sierra_de_neiba_478140_2.gpkg",
  package = "mapme.biodiversity"
) \%>\%
  read_sf() \%>\%
  get_resources(get_nasa_grace(years = 2022)) \%>\%
  calc_indicators(
    calc_drought_indicator(
      engine = "extract",
      stats = c("mean", "median")
    )
  ) \%>\%
  portfolio_long()

aoi
}
}
\keyword{indicator}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/register.R
\name{resources}
\alias{resources}
\alias{register_resource}
\alias{available_resources}
\title{Register or list resources in mapme.biodiversity}
\usage{
register_resource(
  name = NULL,
  description = NULL,
  licence = NULL,
  source = NULL,
  type = NULL
)

available_resources(resources = NULL)
}
\arguments{
\item{name}{A character vector indicating the name of the resource.}

\item{description}{A character vector with a basic description}

\item{licence}{A character vector indicating the licence of the resource.
In case it is a custom licence, put a link to the licence text.}

\item{source}{Optional, preferably a URL where the data is found.}

\item{type}{A character vector indicating the type of the resource. Either
'vector' or 'raster'.}

\item{resources}{If \code{NULL} returns a list of all resources (default).
Otherwise only the ones specified.}
}
\value{
\code{register_resource()} is called for the side-effect of registering a
resource.

\code{available_resources()} returns a tibble listing available resources.
}
\description{
\code{register_resource()} is used to register a new resource function with base
information to the package's internal environment used to inform users about
available resources. Note, registering a custom resource will
only have effect for the current R session.

\code{available_resources()} returns a tibble of registered resources with basic
information such as the source and the licence.
}
\examples{
\dontrun{
register_resource(
  name = "gfw_treecover",
  description = "Global Forest Watch - Percentage of canopy closure in 2000",
  licence = "CC-BY 4.0",
  source = "https://data.globalforestwatch.org/documents/tree-cover-2000/explore",
  type = "raster"
)
}
available_resources()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_exposed_population_ucdp.R
\name{exposed_population_ucdp}
\alias{exposed_population_ucdp}
\alias{calc_exposed_population_ucdp}
\title{Calculate population exposed to violent conflict from UCDP GED}
\usage{
calc_exposed_population_ucdp(
  distance = 5000,
  violence_types = 1:3,
  years = c(1989:2023),
  precision_location = 1,
  precision_time = 1
)
}
\arguments{
\item{distance}{A numeric vector indicating the buffer size around
included conflict events to calculate the exposed population. Either of
length 1 to apply for all types of events, or discrete values for
each category included in \code{violence_types}.}

\item{violence_types}{A numeric vector indicating the types of violence
to be included (see Details).}

\item{years}{A numeric vector indicating for which years to calculate
the exposed population. Restricted to available years for UCDP GED.
For years not intersecting with available WorldPop layers, the first layer
is used for earlier years and the last layer to more recent years.}

\item{precision_location}{A numeric indicating precision value for the
geolocation up to which events are included. Defaults to 1.}

\item{precision_time}{A numeric indicating the precision value of the
temporal coding up to which events are included. Defaults to 1.}
}
\value{
A function that returns an indicator tibble with conflict exposure
as variable and percentage of the population as its value.
}
\description{
The indicator calculates the population exposed to conflict events within a
specified buffer distance around violent events in UCDP GED. Per default,
the first available WorldPop layer is used to estimate exposed populations
for years before the respective year, while the most recent layer is
used for years after.
}
\details{
The indicator is inspired by the Conflict Exposure tool from ACLED (see
citation below), but differs in the regard that we simply flatten our
buffered event layer instead of applying voronoi tessellation.

The required resources for this indicator are:
\itemize{
\item \link{ucdp_ged}
\item \link{worldpop}
}

You may filter for certain types of violence. The coded types according
to the UCDP codebook are:
value 1: state-based conflict
value 2: non-state conflict
value 3: one-sided conflict

You may apply quality filters based on the precision of the geolocation
of events and the temporal precision. By default, these are set to only
include events with the highest precision scores.

For geo-precision there are levels 1 to 7 with decreasing accuracy:
\itemize{
\item value 1: the location information corresponds exactly to the geographical
coordinates available
\item value 2: the location information refers to a limited area around a
specified location
\item value 3: the source refers to or can be specified to a larger location at
the level of second order administrative divisions (ADM2), such as district
or municipality, the GED uses centroid point coordinates for that ADM2.
\item value 4: the location information refers to a first order administrative
division, such as a province (ADM1), the GED uses the coordinates for the
centroid point of ADM1
\item value 5: is used in different cases if the source refers to parts of a
country which are larger than ADM1, but smaller than the entire country;
if two locations are mentioned a representative point in between is
selected; if the location mentioned is an non-independent island; if
the location is not very specifically mentioned or in relation to another
location
\item value 6: the location mentioned refers to an entire country and its
centroid is used
\item value 7: If the event takes place over water or in international airspace,
the geographical coordinates in the dataset either represent the centroid
point of a certain water area or estimated coordinates
}

For temporal precision there are levels 1 to 5 with decreasing precision:
\itemize{
\item value 1: if the exact date of an event is known
\item value 2: if start and end dates for events are of unspecified character,
spanning more than one calendar day though no longer than six days
\item value 3: if when start and end dates for events are specified to a certain
week, but specific dates are not provided
\item value 4: if start and end dates for events are specified to a certain month
\item value 5: if start and end dates for events are specified to a certain year,
but specific dates are not provided
}
}
\examples{
\dontshow{
mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
}
\dontrun{
if (FALSE) {
  library(sf)
  library(mapme.biodiversity)

  outdir <- file.path(tempdir(), "mapme-data")
  dir.create(outdir, showWarnings = FALSE)

  mapme_options(
    outdir = outdir,
    verbose = FALSE,
    chunk_size = 1e8
  )

  aoi <- system.file("extdata", "burundi.gpkg",
    package = "mapme.biodiversity"
  ) \%>\%
    read_sf() \%>\%
    get_resources(
      get_ucdp_ged(version = "22.1"),
      get_worldpop(years = 2000)
    ) \%>\%
    calc_indicators(
      conflict_exposure(
        distance = 5000,
        violence_types = 1:3,
        years = 2000,
        precision_location = 1,
        precision_time = 1
      )
    ) \%>\%
    portfolio_long()

  aoi
}
}
}
\references{
Raleigh, C; C Dowd; A Tatem; A Linke; N Tejedor-Garavito; M
Bondarenko and K Kishi. 2023. Assessing and Mapping Global and Local
Conflict Exposure. Working Paper.
}
\keyword{indicator}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_gfw_emissions.R
\name{gfw_emissions}
\alias{gfw_emissions}
\alias{get_gfw_emissions}
\title{Forest greenhouse gas emissions}
\source{
\url{https://data.globalforestwatch.org/datasets/gfw::forest-greenhouse-gas-emissions/about}
}
\usage{
get_gfw_emissions()
}
\value{
A function that returns an \code{sf} footprint object.
}
\description{
This resource is part of the publication by Harris et al. (2021)
"Global maps of twenty-first century forest carbon fluxes.". It
represents "the greenhouse gas
emissions arising from stand-replacing forest disturbances that occurred in
each modelled year (megagrams CO2 emissions/ha, between 2001 and 2023).
Emissions include all relevant ecosystem carbon pools (aboveground biomass,
belowground biomass, dead wood, litter, soil) and greenhouse gases (CO2, CH4,
N2O)." The area unit that is downloaded here corresponds to the
"megagrams of CO2 emissions/pixel" layer, in order to support the calculation
of area-wise emissions.
}
\details{
There are no arguments users need to specify. However, users should note
that the spatial extent for this dataset does not totally cover the same
extent as the \code{treecover2000} and \code{lossyear} resources by Hansen
et al. (2013). A missing value (NA) will be inserted for greenhouse gas
emissions for areas where no data is available.
}
\references{
Harris, N.L., Gibbs, D.A., Baccini, A. et al. Global maps of
twenty-first century forest carbon fluxes. Nat. Clim. Chang. 11, 234–240
(2021). \doi{doi:10.1038/s41558-020-00976-6}
}
\keyword{resource}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/portfolio.R
\name{portfolio}
\alias{portfolio}
\alias{write_portfolio}
\alias{read_portfolio}
\alias{portfolio_long}
\alias{portfolio_wide}
\title{Portfolio methods}
\usage{
write_portfolio(x, dsn, ...)

read_portfolio(src, ...)

portfolio_long(x, indicators = NULL, drop_geoms = FALSE)

portfolio_wide(x, indicators = NULL, drop_geoms = FALSE)
}
\arguments{
\item{x}{A portfolio object processed with \code{mapme.biodiversity}.}

\item{dsn}{A file path for the output file (must end with \code{gpkg}).}

\item{...}{Additional arguments supplied to \code{write_sf()} or \code{read_sf()}}

\item{src}{A character vector pointing to a GeoPackage that has been
previously written to disk via \code{write_portfolio()}}

\item{indicators}{If NULL (the default), all indicator columns will be detected
and transformed automatically. If a character vector is supplied, only
those indicators will be transformed.}

\item{drop_geoms}{A logical, indicating if geometries should be dropped.}
}
\value{
\code{write_portfolio()} returns \code{dsn}, invisibly.

\code{read_portfolio()} returns an \code{sf} object object with nested list
columns for every indicator found in the GeoPackage source file.

\code{portfolio_long()} returns the portfolio object in long-format.

\code{portfolio_wide()} returns the portfolio object in wide-format.
}
\description{
\code{write_portfolio()} writes a processed biodiversity portfolio to disk.
Portfolio data will only be serialized to disk as a GeoPackage including
two tables: \code{metadata} and \code{indicators}. The \code{metadata} tables
includes, among other simple variables the geometries and a primary
key called \code{assetid}. The 'indicators' tables includes the foreign key
\code{assetid}, a column called \code{indicator} giving the name of the original
indicator as well as the standard indicator columns \code{datetime}, \code{variable},
\code{unit}, and \code{value}. For convenience, use \code{read_portfolio()} to read
such a portfolio GeoPackage back into R.

\code{portfolio_long()} transforms a portfolio to long-format, potentially
dropping geometries in the process.

\code{portfolio_wide()} transforms a portfolio to wide-format, potentially
dropping geometries in the process.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_gsw.R
\name{global_surface_water_seasonality}
\alias{global_surface_water_seasonality}
\alias{get_global_surface_water_seasonality}
\title{Global Surface Water Seasonality}
\source{
\url{https://global-surface-water.appspot.com/}
}
\usage{
get_global_surface_water_seasonality(version = "v1_4_2021")
}
\arguments{
\item{version}{A character vector indicating the version of the GSW data set
to make available.}
}
\value{
A character of file paths.
}
\description{
The Global Surface Water dataset was developed by the European Commission's
Joint Research Centre in the framework of the Copernicus Programme. It maps
the location and temporal distribution of water surfaces at the global scale
over the past 3.8 decades and provides statistics on their extent and change.
It is provisioned as a global tiled raster resource available for all land
areas. The reported data represent aggregated observations between 1984 - 2021.
}
\details{
GSW seasonality describes the intra-annual distribution of surface water for
each pixel. The raster files have integer cell values between \code{[0, 12]},
indicating how many months per year the pixel was classified as water.
}
\references{
Pekel, JF., Cottam, A., Gorelick, N. et al. High-resolution
mapping of global surface water and its long-term changes. Nature 540,
418–422 (2016). \doi{doi:10.1038/nature20584}
}
\keyword{resource}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{check_namespace}
\alias{check_namespace}
\title{Checks if namespace is available}
\usage{
check_namespace(pkg, error = TRUE)
}
\arguments{
\item{pkg}{A character vector of length one indicating a package name
for which the namespace is tested}

\item{error}{A logical indicating whether or not to promote missing namespace
to error. If FALSE, a warning is emitted.}
}
\value{
TRUE, invisible, if the namespace is available. An error message
if \code{error = TRUE}, FALSE and a warning otherwise.
}
\description{
Use this function if your resource/indicator function requires the
namespace of a certain package to be available. An informative error/warning
message is printed if that is not the case.
}
\keyword{utils}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_hfp.R
\name{humanfootprint_resource}
\alias{humanfootprint_resource}
\alias{get_humanfootprint}
\title{Terrestrial Human Foootprint}
\source{
\url{https://figshare.com/articles/figure/An_annual_global_terrestrial_Human_Footprint_dataset_from_2000_to_2018/16571064}
}
\usage{
get_humanfootprint(years = 2000:2024)
}
\arguments{
\item{years}{A numeric vector indicating the years for which to download
the human footprint data, defaults to \code{2000:2024}.}
}
\value{
A function that returns an \code{sf} footprint object.
}
\description{
This resource is part of the publication by Mu et al. (2022) "A global
record of annual terrestrial Human Footprint dataset from 2000 to 2018".
It is calculated based on 8 variables representing human pressures on
natural ecosystems collected at a yearly cadence between 2000 and 2024
sampled at a 1km spatial resolution. The variables are used are
the expansion of built environments (expressed as percentage of built-up
areas within a grid cell), population density (aggregated at the grid cell),
nighttime lights, crop and pasture lands, roads and railways (excluding trails
and minor roads), and navigable waterways (compares waterways with nighttime
lights dataset). The human footprint was then calculated based on a weighting
scheme proposed by Venter et al. (2016), assigning each pixel a value between
0 and 50, with 50 representing the theoretical value of the highest human
pressure.
}
\note{
It may be required to increase the timeout option to successfully download
theses layers from their source location via e.g. \code{options(timeout = 600)}.
In case an 403 error occurs, you can create an account with Figshare and
create an personal access token. If set as \code{FIGSHARE_PAT} environment
variable, it will be used to authenticate.
}
\references{
Mu, H., Li, X., Wen, Y. et al. A global record of annual
terrestrial Human Footprint dataset from 2000 to 2018. Sci Data 9, 176 (2022).
\doi{doi:10.1038/s41597-022-01284-8}
}
\keyword{resource}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_gmw.R
\docType{data}
\name{gmw}
\alias{gmw}
\alias{get_gmw}
\title{Global Mangrove Extent Polygon}
\source{
\url{https://habitats.oceanplus.org/}
}
\usage{
get_gmw(years = c(1996, 2007:2010, 2015:2020))
}
\arguments{
\item{years}{A numeric vector of the years for which to make GMW available.}
}
\value{
A function that returns an \code{sf} footprint object.
}
\description{
This resource is part of the publication by Bunting et al. (2018)
"The Global Mangrove Watch—A New 2010 Global Baseline of Mangrove Extent".
The polygons represent the mangrove, which is tropical coastal vegetation
and considered the most significant part of the marine ecosystem. This
resource is available for the selected years in the period 1996- 2020 from
Global Mangrove Watch (GMW), providing geospatial information about global
mangrove extent.
}
\references{
Bunting P., Rosenqvist A., Lucas R., Rebelo L-M., Hilarides L.,
Thomas N., Hardy A., Itoh T., Shimada M. and Finlayson C.M. (2018). The Global
Mangrove Watch – a New 2010 Global Baseline of Mangrove Extent. Remote Sensing
10(10): 1669. doi:10.3390/rs10101669.
}
\keyword{resource}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_biome.R
\name{biome}
\alias{biome}
\alias{calc_biome}
\title{Calculate biomes statistics (TEOW) based on WWF}
\usage{
calc_biome()
}
\value{
A function that returns an indicator tibble with variable biome type
and corresponding area (in ha) as values.
}
\description{
This function allows to efficiently retrieve the name of the biomes
and compute the corresponding area from Terrestrial Ecoregions of the
World (TEOW) - World Wildlife Fund (WWF) for polygons. For each polygon,
the name and area of the biomes (in hectare) is returned.
The required resources for this indicator are:
\itemize{
\item \link{teow}
}
}
\examples{
\dontshow{
mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
}
\dontrun{
library(sf)
library(mapme.biodiversity)

outdir <- file.path(tempdir(), "mapme-data")
dir.create(outdir, showWarnings = FALSE)

mapme_options(
  outdir = outdir,
  verbose = FALSE
)

aoi <- system.file("extdata", "sierra_de_neiba_478140_2.gpkg",
  package = "mapme.biodiversity"
) \%>\%
  read_sf() \%>\%
  get_resources(get_teow()) \%>\%
  calc_indicators(calc_biome()) \%>\%
  portfolio_long()

aoi
}
}
\keyword{indicator}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spatial-utils.R
\name{make_footprints}
\alias{make_footprints}
\title{Create footprints for vector or raster data sets}
\usage{
make_footprints(
  srcs = NULL,
  filenames = if (inherits(srcs, "sf")) basename(srcs[["source"]]) else basename(srcs),
  what = c("vector", "raster"),
  oo = NULL,
  co = NULL,
  precision = 1e+05
)
}
\arguments{
\item{srcs}{A character vector with GDAL readable paths to either vector
or raster sources, then internal footprint functions are called, or an
sf object which will be appended for filenames and potential options.}

\item{filenames}{A character vector indicating the filenames of the source
data sets if they were written to a destination. Defaults to \code{basename(srcs)}
in case of character type or \code{basename(srcs[["source"]])} in case of
an sf object.}

\item{what}{A character vector indicating if the files are vector or raster
files.}

\item{oo}{Either a list or a character vector with opening options (-oo)
of the respective GDAL driver. A list must have equal length of the
input sources, a vector will be recycled.}

\item{co}{Either a list or a character vector with creation options (-co)
of the respective GDAL driver. A list must have equal length of the
input sources, a vector will be recycled.}

\item{precision}{A numeric indicating the precision of coordinates when
a binary round-trip is done (see \code{?sf::st_as_binary()}).}
}
\value{
An sf object with a the files sources and the geometry indicating
their spatial footprint.
}
\description{
With this function you can create footprints for vector or raster datasets.
Specify a character vector of GDAL readable sources of either vector
or raster type. Internally, GDAL will be used to create an sf object
with a single column indicating the source and the geometry indicating
the bounding box of the respective source.
Note, the performance for remote sources is dependent on your connection to
the server. If you have other means to create footprints in your resource
function (e.g. by using the output \code{{rstac::items_bbox()}}) you should prefer
those means over this function for remote files.
}
\examples{

# a vector resource
# requires GDAL >= 3.7.0
if (FALSE) {
  vec <- system.file("shape/nc.shp", package = "sf")
  make_footprints(vec, what = "vector")
}

# a raster resource
ras <- system.file("ex/elev.tif", package = "terra")
make_footprints(ras, what = "raster")
}
\keyword{utils}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_deforestation_drivers.R
\name{deforestation_drivers}
\alias{deforestation_drivers}
\alias{calc_deforestation_drivers}
\title{Calculate deforestation drivers}
\usage{
calc_deforestation_drivers()
}
\value{
A function that returns an indicator tibble with deforestation
drivers as variable and corresponding area (in ha) as value.
}
\description{
This function extracts areal statistics for the drivers of deforestation
based on the data source produced by Fritz et al (2022).
}
\details{
The required resource for this indicator is:
\itemize{
\item \link{fritz_et_al}
}
}
\examples{
\dontshow{
mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
}
\dontrun{
library(sf)
library(mapme.biodiversity)

outdir <- file.path(tempdir(), "mapme-data")
dir.create(outdir, showWarnings = FALSE)

mapme_options(
  outdir = outdir,
  verbose = FALSE
)

aoi <- system.file("extdata", "sierra_de_neiba_478140_2.gpkg",
  package = "mapme.biodiversity"
) \%>\%
  read_sf() \%>\%
  get_resources(get_fritz_et_al(resolution = 100)) \%>\%
  calc_indicators(calc_deforestation_drivers()) \%>\%
  portfolio_long()

aoi
}
}
\keyword{indicator}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_gsw.R
\name{global_surface_water_change}
\alias{global_surface_water_change}
\alias{get_global_surface_water_change}
\title{Global Surface Water Change}
\source{
\url{https://global-surface-water.appspot.com/}
}
\usage{
get_global_surface_water_change(version = "v1_4_2021")
}
\arguments{
\item{version}{A character vector indicating the version of the GSW data set
to make available.}
}
\value{
A function that returns an \code{sf} footprint object.
}
\description{
The Global Surface Water dataset was developed by the European Commission's
Joint Research Centre in the framework of the Copernicus Programme. It maps
the location and temporal distribution of water surfaces at the global scale
over the past 3.8 decades and provides statistics on their extent and change.
It is provisioned as a global tiled raster resource available for all land
areas. The reported data represent aggregated observations between 1984 - 2021.
}
\details{
The change in water occurrence intensity between the two periods is derived
from homologous pairs of months (i.e. same months containing valid
observations in both periods). The difference in the occurrence of surface
water was calculated for each homologous pair of months. The average of all
of these differences constitutes the Surface Water Occurrence change
intensity. The raster files have integer cell values between \code{[0, 200]}
where 0 represents surface water loss and 200 represents surface water gain.
}
\references{
Pekel, JF., Cottam, A., Gorelick, N. et al. High-resolution
mapping of global surface water and its long-term changes. Nature 540,
418–422 (2016). \doi{doi:10.1038/nature20584}
}
\keyword{resource}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_precipitation_chelsa.R
\name{precipitation_chelsa}
\alias{precipitation_chelsa}
\alias{calc_precipitation_chelsa}
\title{Calculate precipitation average based on CHELSA}
\usage{
calc_precipitation_chelsa(years = 1979:2018, engine = "extract")
}
\arguments{
\item{years}{A numeric vector indicating the years for which to calculate
precipitation statistics.}

\item{engine}{The preferred processing functions from either one of "zonal",
"extract" or "exactextract" as character.}
}
\value{
A function that returns an indicator tibble with variable
precipitation and sum of precipitation (in mm/m^2) as value.
}
\description{
This functions allows to calculate averaged precipitation
from the CHELSA downscaled precipitation layers. Based on
user-selected years, monthly averages of precipitation are calculated.
}
\details{
The required resources for this indicator are:
\itemize{
\item \link{chelsa}
}
}
\examples{
\dontshow{
mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
}
\dontrun{
library(sf)
library(mapme.biodiversity)

outdir <- file.path(tempdir(), "mapme-data")
dir.create(outdir, showWarnings = FALSE)

mapme_options(
  outdir = outdir,
  verbose = FALSE
)

aoi <- system.file("extdata", "sierra_de_neiba_478140_2.gpkg",
  package = "mapme.biodiversity"
) \%>\%
  read_sf() \%>\%
  get_resources(get_chelsa(years = 2010)) \%>\%
  calc_indicators(
    calc_precipitation_chelsa(
      years = 2010,
      engine = "extract"
    )
  ) \%>\%
  portfolio_long()

aoi
}
}
\keyword{indicator}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_mcd64A1.R
\name{mcd64a1}
\alias{mcd64a1}
\alias{get_mcd64a1}
\title{MODIS Burned Area Monthly (MCD64A1)}
\source{
\url{https://planetarycomputer.microsoft.com/dataset/modis-64A1-061}
}
\usage{
get_mcd64a1(years = 2000:2022)
}
\arguments{
\item{years}{Numeric vector of years to make the MCD64A1 product available
for. Must be greater than the year 2000.}
}
\value{
A function that returns an \code{sf} footprint object.
}
\description{
The Terra and Aqua combined MCD64A1 Version 6.1 Burned Area data product is
a monthly, global gridded 500 meter (m) product containing per-pixel
burned-area and quality information. The MCD64A1 burned-area mapping approach
employs 500 m Moderate Resolution Imaging Spectroradiometer (MODIS) Surface
Reflectance imagery coupled with 1 kilometre (km) MODIS active fire observations.
}
\details{
The algorithm uses a burn sensitive Vegetation Index (VI) to create dynamic
thresholds that are applied to the composite data. The VI is derived from
MODIS shortwave infrared atmospherically corrected surface reflectance bands
5 and 7 with a measure of temporal texture. The algorithm identifies the date
of burn for the 500 m grid cells within each individual MODIS tile. The date
is encoded in a single data layer as the ordinal day of the calendar year on
which the burn occurred with values assigned to unburned land pixels and
additional special values reserved for missing data and water grid cells.
}
\references{
Giglio, L., C. Justice, L. Boschetti, D. Roy. MODIS/Terra+Aqua
Burned Area Monthly L3 Global 500m SIN Grid V061. 2021, distributed by
NASA EOSDIS Land Processes Distributed Active Archive Center.
\doi{doi:10.5067/MODIS/MCD64A1.061}
}
\keyword{resource}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_temperature_min_wc.R
\name{temperature_min_wc}
\alias{temperature_min_wc}
\alias{calc_temperature_min_wc}
\title{Calculate minimum temperature statistics based on WorldClim}
\usage{
calc_temperature_min_wc(engine = "extract", stats = "mean")
}
\arguments{
\item{engine}{The preferred processing functions from either one of "zonal",
"extract" or "exactextract" as character.}

\item{stats}{Function to be applied to compute statistics for polygons either
single or multiple inputs as character. Supported statistics are: "mean",
"median", "sd", "min", "max", "sum" "var".}
}
\value{
A function that returns an indicator tibble with minimum temperature
statistics as variables and corresponding values as value.
}
\description{
This function allows to efficiently calculate minimum temperature statistics
from Worldclim for polygons. For each polygon, the desired statistic/s (min,
max, sum, mean, median, sd or var) is/are returned.
}
\details{
The required resources for this indicator are:
\itemize{
\item minimum temperature layer from \link{worldclim_min_temperature}
}
}
\examples{
\dontshow{
mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
}
\dontrun{
library(sf)
library(mapme.biodiversity)

outdir <- file.path(tempdir(), "mapme-data")
dir.create(outdir, showWarnings = FALSE)

mapme_options(
  outdir = outdir,
  verbose = FALSE
)

aoi <- system.file("extdata", "sierra_de_neiba_478140_2.gpkg",
  package = "mapme.biodiversity"
) \%>\%
  read_sf() \%>\%
  get_resources(get_worldclim_min_temperature(years = 2018)) \%>\%
  calc_indicators(
    calc_temperature_min_wc(
      engine = "extract",
      stats = c("mean", "median")
    )
  ) \%>\%
  portfolio_long()

aoi
}
}
\keyword{indicator}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_gsw_change.R
\name{gsw_change}
\alias{gsw_change}
\alias{calc_gsw_change}
\title{Calculate Global Surface Water (GSW) Change}
\usage{
calc_gsw_change(engine = "extract", stats = "mean")
}
\arguments{
\item{engine}{The preferred processing functions from either one of "zonal",
"extract" or "exactextract". Default: "extract".}

\item{stats}{Aggregation function with which the data are combined.
Default: "mean".}
}
\value{
A function that returns an indicator tibble with change intensity
as variable and corresponding (unitless) values as value.
}
\description{
The change in water occurrence intensity between the two periods is derived
from homologous pairs of months (i.e. same months containing valid
observations in both periods). The difference in the occurrence of surface
water was calculated for each homologous pair of months. The average of all
of these differences constitutes the Surface Water Occurrence change
intensity. The raster files have integer cell values between \verb{[0, 200]}
where 0 represents surface water loss and 200 represents surface water gain.
}
\details{
The pixel values are aggregated using method provided via the
\code{stats} parameter using the specified \code{engine}.

The required resources for this indicator are:
\itemize{
\item \link{global_surface_water_change}
}
}
\examples{
\dontshow{
mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
}
\dontrun{
library(sf)
library(mapme.biodiversity)

outdir <- file.path(tempdir(), "mapme-data")
dir.create(outdir, showWarnings = FALSE)

mapme_options(
  outdir = outdir,
  verbose = FALSE
)

aoi <- system.file("extdata", "shell_beach_protected_area_41057_B.gpkg",
  package = "mapme.biodiversity"
) \%>\%
  read_sf() \%>\%
  get_resources(get_global_surface_water_change()) \%>\%
  calc_indicators(
    calc_gsw_change(engine = "extract", stats = "mean")
  ) \%>\%
  portfolio_long()

aoi
}
}
\keyword{indicator}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_nasa_grace.R
\name{nasa_grace}
\alias{nasa_grace}
\alias{get_nasa_grace}
\title{NASA GRACE-based Drought Indicator layer}
\usage{
get_nasa_grace(years = 2003:2022)
}
\arguments{
\item{years}{A numeric vector indicating the years for which to make the
resource available.}
}
\value{
A function that returns an \code{sf} footprint object.
}
\description{
The resource is published by NASA GRACE Tellus. This data set
reflects on potential drought conditions in the shallow groundwater section
relative to a reference period spanning from 1948 to 2012.
It is available as a global raster with a weekly temporal resolution starting
with the year 2003. The value indicates the wetness percentile of a given
pixel with regard to the reference period.
}
\keyword{resource}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_slope.R
\name{slope}
\alias{slope}
\alias{calc_slope}
\title{Calculate slope statistics}
\usage{
calc_slope(engine = "exactextract", stats = "mean")
}
\arguments{
\item{engine}{The preferred processing function from either one of "zonal",
"extract" or "exactextract" as a character string.}

\item{stats}{Function to be applied to compute statistics for polygons.
Accepts either a single string or a vector of strings, such as "mean",
"median", "sd", "min", "max", "sum", or "var".}
}
\value{
A function that returns an indicator tibble with specified slope
statistics as variables and corresponding values (in degrees).
}
\description{
This function allows to calculate slope statistics for
polygons. For each polygon, the desired statistic(s)
are returned.
}
\details{
The required resource for this indicator is:
\itemize{
\item \link{nasa_srtm}
}
}
\examples{
\dontshow{
mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
}
\dontrun{
library(sf)
library(mapme.biodiversity)

outdir <- file.path(tempdir(), "mapme-data")
dir.create(outdir, showWarnings = FALSE)

mapme_options(
  outdir = outdir,
  verbose = FALSE
)

aoi <- system.file("extdata", "sierra_de_neiba_478140_2.gpkg",
  package = "mapme.biodiversity"
) \%>\%
  read_sf() \%>\%
  get_resources(get_nasa_srtm()) \%>\%
  calc_indicators(
    calc_slope(stats = c("mean", "median", "sd", "var"), engine = "extract")
  ) \%>\%
  portfolio_long()

aoi
}
}
\keyword{indicator}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_gsw.R
\name{global_surface_water_occurrence}
\alias{global_surface_water_occurrence}
\alias{get_global_surface_water_occurrence}
\title{Global Surface Water Occurrence}
\source{
\url{https://global-surface-water.appspot.com/}
}
\usage{
get_global_surface_water_occurrence(version = "v1_4_2021")
}
\arguments{
\item{version}{A character vector indicating the version of the GSW data set
to make available.}
}
\value{
A character of file paths.
}
\description{
The Global Surface Water dataset was developed by the European Commission's
Joint Research Centre in the framework of the Copernicus Programme. It maps
the location and temporal distribution of water surfaces at the global scale
over the past 3.8 decades and provides statistics on their extent and change.
It is provisioned as a global tiled raster resource available for all land
areas. The reported data represent aggregated observations between 1984 - 2021.
}
\details{
GSW occurrence raw data comes in raster files with integer cell values
between \code{[0, 100]}. This value gives the percentage of the time that a
given pixel was classified as water during the entire observation period. So
a 0 denotes a pixel that was never classified as water, 100 denotes a pixel
with permanent water.
}
\references{
Pekel, JF., Cottam, A., Gorelick, N. et al. High-resolution
mapping of global surface water and its long-term changes. Nature 540,
418–422 (2016). \doi{doi:10.1038/nature20584}
}
\keyword{resource}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_gsw_transitions.R
\name{gsw_transitions}
\alias{gsw_transitions}
\alias{calc_gsw_transitions}
\title{Calculate Global Surface Water (GSW) Transitions}
\usage{
calc_gsw_transitions()
}
\value{
A function that returns an indicator tibble with transition classes
as variable and corresponding areas (in ha) as value.
}
\description{
GSW transition data contains information about the type of surface water
change for each pixel. The raster files have integer cell values between
\code{[0, 10]} that code for different transition classes:
}
\details{
\tabular{ll}{
   Value \tab Transition Class \cr
   1 \tab Permanent \cr
   2 \tab New Permanent \cr
   3 \tab Lost Permanent \cr
   4 \tab Seasonal \cr
   5 \tab New Seasonal \cr
   6 \tab Lost Seasonal \cr
   7 \tab Seasonal to Permanent \cr
   8 \tab Permanent to Seasonal \cr
   9 \tab Ephemeral Permanent \cr
   10 \tab Ephemeral Seasonal \cr
}


To aggregate, we sum up the area of each transition class for a given region.

The required resources for this indicator are:
\itemize{
\item \link{global_surface_water_transitions}
}
}
\examples{
\dontshow{
mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
}
\dontrun{
library(sf)
library(mapme.biodiversity)

outdir <- file.path(tempdir(), "mapme-data")
dir.create(outdir, showWarnings = FALSE)

mapme_options(
  outdir = outdir,
  verbose = FALSE
)

aoi <- system.file("extdata", "shell_beach_protected_area_41057_B.gpkg",
  package = "mapme.biodiversity"
) \%>\%
  read_sf() \%>\%
  get_resources(get_global_surface_water_transitions()) \%>\%
  calc_indicators(calc_gsw_transitions()) \%>\%
  portfolio_long()

aoi
}
}
\keyword{indicator}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_carbon.R
\name{carbon_indicators}
\alias{carbon_indicators}
\alias{calc_irr_carbon}
\alias{calc_man_carbon}
\alias{calc_vul_carbon}
\title{Calculate carbon statistics}
\usage{
calc_irr_carbon(
  type = c("total", "soil", "biomass", "all"),
  engine = "extract",
  stats = "mean"
)

calc_man_carbon(
  type = c("total", "soil", "biomass", "all"),
  engine = "extract",
  stats = "mean"
)

calc_vul_carbon(
  type = c("total", "soil", "biomass", "all"),
  engine = "extract",
  stats = "mean"
)
}
\arguments{
\item{type}{One of "total", "soil", "biomass", "all". Determines
for which data layer the statistics are calculated.}

\item{engine}{The preferred processing functions from either one of "zonal",
"extract" or "exactextract" as character.}

\item{stats}{Function to be applied to compute statistics for polygons either
one or multiple inputs as character. Supported statistics are: "mean",
"median", "sd", "min", "max", "sum", and "var".}
}
\value{
A function that returns an indicator tibble with \verb{(type)_carbon_(stat)}
as variable and the respective statistic (in Mg) as value.
}
\description{
These functions allow to calculated statistics based on the harmonized
carbon layers for 2010 and 2018 by Noon et al. (2022).
}
\details{
The required resources for these indicators are:
\itemize{
\item \link{carbon_resources}
}

Irrecoverable carbon is the amount of carbon that, if lost today, could not
be recovered until 2050. It can be calculated for above- and below-ground
carbon, the total amount of carbon, or for all layers.

Manageable carbon is the amount of carbon that, in principle, is manageable
by human activities, e.g. its release to the atmosphere can be prevented.
It can be calculated for above- and below-ground carbon, the total amount
of carbon, or for all layers.

Vulnerable carbon is the amount of carbon that would be released in a typical
land conversion activity.
It can be calculated for above- and below-ground carbon, the total amount
of carbon, or for all layers.
}
\examples{
\dontshow{
mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
}
\dontrun{
library(sf)
library(mapme.biodiversity)

outdir <- file.path(tempdir(), "mapme-data")
dir.create(outdir, showWarnings = FALSE)

mapme_options(
  outdir = outdir,
  verbose = FALSE
)

aoi <- system.file("extdata", "shell_beach_protected_area_41057_B.gpkg",
  package = "mapme.biodiversity"
) \%>\%
  read_sf() \%>\%
  get_resources(
    get_man_carbon(),
    get_vul_carbon(),
    get_irr_carbon()
  ) \%>\%
  calc_indicators(
    calc_man_carbon(stats = "sum"),
    calc_vul_carbon(stats = "sum"),
    calc_irr_carbon(stats = "sum")
  ) \%>\%
  portfolio_long()

aoi
}
}
\keyword{indicator}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{make_global_grid}
\alias{make_global_grid}
\title{Helper to create a grid of regular resolution and CRS}
\usage{
make_global_grid(
  xmin = -180,
  xmax = 170,
  dx = 10,
  ymin = -50,
  ymax = 80,
  dy = 10,
  proj = NULL
)
}
\arguments{
\item{xmin}{minimum longitude value (E/W)}

\item{xmax}{maximum longitude value (E/W)}

\item{dx}{difference in longitude value per grid}

\item{ymin}{minimum latitude value (S/N)}

\item{ymax}{maximum latitude value (E/W)}

\item{dy}{difference in latitude value per grid}

\item{proj}{projection system}
}
\value{
An sf object with a defined grid.
}
\description{
Use this function to create a regular grid in a custom CRS. This is used
e.g. to create the tile grid for Global Forest Watch in order to retrieve
the intersecting tiles with a given portfolio.
}
\keyword{utils}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_biodiversity_intactness_index.R
\name{biodiversity_intactness_index_resource}
\alias{biodiversity_intactness_index_resource}
\alias{get_biodiversity_intactness_index}
\title{Biodiversity Intactness Index}
\usage{
get_biodiversity_intactness_index(path = NULL)
}
\arguments{
\item{path}{A character vector to the biodiversity intactness index ASCII file.}
}
\value{
A function that returns an \code{sf} footprints object.
}
\description{
The variable is the modelled average abundance of originally-present species,
relative to their abundance in an intact ecosystem. Please refer to
Newbold et al. (2016) for all details, and please cite it when using these
data.
}
\details{
To use this data in mapme workflows, you will have to manually download the
global data set and point towards the file path on your local machine.
Please find the available data under the source link given below.
}
\references{
Tim Newbold; Lawrence Hudson; Andy Arnell; Sara Contu et al.
(2016). Global map of the Biodiversity Intactness Index, from Newbold et al.
(2016) Science [Data set]. Natural History Museum.
\doi{doi:10.5519/0009936}
}
\keyword{resource}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_worldclim.R
\name{worldclim_max_temperature}
\alias{worldclim_max_temperature}
\alias{get_worldclim_max_temperature}
\title{Downloads WorldClim Maximum Temperature layer}
\source{
\url{https://www.worldclim.org/data/index.html}
}
\usage{
get_worldclim_max_temperature(
  years = 2000:2024,
  resolution = c("2.5m", "5m", "10m")
)
}
\arguments{
\item{years}{A numeric vector indicating for which years to make the
resource available. Defaults to 2000:2024.}

\item{resolution}{A character vector indicating the desired resolution:
"2.5m", "5m", or "10m".}
}
\value{
A function that returns an \code{sf} footprint object.
}
\description{
This resource is published by Fick et al. (2017) "WorldClim 2: new 1-km
spatial resolution climate surfaces for global land areas" and
represents multiple climatic variables from which we will be requiring minimum
temperature, maximum temperature, and mean precipitation layers. The layers are
available to download for the period 1960 - 2024 on monthly basis from WorldClim.
}
\details{
This resource represents the maximum temperature, layers available to
download for the period 1960 - 2024 on monthly basis from WorldClim. Encoded
as (°C), representing the maximum temperature per output grid cell.
}
\keyword{resource}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_gfw_treecover.R
\name{gfw_treecover}
\alias{gfw_treecover}
\alias{get_gfw_treecover}
\title{Treecover for the year 2000}
\source{
\url{https://data.globalforestwatch.org/documents/tree-cover-2000/explore}
}
\usage{
get_gfw_treecover(version = "GFC-2024-v1.12")
}
\arguments{
\item{version}{The version of the dataset to download. Defaults to
"GFC-2024-v1.12". Check mapme.biodiversity:::.available_gfw_versions()
to get a list of available versions}
}
\value{
A function that returns an \code{sf} footprint object.
}
\description{
This resource is part of the publication by Hansen et al. (2013)
represents "tree cover in the year 2000, defined as canopy closure for all
vegetation taller than 5m in height. Encoded as a percentage per output grid
cell, in the range 0–100." Due to changes in the satellites products used
in the compilation of the treecover product, results before the year 2011
and afterwards are not directly comparable until reprocessing has finished.
Users should be aware of this limitation, especially when the timeframe
of the analysis spans over the two periods delimited by the year 2011.
}
\references{
Hansen, M. C., P. V. Potapov, R. Moore, M. Hancher, S. A.
Turubanova, A. Tyukavina, D. Thau, S. V. Stehman, S. J. Goetz, T. R.
Loveland, A. Kommareddy, A. Egorov, L. Chini, C. O. Justice, and J. R. G.
Townshend. 2013. “High-Resolution Global Maps of 21st-Century Forest Cover
Change.” Science 342 (15 November): 850–53.
}
\keyword{resource}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_treecover_area_and_emissions.R
\name{treecover_area_and_emissions}
\alias{treecover_area_and_emissions}
\alias{calc_treecover_area_and_emissions}
\title{Calculate treeloss statistics}
\usage{
calc_treecover_area_and_emissions(
  years = 2000:2024,
  min_size = 10,
  min_cover = 35
)
}
\arguments{
\item{years}{A numeric vector with the years for which to calculate treecover
area and emissions.}

\item{min_size}{The minimum size of a forest patch in ha.}

\item{min_cover}{The minimum threshold of stand density for a pixel to be
considered forest in the year 2000.}
}
\value{
A function that returns an indicator tibble with variables treecover
and emissions and corresponding values (in ha and Mg) as value.
}
\description{
This functions allows to efficiently calculate the treecover and emissions
indicators in a single function call together. Since most of the pre-processing
operations for treecover and emissions are the same, it is more efficient
to calculate them in one run if users are actually interested in both statistics.
Otherwise users are advised to use the respective single indicator functions.
}
\details{
The required resources for this indicator are:
\itemize{
\item \link{gfw_treecover}
\item \link{gfw_lossyear}
\item \link{gfw_emissions}
}
}
\examples{
\dontshow{
mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
}
\dontrun{
library(sf)
library(mapme.biodiversity)

outdir <- file.path(tempdir(), "mapme-data")
dir.create(outdir, showWarnings = FALSE)

mapme_options(
  outdir = outdir,
  verbose = FALSE
)

aoi <- system.file("extdata", "sierra_de_neiba_478140_2.gpkg",
  package = "mapme.biodiversity"
) \%>\%
  read_sf() \%>\%
  get_resources(
    get_gfw_treecover(version = "GFC-2024-v1.12"),
    get_gfw_lossyear(version = "GFC-2024-v1.12"),
    get_gfw_emissions()
  ) \%>\%
  calc_indicators(
    calc_treecover_area_and_emissions(years = 2016:2017, min_size = 1, min_cover = 30)
  ) \%>\%
  portfolio_long()

aoi
}
}
\keyword{indicator}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_chelsa.R
\name{chelsa}
\alias{chelsa}
\alias{get_chelsa}
\title{Climatologies at High resolution for the Earth Land Surface Areas (CHELSA)}
\source{
\url{https://envicloud.wsl.ch/#/?prefix=chelsa/chelsa_V2/GLOBAL/}
}
\usage{
get_chelsa(years = 1979:2019)
}
\arguments{
\item{years}{A numeric vector of the years to make CHELSA monthly precipitation
layers available for. Must be greater 1979, defaults to \code{c(1979:2019)}.}
}
\value{
A function that returns an \code{sf} footprint object.
}
\description{
The CHELSA data (Karger et al. 2017) consists of downscaled model output
temperature and precipitation estimates at a horizontal resolution of 30 arc
sec. The precipitation algorithm incorporates orographic predictors including
wind fields, valley exposition, and boundary layer height, with a subsequent
bias correction. The spatial resolution is about 1-arc second (~1km at the
equator). This resource makes V2 available.
}
\references{
Karger, D.N., Conrad, O., Böhner, J., Kawohl, T., Kreft, H.,
Soria-Auza, R.W., Zimmermann, N.E., Linder, H.P. & Kessler, M. (2021)
Climatologies at high resolution for the earth’s land surface areas.
EnviDat. \doi{doi:10.16904/envidat.228.v2.1}

Karger, D.N., Conrad, O., Böhner, J., Kawohl, T., Kreft, H., Soria-Auza,
R.W., Zimmermann, N.E., Linder, P., Kessler, M. (2017): Climatologies at high
resolution for the Earth land surface areas. Scientific Data. 4
170122. \doi{doi:10.1038/sdata.2017.122}
}
\keyword{resource}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_iucn.R
\name{iucn}
\alias{iucn}
\alias{get_iucn}
\title{IUCN Red List of Threatened Species}
\source{
\url{https://www.iucnredlist.org/resources/other-spatial-downloads}
}
\usage{
get_iucn(paths = NULL)
}
\arguments{
\item{paths}{A character vector to the respective species range files in GTiff
format. Note, that theses files have to be downloaded manually.}
}
\value{
A function that returns an \code{sf} footprint object.
}
\description{
This resource is part of the spatial data set Red List of Threatened Species
released by IUCN. It is free to use under a non-commercial licence. For
commercial uses, a request has to be sent to Integrated Biodiversity
Assessment Tool (IBAT).
}
\details{
To use this data in mapme workflows, you will have to manually download the
global data set and point towards the file path on your local machine.
Please find the available data under the source link given below.
}
\references{
IUCN (2024). The IUCN Red List of Threatened Species.
\url{https://www.iucnredlist.org}
}
\keyword{resource}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_gfw_lossyear.R
\name{gfw_lossyear}
\alias{gfw_lossyear}
\alias{get_gfw_lossyear}
\title{Year of forest loss occurrence}
\source{
\url{https://data.globalforestwatch.org/documents/tree-cover-loss/explore}
}
\usage{
get_gfw_lossyear(version = "GFC-2024-v1.12")
}
\arguments{
\item{version}{The version of the dataset to download. Defaults to
"GFC-2024-v1.12". Check \code{mapme.biodiversity:::.available_gfw_versions()}
to get a list of available versions}
}
\value{
A function that returns an \code{sf} footprint object.
}
\description{
This resource is part of the publication by Hansen et al. (2013)
"High-Resolution Global Maps of 21st-Century Forest Cover Change". It
represents "Forest loss during the period 2000–2021, defined as a
stand-replacement disturbance, or a change from a forest to non-forest state.
Encoded as either 0 (no loss) or else a value in the range 1–20, representing
loss detected primarily in the year 2001–2021, respectively." Due to changes
in the satellites products used in the compilation of the tree loss product,
results before the year 2011 and afterwards are not directly comparable
until reprocessing has finished. Users should be aware of this limitation,
especially when the timeframe of the analysis spans over the two periods
delimited by the year 2011.
}
\references{
Hansen, M. C., P. V. Potapov, R. Moore, M. Hancher, S. A.
Turubanova, A. Tyukavina, D. Thau, S. V. Stehman, S. J. Goetz, T. R.
Loveland, A. Kommareddy, A. Egorov, L. Chini, C. O. Justice, and J. R. G.
Townshend. 2013. “High-Resolution Global Maps of 21st-Century Forest Cover
Change.” Science 342 (15 November): 850–53.
}
\keyword{resource}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/register.R, R/get_resources.R,
%   R/calc_indicators.R, R/spatial-utils.R
\name{mapme}
\alias{mapme}
\alias{mapme_options}
\alias{get_resources}
\alias{calc_indicators}
\alias{prep_resources}
\title{Portfolio methods for mapme.biodiversity}
\usage{
mapme_options(..., outdir, chunk_size, retries, delay, verbose, log_dir)

get_resources(x, ...)

calc_indicators(x, ...)

prep_resources(
  x,
  avail_resources = NULL,
  resources = NULL,
  mode = c("portfolio", "asset")
)
}
\arguments{
\item{...}{One or more functions for resources/indicators}

\item{outdir}{A length one character indicating the output path.}

\item{chunk_size}{A numeric of length one giving the maximum chunk area in ha.
Defaults to 100,000 ha. It refers to the area of an asset's bounding box.
If it lies above the value \code{chunk_size}, splitting and chunking is considered.
An asset will be processes as-is with an bounding box area below the specified
value.}

\item{retries}{An integer of length one indicating the number of retries
the package should attempt to make a resource available.
This commands the GDAL option \code{GDAL_HTTP_MAX_RETRY} (see
\href{https://gdal.org/en/stable/user/configoptions.html#:~:text=GDAL_HTTP_MAX_RETRY}{here}).
Defaults to 3.}

\item{delay}{An integer of length one indicating the number of seconds
the package should wait between the successive retries.
This commands the GDAL option \code{GDAL_HTTP_RETRY_DELAY}  (see
\href{https://gdal.org/en/stable/user/configoptions.html#:~:text=GDAL_HTTP_RETRY_DELAY}{here}).
Defaults to 10.}

\item{verbose}{A logical, indicating if informative messages should be printed.}

\item{log_dir}{A character path pointing toward a GDAL-writable destination
used to log erroneous assets. Defaults to NULL, meaning that erroneous
assets will not be serialized to disk. If specified, a GPKG file named
\code{file.path(log_dir, paste0(Sys.Date(), "_mapme-error-assets.gpkg"))} will
be created and appended to in case of erroneous assets. In the case some of the
resources could not be downloaded, their list will be written to a GPKG file named
\code{file.path(log_dir, paste0(Sys.Date(), "_", {resource_name}, "_mapme-error-resources.gpkg"))}.}

\item{x}{An \code{sf} object with features of type \code{"POLYGON"}}

\item{avail_resources}{A list object of available resources. If NULL (the default),
the available resources will automatically be determined.}

\item{resources}{A character vector with the resources to be prepared. If it
it is NULL (the default) all available resources will be prepared.}

\item{mode}{A character indicating the reading mode, e.g. either "portfolio"
(the default) or "asset".}
}
\value{
\code{mapme_options()} returns a list of options if no arguments are
specified. Otherwise sets matching arguments to new values in the package's
internal environment.

\code{get_resources()} is called for its side effect of making resources
available in the package environment. Returns \code{x}, invisibly.

\code{calc_indicators()} returns \code{x}, invisibly, with an additional nested
list column per requested indicator.

\code{prep_resources()} returns a list with prepared vector and raster
resources as \code{sf} and \code{SpatRaster}-objects.
}
\description{
\code{mapme_options()} sets default options for mapme.biodiversity to control the
behaviour of downstream functions. Mainly, the output path as well as the
chunk size (in ha), can be set. Additionally, the verbosity can be set and
the path to a log directory can be controlled. It also allow to set the number of
retries  and the delay between the successive retries when retrieving the resources.
Might be extended by other
options in the future.

\code{get_resources()} data sets required for the
calculation of indicators can be made available. The function supports the
specification of several resource functions. To determine the output path,
temporary directory and verbosity, the output of \code{mapme_options()} is used.

\code{calc_indicators()} calculates specific biodiversity indicators. A
requirement is that the resources that are mandatory inputs for the requested
indicators are available locally. Multiple indicators and their respective
additional arguments can be supplied.

This function reads and crops available resources to the extent of a single
asset. Specific resources can be queried. If not supplied (the default), all
available resources will be prepared.
}
\examples{
library(mapme.biodiversity)
mapme_options()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_iucn.R
\name{species_richness}
\alias{species_richness}
\alias{calc_species_richness}
\title{Species richness based on IUCN raster data}
\usage{
calc_species_richness(engine = "extract", stats = "mean")
}
\arguments{
\item{engine}{The preferred processing functions from either one of "zonal",
"extract" or "exactextract" as character.}

\item{stats}{Function to be applied to compute statistics for polygons either
one or multiple inputs as character. Supported statistics are: "mean",
"median", "sd", "min", "max", "sum" "var".}
}
\value{
A function that returns an indicator tibble with IUCN layers with
specified statistics as variable and respective species richness (count)
as value.
}
\description{
Species richness counts the number of potential species intersecting with a
polygon grouped by the IUCN threat categorization. Note, that
this indicator function requires the manual download of the respective
raster files.
}
\details{
The specific meaning of the species richness indicator depends on the
supplied raster file.

The required resources for this indicator are:
\itemize{
\item \link{iucn}
}
}
\examples{
\dontshow{
mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
}
\dontrun{
library(sf)
library(mapme.biodiversity)

outdir <- file.path(tempdir(), "mapme-data")
dir.create(outdir, showWarnings = FALSE)

mapme_options(
  outdir = outdir,
  verbose = FALSE
)

iucn_dir <- system.file("res", "iucn", package = "mapme.biodiversity")
sr_rasters <- list.files(iucn_dir, pattern = "*_SR_*", full.names = TRUE)

aoi <- system.file("extdata", "sierra_de_neiba_478140_2.gpkg",
  package = "mapme.biodiversity"
) \%>\%
  read_sf() \%>\%
  get_resources(get_iucn(sr_rasters)) \%>\%
  calc_indicators(calc_species_richness(stats = "median")) \%>\%
  portfolio_long()

aoi
}
}
\keyword{indicator}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_ipbes_biomes.R
\name{ipbes_biomes}
\alias{ipbes_biomes}
\alias{get_ipbes_biomes}
\title{Terrestrial and Aquatic Biomes}
\source{
\url{https://zenodo.org/records/3975694}
}
\usage{
get_ipbes_biomes()
}
\value{
A function that returns an \code{sf} footprint object.
}
\description{
This resource is part of the Global Assessment Report on Biodiversity and
Ecosystem Services and represents a division of the Earth's surface
into several subcategories. The classification differentiates between
biomes and anthromes. Biomes are differentiated between terrestrial and aquatic
biomes.
}
\details{
Terrestrial biomes include:
\itemize{
\item Tropical and subtropical dry and humid forests
\item Temperate and boreal forests and woodlands
\item Mediterranean forests, woodlands and scrub
\item Tundra and High Mountain habitats
\item Tropical and subtropical savannas and grasslands
\item Temperate Grasslands
\item Deserts and xeric shrublands
\item Wetlands – peatlands, mires, bogs
}

Aquatic biomes include:
\itemize{
\item Cryosphere
\item Aquaculture areas
\item Inland surface waters and water bodies/freshwater
\item Shelf ecosystems (neritic and intertidal/littoral zone)
\item Open ocean pelagic systems (euphotic zone)
}
}
\references{
IPBES (2019): Summary for policymakers of the global assessment
report on biodiversity and ecosystem services of the Intergovernmental
Science-Policy Platform on Biodiversity and Ecosystem Services. S. Díaz, J.
Settele, E. S. Brondízio, H. T. Ngo, M. Guèze, J. Agard, A. Arneth, P.
Balvanera, K. A. Brauman, S. H. M. Butchart, K. M. A. Chan, L. A. Garibaldi,
K. Ichii, J. Liu, S. M. Subramanian, G. F. Midgley, P. Miloslavich, Z.
Molnár, D. Obura, A. Pfaff, S. Polasky, A. Purvis, J. Razzaque, B. Reyers, R.
Roy Chowdhury, Y. J. Shin, I. J. Visseren-Hamakers, K. J. Willis, and C. N.
Zayas (eds.). IPBES secretariat, Bonn, Germany. 56 pages.
\doi{doi:10.5281/zenodo.3553579}
}
\keyword{resource}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_tri.R
\name{tri}
\alias{tri}
\alias{calc_tri}
\title{Calculate Terrain Ruggedness Index (TRI) statistics}
\usage{
calc_tri(engine = "extract", stats = "mean")
}
\arguments{
\item{engine}{The preferred processing functions from either one of "zonal",
"extract" or "exactextract" as character.}

\item{stats}{Function to be applied to compute statistics for polygons either
single or multiple inputs as character. Supported statistics are: "mean",
"median", "sd", "min", "max", "sum" "var".}
}
\value{
A function that returns an indicator tibble with tri as variable and
the respective statistic as value.
}
\description{
Terrain Ruggedness Index is a measurement developed by Riley, et al. (1999).
The elevation difference between the centre pixel and its eight immediate
pixels are squared and then averaged and its square root is taken to get
the TRI value. This function allows to calculate terrain ruggedness
index (tri) statistics for polygons. For each polygon, the desired statistic(s)
are returned.
}
\details{
The range of index values and corresponding meaning:
\itemize{
\item 0-80 m - level surface
\item 81-116 m - nearly level surface
\item 117-161 m - slightly rugged surface
\item 162-239 m - intermediately rugged surface
\item 240-497 m - moderately rugged surface
\item 498-958 m - highly rugged surface
\item 959-4367 m  extremely rugged surface
}

The required resources for this indicator are:
\itemize{
\item \link{nasa_srtm}
}
}
\examples{
\dontshow{
mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
}
\dontrun{
library(sf)
library(mapme.biodiversity)

outdir <- file.path(tempdir(), "mapme-data")
dir.create(outdir, showWarnings = FALSE)

mapme_options(
  outdir = outdir,
  verbose = FALSE
)

aoi <- system.file("extdata", "sierra_de_neiba_478140_2.gpkg",
  package = "mapme.biodiversity"
) \%>\%
  read_sf() \%>\%
  get_resources(get_nasa_srtm()) \%>\%
  calc_indicators(
    calc_tri(stats = c("mean", "median", "sd", "var"), engine = "extract")
  ) \%>\%
  portfolio_long()

aoi
}
}
\references{
Riley, S. J., DeGloria, S. D., & Elliot, R. (1999). Index that quantifies
topographic heterogeneity. Intermountain Journal of Sciences, 5(1-4), 23-27.
}
\keyword{indicator}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_esalandcover.R
\name{esalandcover}
\alias{esalandcover}
\alias{get_esalandcover}
\title{ESA Copernicus Global Land Cover layer}
\source{
\url{https://zenodo.org/records/3939050}
}
\usage{
get_esalandcover(years = 2015L:2019L)
}
\arguments{
\item{years}{A numeric vector indicating the years for which to make the
resource available.}
}
\value{
A function that returns an \code{sf} footprint object.
}
\description{
This 100 meter spatial resolution land cover resource is published by Buchhorn
et al. (2020) "Copernicus Global Land Cover Layers—Collection 2". The resource
represents the actual surface cover of ground available annually for the period
2015 to 2019. The cell values range from 0 to 200, representing total of 23
discrete classifications from ESA.
}
\details{
This function mimics the behavior of the original \code{get_esalandcover()} function by
downloading the parts of the global raster that correspond to the 20° x 20° tiles used
previously. The files are named the same as before. This allows other code that uses
this resource to remain unchanged.
}
\references{
Buchhorn, M., Lesiv, M., Tsendbazar, N.-E., Herold, M., Bertels, L.,
& Smets, B. (2020). Copernicus Global Land Cover Layers - Collection 2.
Remote Sensing, 12(6), 1044.
\doi{doi:10.3390/rs12061044}
}
\keyword{resource}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_soilgrids.R
\name{soilgrids}
\alias{soilgrids}
\alias{get_soilgrids}
\title{SoilGrids data layers}
\source{
\url{https://isric.org/explore/soilgrids}
}
\usage{
get_soilgrids(layers, depths, stats)
}
\arguments{
\item{layers}{A character vector indicating the layers to download from
soilgrids}

\item{depths}{A character vector indicating the depths to download}

\item{stats}{A character vector indicating the statistics to download.}
}
\value{
A function that returns an \code{sf} footprint object.
}
\description{
SoilGrids is a project combining global observation data with
machine learning to map the spatial distribution of soil properties across
the globe. It is produced at a spatial resolution of 250 meters and each
parameter is mapped at different depths. In order to be able to assess
prediction uncertainty, besides the mean and median prediction, the 0.05 and
0.95 percentile predictions are available.
The following parameters are available:
\describe{
\item{bdod}{Bulk density of the fine earth fraction (kg/dm3)}
\item{cec}{Cation Exchange Capacity of the soil (cmol(c)/kg)}
\item{cfvo}{Volumetric fraction of coarse fragments > 2 mm (cm3/100cm3 (volPerc))}
\item{clay}{Proportion of clay particles < 0.002 mm in the fine earth fraction (g/100g)}
\item{nitrogen}{Total nitrogen (g/kg)}
\item{phh2o}{Soil pH (pH)}
\item{sand}{Proportion of sand particles > 0.05 mm in the fine earth fraction (g/100g)}
\item{silt}{Proportion of silt particles >= 0.002 mm and <= 0.05 mm in the fine earth fraction (g/100g)}
\item{soc}{Soil organic carbon content in the fine earth fraction (g/kg)}
\item{ocd}{Organic carbon density (kg/m3)}
\item{ocs}{Organic carbon stocks (kg/m²)}
}
}
\details{
Except for \code{ocs}, which is only available for a depth of \code{"0-30cm"},
all other parameters are available at the following depths:
\itemize{
\item "0-5cm"
\item "5-15cm"
\item "15-30cm"
\item "30-60cm"
\item "60-100cm"
\item "100-200cm"
}

Each parameter and depth is available for the following statistics:
\itemize{
\item "Q0.05"
\item "Q0.50"
\item "mean"
\item "Q0.95"
}
}
\references{
Poggio, L., de Sousa, L. M., Batjes, N. H., Heuvelink, G. B. M.,
Kempen, B., Ribeiro, E., and Rossiter, D.: SoilGrids 2.0: producing soil
information for the globe with quantified spatial uncertainty, SOIL, 7,
217–240, 2021. \doi{doi:10.5194/soil-7-217-2021}
}
\keyword{resource}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_biodiversity_intactness_index.R
\docType{data}
\name{biodiversity_intactness_index_indicator}
\alias{biodiversity_intactness_index_indicator}
\alias{calc_biodiversity_intactness_index}
\title{Calculate Biodiversity Intactness Index}
\format{
A function that returns an indicator tibble with variable
biodiversity_intactness_index and corresponding values (unitless) as value.
}
\usage{
calc_biodiversity_intactness_index()
}
\description{
This function calculates the mean biodiversity intactness index for a region.
}
\details{
The required resources for this indicator are:
\itemize{
\item \link{biodiversity_intactness_index_resource}
}
}
\examples{
\dontshow{
mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
}
\dontrun{
library(sf)
library(mapme.biodiversity)

outdir <- file.path(tempdir(), "mapme-data")
dir.create(outdir, showWarnings = FALSE)

mapme_options(
  outdir = outdir,
  verbose = FALSE
)

lbii <- system.file("res", "biodiversity_intactness_index", "lbii.asc",
                    package = "mapme.biodiversity")

aoi <- read_sf(
  system.file("extdata", "shell_beach_protected_area_41057_B.gpkg",
              package = "mapme.biodiversity"
))
aoi <- get_resources(aoi, get_biodiversity_intactness_index(lbii))
aoi <- calc_indicators(aoi, calc_biodiversity_intactness_index())
aoi <- portfolio_long(aoi)

aoi
}
}
\keyword{indicator}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_exposed_population_acled.R
\name{calc_exposed_population_acled}
\alias{calc_exposed_population_acled}
\title{Calculate population exposed to violent conflict from ACLED}
\usage{
calc_exposed_population_acled(
  distance = 5000,
  filter_category = c("event_type", "sub_event_type", "disorder_type"),
  filter_types = NULL,
  years = c(1997:2024),
  precision_location = 1,
  precision_time = 1
)
}
\arguments{
\item{distance}{A numeric vector indicating the buffer radius in meters.
If length is 1, the same buffer size around included conflict events is
drawn. Otherwise, it must be equal to the length of included categories
selected with \code{filter_types}.}

\item{filter_category}{A character indicating the categories to be used
to calculate the exposed population by. Defaults to \code{event_type} meaning
one estimation per event type will be returned.}

\item{filter_types}{A character vector of event types of the respective
category specified in \code{filter_category} to retain. Defaults to NULL,
meaning that no filter is applied and all types are retained.}

\item{years}{A numeric vector indicating for which years to calculate
the exposed population. Restricted to available years for ACLED.
For years not intersecting with available WorldPop layers, the first layer
is used for earlier years and the last layer to more recent years.}

\item{precision_location}{A numeric indicating precision value for the
geolocation up to which events are included. Defaults to 1.}

\item{precision_time}{A numeric indicating the precision value of the
temporal coding up to which events are included. Defaults to 1.}
}
\value{
A function that returns an indicator tibble with conflict exposure
as variable and percentage of the population as its value.
}
\description{
The indicator calculates the population exposed to conflict events within a
specified buffer distance around events in ACLED. Per default,
the first available WorldPop layer is used to estimate exposed populations
for years before the respective year, while the most recent layer is
used for years after.
}
\details{
The indicator is inspired by the Conflict Exposure tool from ACLED (see
citation below), but differs in the regard that we simply flatten our
buffered event layer instead of applying voronoi tessellation.

The required resources for this indicator are:
\itemize{
\item \link{acled}
\item \link{worldpop}
}

Events in ACLED are classified according to the schema described extensively
in their codebook.
You may filter for certain types of events. The categories for which a
filter can be applied are either "event_type", "event_sub_type", or
"disorder_type". These are translated into the following categories:
\itemize{
\item event_type:
\itemize{
\item battles
\item protests
\item riots
\item explosions/remote_violence
\item violence_against_civilians
\item strategic_developments
}
\item event_sub_type:
\itemize{
\item government_regains_territory
\item non-state_actor_overtakes_territory
\item armed_clash
\item excessive_force_against_protesters
\item protest_with_intervention
\item peaceful_protest
\item violent_demonstration
\item mob_violence
\item chemical_weapon
\item air/drone_strike
\item suicide_bomb
\item shelling/artillery/missile_attack
\item remote_explosive/landmine/ied
\item grenade
\item sexual_violence
\item attack
\item abduction/forced_disappearance
\item agreement
\item arrests
\item change_to_group/activity
\item disrupted_weapons_use
\item headquarters_or_base_established
\item looting/property_destruction
\item non-violent_transfer_of_territory
\item other
}
\item disorder_type:
\itemize{
\item political_violence
\item political_violence;_demonstrations
\item demonstrations
\item political_violence
\item strategic_developments
}
}

You may supply buffer distances for each of the event categories. Custom
buffers will then be drawn per category. Supply a single value if you
do not wish do differentiate between categories. Otherwise, supply a
vector of distances equal to the length of included categories.

You may apply quality filters based on the precision of the geolocation
of events and the temporal precision. By default, these are set to only
include events with the highest precision scores.

For geo-precision there are levels 1 to 3 with decreasing accuracy:
\itemize{
\item value 1: the source reporting indicates a particular town, and coordinates
are available for that town
\item value 2: the source material indicates that activity took place in a
small part of a region, and mentions a general area or if an activity
occurs near a town or a city, the event is coded to a town with
geo-referenced coordinates to represent that area
\item value 3: a larger region is mentioned, the closest natural location noted
in reporting (like “border area,” “forest,” or “sea,” among others) – or a
provincial capital is used if no other information at all is available
}

For temporal precision there are levels 1 to 3 with decreasing precision:
\itemize{
\item value 1: the source material includes an actual date of an event
\item value 2: the source material indicates that an event happened sometime
during the week or within a similar period of time
\item value 3: the source material only indicates that an event took place
sometime during a month (i.e. in the past two or three weeks, or in
January), without reference to the particular date, the month mid-point
is chosen
}
}
\examples{
\dontshow{
mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
}
\dontrun{
if (FALSE) {
  library(sf)
  library(mapme.biodiversity)

  outdir <- file.path(tempdir(), "mapme-data")
  dir.create(outdir, showWarnings = FALSE)

  mapme_options(
    outdir = outdir,
    verbose = FALSE,
    chunk_size = 1e8
  )

  aoi <- system.file("extdata", "burundi.gpkg",
    package = "mapme.biodiversity"
  ) \%>\%
    read_sf() \%>\%
    get_resources(
      get_acled(year = 2000),
      get_worldpop(years = 2000)
    ) \%>\%
    calc_indicators(
      calc_exposed_population_acled(
        distance = 5000,
        years = 2000,
        precision_location = 1,
        precision_time = 1
      )
    ) \%>\%
    portfolio_long()

  aoi
}
}
}
\references{
Raleigh, C; C Dowd; A Tatem; A Linke; N Tejedor-Garavito; M
Bondarenko and K Kishi. 2023. Assessing and Mapping Global and Local
Conflict Exposure. Working Paper.
}
\keyword{indicator}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_gsw.R
\name{global_surface_water_recurrence}
\alias{global_surface_water_recurrence}
\alias{get_global_surface_water_recurrence}
\title{Global Surface Water Recurrence}
\source{
\url{https://global-surface-water.appspot.com/}
}
\usage{
get_global_surface_water_recurrence(version = "v1_4_2021")
}
\arguments{
\item{version}{A character vector indicating the version of the GSW data set
to make available.}
}
\value{
A character of file paths.
}
\description{
The Global Surface Water dataset was developed by the European Commission's
Joint Research Centre in the framework of the Copernicus Programme. It maps
the location and temporal distribution of water surfaces at the global scale
over the past 3.8 decades and provides statistics on their extent and change.
It is provisioned as a global tiled raster resource available for all land
areas. The reported data represent aggregated observations between 1984 - 2021.
}
\details{
Water Recurrence is a measurement of the degree of variability in the
presence of water from year to year. It describes the frequency with which
water returned to a particular location from one year to another, and is
expressed as a percentage. The raster files have integer cell values between
\code{[0, 100]}, where 100 represents that water reoccurs predictably every
year, whereas lower values indicate that water only occurs episodically.
}
\references{
Pekel, JF., Cottam, A., Gorelick, N. et al. High-resolution
mapping of global surface water and its long-term changes. Nature 540,
418–422 (2016). \doi{doi:10.1038/nature20584}
}
\keyword{resource}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_nelson_et_al.R
\name{nelson_et_al}
\alias{nelson_et_al}
\alias{get_nelson_et_al}
\title{Accessibility to Cities layer}
\source{
\url{https://figshare.com/articles/dataset/Travel_time_to_cities_and_ports_in_the_year_2015/7638134/3}
}
\usage{
get_nelson_et_al(ranges = "20k_50k")
}
\arguments{
\item{ranges}{A character vector indicating one or more ranges
to download.}
}
\value{
A function that returns an \code{sf} footprint object.
}
\description{
This resource is published by Weiss et al. (2018) "A global map of travel
time to cities to assess inequalities in accessibility in 2015" on journal
nature. Accessibility is the ease with which larger cities can be reached
from a certain location. This resource represents the travel time to major
cities in the year 2015. Encoded as minutes, representing the time needed
to reach that particular cell from nearby city of target population range.
The following ranges to nearby cities are available:
\itemize{
\item "5k_10k"
\item "10k_20k"
\item "20k_50k"
\item "50k_100k"
\item "100k_200k"
\item "200k_500k"
\item "500k_1mio"
\item "1mio_5mio"
\item "50k_50mio"
\item "5k_110mio"
\item "20k_110mio"
\item "5mio_50mio"
}
}
\note{
Note, that the 'figshare' server applies a rather restrictive rate limit
thus frequently resulting in opaque error codes (see \url{https://github.com/mapme-initiative/mapme.biodiversity/issues/308}).
Please set GDAL configuration options to sensible values in case
you are running into this issue, e.g.: \code{Sys.setenv("GDAL_HTTP_MAX_RETRY" = "5", "GDAL_HTTP_RETRY_DELAY" = "15")}.
}
\references{
Weiss, D. J., Nelson, A., Gibson, H. S., Temperley, W., Peedell,
S., Lieber, A., … & Gething, P. W. (2018). A global map of travel time to cities
to assess inequalities in accessibility in 2015. Nature, 553(7688), 333-336.
}
\keyword{resource}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_gsw_occurrence.R
\name{gsw_occurrence}
\alias{gsw_occurrence}
\alias{calc_gsw_occurrence}
\title{Calculate Global Surface Water (GSW) Occurrence}
\usage{
calc_gsw_occurrence(engine = "extract", min_occurrence = NULL)
}
\arguments{
\item{engine}{The preferred processing functions from either one of "zonal",
"extract" or "exactextract". Default: "extract".}

\item{min_occurrence}{Threshold to define which pixels count towards the GSW
occurrence area \verb{[0, 100]}.}
}
\value{
A function that returns an indicator tibble with occurrence as
variable and the corresponding area (in ha) as value.
}
\description{
GSW occurrence raw data comes in raster files with integer cell values
between \verb{[0, 100]}. This value gives the percentage of the time that a
given pixel was classified as water during the entire observation period. So
a 0 denotes a pixel that was never classified as water, 100 denotes a pixel
with permanent water.
}
\details{
The raw data values are aggregated based on a provided threshold parameter
\code{min_occurrence}, the function returns the area covered by values
greater or equal than this threshold.

The required resources for this indicator are:
\itemize{
\item \link{global_surface_water_occurrence}
}
}
\examples{
\dontshow{
mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
}
\dontrun{
library(sf)
library(mapme.biodiversity)

outdir <- file.path(tempdir(), "mapme-data")
dir.create(outdir, showWarnings = FALSE)

mapme_options(
  outdir = outdir,
  verbose = FALSE
)

aoi <- system.file("extdata", "shell_beach_protected_area_41057_B.gpkg",
  package = "mapme.biodiversity"
) \%>\%
  read_sf() \%>\%
  get_resources(get_global_surface_water_occurrence()) \%>\%
  calc_indicators(
    calc_gsw_occurrence(engine = "extract", min_occurrence = 10)
  ) \%>\%
  portfolio_long()

aoi
}
}
\keyword{indicator}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_population_count.R
\name{population_count}
\alias{population_count}
\alias{calc_population_count}
\title{Calculate population count statistics}
\usage{
calc_population_count(engine = "extract", stats = "sum")
}
\arguments{
\item{engine}{The preferred processing functions from either one of "zonal",
"extract" or "exactextract" as character.}

\item{stats}{Function to be applied to compute statistics for polygons
either one or multiple inputs as character "min", "max", "sum", "mean", "median"
"sd" or "var".}
}
\value{
A function that returns an indicator tibble with the specified
populations statistics as variable and the corresponding values as value.
}
\description{
WorldPop, which was initiated in 2013, offers easy access to spatial demographic
datasets, claiming to use peer-reviewed and fully transparent methods to create
global mosaics for the years 2000 to 2020. This function allows to efficiently
calculate population count statistics (e.g. total number of population) for
polygons. For each polygon, the desired statistic/s (min, max, sum, mean,
median, sd or var) is/are returned.
}
\details{
The required resources for this indicator are:
\itemize{
\item \link{worldpop}
}
}
\examples{
\dontshow{
mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
}
\dontrun{
library(sf)
library(mapme.biodiversity)

outdir <- file.path(tempdir(), "mapme-data")
dir.create(outdir, showWarnings = FALSE)

mapme_options(
  outdir = outdir,
  verbose = FALSE
)

aoi <- system.file("extdata", "sierra_de_neiba_478140_2.gpkg",
  package = "mapme.biodiversity"
) \%>\%
  read_sf() \%>\%
  get_resources(get_worldpop(years = 2010:2020)) \%>\%
  calc_indicators(
    calc_population_count(engine = "extract", stats = c("sum", "median"))
  ) \%>\%
  portfolio_long()

aoi
}
}
\keyword{indicator}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_gsw_seasonality.R
\name{gsw_seasonality}
\alias{gsw_seasonality}
\alias{calc_gsw_seasonality}
\title{Calculate Global Surface Water (GSW) Seasonality}
\usage{
calc_gsw_seasonality()
}
\value{
A function that returns an indicator tibble with seasonality
categories as variables and corresponding areas (in ha) as value.
}
\description{
GSW seasonality describes the intra-annual distribution of surface water for
each pixel. The raster files have integer cell values between \code{[0, 12]},
indicating how many months per year the pixel was classified as water.
}
\details{
The pixel values are aggregated using method provided via the
\code{stats} parameter.

The required resources for this indicator are:
\itemize{
\item \link{global_surface_water_seasonality}
}
}
\examples{
\dontshow{
mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
}
\dontrun{
library(sf)
library(mapme.biodiversity)

outdir <- file.path(tempdir(), "mapme-data")
dir.create(outdir, showWarnings = FALSE)

mapme_options(
  outdir = outdir,
  verbose = FALSE
)

aoi <- system.file("extdata", "shell_beach_protected_area_41057_B.gpkg",
  package = "mapme.biodiversity"
) \%>\%
  read_sf() \%>\%
  get_resources(get_global_surface_water_seasonality()) \%>\%
  calc_indicators(calc_gsw_seasonality()) \%>\%
  portfolio_long()

aoi
}
}
\keyword{indicator}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_treecoverloss_emissions.R
\name{treecoverloss_emissions}
\alias{treecoverloss_emissions}
\alias{calc_treecoverloss_emissions}
\title{Calculate emission statistics}
\usage{
calc_treecoverloss_emissions(years = 2000:2024, min_size = 10, min_cover = 35)
}
\arguments{
\item{years}{A numeric vector with the years for which to calculate emissions
caused by treecover loss.}

\item{min_size}{The minimum size of a forest patch in ha.}

\item{min_cover}{The minimum threshold of stand density for a pixel to be
considered forest in the year 2000.}
}
\value{
A function that returns an indicator tibble with emissions as variable
and emitted CO2 equivalent (in Mg) as value.
}
\description{
This functions allows to efficiently calculate emission statistics for
areas of interest. For each year in the analysis timeframe, the forest losses
from Hansen et al. (2013) are overlayed with the respective emission layer
from Harris et al. (2021) and area-wise emission statistics are calculated
for each year.
}
\details{
The required resources for this indicator are:
\itemize{
\item \link{gfw_treecover}
\item \link{gfw_lossyear}
\item \link{gfw_emissions}
}
}
\examples{
\dontshow{
mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
}
\dontrun{
library(sf)
library(mapme.biodiversity)

outdir <- file.path(tempdir(), "mapme-data")
dir.create(outdir, showWarnings = FALSE)

mapme_options(
  outdir = outdir,
  verbose = FALSE
)

aoi <- system.file("extdata", "sierra_de_neiba_478140_2.gpkg",
  package = "mapme.biodiversity"
) \%>\%
  read_sf() \%>\%
  get_resources(
    get_gfw_treecover(version = "GFC-2024-v1.12"),
    get_gfw_lossyear(version = "GFC-2024-v1.12"),
    get_gfw_emissions()
  ) \%>\%
  calc_indicators(
    calc_treecoverloss_emissions(years = 2016:2017, min_size = 1, min_cover = 30)
  ) \%>\%
  portfolio_long()

aoi
}
}
\keyword{indicator}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_acled.R
\name{acled}
\alias{acled}
\alias{get_acled}
\title{Armed Conflict Location & Event Data (ACLED)}
\source{
Armed Conflict Location & Event Data Project (ACLED).
}
\usage{
get_acled(
  years = 1997,
  email = Sys.getenv("ACLED_EMAIL"),
  password = Sys.getenv("ACLED_PASSWORD"),
  accept_terms = FALSE
)
}
\arguments{
\item{years}{A numeric vector specifying the years for which to make
ACLED data available (between 1997 and today).}

\item{email}{Email addressed used to register with ACLED (see Details).}

\item{password}{Password used to register with ACLED (see Details).}

\item{accept_terms}{A logical indicating if you agree to abide by ACLED's terms
of use. Defaults to FALSE, thus must be manually set to TRUE.}
}
\value{
A function that returns an \code{sf} footprint object.
}
\description{
From ACLED's homepage: \emph{The Armed Conflict Location & Event Data Project
(ACLED) is a disaggregated data collection, analysis, and crisis mapping
project. ACLED collects information on the dates, actors, locations,
fatalities, and types of all reported political violence and protest events
around the world. The ACLED team conducts analysis to describe, explore, and
test conflict scenarios, and makes both data and analysis open for free use
by the public.}
}
\details{
In order to access data from the ACLED API, you first must register
an account. Make sure you register with your institutional domain
(e.g., organization, university, or company) email address rather than
your personal email address, in order to be able to use the API,
as explained \href{https://acleddata.com/myacled-faqs}{here}.
Note that the ACLED API provides a \emph{living database}
with single events being altered or removed altogether over time.
}
\references{
Raleigh, C., Kishi, R. & Linke, A. Political instability patterns
are obscured by conflict dataset scope conditions, sources, and coding
choices. Humanit Soc Sci Commun 10, 74 (2023).
\doi{doi:10.1057/s41599-023-01559-4}
}
\keyword{resource}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_soilproperties.R
\name{soilproperties}
\alias{soilproperties}
\alias{calc_soilproperties}
\title{Calculate Zonal Soil Properties}
\usage{
calc_soilproperties(engine = "extract", stats = "mean")
}
\arguments{
\item{engine}{The preferred processing functions from either one of "zonal",
"extract" or "exactextract" as character.}

\item{stats}{Function to be applied to compute statistics for polygons either
single or multiple inputs as character. Supported statistics are: "mean",
"median", "sd", "min", "max", "sum" "var".}
}
\value{
A function that returns an indicator tibble with soilgrid layers and
statistics as variables and the corresponding statistics as value.
}
\description{
This indicator allows the extraction of zonal statistics for resource layers
previously downloaded from SoilGrids, thus in total supporting the calculation
of zonal statistics for 10 different soil properties at 6 different depths for
a total of 4 different model outputs (stat). Zonal statistics will be calculated
for all SoilGrid layers that have been previously made available vie \code{get_resources()}.
}
\details{
The required resource for this indicator is:
\itemize{
\item \link{soilgrids}
}
}
\examples{
if (FALSE) {
  library(sf)
  library(mapme.biodiversity)

  mapme_options(
    outdir = NULL,
    verbose = FALSE
  )

  aoi <- system.file("extdata", "sierra_de_neiba_478140_2.gpkg",
    package = "mapme.biodiversity"
  ) \%>\%
    read_sf() \%>\%
    get_resources(
      get_soilgrids(
        layers = "clay",
        depths = "0-5cm",
        stats = "mean"
      )
    ) \%>\%
    calc_indicators(
      calc_soilproperties(engine = "extract", stats = c("mean", "median"))
    ) \%>\%
    portfolio_long()

  aoi
}
}
\keyword{indicator}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spatial-utils.R
\name{spds_exists}
\alias{spds_exists}
\title{Check if a spatial data sets exists}
\usage{
spds_exists(path, oo = character(0), what = c("vector", "raster"))
}
\arguments{
\item{path}{A length 1 character vector with a GDAL readable file path.}

\item{oo}{Either a list or a character vector with opening options (-oo)
of the respective GDAL driver. A list must have equal length of the
input sources, a vector will be recycled.}

\item{what}{A character vector indicating if the resource is a vector or raster file.}
}
\value{
A logical, TRUE if the file exists, FALSE if it does not.
}
\description{
This function uses a file path readable by GDAL to check if it can query
it for information. Note, this should also work for remote files, e.g.
in an S3 bucket. You can use this function in your custom resource
function to query if a file is already present at the destination.
Note, that performance will be dependent on your connection to the
server. It can also be used for files on the local file system.
}
\examples{

# a vector resource
vec <- system.file("shape/nc.shp", package = "sf")
spds_exists(vec, what = "vector")

# a raster resource
ras <- system.file("ex/elev.tif", package = "terra")
spds_exists(ras, what = "raster")

# a non existing file
spds_exists("not-here.gpkg", what = "vector")

}
\keyword{utils}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_mangroves_area.R
\docType{data}
\name{mangroves_area}
\alias{mangroves_area}
\alias{calc_mangroves_area}
\title{Calculate mangrove extent based on Global Mangrove Watch (GMW)}
\usage{
calc_mangroves_area()
}
\value{
A function that returns an indicator tibble with mangroves as variable
and corresponding areas (in ha) as value.
}
\description{
This function allows to efficiently calculate area of mangrove from
Global Mangrove Watch - World Conservation Monitoring Centre (WCMC)
for polygons. For each polygon, the area of the mangrove (in hectare)
for desired year is returned.
}
\details{
The required resources for this indicator are:
\itemize{
\item \link{gmw}
}
}
\examples{
\dontshow{
mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
}
\dontrun{
library(sf)
library(mapme.biodiversity)

outdir <- file.path(tempdir(), "mapme-data")
dir.create(outdir, showWarnings = FALSE)

mapme_options(
  outdir = outdir,
  verbose = FALSE
)

aoi <- system.file("extdata", "shell_beach_protected_area_41057_B.gpkg",
  package = "mapme.biodiversity"
) \%>\%
  read_sf() \%>\%
  get_resources(get_gmw(years = c(1996, 2016))) \%>\%
  calc_indicators(calc_mangroves_area()) \%>\%
  portfolio_long()

aoi
}
}
\keyword{indicator}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_landcover.R
\name{landcover}
\alias{landcover}
\alias{calc_landcover}
\title{Calculate area of different landcover classes}
\usage{
calc_landcover()
}
\value{
A function that returns an indicator tibble with landcover classes
as variables and corresponding areas (in ha) as value.
}
\description{
The land cover data shows us how much of the region is covered by forests,
rivers, wetlands, barren land, or urban infrastructure thus allowing the
observation of land cover dynamics over a period of time. This function
allows to efficiently calculate area of different landcover classes for
polygons. For each polygon, the area of the classes in hectare(ha) is
returned.
}
\details{
The required resources for this indicator are:
\itemize{
\item \link{esalandcover}
}
}
\examples{
\dontshow{
mapme.biodiversity:::.copy_resource_dir(file.path(tempdir(), "mapme-data"))
}
\dontrun{
library(sf)
library(mapme.biodiversity)

outdir <- file.path(tempdir(), "mapme-data")
dir.create(outdir, showWarnings = FALSE)

mapme_options(
  outdir = outdir,
  verbose = FALSE
)

aoi <- system.file("extdata", "sierra_de_neiba_478140_2.gpkg",
  package = "mapme.biodiversity"
) \%>\%
  read_sf() \%>\%
  get_resources(get_esalandcover(years = 2016:2017)) \%>\%
  calc_indicators(calc_landcover()) \%>\%
  portfolio_long()

aoi
}
}
\keyword{indicator}
